@@ -79,6 +79,7 @@
 /* Check for non-matching prototypes */
 #include "vmware.h"
 #include "str.h"
+#include "err.h"
 #include "posix.h"
 #include "file.h"
 #include "fileIO.h"
@@ -86,7 +87,7 @@
 #include "config.h"
 #include "util.h"
 #include "iovector.h"
-#include "stats_file.h"
+#include "hostType.h"
 
 #include "unicodeOperations.h"
 #include "memaligned.h"
@@ -172,7 +173,7 @@ static FilePosixOptions filePosixOptions;
  */
 
 static FileIOResult
-FileIOErrno2Result(int error) // IN: errno to convert
+FileIOErrno2Result(int error)  // IN: errno to convert
 {
    switch (error) {
    case EEXIST:
@@ -229,21 +230,26 @@ FileIO_OptionalSafeInitialize(void)
 {
    if (!filePosixOptions.initialized) {
       filePosixOptions.enabled =
-         Config_GetBool(TRUE, "filePosix.coalesce.enable");
+                            Config_GetBool(TRUE, "filePosix.coalesce.enable");
+
       /*
-       * Aligned malloc starts failing to allocate memory during
-       * heavy I/O on Linux.  We're not sure why -- maybe we
-       * are running out of mmaps?  Turn it off by default
-       * for now.
+       * Aligned malloc starts failing to allocate memory during heavy I/O on
+       * Linux. We're not sure why -- maybe we are running out of mmaps?
+       * Turn it off by default for now.
        */
+
       filePosixOptions.aligned =
-         Config_GetBool(FALSE, "filePosix.coalesce.aligned");
+                           Config_GetBool(FALSE, "filePosix.coalesce.aligned");
+
       filePosixOptions.countThreshold =
-         Config_GetLong(5, "filePosix.coalesce.count");
+                           Config_GetLong(5, "filePosix.coalesce.count");
+
       filePosixOptions.sizeThreshold =
-         Config_GetLong(16*1024, "filePosix.coalesce.size");
+                           Config_GetLong(16*1024, "filePosix.coalesce.size");
+
       filePosixOptions.aioNumThreads =
-         Config_GetLong(0, "aiomgr.numThreads");
+                           Config_GetLong(0, "aiomgr.numThreads");
+
       filePosixOptions.initialized = TRUE;
    }
 }
@@ -266,7 +272,7 @@ FileIO_OptionalSafeInitialize(void)
  */
 
 void
-FileIO_Invalidate(FileIODescriptor *fd) // OUT
+FileIO_Invalidate(FileIODescriptor *fd)  // OUT:
 {
    ASSERT(fd);
 
@@ -292,7 +298,7 @@ FileIO_Invalidate(FileIODescriptor *fd) // OUT
  */
 
 Bool
-FileIO_IsValid(const FileIODescriptor *fd)      // IN
+FileIO_IsValid(const FileIODescriptor *fd)  // IN:
 {
    ASSERT(fd);
 
@@ -330,11 +336,6 @@ FileIO_CreateFDPosix(int posix,  // IN: UNIX file descriptor
 
    FileIO_Invalidate(&fd);
 
-#if defined(VMX86_STATS)
-   STATS_USER_INIT_MODULE_ONCE();
-   fd.stats = STATS_USER_INIT_INST("Created");
-#endif
-
    if (flags & O_RDWR) {
       fd.flags |= (FILEIO_OPEN_ACCESS_READ | FILEIO_OPEN_ACCESS_WRITE);
    } else if (flags & O_WRONLY) {
@@ -350,6 +351,7 @@ FileIO_CreateFDPosix(int posix,  // IN: UNIX file descriptor
 #endif
 
    fd.posix = posix;
+
    return fd;
 }
 
@@ -402,9 +404,9 @@ FileIO_GetVolumeSectorSize(ConstUnicode pathName,  // IN:
  */
 
 static void
-ProxySendResults(int sock_fd,    // IN:
-                 int send_fd,    // IN:
-                 int send_errno) // IN:
+ProxySendResults(int sock_fd,     // IN:
+                 int send_fd,     // IN:
+                 int send_errno)  // IN:
 {
    struct iovec iov;
    struct msghdr msg;
@@ -460,9 +462,9 @@ ProxySendResults(int sock_fd,    // IN:
  */
 
 static void
-ProxyReceiveResults(int sock_fd,     // IN:
-                    int *recv_fd,    // OUT:
-                    int *recv_errno) // OUT:
+ProxyReceiveResults(int sock_fd,      // IN:
+                    int *recv_fd,     // OUT:
+                    int *recv_errno)  // OUT:
 {
    int err;
    struct iovec iov;
@@ -609,6 +611,7 @@ ProxyUse(ConstUnicode pathName,  // IN:
    if ((Posix_Lstat(pathName, &statbuf) == 0) &&
        S_ISLNK(statbuf.st_mode)) {
       *useProxy = TRUE;
+
       return 0;
    }
 
@@ -692,6 +695,7 @@ PosixFileOpener(ConstUnicode pathName,  // IN:
       err = ProxyUse(pathName, &useProxy);
       if (err != 0) {
          errno = err;
+
          return -1;
       }
    } else {
@@ -728,11 +732,11 @@ PosixFileOpener(ConstUnicode pathName,  // IN:
  */
 
 FileIOResult
-FileIO_Create(FileIODescriptor *file,    // OUT:
-              ConstUnicode pathName,     // IN:
-              int access,                // IN:
-              FileIOOpenAction action,   // IN:
-              int mode)                  // IN: mode_t for creation
+FileIO_Create(FileIODescriptor *file,   // OUT:
+              ConstUnicode pathName,    // IN:
+              int access,               // IN:
+              FileIOOpenAction action,  // IN:
+              int mode)                 // IN: mode_t for creation
 {
    uid_t uid = -1;
    int fd = -1;
@@ -744,19 +748,10 @@ FileIO_Create(FileIODescriptor *file,    // OUT:
 
    if (pathName == NULL) {
       errno = EFAULT;
+
       return FILEIO_ERROR;
    }
 
-#if defined(VMX86_STATS)
-   {
-      Unicode tmp;
-      File_SplitName(pathName, NULL, NULL, &tmp);
-      STATS_USER_INIT_MODULE_ONCE();
-      file->stats = STATS_USER_INIT_INST(tmp);
-      Unicode_Free(tmp);
-   }
-#endif
-
    ASSERT(!FileIO_IsValid(file));
    ASSERT(file->lockToken == NULL);
    ASSERT_ON_COMPILE(FILEIO_ERROR_LAST < 16); /* See comment in fileIO.h */
@@ -771,7 +766,7 @@ FileIO_Create(FileIODescriptor *file,    // OUT:
     * (FILEIO_OPEN_ACCESS_READ | FILEIO_OPEN_LOCKED) are passed, and we are
     * on VMFS, then pass in special flags to get exclusive, multiwriter, or
     * cross-host read-only mode.  The first if statement is to avoid calling
-    * File_OnVMFS() unless really necessary.
+    * HostType_OSIsVMK() unless really necessary.
     *
     * If the above conditions are met FILEIO_OPEN_LOCKED, is filtered out --
     * vmfs will be handling the locking, so there is no need to create
@@ -782,8 +777,9 @@ FileIO_Create(FileIODescriptor *file,    // OUT:
        (access & (FILEIO_OPEN_ACCESS_READ | FILEIO_OPEN_ACCESS_WRITE |
                   FILEIO_OPEN_LOCKED)) ==
        (FILEIO_OPEN_ACCESS_READ | FILEIO_OPEN_LOCKED)) {
-      if (File_OnVMFS(pathName)) {
+      if (HostType_OSIsVMK()) {
          access &= ~FILEIO_OPEN_LOCKED;
+
          if ((access & FILEIO_OPEN_MULTIWRITER_LOCK) != 0) {
             flags |= O_MULTIWRITER_LOCK;
          } else {
@@ -868,6 +864,7 @@ FileIO_Create(FileIODescriptor *file,    // OUT:
     *   Time Machine backs up EVERY file unless explicitly told not to, so this
     *   option uses the API to exclude the file that was just opened.
     */
+
    if ((access & FILEIO_OPEN_NO_TIME_MACHINE)) {
       if (!FileIO_SetExcludedFromTimeMachine(pathName, TRUE)) {
          ret = FILEIO_ERROR;
@@ -880,10 +877,11 @@ FileIO_Create(FileIODescriptor *file,    // OUT:
        *   10.5.2, so if the user is in 10.5.2 and the option isn't set, then
        *   we want to reset the exclusion of the file.
        *
-       *   Note that this call ignores errors because there are some files (like
-       *   raw devices) that will fail checking xattrs and Time Machine Exclusion
-       *   status, but we can't detect them at this point.
+       *   Note that this call ignores errors because there are some files
+       *   (like raw devices) that will fail checking xattrs and Time
+       *   Machine Exclusion status, but we can't detect them at this point.
        */
+
       FileIO_ResetExcludedFromTimeMachine(pathName);
    }
 #endif
@@ -893,6 +891,7 @@ FileIO_Create(FileIODescriptor *file,    // OUT:
        * Remove the name from the name space. The file remains laid out on the
        * disk and accessible through the file descriptor until it is closed.
        */
+
       if (Posix_Unlink(pathName) == -1) {
          ret = FileIOErrno2Result(errno);
          goto error;
@@ -901,8 +900,6 @@ FileIO_Create(FileIODescriptor *file,    // OUT:
 
    file->posix = fd;
 
-   FileIO_StatsInit(file);
-
    return FILEIO_SUCCESS;
 
 error:
@@ -988,9 +985,9 @@ _llseek(unsigned int fd,
 #endif
 
 uint64
-FileIO_Seek(const FileIODescriptor *file, // IN
-            int64 distance,               // IN
-            FileIOSeekOrigin origin)      // IN
+FileIO_Seek(const FileIODescriptor *file,  // IN:
+            int64 distance,                // IN:
+            FileIOSeekOrigin origin)       // IN:
 {
    ASSERT(file);
 
@@ -1017,12 +1014,13 @@ FileIO_Seek(const FileIODescriptor *file, // IN
     */
 
    loff_t res;
+
    if (_llseek(file->posix, distance >> 32, distance & 0xFFFFFFFF,
                &res, FileIO_SeekOrigins[origin]) == -1) {
       res = -1;
    }
-   return res;
 
+   return res;
 #elif defined(__USE_LARGEFILE64) && !defined(__USE_LARGEFILE)
    /*
     * This is a Linux system with glibc that has lseek64 available, but not a
@@ -1033,7 +1031,6 @@ FileIO_Seek(const FileIODescriptor *file, // IN
     */
 
     return lseek64(file->posix, distance, FileIO_SeekOrigins[origin]);
-
 #else
     /*
      * We're taking this route because either we know lseek() can support
@@ -1045,7 +1042,6 @@ FileIO_Seek(const FileIODescriptor *file, // IN
      */
 
     return lseek(file->posix, distance, FileIO_SeekOrigins[origin]);
-
 #endif
 }
 
@@ -1076,10 +1072,10 @@ FileIO_Seek(const FileIODescriptor *file, // IN
  */
 
 FileIOResult
-FileIO_Write(FileIODescriptor *fd,      // IN
-             const void *bufIn,         // IN
-             size_t requested,          // IN
-             size_t *actual)            // OUT
+FileIO_Write(FileIODescriptor *fd,  // IN:
+             const void *bufIn,     // IN:
+             size_t requested,      // IN:
+             size_t *actual)        // OUT:
 {
    const uint8 *buf = (const uint8 *)bufIn;
    size_t initial_requested;
@@ -1087,20 +1083,12 @@ FileIO_Write(FileIODescriptor *fd,      // IN
 
    ASSERT(fd);
 
-   STAT_INST_INC(fd->stats, NumWrites);
-   STAT_INST_INC_BY(fd->stats, BytesWritten, requested);
-   STATS_ONLY({
-      fd->writeIn++;
-      fd->bytesWritten += requested;
-   })
-
    ASSERT_NOT_IMPLEMENTED(requested < 0x80000000);
 
    initial_requested = requested;
    while (requested > 0) {
       ssize_t res;
 
-      STATS_ONLY(fd->writeDirect++;)
       res = write(fd->posix, buf, requested);
 
       if (res == -1) {
@@ -1147,31 +1135,23 @@ FileIO_Write(FileIODescriptor *fd,      // IN
  */
 
 FileIOResult
-FileIO_Read(FileIODescriptor *fd,       // IN
-            void *bufIn,                // OUT
-            size_t requested,           // IN
-            size_t *actual)             // OUT
+FileIO_Read(FileIODescriptor *fd,  // IN:
+            void *bufIn,           // OUT:
+            size_t requested,      // IN:
+            size_t *actual)        // OUT:
 {
-   uint8 *buf = (uint8 *)bufIn;
+   uint8 *buf = (uint8 *) bufIn;
    size_t initial_requested;
    FileIOResult fret = FILEIO_SUCCESS;
 
    ASSERT(fd);
 
-   STAT_INST_INC(fd->stats, NumReads);
-   STAT_INST_INC_BY(fd->stats, BytesRead, requested);
-   STATS_ONLY({
-      fd->readIn++;
-      fd->bytesRead += requested;
-   })
-
    ASSERT_NOT_IMPLEMENTED(requested < 0x80000000);
 
    initial_requested = requested;
    while (requested > 0) {
       ssize_t res;
 
-      STATS_ONLY(fd->readDirect++;)
       res = read(fd->posix, buf, requested);
       if (res == -1) {
          if (errno == EINTR) {
@@ -1180,7 +1160,7 @@ FileIO_Read(FileIODescriptor *fd,       // IN
          }
          fret = FileIOErrno2Result(errno);
          if (FILEIO_ERROR == fret) {
-            Log("read failed, errno=%d, %s\n", errno, strerror(errno));
+            Log("read failed, errno=%d, %s\n", errno, Err_Errno2String(errno));
          }
          break;
       }
@@ -1219,8 +1199,8 @@ FileIO_Read(FileIODescriptor *fd,       // IN
  */
 
 Bool
-FileIO_Truncate(FileIODescriptor *file, // IN
-                uint64 newLength)       // IN
+FileIO_Truncate(FileIODescriptor *file,  // IN:
+                uint64 newLength)        // IN:
 {
    ASSERT(file);
 
@@ -1247,7 +1227,7 @@ FileIO_Truncate(FileIODescriptor *file, // IN
  */
 
 Bool
-FileIO_Close(FileIODescriptor *file) // IN
+FileIO_Close(FileIODescriptor *file)  // IN:
 {
    int err;
 
@@ -1255,8 +1235,6 @@ FileIO_Close(FileIODescriptor *file) // IN
 
    err = (close(file->posix) == -1) ? errno : 0;
 
-   FileIO_StatsExit(file);
-
    /* Unlock the file if it was locked */
    FileIO_Unlock(file);
    FileIO_Cleanup(file);
@@ -1289,7 +1267,7 @@ FileIO_Close(FileIODescriptor *file) // IN
  */
 
 int
-FileIO_Sync(const FileIODescriptor *file) // IN
+FileIO_Sync(const FileIODescriptor *file)  // IN:
 {
    ASSERT(file);
 
@@ -1321,13 +1299,13 @@ FileIO_Sync(const FileIODescriptor *file) // IN
  */
 
 static Bool
-FileIOCoalesce(struct iovec *inVec,     // IN:  Vector to coalesce from
-               int inCount,             // IN:  count for inVec
-               size_t inTotalSize,      // IN:  totalSize (bytes) in inVec
-               Bool isWrite,            // IN:  coalesce for writing (or reading)
-               Bool forceCoalesce,      // IN:  if TRUE always coalesce
-               int flags,               // IN: fileIO open flags
-               struct iovec *outVec)    // OUT: Coalesced (1-entry) iovec
+FileIOCoalesce(struct iovec *inVec,   // IN:  Vector to coalesce from
+               int inCount,           // IN:  count for inVec
+               size_t inTotalSize,    // IN:  totalSize (bytes) in inVec
+               Bool isWrite,          // IN:  coalesce for writing (or reading)
+               Bool forceCoalesce,    // IN:  if TRUE always coalesce
+               int flags,             // IN: fileIO open flags
+               struct iovec *outVec)  // OUT: Coalesced (1-entry) iovec
 {
    uint8 *cBuf;
 
@@ -1345,6 +1323,7 @@ FileIOCoalesce(struct iovec *inVec,     // IN:  Vector to coalesce from
     * Only coalesce when the number of entries is above our count threshold
     * and the average size of an entry is less than our size threshold
     */
+
    if (!forceCoalesce &&
        (!filePosixOptions.enabled ||
        inCount <= filePosixOptions.countThreshold ||
@@ -1355,6 +1334,7 @@ FileIOCoalesce(struct iovec *inVec,     // IN:  Vector to coalesce from
    // XXX: Wouldn't it be nice if we could log from here!
    //LOG(5, ("FILE: Coalescing %s of %d elements and %d size\n",
    //        isWrite ? "write" : "read", inCount, inTotalSize));
+
    if (filePosixOptions.aligned || flags & FILEIO_OPEN_UNBUFFERED) {
       cBuf = Aligned_Malloc(sizeof(uint8) * inTotalSize);
    } else {
@@ -1370,6 +1350,7 @@ FileIOCoalesce(struct iovec *inVec,     // IN:  Vector to coalesce from
 
    outVec->iov_base = cBuf;
    outVec->iov_len = inTotalSize;
+
    return TRUE;
 }
 
@@ -1442,11 +1423,11 @@ FileIODecoalesce(struct iovec *coVec,   // IN: Coalesced (1-entry) vector
  */
 
 FileIOResult
-FileIO_Readv(FileIODescriptor *fd,      // IN
-             struct iovec *v,           // IN
-             int numEntries,            // IN
-             size_t totalSize,          // IN
-             size_t *actual)            // OUT
+FileIO_Readv(FileIODescriptor *fd,  // IN:
+             struct iovec *v,       // IN:
+             int numEntries,        // IN:
+             size_t totalSize,      // IN:
+             size_t *actual)        // OUT:
 {
    size_t bytesRead = 0, sum = 0;
    FileIOResult fret = FILEIO_ERROR;
@@ -1461,16 +1442,6 @@ FileIO_Readv(FileIODescriptor *fd,      // IN
    didCoalesce = FileIOCoalesce(v, numEntries, totalSize, FALSE,
                                 FALSE, fd->flags, &coV);
 
-   STAT_INST_INC(fd->stats, NumReadvs);
-   STAT_INST_INC_BY(fd->stats, BytesReadv, totalSize);
-   STATS_ONLY({
-      fd->readvIn++;
-      fd->bytesRead += totalSize;
-      if (didCoalesce) {
-         fd->numReadCoalesced++;
-      }
-   })
-
    ASSERT_NOT_IMPLEMENTED(totalSize < 0x80000000);
 
    numVec = didCoalesce ? 1 : numEntries;
@@ -1478,9 +1449,10 @@ FileIO_Readv(FileIODescriptor *fd,      // IN
 
    while (nRetries < maxRetries) {
       ssize_t retval;
+
       ASSERT(numVec > 0);
-      STATS_ONLY(fd->readvDirect++;)
       retval = readv(fd->posix, vPtr, numVec);
+
       if (retval == -1) {
          if (errno == EINTR) {
             NOT_TESTED();
@@ -1498,6 +1470,7 @@ FileIO_Readv(FileIODescriptor *fd,      // IN
          fret = FILEIO_READ_ERROR_EOF;
          break;
       }
+
       /*
        * Ambigous case. Stupid Linux. If the bytesRead matches an
        * exact iovector boundary, we need to retry from the next
@@ -1506,14 +1479,17 @@ FileIO_Readv(FileIODescriptor *fd,      // IN
        * ambiguity handling may need to change.
        * --Ganesh, 08/15/2001.
        */
+
       for (; sum <= bytesRead; vPtr++, numVec--) {
          sum += vPtr->iov_len;
+
          /*
           * In each syscall, we will process atleast one iovec
           * or get an error back. We will therefore retry atmost
           * count times. If multiple iovecs were processed before
           * an error hit, we will retry a lesser number of times.
           */
+
          nRetries++;
       }
       if (sum > bytesRead) {
@@ -1530,6 +1506,7 @@ FileIO_Readv(FileIODescriptor *fd,      // IN
    if (actual) {
       *actual = bytesRead;
    }
+
    return fret;
 }
 
@@ -1558,11 +1535,11 @@ FileIO_Readv(FileIODescriptor *fd,      // IN
  */
 
 FileIOResult
-FileIO_Writev(FileIODescriptor *fd,     // IN
-              struct iovec *v,          // IN
-              int numEntries,           // IN
-              size_t totalSize,         // IN
-              size_t *actual)           // OUT
+FileIO_Writev(FileIODescriptor *fd,  // IN:
+              struct iovec *v,       // IN:
+              int numEntries,        // IN:
+              size_t totalSize,      // IN:
+              size_t *actual)        // OUT:
 {
    size_t bytesWritten = 0, sum = 0;
    FileIOResult fret = FILEIO_ERROR;
@@ -1577,16 +1554,6 @@ FileIO_Writev(FileIODescriptor *fd,     // IN
    didCoalesce = FileIOCoalesce(v, numEntries, totalSize, TRUE,
                                 FALSE, fd->flags, &coV);
 
-   STAT_INST_INC(fd->stats, NumWritevs);
-   STAT_INST_INC_BY(fd->stats, BytesWritev, totalSize);
-   STATS_ONLY({
-      fd->writevIn++;
-      fd->bytesWritten += totalSize;
-      if (didCoalesce) {
-         fd->numWriteCoalesced++;
-      }
-   })
-
    ASSERT_NOT_IMPLEMENTED(totalSize < 0x80000000);
 
    numVec = didCoalesce ? 1 : numEntries;
@@ -1594,9 +1561,10 @@ FileIO_Writev(FileIODescriptor *fd,     // IN
 
    while (nRetries < maxRetries) {
       ssize_t retval;
+
       ASSERT(numVec > 0);
-      STATS_ONLY(fd->writevDirect++;)
       retval = writev(fd->posix, vPtr, numVec);
+
       if (retval == -1) {
          fret = FileIOErrno2Result(errno);
          break;
@@ -1612,10 +1580,12 @@ FileIO_Writev(FileIODescriptor *fd,     // IN
          sum += vPtr->iov_len;
          nRetries++;
       }
+
       /*
        * writev only seems to produce a partial iovec when the disk is
        * out of space.  Just call it an error. --probin
        */
+
       if (sum != bytesWritten) {
          fret = FILEIO_WRITE_ERROR_NOSPC;
          break;
@@ -1629,6 +1599,7 @@ FileIO_Writev(FileIODescriptor *fd,     // IN
    if (actual) {
       *actual = bytesWritten;
    }
+
    return fret;
 }
 
@@ -1655,11 +1626,11 @@ FileIO_Writev(FileIODescriptor *fd,     // IN
  */
 
 FileIOResult
-FileIO_Preadv(FileIODescriptor *fd,    // IN: File descriptor
-              struct iovec *entries,   // IN: Vector to read into
-              int numEntries,          // IN: Number of vector entries
-              uint64 offset,           // IN: Offset to start reading
-              size_t totalSize)        // IN: totalSize (bytes) in entries
+FileIO_Preadv(FileIODescriptor *fd,   // IN: File descriptor
+              struct iovec *entries,  // IN: Vector to read into
+              int numEntries,         // IN: Number of vector entries
+              uint64 offset,          // IN: Offset to start reading
+              size_t totalSize)       // IN: totalSize (bytes) in entries
 {
    size_t sum = 0;
    struct iovec *vPtr;
@@ -1680,21 +1651,10 @@ FileIO_Preadv(FileIODescriptor *fd,    // IN: File descriptor
    count = didCoalesce ? 1 : numEntries;
    vPtr = didCoalesce ? &coV : entries;
 
-   STAT_INST_INC(fd->stats, NumPreadvs);
-   STAT_INST_INC_BY(fd->stats, BytesPreadv, totalSize);
-   STATS_ONLY({
-      fd->preadvIn++;
-      fd->bytesRead += totalSize;
-      if (didCoalesce) {
-         fd->numReadCoalesced++;
-      }
-   })
-
    fileOffset = offset;
    while (count > 0) {
       size_t leftToRead = vPtr->iov_len;
-      uint8 *buf = (uint8 *)vPtr->iov_base;
-      STATS_ONLY(fd->preadDirect++;)
+      uint8 *buf = (uint8 *) vPtr->iov_base;
 
       while (leftToRead > 0) {
          ssize_t retval = pread(fd->posix, buf, leftToRead, fileOffset);
@@ -1779,21 +1739,10 @@ FileIO_Pwritev(FileIODescriptor *fd,   // IN: File descriptor
    count = didCoalesce ? 1 : numEntries;
    vPtr = didCoalesce ? &coV : entries;
 
-   STAT_INST_INC(fd->stats, NumPwritevs);
-   STAT_INST_INC_BY(fd->stats, BytesPwritev, totalSize);
-   STATS_ONLY({
-      fd->pwritevIn++;
-      fd->bytesWritten += totalSize;
-      if (didCoalesce) {
-         fd->numWriteCoalesced++;
-      }
-   })
-
    fileOffset = offset;
    while (count > 0) {
       size_t leftToWrite = vPtr->iov_len;
       uint8 *buf = (uint8 *)vPtr->iov_base;
-      STATS_ONLY(fd->pwriteDirect++;)
 
       while (leftToWrite > 0) {
          ssize_t retval = pwrite(fd->posix, buf, leftToWrite, fileOffset);
@@ -1919,8 +1868,8 @@ FileIO_GetAllocSize(const FileIODescriptor *fd)  // IN
  */
 
 Bool
-FileIO_SetAllocSize(const FileIODescriptor *fd,  // IN
-                    uint64 size)                 // IN
+FileIO_SetAllocSize(const FileIODescriptor *fd,  // IN:
+                    uint64 size)                 // IN:
 {
 
 #if defined(__APPLE__) || defined(__linux__)
@@ -1933,6 +1882,7 @@ FileIO_SetAllocSize(const FileIODescriptor *fd,  // IN
    curSize = FileIO_GetAllocSize(fd);
    if (curSize > size) {
       errno = EINVAL;
+
       return FALSE;
    }
    preallocLen = size - curSize;
@@ -1952,6 +1902,7 @@ FileIO_SetAllocSize(const FileIODescriptor *fd,  // IN
 
 #else
    errno = ENOSYS;
+
    return FALSE;
 #endif
 }
@@ -2004,13 +1955,14 @@ FileIO_GetSizeByPath(ConstUnicode pathName)  // IN:
  */
 
 FileIOResult
-FileIO_Access(ConstUnicode pathName,  // IN: Path name to be tested. May be NULL.
+FileIO_Access(ConstUnicode pathName,  // IN: Path name. May be NULL.
               int accessMode)         // IN: Access modes to be asserted
 {
    int mode = 0;
 
    if (pathName == NULL) {
       errno = EFAULT;
+
       return FILEIO_ERROR;
    }
 
@@ -2048,7 +2000,7 @@ FileIO_Access(ConstUnicode pathName,  // IN: Path name to be tested. May be NULL
  */
 
 uint32
-FileIO_GetFlags(FileIODescriptor *fd)   // IN
+FileIO_GetFlags(FileIODescriptor *fd)  // IN:
 {
    ASSERT(fd);
    ASSERT(FileIO_IsValid(fd));
@@ -2101,12 +2053,14 @@ FileIO_SupportsFileSize(const FileIODescriptor *fd,  // IN:
       newPos = FileIO_Seek(fd, oldPos, FILEIO_SEEK_BEGIN);
       ASSERT_NOT_IMPLEMENTED(oldPos == newPos);
    }
+
    return supported;
 #elif defined(__APPLE__)
    struct statfs buf;
 
    if (fstatfs(fd->posix, &buf) == -1) {
-      Log(LGPFX" %s fstatfs failure: %s\n", __FUNCTION__, strerror(errno));
+      Log(LGPFX" %s fstatfs failure: %s\n", __FUNCTION__,
+          Err_Errno2String(errno));
       /* Be optimistic despite failure */
       return TRUE;
    }
@@ -2144,7 +2098,7 @@ FileIO_SupportsFileSize(const FileIODescriptor *fd,  // IN:
  */
 
 int64
-FileIO_GetModTime(const FileIODescriptor *fd)
+FileIO_GetModTime(const FileIODescriptor *fd)  // IN:
 {
    struct stat statbuf;
 
@@ -2182,6 +2136,7 @@ FileIO_PrivilegedPosixOpen(ConstUnicode pathName,  // IN:
 
    if (pathName == NULL) {
       errno = EFAULT;
+
       return -1;
    }
 
@@ -2297,7 +2252,7 @@ FileIO_DescriptorToStream(FileIODescriptor *fdesc,  // IN:
  */
 
 Bool
-FileIO_ResetExcludedFromTimeMachine(char const *pathName) // IN
+FileIO_ResetExcludedFromTimeMachine(char const *pathName)  // IN:
 {
    bool result = TRUE;
    char xattr;
@@ -2311,7 +2266,7 @@ FileIO_ResetExcludedFromTimeMachine(char const *pathName) // IN
    }
    if (errno != ENOATTR) {
       LOG_ONCE((LGPFX" %s Couldn't get xattr on path [%s]: %s.\n",
-                __func__, pathName, strerror(errno)));
+                __func__, pathName, Err_Errno2String(errno)));
       result = FALSE;
       goto exit;
    }
@@ -2324,7 +2279,7 @@ FileIO_ResetExcludedFromTimeMachine(char const *pathName) // IN
                            &xattr, sizeof(xattr), 0, 0);
    if (sXattrResult == -1) {
       LOG_ONCE((LGPFX" %s Couldn't set xattr on path [%s]: %s.\n",
-                __func__, pathName, strerror(errno)));
+                __func__, pathName, Err_Errno2String(errno)));
       result = FALSE;
       goto exit;
    }
@@ -2352,8 +2307,8 @@ exit:
  */
 
 Bool
-FileIO_SetExcludedFromTimeMachine(char const *pathName, // IN
-                                  Bool isExcluded)      // IN
+FileIO_SetExcludedFromTimeMachine(char const *pathName,  // IN:
+                                  Bool isExcluded)       // IN:
 {
    Bool result = TRUE;
    CSBackupSetItemExcludedFunction *backupFunc;
@@ -2368,25 +2323,23 @@ FileIO_SetExcludedFromTimeMachine(char const *pathName, // IN
       goto exit;
    }
 
-   image = dlopen(libPath, RTLD_LAZY|RTLD_GLOBAL);
+   image = dlopen(libPath, RTLD_LAZY | RTLD_GLOBAL);
 
    if (!image) {
       LOG_ONCE((LGPFX" %s Couldn't dlopen [%s]: %s.\n",
-               __func__, libPath, strerror(errno)));
+               __func__, libPath, Err_Errno2String(errno)));
       goto exit;
    }
 
-   backupFunc = (CSBackupSetItemExcludedFunction *)dlsym(image,
-                                                         symbolName);
+   backupFunc = (CSBackupSetItemExcludedFunction *) dlsym(image, symbolName);
 
    if (!backupFunc) {
       LOG_ONCE((LGPFX" %s Couldn't dlsym [%s]: %s.\n",
-               __func__, symbolName, strerror(errno)));
+               __func__, symbolName, Err_Errno2String(errno)));
       goto exit;
    }
 
-   item = CFURLCreateFromFileSystemRepresentation(NULL,
-                                                  pathName,
+   item = CFURLCreateFromFileSystemRepresentation(NULL, pathName,
                                                   strlen((const char *)pathName),
                                                   FALSE);
 
@@ -2413,6 +2366,7 @@ FileIO_SetExcludedFromTimeMachine(char const *pathName, // IN
        * mark as excluded. Simply logging produces way too many log satements
        * and logging once would only note when this fails on the first file.
        */
+
       goto exit;
    }
 
@@ -2479,6 +2433,7 @@ HostSupportsPrealloc(void)
 
    num = sscanf(curRel, "%u.%u.%u%c%u", &cur[0], &cur[1], &cur[2], &type,
                 &cur[3]);
+
    if (num < 3) {
       goto exit;
    }
@@ -2495,17 +2450,20 @@ HostSupportsPrealloc(void)
       ret = req[3] <= cur[3];
       goto exit;
    }
+
    /*
     * If we get a type with no letter (num == 4), thats odd.
     * Consider it mal-formatted and fail.
     */
+
    ret = num != 4;
 
 exit:
    if (!ret && filePosixOptions.initialized && 
        filePosixOptions.aioNumThreads == 1) {
       ret =TRUE;
-   } 
+   }
+
    return  ret;
 }
 
@@ -2560,8 +2518,8 @@ HostSupportsPrealloc(void)
  */
 
 Bool
-FileIO_SupportsPrealloc(const char *pathName,   // IN
-                        Bool fsCheck)           // IN
+FileIO_SupportsPrealloc(const char *pathName,  // IN:
+                        Bool fsCheck)          // IN:
 {
    Bool ret = TRUE;
 
@@ -2570,7 +2528,7 @@ FileIO_SupportsPrealloc(const char *pathName,   // IN
    }
 
    if (!fsCheck) {
-         return ret;
+      return ret;
    }
 
 #if (defined( __linux__) && !defined(VMX86_SERVER))
@@ -2595,6 +2553,7 @@ FileIO_SupportsPrealloc(const char *pathName,   // IN
       Unicode_Free(fullPath);
    }
 #endif
+
    return ret;
 }
 
