@@ -51,43 +51,20 @@
 #include "fileInt.h"
 #include "util.h"
 #include "str.h"
+#include "err.h"
 #include "vm_version.h"
 #include "localconfig.h"
 #include "hostinfo.h"
 #include "su.h"
+#include "hostType.h"
 
 #include "unicodeOperations.h"
 
-#if defined(VMX86_SERVER)
-#include "hostType.h"
-/*
- * UserWorlds have to handle file locking slightly differently than COS
- * applications, as they are in a different pid space.  The problem is that
- * if a UserWorld tries to write its UserWorld pid to a lock file, a COS
- * program will think the lock file is stale, as that pid won't be valid on
- * the COS. Luckily, every UW is attached to a COS proxy, so, for the
- * purposes of this file, the UW can use its COS proxy's pid as its pid when
- * locking files. This way, if a COS app looks up the proxy pid, it will be
- * valid and the COS app will wait for the lock.
- */
-#include "uwvmk.h"
-#endif
-
 #define LOGLEVEL_MODULE main
 #include "loglevel_user.h"
 
 #define DEVICE_LOCK_DIR "/var/lock"
 
-/*
- * Parameters used by the file library.
- */
-typedef struct FileLockOptions {
-   int lockerPid;
-   Bool userWorld;
-} FileLockOptions;
-
-static FileLockOptions fileLockOptions = { 0, FALSE };
-
 /*
  * XXX
  * Most of these warnings must be turned back into Msg_Appends, or the
@@ -97,30 +74,6 @@ static FileLockOptions fileLockOptions = { 0, FALSE };
  * into the log file.
  */
 
-/*
- *----------------------------------------------------------------------
- *
- * FileLock_Init --
- *
- *      Initialize the file locking library.
- *
- * Results:
- *      None.
- *
- * Side effects:
- *      Allows the user to enable locking on random file systems.
- *
- *----------------------------------------------------------------------
- */
-
-void
-FileLock_Init(int lockerPid,            // IN
-              Bool userWorld)           // IN
-{
-   fileLockOptions.lockerPid = lockerPid;
-   fileLockOptions.userWorld = userWorld;
-}
-
 #if !defined(__FreeBSD__) && !defined(sun)
 /*
  *----------------------------------------------------------------------
@@ -140,24 +93,27 @@ FileLock_Init(int lockerPid,            // IN
  */
 
 static Bool
-IsLinkingAvailable(const char *fileName)     // IN:
+IsLinkingAvailable(const char *fileName)  // IN:
 {
    struct statfs buf;
-   int           status;
+   int status;
 
    ASSERT(fileName);
 
-#if defined(VMX86_SERVER)
-   // Never supported on VMvisor
-   if (HostType_OSIsPureVMK()) {
+   /*
+    * Don't use linking on ESX/VMFS... the overheads are expensive and this
+    * path really isn't used.
+    */
+
+   if (HostType_OSIsVMK()) {
       return FALSE;
    }
-#endif
 
    status = statfs(fileName, &buf);
 
    if (status == -1) {
-      Log(LGPFX" Bad statfs using %s (%s).\n", fileName, strerror(errno));
+      Log(LGPFX" Bad statfs using %s (%s).\n", fileName,
+          Err_Errno2String(errno));
 
       return FALSE;
    }
@@ -194,7 +150,6 @@ IsLinkingAvailable(const char *fileName)     // IN:
    case TMPFS_SUPER_MAGIC:
    case JFS_SUPER_MAGIC:
         return TRUE;                        // these are known to work
-   case VMFS_SUPER_MAGIC:
    case SMB_SUPER_MAGIC:
    case MSDOS_SUPER_MAGIC:
         return FALSE;
@@ -212,45 +167,6 @@ IsLinkingAvailable(const char *fileName)     // IN:
 }
 
 
-/*
- *---------------------------------------------------------------------------
- *
- * FileLockGetPid --
- *
- *      Returns the pid of the main thread if we're running in a
- *      multithreaded process, otherwise return the result of getpid().
- *
- * Results:
- *      a pid.
- *
- * Side effects:
- *      None.
- *
- *---------------------------------------------------------------------------
- */
-
-static int
-FileLockGetPid(void)
-{
-#if defined(VMX86_SERVER)
-   /*
-    * For a UserWorld, we want to get this cartel's proxy's cos pid.
-    */
-   if (fileLockOptions.userWorld) {
-      int pid, err;
-
-      err = VMKernel_GetLockPid(&pid);
-      ASSERT_NOT_IMPLEMENTED(err == 0);
-
-      return pid;
-   }
-#endif
-
-   return fileLockOptions.lockerPid > 0 ?
-                         fileLockOptions.lockerPid : (int) getpid();
-}
-
-
 /*
  *----------------------------------------------------------------------
  *
@@ -268,7 +184,7 @@ FileLockGetPid(void)
  */
 
 static Bool
-RemoveStaleLockFile(const char *lockFileName)      // IN:
+RemoveStaleLockFile(const char *lockFileName)  // IN:
 {
    uid_t uid;
    int ret;
@@ -287,7 +203,7 @@ RemoveStaleLockFile(const char *lockFileName)      // IN:
 
    if (ret < 0) {
       Warning(LGPFX" Failed to remove stale lock file %s (%s).\n",
-              lockFileName, strerror(saveErrno));
+              lockFileName, Err_Errno2String(saveErrno));
 
       return FALSE;
    }
@@ -313,9 +229,9 @@ RemoveStaleLockFile(const char *lockFileName)      // IN:
  */
 
 static int
-GetLockFileValues(const char *lockFileName, // IN:
-                  int *pid,                 // OUT:
-                  char *hostID)             // OUT:
+GetLockFileValues(const char *lockFileName,  // IN:
+                  int *pid,                  // OUT:
+                  char *hostID)              // OUT:
 {
    char *p;
    int  saveErrno;
@@ -336,7 +252,7 @@ GetLockFileValues(const char *lockFileName, // IN:
 
    if (lockFile == NULL) {
       Warning(LGPFX" Failed to open existing lock file %s (%s).\n",
-              lockFileName, strerror(saveErrno));
+              lockFileName, Err_Errno2String(saveErrno));
 
       return (saveErrno == ENOENT) ? 0 : -1;
    }
@@ -348,7 +264,7 @@ GetLockFileValues(const char *lockFileName, // IN:
 
    if (p == NULL) {
       Warning(LGPFX" Failed to read line from lock file %s (%s).\n",
-              lockFileName, strerror(saveErrno));
+              lockFileName, Err_Errno2String(saveErrno));
 
       deleteLockFile = TRUE;
    } else {
@@ -401,12 +317,6 @@ FileLockIsValidProcess(int pid)  // IN:
    int err;
    Bool ret;
 
-#if defined(VMX86_SERVER)
-   if (fileLockOptions.userWorld) {
-      return (VMKernel_IsLockPidAlive(pid) == 0) ? TRUE : FALSE;
-   }
-#endif
-
    uid = Id_BeginSuperUser();
    err = (kill(pid, 0) == -1) ? errno : 0;
    Id_EndSuperUser(uid);
@@ -469,7 +379,7 @@ FileLockCreateLockFile(const char *lockFileName,  // IN:
 
       if (lockFD == -1) {
          Log(LGPFX" Failed to create new lock file %s (%s).\n",
-             lockFileLink, strerror(saveErrno));
+             lockFileLink, Err_Errno2String(saveErrno));
 
          return (saveErrno == EEXIST) ? 0 : -1;
       }
@@ -490,7 +400,7 @@ FileLockCreateLockFile(const char *lockFileName,  // IN:
 
       if (lockFD == -1) {
          Log(LGPFX" Failed to create new lock file %s (%s).\n",
-             lockFileName, strerror(saveErrno));
+             lockFileName, Err_Errno2String(saveErrno));
 
          return (saveErrno == EEXIST) ? 0 : -1;
       }
@@ -503,7 +413,7 @@ FileLockCreateLockFile(const char *lockFileName,  // IN:
 
    if (err != strlen(uniqueID)) {
       Warning(LGPFX" Failed to write to new lock file %s (%s).\n",
-              lockFileName, strerror(saveErrno));
+              lockFileName, Err_Errno2String(saveErrno));
       status = -1;
       goto exit;
    }
@@ -524,7 +434,7 @@ exit:
 
       if (err < 0) {
          Warning(LGPFX" Failed to remove temporary lock file %s (%s).\n",
-                 lockFileLink, strerror(errno));
+                 lockFileLink, Err_Errno2String(errno));
       }
 
    }
@@ -556,7 +466,7 @@ exit:
  */
 
 int
-FileLock_LockDevice(const char *deviceName)   // IN:
+FileLock_LockDevice(const char *deviceName)  // IN:
 {
    const char *hostID;
    char       uniqueID[1000];
@@ -571,14 +481,14 @@ FileLock_LockDevice(const char *deviceName)   // IN:
                                    deviceName);
 
    lockFileLink = Str_SafeAsprintf(NULL, "%s/LTMP..%s.t%05d", DEVICE_LOCK_DIR,
-                                   deviceName, FileLockGetPid());
+                                   deviceName, getpid());
 
    LOG(1, ("Requesting lock %s (temp = %s).\n", lockFileName,
            lockFileLink));
 
    hostID = FileLockGetMachineID();
    Str_Sprintf(uniqueID, sizeof uniqueID, "%d %s\n",
-               FileLockGetPid(), hostID);
+               getpid(), hostID);
 
    while ((status = FileLockCreateLockFile(lockFileName, lockFileLink,
                                            uniqueID)) == 0) {
@@ -666,7 +576,7 @@ FileLock_UnlockDevice(const char *deviceName)  // IN:
 
    if (ret < 0) {
       Log(LGPFX" Cannot remove lock file %s (%s).\n",
-          path, strerror(saveErrno));
+          path, Err_Errno2String(saveErrno));
       free(path);
 
       return FALSE;
@@ -696,9 +606,9 @@ FileLock_UnlockDevice(const char *deviceName)  // IN:
  */
 
 static int
-ReadSlashProc(const char *procPath, // IN:
-              char *buffer,         // OUT:
-              size_t bufferSize)    // IN:
+ReadSlashProc(const char *procPath,  // IN:
+              char *buffer,          // OUT:
+              size_t bufferSize)     // IN:
 {
    int fd;
    int err;
@@ -754,11 +664,11 @@ ReadSlashProc(const char *procPath, // IN:
  */
 
 static uint64
-ProcessCreationTime(pid_t pid)
+ProcessCreationTime(pid_t pid)  // IN:
 {
-   int    err;
-   char   path[64];
-   char   buffer[1024];
+   int err;
+   char path[64];
+   char buffer[1024];
    uint64 creationTime;
 
    Str_Sprintf(path, sizeof path, "/proc/%d/stat", pid);
@@ -805,12 +715,12 @@ ProcessCreationTime(pid_t pid)
 }
 #elif defined(__APPLE__)
 static uint64
-ProcessCreationTime(pid_t pid)
+ProcessCreationTime(pid_t pid)  // IN:
 {
-    int                err;
-    size_t             size;
+    int err;
+    size_t size;
     struct kinfo_proc  info;
-    int                mib[4];
+    int mib[4];
   
     /* Request information about the specified process */
     mib[0] = CTL_KERN;
@@ -825,7 +735,7 @@ ProcessCreationTime(pid_t pid)
     /* Log any failures */
     if (err == -1) {
        Warning(LGPFX" %s sysctl for pid %d failed: %s\n", __FUNCTION__,
-               pid, strerror(errno));
+               pid, Err_Errno2String(errno));
 
        return 0;
     }
@@ -836,7 +746,7 @@ ProcessCreationTime(pid_t pid)
 }
 #else
 static uint64
-ProcessCreationTime(pid_t pid)
+ProcessCreationTime(pid_t pid)  // IN:
 {
    return 0;
 }
@@ -861,8 +771,8 @@ ProcessCreationTime(pid_t pid)
  */
 
 Bool
-FileLockValidOwner(const char *executionID, // IN:
-                   const char *payload)     // IN:
+FileLockValidOwner(const char *executionID,  // IN:
+                   const char *payload)      // IN:
 {
    int pid;
 
@@ -927,9 +837,9 @@ FileLockValidOwner(const char *executionID, // IN:
  */
 
 int
-FileLockOpenFile(ConstUnicode pathName,        // IN:
-                 int flags,                    // IN:
-                 FILELOCK_FILE_HANDLE *handle) // OUT:
+FileLockOpenFile(ConstUnicode pathName,         // IN:
+                 int flags,                     // IN:
+                 FILELOCK_FILE_HANDLE *handle)  // OUT:
 {
    ASSERT(pathName);
 
@@ -957,7 +867,7 @@ FileLockOpenFile(ConstUnicode pathName,        // IN:
  */
 
 int
-FileLockCloseFile(FILELOCK_FILE_HANDLE handle) // IN:
+FileLockCloseFile(FILELOCK_FILE_HANDLE handle)  // IN:
 {
    return (close(handle) == -1) ? errno : 0;
 }
@@ -1064,7 +974,7 @@ FileLockWriteFile(FILELOCK_FILE_HANDLE handle,  // IN:
 char *
 FileLockGetExecutionID(void)
 {
-   return Str_SafeAsprintf(NULL, "%d", FileLockGetPid());
+   return Str_SafeAsprintf(NULL, "%d", getpid());
 }
 
 
@@ -1078,7 +988,7 @@ FileLockGetExecutionID(void)
  *      points to.
  *
  * Results:
- *      The normalized path
+ *      The normalized path or NULL on error
  *
  * Side effects:
  *      None
@@ -1090,41 +1000,28 @@ static Unicode
 FileLockNormalizePath(ConstUnicode filePath)  // IN:
 {
    Unicode result;
-   struct stat statbuf;
+   Unicode fullPath;
 
-   if ((Posix_Lstat(filePath, &statbuf) == 0) &&
-        (S_ISLNK(statbuf.st_mode))) {
-      Unicode fullPath;
-      Unicode baseDir = NULL;
-      Unicode fileName = NULL;
+   Unicode dirName = NULL;
+   Unicode fileName = NULL;
 
-      /*
-       * The file to be locked is a symbolic path. Place the lock file
-       * next to the file, not where the symbolic path points to.
-       */
+   /*
+    * If the file to be locked is a symbolic link the lock file belongs next
+    * to the symbolic link, not "off" where the symbolic link points to.
+    * Translation: Don't "full path" the entire path of the file to be locked;
+    * "full path" the dirName of the path, leaving the fileName alone.
+    */
 
-       File_GetPathName(filePath, &baseDir, &fileName);
+   File_GetPathName(filePath, &dirName, &fileName);
 
-       fullPath = File_FullPath(baseDir);
+   fullPath = File_FullPath(dirName);
 
-       if (fullPath == NULL) {
-          result = NULL;
-       } else {
-          result = Unicode_Join(fullPath, DIRSEPS, fileName, NULL);
+   result = (fullPath == NULL) ? NULL : Unicode_Join(fullPath, DIRSEPS,
+                                                     fileName, NULL);
 
-          Unicode_Free(fullPath);
-       }
-
-       Unicode_Free(baseDir);
-       Unicode_Free(fileName);
-   } else {
-      /*
-       * The stat failed or the file is not a symbolic link - do a fullpath
-       * on the argument path to normalize it.
-       */
-
-      result = File_FullPath(filePath);
-   }
+   Unicode_Free(fullPath);
+   Unicode_Free(dirName);
+   Unicode_Free(fileName);
 
    return result;
 }
@@ -1165,7 +1062,6 @@ FileLock_Lock(ConstUnicode filePath,         // IN:
 {
    void *lockToken;
    Unicode normalizedPath;
-   char creationTimeString[32];
 
    ASSERT(filePath);
    ASSERT(err);
@@ -1174,17 +1070,19 @@ FileLock_Lock(ConstUnicode filePath,         // IN:
    if (normalizedPath == NULL) {
       *err = EINVAL;
 
-      return NULL;
+      lockToken = NULL;
+   } else {
+      char creationTimeString[32];
+
+      Str_Sprintf(creationTimeString, sizeof creationTimeString, "%"FMT64"u",
+                  ProcessCreationTime(getpid()));
+
+      lockToken = FileLockIntrinsic(normalizedPath, !readOnly, msecMaxWaitTime,
+                                    creationTimeString, err);
+
+      Unicode_Free(normalizedPath);
    }
 
-   Str_Sprintf(creationTimeString, sizeof creationTimeString, "%"FMT64"u",
-               ProcessCreationTime(FileLockGetPid()));
-
-   lockToken = FileLockIntrinsic(normalizedPath, !readOnly, msecMaxWaitTime,
-                                 creationTimeString, err);
-
-   Unicode_Free(normalizedPath);
-
    return lockToken;
 }
 
@@ -1221,13 +1119,13 @@ FileLock_IsLocked(ConstUnicode filePath,  // IN:
          *err = EINVAL;
       }
 
-      return FALSE;
+      isLocked = FALSE;
+   } else {
+      isLocked = FileLockIsLocked(normalizedPath, err);
+
+      Unicode_Free(normalizedPath);
    }
 
-   isLocked = FileLockIsLocked(normalizedPath, err);
-
-   Unicode_Free(normalizedPath);
-
    return isLocked;
 }
 
@@ -1261,13 +1159,13 @@ FileLock_Unlock(ConstUnicode filePath,  // IN:
 
    normalizedPath = FileLockNormalizePath(filePath);
    if (normalizedPath == NULL) {
-      return EINVAL;
+      err = EINVAL;
+   } else {
+      err = FileUnlockIntrinsic(normalizedPath, lockToken);
+
+      Unicode_Free(normalizedPath);
    }
 
-   err = FileUnlockIntrinsic(normalizedPath, lockToken);
-
-   Unicode_Free(normalizedPath);
-
    return err;
 }
 
@@ -1302,13 +1200,13 @@ FileLock_DeleteFileVMX(ConstUnicode filePath)  // IN:
 
    normalizedPath = FileLockNormalizePath(filePath);
    if (normalizedPath == NULL) {
-      return EINVAL;
+      err = EINVAL;
+   } else {
+      err = FileLockHackVMX(normalizedPath);
+
+      Unicode_Free(normalizedPath);
    }
 
-   err = FileLockHackVMX(normalizedPath);
-
-   Unicode_Free(normalizedPath);
-
    return err;
 }
 
