@@ -25,6 +25,7 @@
  * if the directory contains any files.
  */
 
+#define _GNU_SOURCE /* For O_NOFOLLOW */
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -121,12 +122,11 @@ Sleeper(LockValues *myValues,  // IN/OUT:
 
    myValues->waitTime += msecSleepTime;
 
+   /* Clamp individual sleeps to avoid Windows issues */
    while (msecSleepTime) {
       uint32 sleepTime = (msecSleepTime > 900) ? 900 : msecSleepTime;
 
-      FileSleeper(sleepTime);
-
-      msecSleepTime -= sleepTime;
+      msecSleepTime -= FileSleeper(sleepTime, sleepTime);
    }
 
    return 0;
@@ -170,7 +170,7 @@ RemoveLockingFile(ConstUnicode lockDir,   // IN:
          err = 0;
       } else {
          Warning(LGPFX" %s of '%s' failed: %s\n", __FUNCTION__,
-                 UTF8(path), strerror(err));
+                 UTF8(path), Err_Errno2String(err));
       }
    }
 
@@ -302,7 +302,7 @@ FileLockMemberValues(ConstUnicode lockDir,      // IN:
 
       if (err != ENOENT) {
          Warning(LGPFX" %s open failure on '%s': %s\n", __FUNCTION__,
-                 UTF8(path), strerror(err));
+                 UTF8(path), Err_Errno2String(err));
       }
 
       goto bail;
@@ -313,7 +313,7 @@ FileLockMemberValues(ConstUnicode lockDir,      // IN:
 
    if (err != 0) {
       Warning(LGPFX" %s file size failure on '%s': %s\n", __FUNCTION__,
-              UTF8(path), strerror(err));
+              UTF8(path), Err_Errno2String(err));
 
       FileLockCloseFile(handle);
 
@@ -337,7 +337,7 @@ FileLockMemberValues(ConstUnicode lockDir,      // IN:
 
    if (err != 0) {
       Warning(LGPFX" %s read failure on '%s': %s\n",
-              __FUNCTION__, UTF8(path), strerror(err));
+              __FUNCTION__, UTF8(path), Err_Errno2String(err));
 
       goto bail;
    }
@@ -883,7 +883,8 @@ Scanner(ConstUnicode lockDir,    // IN:
          ptr = ptr->next;
       }
 
-      FileSleeper(FILELOCK_PROGRESS_SAMPLE); // relax
+      FileSleeper(FILELOCK_PROGRESS_SAMPLE,
+                  FILELOCK_PROGRESS_SAMPLE); // relax
    }
 
    /* Clean up anything still on the list; they are no longer important */
@@ -952,7 +953,7 @@ FileUnlockIntrinsic(ConstUnicode pathName,  // IN:
 
       if (err && vmx86_debug) {
          Log(LGPFX" %s failed for '%s': %s\n",
-             __FUNCTION__, (char *) lockToken, strerror(err));
+             __FUNCTION__, (char *) lockToken, Err_Errno2String(err));
       }
 
       /*
@@ -1099,62 +1100,6 @@ NumberScan(ConstUnicode lockDir,      // IN:
 }
 
 
-/*
- *-----------------------------------------------------------------------------
- *
- * SimpleRandomNumber --
- *
- *      Return a random number in the range of 0 and 2^16-1.
- *
- * Results:
- *      Random number is returned.
- *
- * Side Effects:
- *      None.
- *
- *-----------------------------------------------------------------------------
- */
-
-static uint32
-SimpleRandomNumber(const char *machineID,    // IN:
-                   const char *executionID)  // IN:
-{
-   static Atomic_Ptr atomic; /* Implicitly initialized to NULL. --mbellon */
-   char *context;
-
-   context = Atomic_ReadPtr(&atomic);
-
-   if (context == NULL) {
-      void *p;
-      uint32 value = 0;
-
-      /*
-       * Use the machineID and executionID to hopefully start each machine
-       * and process/thread at a different place in the answer stream.
-       */
-
-      while (*machineID) {
-         value += *machineID++;
-      }
-
-      while (*executionID) {
-         value += *executionID++;
-      }
-
-      p = Random_QuickSeed(value);
-
-      if (Atomic_ReadIfEqualWritePtr(&atomic, NULL, p)) {
-         free(p);
-      }
-
-      context = Atomic_ReadPtr(&atomic);
-      ASSERT(context);
-   }
-
-   return (Random_Quick(context) >> 8) & 0xFFFF;
-}
-
-
 /*
  *-----------------------------------------------------------------------------
  *
@@ -1221,9 +1166,7 @@ MakeDirectory(ConstUnicode pathName)  // IN:
  */
 
 static int
-CreateEntryDirectory(const char *machineID,    // IN:
-                     const char *executionID,  // IN:
-                     ConstUnicode lockDir,     // IN:
+CreateEntryDirectory(ConstUnicode lockDir,     // IN:
                      Unicode *entryDirectory,  // OUT:
                      Unicode *entryFilePath,   // OUT:
                      Unicode *memberFilePath,  // OUT:
@@ -1282,20 +1225,20 @@ CreateEntryDirectory(const char *machineID,    // IN:
 
             if ((err != 0) && (err != EEXIST)) {
                Warning(LGPFX" %s creation failure on '%s': %s\n",
-                       __FUNCTION__, UTF8(lockDir), strerror(err));
+                       __FUNCTION__, UTF8(lockDir), Err_Errno2String(err));
 
                break;
             }
          } else {
             Warning(LGPFX" %s stat failure on '%s': %s\n",
-                    __FUNCTION__, UTF8(lockDir), strerror(err));
+                    __FUNCTION__, UTF8(lockDir), Err_Errno2String(err));
 
             break;
          }
       }
 
       /* There is a small chance of collision/failure; grab stings now */
-      randomNumber = SimpleRandomNumber(machineID, executionID);
+      randomNumber = (FileSimpleRandom() >> 8) & 0xFFFF;
 
       *memberName = Unicode_Format("M%05u%s", randomNumber, FILELOCK_SUFFIX);
 
@@ -1332,7 +1275,7 @@ CreateEntryDirectory(const char *machineID,    // IN:
 
             if (vmx86_debug) {
                Log(LGPFX" %s stat failure on '%s': %s\n",
-                   __FUNCTION__, UTF8(*memberFilePath), strerror(err));
+                   __FUNCTION__, UTF8(*memberFilePath), Err_Errno2String(err));
              }
          }
 
@@ -1341,7 +1284,8 @@ CreateEntryDirectory(const char *machineID,    // IN:
           if ((err != EEXIST) &&  // Another process/thread created it...
               (err != ENOENT)) {  // lockDir is gone...
              Warning(LGPFX" %s creation failure on '%s': %s\n",
-                     __FUNCTION__, UTF8(*entryDirectory), strerror(err));
+                     __FUNCTION__, UTF8(*entryDirectory),
+                     Err_Errno2String(err));
 
              break;
           }
@@ -1429,7 +1373,7 @@ CreateMemberFile(FILELOCK_FILE_HANDLE entryHandle,  // IN:
 
    if (err != 0) {
       Warning(LGPFX" %s write of '%s' failed: %s\n", __FUNCTION__,
-              UTF8(entryFilePath), strerror(err));
+              UTF8(entryFilePath), Err_Errno2String(err));
 
       FileLockCloseFile(entryHandle);
 
@@ -1440,7 +1384,7 @@ CreateMemberFile(FILELOCK_FILE_HANDLE entryHandle,  // IN:
 
    if (err != 0) {
       Warning(LGPFX" %s close of '%s' failed: %s\n", __FUNCTION__,
-              UTF8(entryFilePath), strerror(err));
+              UTF8(entryFilePath), Err_Errno2String(err));
 
       return err;
    }
@@ -1457,16 +1401,16 @@ CreateMemberFile(FILELOCK_FILE_HANDLE entryHandle,  // IN:
    if (err != 0) {
       Warning(LGPFX" %s FileRename of '%s' to '%s' failed: %s\n",
               __FUNCTION__, UTF8(entryFilePath), UTF8(memberFilePath),
-              strerror(err));
+              Err_Errno2String(err));
 
       if (vmx86_debug) {
          Log(LGPFX" %s FileLockFileType() of '%s': %s\n",
              __FUNCTION__, UTF8(entryFilePath),
-            strerror(FileAttributesRobust(entryFilePath, NULL)));
+            Err_Errno2String(FileAttributesRobust(entryFilePath, NULL)));
 
          Log(LGPFX" %s FileLockFileType() of '%s': %s\n",
              __FUNCTION__, UTF8(memberFilePath),
-            strerror(FileAttributesRobust(memberFilePath, NULL)));
+            Err_Errno2String(FileAttributesRobust(memberFilePath, NULL)));
       }
 
       return err;
@@ -1511,6 +1455,7 @@ FileLockIntrinsic(ConstUnicode pathName,   // IN:
 {
    FILELOCK_FILE_HANDLE handle;
    LockValues myValues;
+   int createFlags;
 
    Unicode lockDir = NULL;
    Unicode entryFilePath = NULL;
@@ -1544,9 +1489,7 @@ FileLockIntrinsic(ConstUnicode pathName,   // IN:
     * entry and member path names.
     */
 
-   *err = CreateEntryDirectory(myValues.machineID, myValues.executionID,
-                               lockDir,
-                               &entryDirectory, &entryFilePath,
+   *err = CreateEntryDirectory(lockDir, &entryDirectory, &entryFilePath,
                                &memberFilePath, &myValues.memberName);
 
    switch (*err) {
@@ -1579,7 +1522,11 @@ FileLockIntrinsic(ConstUnicode pathName,   // IN:
           Unicode_LengthInCodeUnits(pathName) <= FILELOCK_OVERHEAD);
 
    /* Attempt to create the entry file */
-   *err = FileLockOpenFile(entryFilePath, O_CREAT | O_WRONLY, &handle);
+   createFlags = O_CREAT | O_WRONLY; 
+#ifndef _WIN32
+   createFlags |= O_NOFOLLOW;
+#endif
+   *err = FileLockOpenFile(entryFilePath, createFlags, &handle);
 
    if (*err != 0) {
       /* clean up */
@@ -1681,7 +1628,7 @@ FileLockIsLocked(ConstUnicode pathName,  // IN:
                  int *err)               // OUT:
 {
    uint32 i;
-   int errValue;
+   int errValue = 0;
    int numEntries;
    Unicode lockDir;
 
@@ -1693,7 +1640,13 @@ FileLockIsLocked(ConstUnicode pathName,  // IN:
    numEntries = FileListDirectoryRobust(lockDir, &fileList);
 
    if (numEntries == -1) {
-      errValue = errno;
+      /*
+       * If the lock directory doesn't exist, we should not count this
+       * as an error.  This is expected if the file isn't locked.
+       */
+      if (errno != ENOENT) {
+         errValue = errno;
+      }
 
       goto bail;
    }
@@ -1711,8 +1664,6 @@ FileLockIsLocked(ConstUnicode pathName,  // IN:
 
    free(fileList);
 
-   errValue = 0;
-
 bail:
    Unicode_Free(lockDir);
 
@@ -1802,9 +1753,7 @@ FileLockHackVMX(ConstUnicode pathName)  // IN:
    LOG(1, ("%s on %s (%s, %s).\n", __FUNCTION__, UTF8(pathName),
        myValues.machineID, myValues.executionID));
 
-   err = CreateEntryDirectory(myValues.machineID, myValues.executionID,
-                              lockDir,
-                              &entryDirectory, &entryFilePath,
+   err = CreateEntryDirectory(lockDir, &entryDirectory, &entryFilePath,
                               &memberFilePath, &myValues.memberName);
 
    if (err != 0) {
@@ -1822,7 +1771,7 @@ FileLockHackVMX(ConstUnicode pathName)  // IN:
    } else {
       if (vmx86_debug) {
          Warning(LGPFX" %s clean-up failure for '%s': %s\n",
-                 __FUNCTION__, UTF8(pathName), strerror(err));
+                 __FUNCTION__, UTF8(pathName), Err_Errno2String(err));
       }
    }
 
