@@ -66,6 +66,8 @@
 #include "timeutil.h"
 #include "dynbuf.h"
 #include "localconfig.h"
+#include "hostType.h"
+#include "vmfs.h"
 
 #include "unicodeOperations.h"
 
@@ -112,6 +114,11 @@ struct WalkDirContextImpl
 
 #endif
 
+/* A string for NFS on ESX file system type */
+#define FS_NFS_ON_ESX "NFS"
+/* A string for VMFS on ESX file system type */
+#define FS_VMFS_ON_ESX "VMFS"
+
 
 /*
  *-----------------------------------------------------------------------------
@@ -166,8 +173,8 @@ FileRename(ConstUnicode oldName,  // IN:
  *----------------------------------------------------------------------
  *
  *  FileDeletion --
- *	     Delete the specified file.  A NULL pathName will result in an error
- *	     and errno will be set to EFAULT.
+ *      Delete the specified file.  A NULL pathName will result in an error
+ *      and errno will be set to EFAULT.
  *
  * Results:
  *	0	success
@@ -189,12 +196,14 @@ FileDeletion(ConstUnicode pathName,   // IN:
 
    if (pathName == NULL) {
       errno = EFAULT;
+
       return errno;
    } else if ((primaryPath = Unicode_GetAllocBytes(pathName,
                                STRING_ENCODING_DEFAULT)) == NULL) {
       Log(LGPFX" %s: failed to convert \"%s\" to current encoding\n",
           __FUNCTION__, UTF8(pathName));
       errno = UNICODE_CONVERSION_ERRNO;
+
       return errno;
    }
 
@@ -231,6 +240,7 @@ FileDeletion(ConstUnicode pathName,   // IN:
 bail:
    free(primaryPath);
    free(linkPath);
+
    return err;
 }
 
@@ -349,8 +359,22 @@ FileAttributes(ConstUnicode pathName,  // IN:
  * File_IsRemote --
  *
  *      Determine whether a file is on a remote filesystem.
- *      In case of an error be conservative and assume that 
- *      the file is a remote file.
+ *
+ *      On ESX all files are treated as local files, as all
+ *      callers of this function wants to do is to post message
+ *      that performance will be degraded on remote filesystems.
+ *      On ESX (a) performance should be acceptable with remote
+ *      files, and (b) even if it is not, we should not ask users
+ *      whether they are aware that it is poor.  ESX has
+ *      performance monitoring which can notify user if something
+ *      is wrong.
+ *
+ *      On hosted platform we report remote files as faithfully
+ *      as we can because having mainmem file on NFS is known
+ *      to badly affect VM consistency when NFS filesystem gets
+ *      reconnected.  Due to that we are conservative, and report
+ *      filesystem as remote if there was some problem with
+ *      determining file remoteness.
  *
  * Results:
  *      The answer.
@@ -365,44 +389,40 @@ FileAttributes(ConstUnicode pathName,  // IN:
 Bool
 File_IsRemote(ConstUnicode pathName)  // IN: Path name
 {
-   struct statfs sfbuf;
+   if (HostType_OSIsVMK()) {
+      /*
+       * All files and file systems are treated as "directly attached"
+       * on ESX.  See bug 158284.
+       */
 
-#if defined(VMX86_SERVER)
-   /*
-    * On ESX, statfs() will always return VMFS_MAGIC for files on VMFS so this
-    * function is only correct for files on COS, otherwise it always returns
-    * FALSE.
-    * On VMvisor, statfs() could return VMFS_NFS_MAGIC but it is very slow.
-    * Since there is no COS for VMvisor, just be on par with ESX and return
-    * FALSE directly.
-    * XXX See PR 158284. It is not clear what the side-effects are of this
-    * function being incorrect for VMFS files.
-    */
-
-   if (HostType_OSIsPureVMK()) {
       return FALSE;
-   }
-#endif
+   } else {
+      struct statfs sfbuf;
 
-   if (Posix_Statfs(pathName, &sfbuf) == -1) {
-      Log(LGPFX" %s: statfs(%s) failed: %s\n", __func__, UTF8(pathName),
-          strerror(errno));
-      return TRUE;
-   }
+      if (Posix_Statfs(pathName, &sfbuf) == -1) {
+         Log(LGPFX" %s: statfs(%s) failed: %s\n", __func__, UTF8(pathName),
+             Err_Errno2String(errno));
+
+         return TRUE;
+      }
 #if defined(__APPLE__)
-   return sfbuf.f_flags & MNT_LOCAL ? FALSE : TRUE;
+      return sfbuf.f_flags & MNT_LOCAL ? FALSE : TRUE;
 #else
-   if (NFS_SUPER_MAGIC == sfbuf.f_type) {
-      return TRUE;
-   }
-   if (SMB_SUPER_MAGIC == sfbuf.f_type) {
-      return TRUE;
-   }
-   if (CIFS_SUPER_MAGIC == sfbuf.f_type) {
-      return TRUE;
-   }
-   return FALSE;
+      if (NFS_SUPER_MAGIC == sfbuf.f_type) {
+         return TRUE;
+      }
+
+      if (SMB_SUPER_MAGIC == sfbuf.f_type) {
+         return TRUE;
+      }
+
+      if (CIFS_SUPER_MAGIC == sfbuf.f_type) {
+         return TRUE;
+      }
+
+      return FALSE;
 #endif
+   }
 }
 #endif /* !FreeBSD && !sun */
 
@@ -578,9 +598,7 @@ File_FullPath(ConstUnicode pathName)  // IN:
           ret = FileStripFwdSlashes(pathName);
        }
    } else {
-      Unicode path;
-
-      path = Unicode_Join(cwd, DIRSEPS, pathName, NULL);
+      Unicode path = Unicode_Join(cwd, DIRSEPS, pathName, NULL);
 
       ret = Posix_RealPath(path);
 
@@ -640,11 +658,11 @@ File_IsFullPath(ConstUnicode pathName)  // IN:
  */
 
 Bool
-File_GetTimes(ConstUnicode pathName,      // IN:
-              VmTimeType *createTime,     // OUT: Windows NT time format
-              VmTimeType *accessTime,     // OUT: Windows NT time format
-              VmTimeType *writeTime,      // OUT: Windows NT time format
-              VmTimeType *attrChangeTime) // OUT: Windows NT time format
+File_GetTimes(ConstUnicode pathName,       // IN:
+              VmTimeType *createTime,      // OUT: Windows NT time format
+              VmTimeType *accessTime,      // OUT: Windows NT time format
+              VmTimeType *writeTime,       // OUT: Windows NT time format
+              VmTimeType *attrChangeTime)  // OUT: Windows NT time format
 {
    struct stat statBuf;
 
@@ -657,7 +675,7 @@ File_GetTimes(ConstUnicode pathName,      // IN:
 
    if (Posix_Lstat(pathName, &statBuf) == -1) {
       Log(LGPFX" %s: error stating file \"%s\": %s\n", __FUNCTION__,
-          UTF8(pathName), strerror(errno));
+          UTF8(pathName), Err_Errno2String(errno));
       return FALSE;
    }
 
@@ -782,6 +800,7 @@ File_SetTimes(ConstUnicode pathName,      // IN:
    if (path == NULL) {
       Log(LGPFX" %s: failed to convert \"%s\" to current encoding\n",
           __FUNCTION__, UTF8(pathName));
+
       return FALSE;
    }
 
@@ -789,8 +808,9 @@ File_SetTimes(ConstUnicode pathName,      // IN:
 
    if (err != 0) {
       Log(LGPFX" %s: error stating file \"%s\": %s\n", __FUNCTION__,
-          UTF8(pathName), strerror(err));
+          UTF8(pathName), Err_Errno2String(err));
       free(path);
+
       return FALSE;
    }
 
@@ -809,6 +829,7 @@ File_SetTimes(ConstUnicode pathName,      // IN:
 
    if (accessTime > 0) {
       struct timespec ts;
+
       TimeUtil_NtTimeToUnixTime(&ts, accessTime);
       aTime->tv_sec = ts.tv_sec;
       aTime->tv_usec = ts.tv_nsec / 1000;
@@ -816,6 +837,7 @@ File_SetTimes(ConstUnicode pathName,      // IN:
 
    if (writeTime > 0) {
       struct timespec ts;
+
       TimeUtil_NtTimeToUnixTime(&ts, writeTime);
       wTime->tv_sec = ts.tv_sec;
       wTime->tv_usec = ts.tv_nsec / 1000;
@@ -827,7 +849,8 @@ File_SetTimes(ConstUnicode pathName,      // IN:
 
    if (err != 0) {
       Log(LGPFX" %s: utimes error on file \"%s\": %s\n", __FUNCTION__,
-          UTF8(pathName), strerror(err));
+          UTF8(pathName), Err_Errno2String(err));
+
       return FALSE;
    }
 
@@ -852,16 +875,19 @@ File_SetTimes(ConstUnicode pathName,      // IN:
  */
 
 Bool
-File_SetFilePermissions(ConstUnicode pathName,     // IN:
-                        int perms)                 // IN: permissions
+File_SetFilePermissions(ConstUnicode pathName,  // IN:
+                        int perms)              // IN: permissions
 {
    ASSERT(pathName);
+
    if (Posix_Chmod(pathName, perms) == -1) {
       /* The error is not critical, just log it. */
-      Log(LGPFX" %s: failed to change permissions on file \"%s\": %s\n", __FUNCTION__,
-          UTF8(pathName), strerror(errno));
+      Log(LGPFX" %s: failed to change permissions on file \"%s\": %s\n",
+          __FUNCTION__, UTF8(pathName), Err_Errno2String(errno));
+
       return FALSE;
    }
+
    return TRUE;
 }
 
@@ -999,95 +1025,16 @@ File_GetFreeSpace(ConstUnicode pathName,  // IN: File name
 
    fullPath = File_FullPath(pathName);
    if (fullPath == NULL) {
-      ret = -1;
-      goto end;
+      return -1;
    }
 
-   if (!FileGetStats(fullPath, doNotAscend, &statfsbuf)) {
+   if (FileGetStats(fullPath, doNotAscend, &statfsbuf)) {
+      ret = (uint64) statfsbuf.f_bavail * statfsbuf.f_bsize;
+   } else {
       Warning("%s: Couldn't statfs %s\n", __func__, fullPath);
       ret = -1;
-      goto end;
    }
 
-   ret = (uint64)statfsbuf.f_bavail * statfsbuf.f_bsize;
-
-#if defined(VMX86_SERVER)
-   /*
-    * The following test is never true on VMvisor but we do not care as
-    * this is only intended for callers going through vmkfs. Direct callers
-    * as we are always get the right answer from statfs above.
-    */
-
-   if (statfsbuf.f_type == VMFS_MAGIC_NUMBER) {
-      int fd;
-      FS_FreeSpaceArgs args = { 0 };
-      Unicode specialPath = NULL;
-
-      /*
-       * If the file exists and can be opened we're all set. If the file
-       * doesn't exist we can use the parent directory for the ioctl.
-       * However, if the file exists and can't be opened (e.g. permissions
-       * issues) a correct answer can only be returned if the target isn't a
-       * directory. If the target is a directory its parent may be a mount
-       * point - leading across a mount point to a different file system.
-       * PR 412387
-       */
-
-      ret = -1;
-
-      fd = Posix_Open(fullPath, O_RDONLY, 0);
-
-      if (fd == -1) {
-         switch (errno) {
-         case EPERM:
-         case EACCES:
-            {
-               int err = errno;
-               struct stat statbuf;
-
-               if (Posix_Stat(fullPath, &statbuf) == -1) {
-                  errno = err;
-                  break;
-               }
-
-               if (S_ISDIR(statbuf.st_mode)) {
-                  Warning(LGPFX" %s: directory (%s) present but inaccessible\n",
-                          __func__, UTF8(fullPath));
-                  errno = err;
-                  break;
-               }
-            }
-            /* FALLTHROUGH */
-
-         case ENOENT:
-         default:
-            File_SplitName(fullPath, NULL, &specialPath, NULL);
-
-            fd = Posix_Open(specialPath, O_RDONLY, 0);
-         }
-      }
-
-      if (fd == -1) {
-         Warning(LGPFX" %s: open of %s failed with: %s\n", __func__,
-                 (specialPath == NULL) ? UTF8(fullPath) : UTF8(specialPath),
-                 Msg_ErrString());
-      } else {
-         if (ioctl(fd, IOCTLCMD_VMFS_GET_FREE_SPACE, &args) == -1) {
-            Warning(LGPFX" %s: ioctl on %s failed with: %s\n", __func__,
-                    (specialPath == NULL) ? UTF8(fullPath) : UTF8(specialPath),
-                    Msg_ErrString());
-         } else {
-            ret = args.bytesFree;
-         }
-
-         close(fd);
-      }
-
-      Unicode_Free(specialPath);
-   }
-#endif
-
-end:
    Unicode_Free(fullPath);
 
    return ret;
@@ -1130,9 +1077,8 @@ File_GetVMFSAttributes(ConstUnicode pathName,             // IN: File to test
 
    File_SplitName(fullPath, NULL, &parentPath, NULL);
 
-   if (!File_OnVMFS(pathName)) {
-      Log(LGPFX" %s: File %s not on VMFS volume\n", __func__,
-          UTF8(pathName));
+   if (!HostType_OSIsVMK()) {
+      Log(LGPFX" %s: File %s not on VMFS volume\n", __func__, UTF8(pathName));
       ret = -1;
       goto bail;
    }
@@ -1174,46 +1120,6 @@ bail:
 }
 
 
-/*
- *----------------------------------------------------------------------
- *
- * File_GetVMFSVersion --
- *
- *      Acquire the version number for a given file on a VMFS file system.
- *
- * Results:
- *      Integer return value and version number
- *
- * Side effects:
- *      Will fail if file is not on VMFS or not enough memory for partition
- *      query results
- *
- *----------------------------------------------------------------------
- */
-
-static int
-File_GetVMFSVersion(ConstUnicode pathName,  // IN: Filename to test
-                    uint32 *version)        // IN/OUT: version number of VMFS
-{
-   int ret = -1;
-   FS_PartitionListResult *fsAttrs = NULL;
-
-   ret = File_GetVMFSAttributes(pathName, &fsAttrs);
-   if (ret < 0) {
-      Log(LGPFX" %s: File_GetVMFSAttributes failed\n", __func__);
-      goto done;
-   }
-
-   *version = fsAttrs->versionNumber;
-
-done:
-   if (fsAttrs) {
-      free(fsAttrs);
-   }
-   return ret;
-}
-
-
 /*
  *----------------------------------------------------------------------
  *
@@ -1235,62 +1141,21 @@ int
 File_GetVMFSBlockSize(ConstUnicode pathName,  // IN: File name to test
                       uint32 *blockSize)      // IN/OUT: VMFS block size
 {
-   int ret = -1;
+   int ret;
    FS_PartitionListResult *fsAttrs = NULL;
 
    ret = File_GetVMFSAttributes(pathName, &fsAttrs);
+
    if (ret < 0) {
       Log(LGPFX" %s: File_GetVMFSAttributes failed\n", __func__);
-      goto done;
+   } else {
+      *blockSize = fsAttrs->fileBlockSize;
    }
 
-   *blockSize = fsAttrs->fileBlockSize;
-
-done:
    if (fsAttrs) {
       free(fsAttrs);
    }
-   return ret;
-}
 
-
-/*
- *----------------------------------------------------------------------
- *
- * File_GetVMFSfsType --
- *
- *      Acquire the fsType for a given file on a VMFS.
- *
- * Results:
- *      Integer return value and fs type
- *
- * Side effects:
- *      Will fail if file is not on VMFS or not enough memory for partition
- *      query results
- *
- *----------------------------------------------------------------------
- */
-
-int
-File_GetVMFSfsType(ConstUnicode pathName,  // IN: File name to test
-                   char **fsType)          // IN/OUT: VMFS fsType
-{
-   int ret = -1;
-   FS_PartitionListResult *fsAttrs = NULL;
-
-   ret = File_GetVMFSAttributes(pathName, &fsAttrs);
-   if (ret < 0) {
-      Log(LGPFX" %s: File_GetVMFSAttributes failed\n", __func__);
-      goto done;
-   }
-
-   *fsType = Util_SafeMalloc(sizeof(char) * FS_PLIST_DEF_MAX_FSTYPE_LEN);
-   memcpy(*fsType, fsAttrs->fsType, FS_PLIST_DEF_MAX_FSTYPE_LEN);
-
-done:
-   if (fsAttrs) {
-      free(fsAttrs);
-   }
    return ret;
 }
 
@@ -1315,12 +1180,12 @@ done:
  */
 
 int 
-File_GetVMFSMountInfo(ConstUnicode pathName,
-                     char **fsType,
-                     uint32 *version,
-                     char **remoteIP,
-                     char **remoteMountPoint,
-                     char **localMountPoint)
+File_GetVMFSMountInfo(ConstUnicode pathName,  // IN:
+                     char **fsType,           // OUT:
+                     uint32 *version,         // OUT:
+                     char **remoteIP,         // OUT:
+                     char **remoteMountPoint, // OUT:
+                     char **localMountPoint)  // OUT:
 {
    int ret;
    int len;
@@ -1338,7 +1203,7 @@ File_GetVMFSMountInfo(ConstUnicode pathName,
       *version = fsAttrs->versionNumber;
       *fsType = Util_SafeStrdup(fsAttrs->fsType);
  
-      if (strncmp(fsAttrs->fsType, "NFS", sizeof("NFS")) == 0) {
+      if (strncmp(fsAttrs->fsType, FS_NFS_ON_ESX, sizeof(FS_NFS_ON_ESX)) == 0) {
          len = strlen(fsAttrs->logicalDevice);
          *remoteIP = Util_SafeMalloc(len);
          *remoteMountPoint = Util_SafeMalloc(len);
@@ -1350,75 +1215,53 @@ File_GetVMFSMountInfo(ConstUnicode pathName,
    }
 
    free(fsAttrs);
+
    return ret;
 }
-
 #endif 
 
 
 /*
  *----------------------------------------------------------------------
  *
- * File_OnVMFS --
+ * File_SupportsZeroedThick --
  *
- *      Return TRUE if file is on a VMFS file system.
+ *      Check if the given file is on an FS supports creation of
+ *      the zeroed-thick files.
+ *      Currently only VMFS on ESX does support zeroed-thick files, but
+ *      this may change in the future.
  *
  * Results:
- *      Boolean
+ *      TRUE if FS supports creation of the zeroed-thick files.
  *
  * Side effects:
- *      None
+ *       None
  *
  *----------------------------------------------------------------------
  */
 
 Bool
-File_OnVMFS(ConstUnicode pathName)
+File_SupportsZeroedThick(ConstUnicode pathName) // IN: File name to test
 {
+   Bool result = FALSE;
+
 #if defined(VMX86_SERVER)
-   Bool ret;
-   struct statfs statfsbuf;
+   /* Right now only VMFS supports ZeroedThick */
+   FS_PartitionListResult *fsAttrs = NULL;
 
-   /* XXX See Vmfs_IsVMFSDir. Same caveat about fs exclusion. */
-   if (HostType_OSIsPureVMK()) {
-      return TRUE;
+   if (File_GetVMFSAttributes(pathName, &fsAttrs) >= 0) {
+      result = strncmp(fsAttrs->fsType, FS_VMFS_ON_ESX,
+                       sizeof(FS_VMFS_ON_ESX)) == 0;
+   } else {
+      Log(LGPFX" %s: File_GetVMFSAttributes failed\n", __func__);
    }
 
-   /*
-    * Do a quick statfs() for best performance in the case that the file
-    * exists.  If file doesn't exist, then get the full path and do a
-    * FileGetStats() to check each of the parent directories.
-    */
-
-   if (Posix_Statfs(pathName, &statfsbuf) == -1) {
-      int err;
-      Unicode fullPath;
-
-      fullPath = File_FullPath(pathName);
-      if (fullPath == NULL) {
-         ret = FALSE;
-         goto end;
-      }
-
-      err = FileGetStats(fullPath, FALSE, &statfsbuf);
-
-      Unicode_Free(fullPath);
-
-      if (err == -1) {
-         Warning(LGPFX" %s: Couldn't statfs\n", __FUNCTION__);
-         ret = FALSE;
-         goto end;
-      }
+   if (fsAttrs) {
+      free(fsAttrs);
    }
-
-   ret = (statfsbuf.f_type == VMFS_MAGIC_NUMBER);
-
-end:
-
-   return ret;
-#else
-   return FALSE;
 #endif
+ 
+   return result;
 }
 
 
@@ -1448,20 +1291,18 @@ File_GetCapacity(ConstUnicode pathName)  // IN: Path name
 
    fullPath = File_FullPath(pathName);
    if (fullPath == NULL) {
-      ret = -1;
-      goto end;
+      return -1;
    }
 
-   if (!FileGetStats(fullPath, FALSE, &statfsbuf)) {
+   if (FileGetStats(fullPath, FALSE, &statfsbuf)) {
+      ret = (uint64) statfsbuf.f_blocks * statfsbuf.f_bsize;
+   } else {
       Warning(LGPFX" %s: Couldn't statfs\n", __func__);
       ret = -1;
-      goto end;
    }
 
-   ret = (uint64)statfsbuf.f_blocks * statfsbuf.f_bsize;
-
-end:
    Unicode_Free(fullPath);
+
    return ret;
 }
 
@@ -1493,40 +1334,41 @@ end:
  */
 
 char *
-File_GetUniqueFileSystemID(char const *path) // IN: File path
+File_GetUniqueFileSystemID(char const *path)  // IN: File path
 {
-#if defined(VMX86_SERVER)
-   char *canPath;
-   char *existPath;
+   if (HostType_OSIsVMK()) {
+      char *canPath;
+      char *existPath;
 
-   existPath = FilePosixNearestExistingAncestor(path);
-   canPath = Posix_RealPath(existPath);
-   free(existPath);
+      existPath = FilePosixNearestExistingAncestor(path);
+      canPath = Posix_RealPath(existPath);
+      free(existPath);
 
-   if (canPath == NULL) {
-      return NULL;
-   }
-
-   /*
-    * VCFS doesn't have real mount points, so the mount point lookup below
-    * returns "/vmfs", instead of the VCFS mount point.
-    *
-    * See bug 61646 for why we care.
-    */
-
-   if (strncmp(canPath, VCFS_MOUNT_POINT, strlen(VCFS_MOUNT_POINT)) == 0) {
-      char vmfsVolumeName[FILE_MAXPATH];
-
-      if (sscanf(canPath, VCFS_MOUNT_PATH "%[^/]%*s", vmfsVolumeName) == 1) {
-         free(canPath);
-
-         return Str_SafeAsprintf(NULL, "%s/%s", VCFS_MOUNT_POINT,
-                                 vmfsVolumeName);
+      if (canPath == NULL) {
+         return NULL;
       }
-   }
 
-   free(canPath);
-#endif
+      /*
+       * VCFS doesn't have real mount points, so the mount point lookup below
+       * returns "/vmfs", instead of the VCFS mount point.
+       *
+       * See bug 61646 for why we care.
+       */
+
+      if (strncmp(canPath, VCFS_MOUNT_POINT, strlen(VCFS_MOUNT_POINT)) == 0) {
+         char vmfsVolumeName[FILE_MAXPATH];
+
+         if (sscanf(canPath, VCFS_MOUNT_PATH "%[^/]%*s",
+                    vmfsVolumeName) == 1) {
+            free(canPath);
+
+            return Str_SafeAsprintf(NULL, "%s/%s", VCFS_MOUNT_POINT,
+                                    vmfsVolumeName);
+         }
+      }
+
+      free(canPath);
+   }
 
    return FilePosixGetBlockDevice(path);
 }
@@ -1554,22 +1396,64 @@ File_GetUniqueFileSystemID(char const *path) // IN: File path
  */
 
 static char *
-FilePosixLookupMountPoint(char const *canPath, // IN: Canonical file path
-                          Bool *bind)          // OUT: Mounted with --[r]bind?
+FilePosixLookupMountPoint(char const *canPath,  // IN: Canonical file path
+                          Bool *bind)           // OUT: Mounted with --[r]bind?
 {
    FILE *f;
-   struct mntent *mnt;
+   struct mntent mnt;
+   char *buf;
+   size_t size;
+   size_t used;
+   char *ret = NULL;
 
    ASSERT(canPath);
    ASSERT(bind);
 
+   size = 4 * FILE_MAXPATH;  // Should suffice for most locales
+
+retry:
    f = setmntent(MOUNTED, "r");
    if (f == NULL) {
       return NULL;
    }
 
-   /* XXX getmntent() is not thread-safe. Use getmntent_r() instead. */
-   while ((mnt = getmntent(f)) != NULL) {
+   buf = Util_SafeMalloc(size);
+
+   while (Posix_Getmntent_r(f, &mnt, buf, size) != NULL) {
+
+      /*
+       * Our Posix_Getmntent_r graciously sets errno when the buffer 
+       * is too small, but on UTF-8 based platforms Posix_Getmntent_r
+       * is #defined to the system's getmntent_r, which can simply 
+       * truncate the strings with no other indication.  See how much 
+       * space it used and increase the buffer size if needed.  Note
+       * that if some of the strings are empty, they may share a
+       * common nul in the buffer, and the resulting size calculation 
+       * will be a little over-zealous.
+       */
+
+      used = 0;  
+      if (mnt.mnt_fsname) {
+         used += strlen(mnt.mnt_fsname) + 1;
+      } 
+      if (mnt.mnt_dir) {
+         used += strlen(mnt.mnt_dir) + 1;
+      } 
+      if (mnt.mnt_type) {
+         used += strlen(mnt.mnt_type) + 1;
+      } 
+      if (mnt.mnt_opts) {
+         used += strlen(mnt.mnt_opts) + 1;
+      } 
+      if (used >= size || !mnt.mnt_fsname || !mnt.mnt_dir || 
+          !mnt.mnt_type || !mnt.mnt_opts) {
+         size += 4 * FILE_MAXPATH;
+         ASSERT(size <= 32 * FILE_MAXPATH);
+         free(buf);
+         endmntent(f);
+         goto retry;
+      }
+
       /*
        * NB: A call to realpath is not needed as getmntent() already
        *     returns it in canonical form.  Additionally, it is bad
@@ -1579,9 +1463,7 @@ FilePosixLookupMountPoint(char const *canPath, // IN: Canonical file path
        *     all expecting.
        */
 
-      if (strcmp(mnt->mnt_dir, canPath) == 0) {
-         endmntent(f);
-
+      if (strcmp(mnt.mnt_dir, canPath) == 0) {
          /*
           * The --bind and --rbind options behave differently. See 
           * FilePosixGetBlockDevice() for details.
@@ -1591,15 +1473,20 @@ FilePosixLookupMountPoint(char const *canPath, // IN: Canonical file path
           * always "bind".
           */
 
-         *bind = strstr(mnt->mnt_opts, "bind") != NULL;
+         *bind = strstr(mnt.mnt_opts, "bind") != NULL;
 
-         return Util_SafeStrdup(mnt->mnt_fsname);
+         ret = Util_SafeStrdup(mnt.mnt_fsname);
+
+	 break;
       }
    }
 
    // 'canPath' is not a mount point.
    endmntent(f);
-   return NULL;
+
+   free(buf);
+
+   return ret;
 }
 #endif
 
@@ -1625,7 +1512,7 @@ FilePosixLookupMountPoint(char const *canPath, // IN: Canonical file path
  */
 
 char *
-FilePosixGetBlockDevice(char const *path) // IN: File path
+FilePosixGetBlockDevice(char const *path)  // IN: File path
 {
    char *existPath;
    Bool failed;
@@ -1778,7 +1665,7 @@ retry:
  */
 
 static char *
-FilePosixNearestExistingAncestor(char const *path) // IN: File path
+FilePosixNearestExistingAncestor(char const *path)  // IN: File path
 {
    size_t resultSize;
    char *result;
@@ -1786,8 +1673,8 @@ FilePosixNearestExistingAncestor(char const *path) // IN: File path
 
    resultSize = MAX(strlen(path), 1) + 1;
    result = Util_SafeMalloc(resultSize);
-
    Str_Strcpy(result, path, resultSize);
+
    for (;;) {
       char *ptr;
 
@@ -1914,6 +1801,7 @@ File_IsSameFile(ConstUnicode path1,  // IN:
        (st1.st_blocks == st2.st_blocks)) {
       return TRUE;
    }
+
    return FALSE;
 }
 
@@ -1996,52 +1884,11 @@ bail:
    free(oldPath);
 
    errno = status;
+
    return result;
 }
 
 
-/*
- *----------------------------------------------------------------------
- *
- * FileIsVMFS --
- *
- *      Determine whether specified file lives on VMFS filesystem.
- *      Only Linux host can have VMFS, so skip it on Solaris
- *      and FreeBSD.
- *
- * Results:
- *      TRUE if specified file lives on VMFS
- *      FALSE if file is not on VMFS or does not exist
- *
- * Side effects:
- *      None.
- *
- *----------------------------------------------------------------------
- */
-
-static Bool
-FileIsVMFS(ConstUnicode pathName)  // IN: file name to test
-{
-#if defined(linux)
-   struct statfs statfsbuf;
-
-#if defined(VMX86_SERVER)
-   /* XXX See Vmfs_IsVMFSFile. Same caveat about fs exclusion. */
-   if (HostType_OSIsPureVMK()) {
-      return TRUE;
-   }
-#endif
-
-   if (Posix_Statfs(pathName, &statfsbuf) == 0) {
-      return statfsbuf.f_type == VMFS_SUPER_MAGIC;
-   }
-
-#endif
-
-   return FALSE;
-}
-
-
 /*
  *----------------------------------------------------------------------
  *
@@ -2063,8 +1910,8 @@ FileIsVMFS(ConstUnicode pathName)  // IN: file name to test
  */
 
 static Bool
-FilePosixCreateTestFileSize(ConstUnicode dirName, // IN: directory to create large file
-                            uint64 fileSize)      // IN: test file size
+FilePosixCreateTestFileSize(ConstUnicode dirName,  // IN: test directory
+                            uint64 fileSize)       // IN: test file size
 {
    Bool retVal;
    int posixFD;
@@ -2121,42 +1968,35 @@ File_VMFSSupportsFileSize(ConstUnicode pathName,  // IN:
                           uint64 fileSize)        // IN:
 {
 #if defined(VMX86_SERVER)
-   uint32 version = -1;
-   uint32 blockSize = -1;
    uint64 maxFileSize = -1;
    Bool supported;
-   char *fsType = NULL;
+   FS_PartitionListResult *fsAttrs = NULL;
+
+   if (File_GetVMFSAttributes(pathName, &fsAttrs) < 0) {
+      Log(LGPFX" %s: File_GetVMFSAttributes Failed\n", __func__);
 
-   if (File_GetVMFSVersion(pathName, &version) < 0) {
-      Log(LGPFX" %s: File_GetVMFSVersion Failed\n", __func__);
-      return FALSE;
-   }
-   if (File_GetVMFSBlockSize(pathName, &blockSize) < 0) {
-      Log(LGPFX" %s: File_GetVMFSBlockSize Failed\n", __func__);
-      return FALSE;
-   }
-   if (File_GetVMFSfsType(pathName, &fsType) < 0) {
-      Log(LGPFX" %s: File_GetVMFSfsType Failed\n", __func__);
       return FALSE;
    }
 
-   if (strcmp(fsType, "VMFS") == 0) {
-      if (version == 3) {
-         maxFileSize = (VMFS3CONST * (uint64) blockSize * 1024);
+   if (strcmp(fsAttrs->fsType, FS_VMFS_ON_ESX) == 0) {
+      if (fsAttrs->versionNumber == 3) {
+         maxFileSize = (VMFS3CONST * (uint64) fsAttrs->fileBlockSize * 1024);
       } else {
          /* Get ready for 64 TB on VMFS5 and perform sanity check on version */
-         ASSERT(version == 5);
+         ASSERT(fsAttrs->versionNumber == 5);
          maxFileSize = (uint64) 0x400000000000ULL;
       }
 
       if (fileSize <= maxFileSize && maxFileSize != -1) {
-         free(fsType);
+         free(fsAttrs);
+
          return TRUE;
       } else {
          Log(LGPFX" %s: Requested file size (%"FMT64"d) larger than maximum "
              "supported filesystem file size (%"FMT64"d)\n", __FUNCTION__,
              fileSize, maxFileSize);
-         free(fsType);
+         free(fsAttrs);
+
          return FALSE;
       }
    } else {
@@ -2167,7 +2007,8 @@ File_VMFSSupportsFileSize(ConstUnicode pathName,  // IN:
 
       if (fullPath == NULL) {
          Log(LGPFX" %s: Error acquiring full path\n", __func__);
-         free(fsType);
+         free(fsAttrs);
+
          return FALSE;
       }
 
@@ -2175,7 +2016,7 @@ File_VMFSSupportsFileSize(ConstUnicode pathName,  // IN:
 
       supported = FilePosixCreateTestFileSize(parentPath, fileSize);
 
-      free(fsType);
+      free(fsAttrs);
       Unicode_Free(fullPath);
       Unicode_Free(parentPath);
 
@@ -2184,6 +2025,7 @@ File_VMFSSupportsFileSize(ConstUnicode pathName,  // IN:
 
 #endif
    Log(LGPFX" %s: did not execute properly\n", __func__);
+
    return FALSE; /* happy compiler */
 }
 
@@ -2209,9 +2051,9 @@ File_SupportsFileSize(ConstUnicode pathName,  // IN:
                       uint64 fileSize)        // IN:
 {
    Unicode fullPath;
+   Unicode folderPath;
 
    Bool supported = FALSE;
-   Unicode folderPath = NULL;
 
    /* All supported filesystems can hold at least 2GB - 1 files. */
    if (fileSize <= 0x7FFFFFFF) {
@@ -2232,24 +2074,12 @@ File_SupportsFileSize(ConstUnicode pathName,  // IN:
       goto out;
    }
 
-   /* 
-    * This function expects a filename. If given one, truncate the name to
-    * point to the parent directory so we can get accurate results from
-    * FileIsVMFS. If handed a directory directly, no truncation is necessary.
-    */
-
-   if (File_IsDirectory(pathName)) {
-      folderPath = Unicode_Duplicate(fullPath);
-   } else {
-      File_SplitName(fullPath, NULL, &folderPath, NULL);
-   }
-
    /* 
     * We know that VMFS supports large files - But they have limitations
     * See function File_VMFSSupportsFileSize() - PR 146965
     */
 
-   if (FileIsVMFS(folderPath)) {
+   if (HostType_OSIsVMK()) {
       supported = File_VMFSSupportsFileSize(pathName, fileSize);
       goto out;
    }
@@ -2268,15 +2098,22 @@ File_SupportsFileSize(ConstUnicode pathName,  // IN:
    }
 
    /*
-    * On unknown filesystems create temporary file and use it as a test.
+    * On unknown filesystems create a temporary file in the argument file's
+    * parent directory and use it as a test.
     */
 
+   if (File_IsDirectory(pathName)) {
+      folderPath = Unicode_Duplicate(fullPath);
+   } else {
+      folderPath = NULL;
+      File_SplitName(fullPath, NULL, &folderPath, NULL);
+   }
+
    supported = FilePosixCreateTestFileSize(folderPath, fileSize);
+   Unicode_Free(folderPath);
 
 out:
    Unicode_Free(fullPath);
-   Unicode_Free(folderPath);
-
    return supported;
 }
 
@@ -2374,6 +2211,7 @@ File_ListDirectory(ConstUnicode pathName,  // IN:
       /* Don't create the file list if we aren't providing it to the caller. */
       if (ids) {
          Unicode id = Unicode_Alloc(entry->d_name, STRING_ENCODING_DEFAULT);
+
          DynBuf_Append(&b, &id, sizeof id);
       }
 
@@ -2423,7 +2261,7 @@ File_ListDirectory(ConstUnicode pathName,  // IN:
  */
 
 WalkDirContext
-File_WalkDirectoryStart(ConstUnicode parentPath) // IN
+File_WalkDirectoryStart(ConstUnicode parentPath)  // IN:
 {
    WalkDirContextImpl *context;
    char * const traversalRoots[] =
@@ -2434,8 +2272,7 @@ File_WalkDirectoryStart(ConstUnicode parentPath) // IN
       return NULL;
    }
 
-   context->fts = fts_open(traversalRoots,
-                           FTS_LOGICAL|FTS_NOSTAT|FTS_NOCHDIR,
+   context->fts = fts_open(traversalRoots, FTS_LOGICAL|FTS_NOSTAT|FTS_NOCHDIR,
                            NULL);
    if (!context->fts) {
       free(context);
@@ -2471,8 +2308,8 @@ File_WalkDirectoryStart(ConstUnicode parentPath) // IN
  */
 
 Bool
-File_WalkDirectoryNext(WalkDirContext context, // IN
-                       Unicode *path)          // OUT
+File_WalkDirectoryNext(WalkDirContext context,  // IN:
+                       Unicode *path)           // OUT:
 {
    FTSENT *nextEntry;
 
@@ -2482,17 +2319,20 @@ File_WalkDirectoryNext(WalkDirContext context, // IN
 
    do {
       nextEntry = fts_read(context->fts);
+
       /*
        * We'll skip any entries that cannot be read, are errors, or
        * are the second traversal (post-order) of a directory.
        */
-      if (   nextEntry
-          && nextEntry->fts_info != FTS_DNR
-          && nextEntry->fts_info != FTS_ERR
-          && nextEntry->fts_info != FTS_DP) {
+
+      if (nextEntry &&
+          nextEntry->fts_info != FTS_DNR &&
+          nextEntry->fts_info != FTS_ERR &&
+          nextEntry->fts_info != FTS_DP) {
          *path = Unicode_AllocWithLength(nextEntry->fts_path,
                                          nextEntry->fts_pathlen,
                                          STRING_ENCODING_DEFAULT);
+
          return TRUE;
       }
    } while (nextEntry);
@@ -2518,7 +2358,7 @@ File_WalkDirectoryNext(WalkDirContext context, // IN
  */
 
 void
-File_WalkDirectoryEnd(WalkDirContext context) // IN
+File_WalkDirectoryEnd(WalkDirContext context)  // IN:
 {
    ASSERT(context);
    ASSERT(context->fts);
@@ -2551,22 +2391,22 @@ File_WalkDirectoryEnd(WalkDirContext context) // IN
  */
 
 WalkDirContext
-File_WalkDirectoryStart(ConstUnicode parentPath) // IN
+File_WalkDirectoryStart(ConstUnicode parentPath)  // IN:
 {
    NOT_IMPLEMENTED();
 }
 
 
 Bool
-File_WalkDirectoryNext(WalkDirContext context, // IN
-                       Unicode *path)          // OUT
+File_WalkDirectoryNext(WalkDirContext context,  // IN:
+                       Unicode *path)           // OUT:
 {
    NOT_IMPLEMENTED();
 }
 
 
 void
-File_WalkDirectoryEnd(WalkDirContext context) // IN
+File_WalkDirectoryEnd(WalkDirContext context)  // IN:
 {
    NOT_IMPLEMENTED();
 }
@@ -2592,7 +2432,7 @@ File_WalkDirectoryEnd(WalkDirContext context) // IN
  */
 
 static Bool
-FileIsGroupsMember(gid_t gid)
+FileIsGroupsMember(gid_t gid)  // IN:
 {
    int nr_members;
    gid_t *members;
@@ -2601,6 +2441,7 @@ FileIsGroupsMember(gid_t gid)
 
    members = NULL;
    nr_members = 0;
+
    for (;;) {
       gid_t *new;
 
@@ -2715,7 +2556,7 @@ FileIsWritableDir(ConstUnicode dirName)  // IN:
  */
 
 static char *
-FileTryDir(const char *dirName) // IN: Is this a writable directory?
+FileTryDir(const char *dirName)  // IN: Is this a writable directory?
 {
    char *edirName;
 
@@ -2753,7 +2594,7 @@ FileTryDir(const char *dirName) // IN: Is this a writable directory?
  */
 
 char *
-File_GetTmpDir(Bool useConf) // IN: Use the config file?
+File_GetTmpDir(Bool useConf)  // IN: Use the config file?
 {
    char *dirName;
    char *edirName;
@@ -2833,7 +2674,7 @@ File_GetTmpDir(Bool useConf) // IN: Use the config file?
  */
 
 Bool
-File_MakeCfgFileExecutable(ConstUnicode pathName)
+File_MakeCfgFileExecutable(ConstUnicode pathName)  // IN:
 {
    struct stat s;
 
