@@ -32,6 +32,12 @@
 #include "vmware.h"
 
 
+/*
+ * Environment used when launching applications.
+ */
+extern const char **guestAppSpawnEnviron;
+
+
 /*
  * Global functions
  */
