@@ -75,20 +75,19 @@ EXTERN FileMacosUnmountStatus FileMacos_UnmountDev(char const *bsdDev,
                                                    Bool eject,
                                                    Bool su);
 
-EXTERN void FileMacos_MountDevAsyncNoResult(char const *bsdSliceDev,
+EXTERN void FileMacos_MountDevAsyncNoResult(char const *bsdDev,
                                             Bool su);
 
 EXTERN Bool FileMacos_IsOnExternalDevice(int fd);
 EXTERN Bool FileMacos_IsOnSparseDmg(int fd);
 EXTERN Bool FileMacos_IsSliceDevice(char const *bsdDev);
 
-EXTERN char *FileMacos_DiskDevToDiskName(char const *bsdDiskDev);
+EXTERN char *FileMacos_DiskDevToUserFriendlyName(char const *bsdDiskDev);
 
 EXTERN char *FileMacos_DiskDeviceToUniqueID(char const *bsdPath);
 EXTERN char *FileMacos_UniqueIDToDiskDevice(char const *identifier);
 #elif defined VMX86_SERVER
 EXTERN int File_GetVMFSBlockSize(ConstUnicode pathName, uint32 *blockSize);
-EXTERN int File_GetVMFSfsType(ConstUnicode pathName, char **fsType);
 EXTERN int File_GetVMFSMountInfo(ConstUnicode pathName,
                                  char **fsType,
                                  uint32 *version,
@@ -97,6 +96,8 @@ EXTERN int File_GetVMFSMountInfo(ConstUnicode pathName,
                                  char **localMountPoint);
 #endif
 
+EXTERN Bool File_SupportsZeroedThick(ConstUnicode pathName);
+
 EXTERN Bool File_Exists(ConstUnicode pathName);
 
 EXTERN int File_Unlink(ConstUnicode pathName);
@@ -299,8 +300,6 @@ EXTERN Unicode File_ReplaceExtension(ConstUnicode pathName,
                                      uint32 numExtensions,
                                      ...);
 
-EXTERN Bool File_OnVMFS(ConstUnicode pathName);
-
 EXTERN Bool File_MakeCfgFileExecutable(ConstUnicode pathName);
 
 EXTERN char *File_ExpandAndCheckDir(const char *dirName);
