@@ -118,13 +118,14 @@ typedef enum {
    HGFS_OP_CREATE_SYMLINK_V3,     /* Create a symlink */
    HGFS_OP_SERVER_LOCK_CHANGE_V3, /* Change the oplock on a file */
    HGFS_OP_WRITE_WIN32_STREAM_V3, /* Write WIN32_STREAM_ID format data to file */
-
    /*
     * Operations for version 4, deprecating version 3 operations.
     */
 
    HGFS_OP_CREATE_SESSION_V4,     /* Create a session and return host capabilities. */
    HGFS_OP_DESTROY_SESSION_V4,    /* Destroy/close session. */
+   HGFS_OP_READ_FAST_V4,          /* Read */
+   HGFS_OP_WRITE_FAST_V4,         /* Write */
    HGFS_OP_OPEN_V4,               /* Open file */
    HGFS_OP_DIRECTORY_READ_V4,     /* Read directory entries. */
    HGFS_OP_ENUMERATE_STREAMS_V4,  /* Enumerate alternative named streams for a file. */
@@ -165,55 +166,6 @@ typedef enum {
 #define HGFS_REQ_GET_PAYLOAD_V3(hgfsReq) ((char *)(hgfsReq) + sizeof(HgfsRequest))
 #define HGFS_REP_GET_PAYLOAD_V3(hgfsRep) ((char *)(hgfsRep) + sizeof(HgfsReply))
 
-/* Some fudged values for TCP over sockets. */
-#define HGFS_HOST_PORT 2000
-
-/* Socket packet magic. */
-#define HGFS_SOCKET_VERSION1   1
-
-/*
- * Socket status codes.
- */
-
-typedef enum {
-   HGFS_SOCKET_STATUS_SUCCESS,                  /* Socket header is good. */
-   HGFS_SOCKET_STATUS_SIZE_MISMATCH,            /* Size and version are incompatible. */
-   HGFS_SOCKET_STATUS_VERSION_NOT_SUPPORTED,    /* Version not handled by remote. */
-   HGFS_SOCKET_STATUS_INVALID_PACKETLEN,        /* Message len exceeds maximum. */
-} HgfsSocketStatus;
-
-/*
- * Socket flags.
- */
-
-typedef uint32 HgfsSocketFlags;
-
-/* Used by backdoor proxy socket client to Hgfs server (out of VMX process). */
-#define HGFS_SOCKET_SYNC         (1 << 0)
-
-/* Socket packet header. */
-typedef
-#include "vmware_pack_begin.h"
-struct HgfsSocketHeader {
-   uint32 version;            /* Header version. */
-   uint32 size;               /* Header size, should match for the specified version. */
-   HgfsSocketStatus status;   /* Status: always success when sending (ignored) valid on replies. */
-   uint32 packetLen;          /* The length of the packet to follow. */
-   HgfsSocketFlags flags;     /* The flags to indicate how to deal with the packet. */
-}
-#include "vmware_pack_end.h"
-HgfsSocketHeader;
-
-#define HgfsSocketHeaderInit(hdr, _version, _size, _status, _pktLen, _flags) \
-   do {                                                                      \
-      (hdr)->version    = (_version);                                        \
-      (hdr)->size       = (_size);                                           \
-      (hdr)->status     = (_status);                                         \
-      (hdr)->packetLen  = (_pktLen);                                         \
-      (hdr)->flags      = (_flags);                                          \
-   } while (0)
-
-
 /*
  * File types, used in HgfsAttr. We support regular files,
  * directories, and symlinks.
@@ -1710,6 +1662,10 @@ struct HgfsHeader {
 #include "vmware_pack_end.h"
 HgfsHeader;
 
+/*
+ * If no flags are set then the capability is not supported by the host.
+ */
+#define HGFS_REQUEST_NOT_SUPPORTED              0
 /*
  * Flag HGFS_REQUEST_SUPPORTED is set for every requests that are supported by the host.
  */
@@ -1808,18 +1764,19 @@ HgfsIdentity;
 
 typedef
 #include "vmware_pack_begin.h"
-struct HgfsRequestCreateSession {
+struct HgfsRequestCreateSessionV4 {
    uint32 numCapabilities;            /* Number of capabilities to follow. */
    uint32 maxPacketSize;              /* Maximum packet size supported. */
    uint64 reserved;                   /* Reserved for future use. */
    HgfsCapability capabilities[1];    /* Array of HgfsCapabilities. */
 }
 #include "vmware_pack_end.h"
-HgfsRequestCreateSession;
+HgfsRequestCreateSessionV4;
 
+#define HGFS_INVALID_SESSION_ID 0xffffffff
 typedef
 #include "vmware_pack_begin.h"
-struct HgfsReplyCreateSession {
+struct HgfsReplyCreateSessionV4 {
    uint64 sessionId;                  /* Session ID. */
    uint32 numCapabilities;            /* Number of capabilities to follow. */
    uint32 maxPacketSize;              /* Maximum packet size supported. */
@@ -1828,24 +1785,23 @@ struct HgfsReplyCreateSession {
    HgfsCapability capabilities[1];    /* Array of HgfsCapabilities. */
 }
 #include "vmware_pack_end.h"
-HgfsReplyCreateSession;
+HgfsReplyCreateSessionV4;
 
 typedef
 #include "vmware_pack_begin.h"
-struct HgfsRequestDestroySession {
-   uint64 sessionId;   /* Session ID. */
+struct HgfsRequestDestroySessionV4 {
    uint64 reserved;    /* Reserved for future use. */
 }
 #include "vmware_pack_end.h"
-HgfsRequestDestroySession;
+HgfsRequestDestroySessionV4;
 
 typedef
 #include "vmware_pack_begin.h"
-struct HgfsReplyDestroySession {
+struct HgfsReplyDestroySessionV4 {
    uint64 reserved;    /* Reserved for future use. */
 }
 #include "vmware_pack_end.h"
-HgfsReplyDestroySession;
+HgfsReplyDestroySessionV4;
 
 /* Adds new error status: HGFS_STATUS_INVALID_SESSION. */
 
