@@ -169,6 +169,8 @@ EXTERN char *Msg_GetStringSafe(const char *idString);
 EXTERN char *Msg_GetPlainButtonText(const char *idString);
 EXTERN const char *Msg_GetLocale(void);
 EXTERN void Msg_SetLocale(const char *locale, const char *binaryName);
+EXTERN void Msg_SetLocaleEx(const char *locale, const char *binaryName,
+                            const char *baseDirPath);
 EXTERN char *Msg_GetMessageFilePath(const char *locale, const char *binaryName,
 				    const char *extension);
 EXTERN char *Msg_FormatFloat(double value, unsigned int precision);
