@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2009-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2009 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -43,33 +43,24 @@
 
 typedef struct MsgList MsgList;
 struct MsgList {
-   MsgList     *next;
-   char        *id;
-   char        *format;
-   MsgFmt_Arg  *args;
-   int          numArgs;
+   MsgList *next;
+   char *id;
+   char *format;
+   MsgFmt_Arg *args;
+   int numArgs;
 };
 
 /*
  * Functions
  */
 
-MsgList *MsgList_Create(const char *idFmt, ...) PRINTF_DECL(1, 2);
-MsgList *MsgList_VCreate(const char *idFmt, va_list args);
-MsgList *MsgList_CreateStr(const char *id);
+EXTERN MsgList *MsgList_Create(const char *idFmt, ...);
+EXTERN MsgList *MsgList_VCreate(const char *idFmt, va_list args);
 
-void MsgList_Append(MsgList **tail, const char *idFmt, ...) PRINTF_DECL(2, 3);
-void MsgList_VAppend(MsgList **tail, const char *idFmt, va_list args);
-void MsgList_AppendStr(MsgList **tail, const char *id);
-void MsgList_AppendMsgList(MsgList **tail, MsgList *messages);
+EXTERN void MsgList_Log(const MsgList *messages);
+EXTERN MsgList *MsgList_Copy(const MsgList *src);
+EXTERN void MsgList_Free(MsgList *messages);
 
-void MsgList_Log(const MsgList *messages);
-char *MsgList_ToString(const MsgList *messages);
-MsgList *MsgList_Copy(const MsgList *src);
-void MsgList_Free(MsgList *messages);
-
-const char *MsgList_GetMsgID(const MsgList *messages);
-
-Bool MsgList_Present(const MsgList *messages);
+EXTERN const char *MsgList_GetMsgID(const MsgList *messages);
 
 #endif // ifndef _MSGLIST_H_
