@@ -50,7 +50,6 @@
 #include "includeCheck.h"
 
 #include "vmware.h"
-#include "rpcin.h"
 
 
 /*
@@ -67,17 +66,31 @@
 #   undef Bool
 typedef Display *       InitHandle;
 
-#elif defined(__APPLE__)
+#elif defined(__APPLE__) || defined(RESOLUTION_WIN32)
 typedef void *          InitHandle;
 #else
 #   error Unknown display backend
 #endif
 
+/*
+ * Arguments to VMwareResolutionSet.exe
+ */
+#define RESOLUTION_SET_APP_NAME "VMwareResolutionSet.exe"
+typedef enum {
+   RESOLUTION_SET_NORESET   = 0,
+   RESOLUTION_SET_RESET     = 1,
+   RESOLUTION_SET_ARBITRARY = 2,
+} ResolutionSetDisplayReset;
+
 
 /*
  * Global functions
  */
 
+#if !defined(VMTOOLS_USE_GLIB)
+
+#include "rpcin.h"
+
 Bool Resolution_Init(const char *tcloChannel, InitHandle handle);
 void Resolution_Cleanup(void);
 
@@ -87,4 +100,6 @@ void Resolution_CleanupBackdoor(void);
 Bool Resolution_RegisterCaps(void);
 Bool Resolution_UnregisterCaps(void);
 
+#endif
+
 #endif // ifndef _LIB_RESOLUTION_H_
