@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2009-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2009 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -56,22 +56,32 @@
 #    define REG_R14 GNU_REG_R14
 #    define REG_R15 GNU_REG_R15
 #  elif defined(__i386__)
-#    define REG_EAX GNU_REG_EAX
-#    define REG_EBX GNU_REG_EBX
-#    define REG_ECX GNU_REG_ECX
-#    define REG_EDX GNU_REG_EDX
-#    define REG_ESI GNU_REG_ESI
-#    define REG_EDI GNU_REG_EDI
-#    define REG_ESP GNU_REG_ESP
-#    define REG_EBP GNU_REG_EBP
-#    define REG_EIP GNU_REG_EIP
+#    if GLIBC_VERSION_22
+#       define REG_EAX GNU_REG_EAX
+#       define REG_EBX GNU_REG_EBX
+#       define REG_ECX GNU_REG_ECX
+#       define REG_EDX GNU_REG_EDX
+#       define REG_ESI GNU_REG_ESI
+#       define REG_EDI GNU_REG_EDI
+#       define REG_ESP GNU_REG_ESP
+#       define REG_EBP GNU_REG_EBP
+#       define REG_EIP GNU_REG_EIP
+#    else
+#       define EAX GNU_REG_EAX
+#       define EBX GNU_REG_EBX
+#       define ECX GNU_REG_ECX
+#       define EDX GNU_REG_EDX
+#       define ESI GNU_REG_ESI
+#       define EDI GNU_REG_EDI
+#       define ESP GNU_REG_ESP
+#       define EBP GNU_REG_EBP
+#       define EIP GNU_REG_EIP
+#    endif
 #  endif
 #endif
 
 #include <signal.h>
-#ifndef __ANDROID__
 #include <sys/ucontext.h>
-#endif
 
 #if __linux__
 #  if defined(__x86_64__)
@@ -93,15 +103,27 @@
 #    undef REG_R14
 #    undef REG_R15
 #  elif defined(__i386__)
-#    undef REG_EAX
-#    undef REG_EBX
-#    undef REG_ECX
-#    undef REG_EDX
-#    undef REG_ESI
-#    undef REG_EDI
-#    undef REG_ESP
-#    undef REG_EBP
-#    undef REG_EIP
+#    if GLIBC_VERSION_22
+#       undef REG_EAX
+#       undef REG_EBX
+#       undef REG_ECX
+#       undef REG_EDX
+#       undef REG_ESI
+#       undef REG_EDI
+#       undef REG_ESP
+#       undef REG_EBP
+#       undef REG_EIP
+#    else
+#       undef EAX
+#       undef EBX
+#       undef ECX
+#       undef EDX
+#       undef ESI
+#       undef EDI
+#       undef ESP
+#       undef EBP
+#       undef EIP
+#    endif
 #  endif
 #endif
 
@@ -125,23 +147,6 @@
 #define SC_R13(uc) ((unsigned long) (uc)->uc_mcontext->__ss.__r13)
 #define SC_R14(uc) ((unsigned long) (uc)->uc_mcontext->__ss.__r14)
 #define SC_R15(uc) ((unsigned long) (uc)->uc_mcontext->__ss.__r15)
-#elif defined(__arm__)
-#define SC_R0(uc)  ((unsigned long) (uc)->uc_mcontext->__ss.__r[0])
-#define SC_R1(uc)  ((unsigned long) (uc)->uc_mcontext->__ss.__r[1])
-#define SC_R2(uc)  ((unsigned long) (uc)->uc_mcontext->__ss.__r[2])
-#define SC_R3(uc)  ((unsigned long) (uc)->uc_mcontext->__ss.__r[3])
-#define SC_R4(uc)  ((unsigned long) (uc)->uc_mcontext->__ss.__r[4])
-#define SC_R5(uc)  ((unsigned long) (uc)->uc_mcontext->__ss.__r[5])
-#define SC_R6(uc)  ((unsigned long) (uc)->uc_mcontext->__ss.__r[6])
-#define SC_R7(uc)  ((unsigned long) (uc)->uc_mcontext->__ss.__r[7])
-#define SC_R8(uc)  ((unsigned long) (uc)->uc_mcontext->__ss.__r[8])
-#define SC_R9(uc)  ((unsigned long) (uc)->uc_mcontext->__ss.__r[9])
-#define SC_R10(uc) ((unsigned long) (uc)->uc_mcontext->__ss.__r[10])
-#define SC_FP(uc)  ((unsigned long) (uc)->uc_mcontext->__ss.__r[11])
-#define SC_IP(uc)  ((unsigned long) (uc)->uc_mcontext->__ss.__r[12])
-#define SC_SP(uc)  ((unsigned long) (uc)->uc_mcontext->__ss.__sp)
-#define SC_LR(uc)  ((unsigned long) (uc)->uc_mcontext->__ss.__lr)
-#define SC_PC(uc)  ((unsigned long) (uc)->uc_mcontext->__ss.__pc)
 #else
 #define SC_EAX(uc) ((unsigned long) (uc)->uc_mcontext->__ss.__eax)
 #define SC_EBX(uc) ((unsigned long) (uc)->uc_mcontext->__ss.__ebx)
@@ -224,16 +229,6 @@
 #define SC_ESP(uc) ((unsigned long) (uc)->uc_mcontext.gregs[ESP])
 #define SC_EIP(uc) ((unsigned long) (uc)->uc_mcontext.gregs[EIP])
 #endif
-#elif defined(ANDROID_X86)
-#define SC_EAX(uc) ((unsigned long) (uc)->uc_mcontext.eax)
-#define SC_EBX(uc) ((unsigned long) (uc)->uc_mcontext.ebx)
-#define SC_ECX(uc) ((unsigned long) (uc)->uc_mcontext.ecx)
-#define SC_EDX(uc) ((unsigned long) (uc)->uc_mcontext.edx)
-#define SC_EDI(uc) ((unsigned long) (uc)->uc_mcontext.edi)
-#define SC_ESI(uc) ((unsigned long) (uc)->uc_mcontext.esi)
-#define SC_EBP(uc) ((unsigned long) (uc)->uc_mcontext.ebp)
-#define SC_ESP(uc) ((unsigned long) (uc)->uc_mcontext.esp)
-#define SC_EIP(uc) ((unsigned long) (uc)->uc_mcontext.eip)
 #else
 #ifdef __x86_64__
 #define SC_EAX(uc) ((unsigned long) (uc)->uc_mcontext.gregs[GNU_REG_RAX])
@@ -253,27 +248,6 @@
 #define SC_R13(uc) ((unsigned long) (uc)->uc_mcontext.gregs[GNU_REG_R13])
 #define SC_R14(uc) ((unsigned long) (uc)->uc_mcontext.gregs[GNU_REG_R14])
 #define SC_R15(uc) ((unsigned long) (uc)->uc_mcontext.gregs[GNU_REG_R15])
-#elif defined(__arm__)
-#define SC_R0(uc)  ((unsigned long) (uc)->uc_mcontext.arm_r0)
-#define SC_R1(uc)  ((unsigned long) (uc)->uc_mcontext.arm_r1)
-#define SC_R2(uc)  ((unsigned long) (uc)->uc_mcontext.arm_r2)
-#define SC_R3(uc)  ((unsigned long) (uc)->uc_mcontext.arm_r3)
-#define SC_R4(uc)  ((unsigned long) (uc)->uc_mcontext.arm_r4)
-#define SC_R5(uc)  ((unsigned long) (uc)->uc_mcontext.arm_r5)
-#define SC_R6(uc)  ((unsigned long) (uc)->uc_mcontext.arm_r6)
-#define SC_R7(uc)  ((unsigned long) (uc)->uc_mcontext.arm_r7)
-#define SC_R8(uc)  ((unsigned long) (uc)->uc_mcontext.arm_r8)
-#define SC_R9(uc)  ((unsigned long) (uc)->uc_mcontext.arm_r9)
-#define SC_R10(uc) ((unsigned long) (uc)->uc_mcontext.arm_r10)
-#define SC_FP(uc)  ((unsigned long) (uc)->uc_mcontext.arm_fp)
-#define SC_IP(uc)  ((unsigned long) (uc)->uc_mcontext.arm_ip)
-#define SC_SP(uc)  ((unsigned long) (uc)->uc_mcontext.arm_sp)
-#define SC_LR(uc)  ((unsigned long) (uc)->uc_mcontext.arm_lr)
-#define SC_PC(uc)  ((unsigned long) (uc)->uc_mcontext.arm_pc)
-#elif defined(__aarch64__)
-#define SC_X(uc,n) ((unsigned long) (uc)->uc_mcontext.regs[n])
-#define SC_SP(uc)  ((unsigned long) (uc)->uc_mcontext.sp)
-#define SC_PC(uc)  ((unsigned long) (uc)->uc_mcontext.pc)
 #else
 #define SC_EAX(uc) ((unsigned long) (uc)->uc_mcontext.gregs[GNU_REG_EAX])
 #define SC_EBX(uc) ((unsigned long) (uc)->uc_mcontext.gregs[GNU_REG_EBX])
