@@ -163,6 +163,11 @@ EXTERN char *Util_GetSafeTmpDir(Bool useConf);
 EXTERN int Util_MakeSafeTemp(ConstUnicode tag,
                              Unicode *presult);
 
+typedef struct UtilSingleUseResource UtilSingleUseResource;
+UtilSingleUseResource *Util_SingleUseAcquire(const char *name);
+void Util_SingleUseRelease(UtilSingleUseResource *res);
+
+
 #if defined(__linux__) || defined(__FreeBSD__) || defined(sun)
 EXTERN Bool Util_GetProcessName(pid_t pid, char *bufOut, size_t bufOutSize);
 #endif
@@ -546,71 +551,4 @@ Util_FreeStringList(char **list,      // IN/OUT: the list to free
 {
    Util_FreeList((void **) list, length);
 }
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * fls0 --
- *
- *      Highest bit set in the 'n', or -1 if all bits are clear.
- *
- * Results:
- *      fls0 (== floor(log2(n)) for n != 0)
- *
- * Side effects:
- *      None.
- *
- *-----------------------------------------------------------------------------
- */
-
-static INLINE int
-fls0(unsigned int n)
-{
-   if (n) {
-#ifdef _MSC_VER
-#  ifdef VM_X86_64
-      unsigned long idx;
-
-      _BitScanReverse(&idx, n);
-      return idx;
-#  else
-      int idx;
-
-      __asm bsr eax, n
-      __asm mov idx, eax
-      return idx;
-#  endif
-#else
-      int idx;
-
-      __asm__ __volatile__("bsrl %1, %0"
-                           :"=r"(idx)
-                           :"rm"(n)
-                           :"cc");
-      return idx;
-#endif
-   }
-   return -1;
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * Util_Log2 --
- *
- *      Compute floor(log2(n)).  Returns -1 for n == 0 (-1 == log2(0.5))
- *
- * Results:
- *      floor(log2(n))
- *
- * Side effects:
- *      None.
- *
- *-----------------------------------------------------------------------------
- */
-
-#define Util_Log2(n) fls0(n)
-
 #endif /* UTIL_H */
