@@ -78,7 +78,7 @@ enum VixCommonCommandOptionValues {
    VIX_COMMAND_FORWARD_TO_GUEST              = 0x04,
    VIX_COMMAND_GUEST_RETURNS_STRING          = 0x08,
    VIX_COMMAND_GUEST_RETURNS_INTEGER_STRING  = 0x10,
-   VIX_COMMAND_GUEST_RETURNS_ENCODED_STRING  = 0x20,
+   /* DEPRECATED VIX_COMMAND_GUEST_RETURNS_ENCODED_STRING  = 0x20, */
    VIX_COMMAND_GUEST_RETURNS_PROPERTY_LIST   = 0x40,
    VIX_COMMAND_GUEST_RETURNS_BINARY          = 0x80,
    // We cannot add more constants here. This is stored in a uint8,
@@ -727,6 +727,22 @@ enum VixListDirectoryOptions {
    VIX_LIST_DIRECTORY_USE_OFFSET = 0x01
 };
 
+typedef
+#include "vmware_pack_begin.h"
+struct VixMsgListFilesRequest {
+   VixCommandRequestHeader header;
+
+   int32                   fileOptions;
+   uint32                  guestPathNameLength;
+   uint32                  patternLength;
+   int32                   index;
+   int32                   maxResults;
+   uint64                  offset;
+}
+#include "vmware_pack_end.h"
+VixMsgListFilesRequest;
+
+
 /*
  * This is used to reply to several operations, like testing whether
  * a file or registry key exists on the client.
@@ -760,27 +776,6 @@ struct VixMsgCreateFileRequest {
 VixMsgCreateFileRequest;
 
 
-/*
- * **********************************************************
- * Hot add and remove a disk in a running VM.
- */
-typedef
-#include "vmware_pack_begin.h"
-struct VixMsgHotDiskRequest {
-   VixCommandRequestHeader header;
-   int32                    hotDiskOptions;
-   uint32                   adapterTypeLength;
-   uint32                   typeLength;
-   uint32                   nameLength;
-   uint32                   modeLength;
-   uint32                   deviceTypeLength;
-   int32                    adapterNum;
-   int32                    targetNum;
-}
-#include "vmware_pack_end.h"
-VixMsgHotDiskRequest;
-
-
 /*
  * **********************************************************
  * Hot extend a disk in a running VM.
@@ -1165,33 +1160,6 @@ struct VixMsgListRollingTierResponse {
 VixMsgListRollingTierResponse;
 
 
-/*
- * Fork a running VM.
- */
-typedef
-#include "vmware_pack_begin.h"
-struct VixMsgVMForkRequest {
-   VixCommandRequestHeader    header;
-
-   int32                      options;
-   Bool                       disconnectRemovable;
-
-   uint32                     cfgFileNameLen;
-   uint32                     displayNameLen;
-}
-#include "vmware_pack_end.h"
-VixMsgVMForkRequest;
-
-
-typedef
-#include "vmware_pack_begin.h"
-struct VixMsgVMForkResponse {
-   VixCommandResponseHeader  header;
-}
-#include "vmware_pack_end.h"
-VixMsgVMForkResponse;
-
-
 /*
  * Stop recording or playback of a snapshot event log.
  */
@@ -1367,35 +1335,6 @@ struct VixMsgSetSharedFolderRequest {
 VixMsgSetSharedFolderRequest;
 
 
-
-/*
- * **********************************************************
- * Get properties of a disk.
- */
-typedef
-#include "vmware_pack_begin.h"
-struct VixMsgGetDiskPropertiesRequest {
-   VixCommandRequestHeader   header;
-
-   int32                     options;
-   uint32                    diskPathNameLength;
-}
-#include "vmware_pack_end.h"
-VixMsgGetDiskPropertiesRequest;
-
-typedef
-#include "vmware_pack_begin.h"
-struct VixMsgGetDiskPropertiesResponse {
-   VixCommandResponseHeader  header;
-
-   int64                     capacity;
-   int64                     spaceUsed;
-   int32                     diskLibDiskType;
-   uint32                    physicalPathLength;
-}
-#include "vmware_pack_end.h"
-VixMsgGetDiskPropertiesResponse;
-
 /*
  * **********************************************************
  * Open a URL in the guest.
@@ -1866,175 +1805,6 @@ VixMsgSampleCommandResponse;
 // End of "HOWTO: Adding a new Vix Command. Step 3."
 
 
-
-/*
- * **********************************************************
- * Report and manage the state of a Msg_Post dialogs.
- */
-
-/*
- * This is used to report a MsgPost is opening.
- * This is the non-localized version. It passes the original format string
- * and a list of vararg-style parameters.
- *
- * See VixMsgOpenMsgPostEventLocalized for the localized version.
- */
-typedef
-#include "vmware_pack_begin.h"
-struct VixMsgOpenMsgPostEvent {
-   VixMsgEventHeader          eventHeader;
-
-   int32                      dialogType;
-   uint64                     dialogCookie;
-   int32                      dialogOptions;
-
-   int32                      severity;
-   int32                      defaultAnswer;
-   int32                      percent;
-   int32                      cancelButton;
-   int32                      hintOptions;
-
-   uint32                     localeStrLength;
-   int32                      numMessages;
-   int32                      numButtons;
-
-   /*
-    * Followed by:
-    *       A locale string (a null-terminated string)
-    *       A list of messages, each is stored in one VixMsgDialogStr.
-    *       A list of button strings (each is a null-terminated string.
-    */
-}
-#include "vmware_pack_end.h"
-VixMsgOpenMsgPostEvent;
-
-/*
- * These are the flags set in the dialogOptions field.
- */
-enum VixMsgPostStateValues {
-   VIX_COMMAND_DIALOG_OPTIONS_VMX_IS_BLOCKED       = 0x01,
-};
-
-
-/*
- * This is one string in the message. It corresponds to a
- * single MsgList object.
- */
-typedef
-#include "vmware_pack_begin.h"
-struct VixMsgDialogStr {
-   uint32         idLength;
-   uint32         formatLength;
-   int32          numArgs;
-   /*
-    * Followed by:
-    *       The ID string (with NULL terminator)
-    *       The format string (with NULL terminator)
-    *       A list of arguments, each is one VixMsgDialogStrArg.
-    */
-}
-#include "vmware_pack_end.h"
-VixMsgDialogStr;
-
-
-/*
- * This is one argument to the message. It corresponds to a
- * single MsgFmt_Arg object.
- */
-typedef
-#include "vmware_pack_begin.h"
-struct VixMsgDialogStrArg {
-   int32          argType;
-   uint32         argSize;
-   /*
-    * Followed by the actual argument data.
-    */
-}
-#include "vmware_pack_end.h"
-VixMsgDialogStrArg;
-
-
-/*
- * This is used to report a MsgPost is closing.
- */
-typedef
-#include "vmware_pack_begin.h"
-struct VixMsgCloseUIDialogEvent {
-   VixMsgEventHeader          eventHeader;
-
-   uint64                     dialogCookie;
-   int32                      numMessages;
-
-   /*
-    * Followed by:
-    *       A list of strings, each is one MsgPost Id.
-    */
-}
-#include "vmware_pack_end.h"
-VixMsgCloseUIDialogEvent;
-
-
-/*
- * Answer a Msg_Post post/hint/question
- */
-typedef
-#include "vmware_pack_begin.h"
-struct VixMsgAnswerMsgPost {
-   VixCommandRequestHeader   header;
-
-   uint64                    dialogCookie;
-
-   int32                     options;
-
-   int32                     answer;
-   void                      *progressState;
-
-   uint32                    idStrSize;
-   uint32                    propertyListBufferSize;
-
-   /*
-    * Followed by:
-    *       msgIdStr.
-    *       The serialized properties
-    */
-}
-#include "vmware_pack_end.h"
-VixMsgAnswerMsgPost;
-
-
-
-typedef
-#include "vmware_pack_begin.h"
-struct VixMsgSetLocaleRequest {
-   VixCommandRequestHeader header;
-
-   int32                   localeOptions;
-   
-   uint32                  localeStrLength;
-   char                    localeStr[1];
-   // This is followed by the country code string.
-}
-#include "vmware_pack_end.h"
-VixMsgSetLocaleRequest;
-
-
-
-/*
- * This is used to report progress.
- */
-typedef
-#include "vmware_pack_begin.h"
-struct VixMsgLazyProgressEvent {
-   VixMsgEventHeader          eventHeader;
-
-   uint64                     dialogCookie;
-   int32                      percent;
-}
-#include "vmware_pack_end.h"
-VixMsgLazyProgressEvent;
-
-
-
 /*
  * **********************************************************
  *  Debugger related commands.
@@ -2257,8 +2027,8 @@ enum {
    VIX_COMMAND_VM_RESET                         = 2,
    VIX_COMMAND_VM_SUSPEND                       = 3,
    VIX_COMMAND_RUN_PROGRAM                      = 4,
-   VIX_COMMAND_GET_PROPERTY                     = 5,
-   VIX_COMMAND_SET_PROPERTY                     = 6,
+   /* DEPRECATED VIX_COMMAND_GET_PROPERTY                     = 5, */
+   /* DEPRECATED VIX_COMMAND_SET_PROPERTY                     = 6, */
    VIX_COMMAND_KEYSTROKES                       = 7,
    VIX_COMMAND_READ_REGISTRY                    = 8,
    VIX_COMMAND_WRITE_REGISTRY                   = 10,
@@ -2285,8 +2055,8 @@ enum {
    VIX_COMMAND_DELETE_VM                        = 32,
    VIX_COMMAND_SYNCDRIVER_FREEZE                = 33,
    VIX_COMMAND_SYNCDRIVER_THAW                  = 34,
-   VIX_COMMAND_HOT_ADD_DISK                     = 35,
-   VIX_COMMAND_HOT_REMOVE_DISK                  = 36,
+   /* DEPRECATED VIX_COMMAND_HOT_ADD_DISK                     = 35, */
+   /* DEPRECATED VIX_COMMAND_HOT_REMOVE_DISK                  = 36, */
    VIX_COMMAND_SET_GUEST_PRINTER                = 37,
    VIX_COMMAND_WAIT_FOR_TOOLS                   = 38,
    VIX_COMMAND_CREATE_RUNNING_VM_SNAPSHOT       = 39,
@@ -2298,10 +2068,10 @@ enum {
    VIX_COMMAND_ADD_SHARED_FOLDER                = 45,
    VIX_COMMAND_RUN_SCRIPT_IN_GUEST              = 46,
    VIX_COMMAND_OPEN_VM                          = 47,
-   VIX_COMMAND_GET_DISK_PROPERTIES              = 48,
+   /* DEPRECATED VIX_COMMAND_GET_DISK_PROPERTIES              = 48, */
    VIX_COMMAND_OPEN_URL                         = 49,
    VIX_COMMAND_GET_HANDLE_STATE                 = 50,
-   VIX_COMMAND_SET_HANDLE_STATE                 = 51,
+   /* DEPRECATED VIX_COMMAND_SET_HANDLE_STATE                 = 51, */
    VIX_COMMAND_CREATE_WORKING_COPY              = 55, // DELETE this when we switch remote foundry to VIM
    VIX_COMMAND_DISCARD_WORKING_COPY             = 56, // DELETE this when we switch remote foundry to VIM
    VIX_COMMAND_SAVE_WORKING_COPY                = 57, // DELETE this when we switch remote foundry to VIM
@@ -2326,7 +2096,7 @@ enum {
    VIX_CREATE_SESSION_KEY_COMMAND               = 83,
    VMXI_HGFS_SEND_PACKET_COMMAND                = 84,
    VIX_COMMAND_KILL_PROCESS                     = 85,
-   VIX_VM_FORK_COMMAND                          = 86,
+   /* DEPRECATED VIX_VM_FORK_COMMAND                          = 86, */
    VIX_COMMAND_LOGOUT_IN_GUEST                  = 87,
    VIX_COMMAND_READ_VARIABLE                    = 88,
    VIX_COMMAND_WRITE_VARIABLE                   = 89,
@@ -2369,11 +2139,11 @@ enum {
 
    VIX_COMMAND_VM_PAUSE                         = 122,
    VIX_COMMAND_VM_UNPAUSE                       = 123,
-   VIX_COMMAND_GET_SNAPSHOT_LOG_INFO            = 124,
+   /* DEPRECATED VIX_COMMAND_GET_SNAPSHOT_LOG_INFO            = 124, */
    VIX_COMMAND_SET_REPLAY_SPEED                 = 125,
 
-   VIX_COMMAND_ANSWER_USER_MESSAGE              = 126,
-   VIX_COMMAND_SET_CLIENT_LOCALE                = 127,
+   /* DEPRECATED VIX_COMMAND_ANSWER_USER_MESSAGE              = 126, */
+   /* DEPRECATED VIX_COMMAND_SET_CLIENT_LOCALE                = 127, */
 
    VIX_COMMAND_GET_PERFORMANCE_DATA             = 128,
 
@@ -2383,8 +2153,8 @@ enum {
    VIX_COMMAND_ADD_TIMEMARKER                   = 131,
 
    VIX_COMMAND_WAIT_FOR_USER_ACTION_IN_GUEST    = 132,
-   VIX_COMMAND_VMDB_END_TRANSACTION             = 133,
-   VIX_COMMAND_VMDB_SET                         = 134,
+   /* DEPRECATED VIX_COMMAND_VMDB_END_TRANSACTION             = 133, */
+   /* DEPRECATED VIX_COMMAND_VMDB_SET                         = 134, */
 
    VIX_COMMAND_CHANGE_VIRTUAL_HARDWARE          = 135,
 
@@ -2404,7 +2174,7 @@ enum {
    VIX_COMMAND_SET_REPLAY_STATE                 = 147,
    VIX_COMMAND_REMOVE_REPLAY_STATE              = 148,
 
-   VIX_COMMAND_CANCEL_USER_PROGRESS_MESSAGE     = 150,
+   /* DEPRECATED VIX_COMMAND_CANCEL_USER_PROGRESS_MESSAGE     = 150, */
    
    VIX_COMMAND_GET_VMX_DEVICE_STATE             = 151,
 
@@ -2446,6 +2216,10 @@ enum {
 
    VIX_COMMAND_CHANGE_DISPLAY_TOPOLOGY_MODES    = 175,
 
+   VIX_COMMAND_QUERY_CHILDREN                   = 176,
+
+   VIX_COMMAND_LIST_FILES                       = 177,
+
    /*
     * HOWTO: Adding a new Vix Command. Step 2a.
     *
@@ -2456,7 +2230,7 @@ enum {
     * Once a new command is added here, a command info field needs to be added
     * in bora/lib/foundryMsg. as well.
     */
-   VIX_COMMAND_LAST_NORMAL_COMMAND              = 176,
+   VIX_COMMAND_LAST_NORMAL_COMMAND              = 178,
 
    VIX_TEST_UNSUPPORTED_TOOLS_OPCODE_COMMAND    = 998,
    VIX_TEST_UNSUPPORTED_VMX_OPCODE_COMMAND      = 999,
