@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2003 VMware, Inc. All rights reserved.
+ * Copyright (C) 2003-2010 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -194,6 +194,14 @@ Max(int a, int b)
 #define XSTR(x)                 MAKESTR(x)
 
 
+/*
+ * Wide versions of string constants.
+ */
+
+#define WSTR_(X)     L ## X
+#define WSTR(X)      WSTR_(X)
+
+
 /*
  * Page operations
  *
@@ -327,36 +335,25 @@ void *_ReturnAddress(void);
 #ifdef __GNUC__
 #ifndef sun
 
-/*
- * Get the frame pointer. We use this assembly hack instead of
- * __builtin_frame_address() due to a bug introduced in gcc 4.1.1
- */
 static INLINE_SINGLE_CALLER uintptr_t
 GetFrameAddr(void)
 {
    uintptr_t bp;
-#if (__GNUC__ < 4 || (__GNUC__ == 4 && __GNUC_MINOR__ == 0))
+#if  !(__GNUC__ == 4 && (__GNUC_MINOR__ == 0 || __GNUC_MINOR__ == 1))
    bp = (uintptr_t)__builtin_frame_address(0);
-#elif (__GNUC__ == 4 && __GNUC_MINOR__ == 1 && __GNUC_PATCHLEVEL__ <= 3)
-#  if defined(VMM) || defined(VM_X86_64)
-     __asm__ __volatile__("movq %%rbp, %0\n" : "=g" (bp));
-#  else
-     __asm__ __volatile__("movl %%ebp, %0\n" : "=g" (bp));
-#  endif
 #else
-   __asm__ __volatile__(
-#ifdef __linux__
-      ".print \"This newer version of GCC may or may not have the "
-               "__builtin_frame_address bug.  Need to update this. "
-               "See bug 147638.\"\n"
-      ".abort"
-#else /* MacOS */
-      ".abort \"This newer version of GCC may or may not have the "
-               "__builtin_frame_address bug.  Need to update this. "
-               "See bug 147638.\"\n"
-#endif
-      : "=g" (bp)
-   );
+   /*
+    * We use this assembly hack due to a bug discovered in gcc 4.1.1.
+    * The bug was fixed in 4.2.0; assume it originated with 4.0.
+    * PR147638, PR554369.
+    */
+     __asm__ __volatile__(
+#  if defined(VM_X86_64)
+                          "movq %%rbp, %0\n"
+#  else
+                          "movl %%ebp, %0\n"
+#  endif
+                          : "=g" (bp));
 #endif
    return bp;
 }
@@ -521,23 +518,12 @@ typedef int pid_t;
  */
 
 #undef DEBUG_ONLY
-#undef SL_DEBUG_ONLY
-#undef VMX86_SL_DEBUG
 #ifdef VMX86_DEBUG
 #define vmx86_debug      1
 #define DEBUG_ONLY(x)    x
-/*
- * Be very, very, very careful with SL_DEBUG. Pls ask ganesh or min before 
- * using it.
- */
-#define VMX86_SL_DEBUG
-#define vmx86_sl_debug   1
-#define SL_DEBUG_ONLY(x) x
 #else
 #define vmx86_debug      0
 #define DEBUG_ONLY(x)
-#define vmx86_sl_debug   0
-#define SL_DEBUG_ONLY(x)
 #endif
 
 #ifdef VMX86_STATS
@@ -662,9 +648,33 @@ typedef int pid_t;
  */
 
 #define RANK_UNRANKED            0
-#define RANK_userlevelLock       100
-#define RANK_LEAF                0xFFFE
-#define RANK_INVALID             0xFFFF
+#define RANK_LEAF                0xFF000000
+#define RANK_INVALID             0xFFFFFFFF
+
+/*
+ * VMX/VMM/device lock rank space is at the bottom, from 1 to
+ * RANK_VMX_LEAF. See vmx/public/mutexRank.h for definitions.
+ */
+
+/*
+ * bora/lib lock rank space is from RANK_libLockBase on up to
+ * RANK_LEAF.
+ */
+#define RANK_libLockBase         0xF0000000
+
+/*
+ * bora/lib/allocTrack is a special case. It hooks malloc/free and the
+ * like, and thus can basically sneak in underneath anyone. To that end
+ * allocTrack uses unranked, native locks internally to avoid any
+ * complications.
+ */
+
+/*
+ * For situations where we need to create locks on behalf of
+ * third-party code, but we don't know what ranking scheme, if any,
+ * that code uses.  For now, the only usage is in bora/lib/ssl.
+ */
+#define RANK_THIRDPARTY          RANK_UNRANKED
 
 /*
  * Use to initialize cbSize for this structure to preserve < Vista
