@@ -170,20 +170,17 @@ typedef signed __int64 int64;
 
 #elif __GNUC__
 /* The Xserver source compiles with -ansi -pendantic */
-#ifndef __STRICT_ANSI__
-#if defined(VM_X86_64)
+#   if !defined(__STRICT_ANSI__) || defined(__FreeBSD__)
+#      if defined(VM_X86_64)
 typedef unsigned long uint64;
 typedef long int64;
-#else
+#      else
 typedef unsigned long long uint64;
 typedef long long int64;
-#endif
-#elif defined __FreeBSD__
-typedef unsigned long long uint64;
-typedef long long int64;
-#endif
+#      endif
+#   endif
 #else
-#error - Need compiler define for int64/uint64
+#   error - Need compiler define for int64/uint64
 #endif /* _MSC_VER */
 
 typedef unsigned int       uint32;
@@ -663,11 +660,6 @@ typedef void * UserVA;
  * extra __always_inline__ attribute.
  */
 #   define INLINE_SINGLE_CALLER INLINE __attribute__((__always_inline__))
-#   if    defined(VMM) \
-       && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ > 1))
-#      warning Verify INLINE_SINGLE_CALLER '__always_inline__' attribute (did \
-             monitor size change?)
-#   endif
 #else
 #   define INLINE_SINGLE_CALLER INLINE
 #endif
@@ -770,10 +762,13 @@ typedef void * UserVA;
 #endif
 
 /*
- * REGPARM defaults to REGPARM3, i.e., a requent that gcc
- * puts the first three arguments in registers.  (It is fine
- * if the function has fewer than three args.)  Gcc only.
+ * REGPARM defaults to REGPARM3; i.e., a request that gcc
+ * put the first three arguments in registers.  (It is fine
+ * if the function has fewer than three arguments.)  Gcc only.
  * Syntactically, put REGPARM where you'd put INLINE or NORETURN.
+ *
+ * Note that 64-bit code already puts the first six arguments in
+ * registers, so these attributes are only useful for 32-bit code.
  */
 
 #if defined(__GNUC__)
