@@ -136,6 +136,8 @@
 #define PRODUCT_VDM_CLIENT_NAME MAKE_NAME("View Client")
 #define PRODUCT_VDM_CLIENT_NAME_FOR_LICENSE PRODUCT_VDM_CLIENT_NAME
 
+#define PRODUCT_RMKSCONTAINER_NAME MAKE_NAME("Remote MKS Container")
+
 // XXX VMvisor is the underlying technology for possibly several products,
 // XXX not the product. Fix when names are decided.
 // #define PRODUCT_VMVISOR_NAME MAKE_NAME("VMvisor")
@@ -154,26 +156,15 @@
 #define PRODUCT_VMRC_EXECUTABLE PRODUCT_GENERIC_NAME_LOWER "-vmrc.exe"
 #else
 #define PRODUCT_VMRC_EXECUTABLE PRODUCT_GENERIC_NAME_LOWER "-vmrc"
+#define PRODUCT_VMRC_DEVICE_EXECUTABLE PRODUCT_GENERIC_NAME_LOWER "-deviceMgr"
 #endif
 
 /*
  * VMRC ActiveX CLSIDs and ProgIDs
  */
-#define PRODUCT_VMRC_PLUGIN_GUID_WEB           B94C2238-346E-4c5e-9B36-8CC627F35574
-#define PRODUCT_VMRC_PLUGIN_GUID_WEB_TYPELIB   84D652E1-9364-4536-B13C-24F16FBEBCFE
-#define PRODUCT_VMRC_PLUGIN_WEB_CLSID          CLSID:##PRODUCT_VMRC_PLUGIN_GUID_WEB
-#define PRODUCT_VMRC_PLUGIN_WEB_PROGID_BASE    "VMware.web.VMwareRemoteConsole"
-#define PRODUCT_VMRC_PLUGIN_WEB_PROGID         PRODUCT_VMRC_PLUGIN_WEB_PROGID_BASE "." \
-                                               XSTR(VMRC_PLUGIN_VERSION_BASE)
-
-#define PRODUCT_VMRC_PLUGIN_GUID               B94C2248-346E-4C5E-9B36-8CC627F35574
-#define PRODUCT_VMRC_PLUGIN_GUID_EMBEDDED      B94C2249-346E-4C5E-9B36-8CC627F35574
-#define PRODUCT_VMRC_PLUGIN_GUID_QUICKMKSAXCTL 338095E4-1806-4BA3-AB51-38A3179200E9
+#define PRODUCT_VMRC_PLUGIN_GUID_EMBEDDED      0567856D-8D8E-43ad-9241-7772F392B3A3
 #define PRODUCT_VMRC_PLUGIN_GUID_TYPELIB       A51B0CB2-1545-43AD-B66E-AD322009406C
-#define PRODUCT_VMRC_PLUGIN_PROGID_BASE        "VMware.hosted.VMwareRemoteConsole"
-#define PRODUCT_VMRC_PLUGIN_PROGID             PRODUCT_VMRC_PLUGIN_PROGID_BASE "." \
-                                               XSTR(VMRC_PLUGIN_VERSION_BASE)
-#define PRODUCT_VMRC_PLUGIN_PROGID_EMBEDDED_BASE "VMware.hosted.VMwareEmbeddedRemoteConsole"
+#define PRODUCT_VMRC_PLUGIN_PROGID_EMBEDDED_BASE "VMware.VMwareRemoteConsole"
 #define PRODUCT_VMRC_PLUGIN_PROGID_EMBEDDED    PRODUCT_VMRC_PLUGIN_PROGID_EMBEDDED_BASE "." \
                                                XSTR(VMRC_PLUGIN_VERSION_BASE)
 
@@ -222,6 +213,18 @@
    PRODUCT_VMRC_PLUGIN_CURRENT_MIMETYPE PRODUCT_VMRC_MIMETYPE_SEPARATOR \
    PRODUCT_VMRC_PLUGIN_PREVIOUS_MIMETYPES PRODUCT_VMRC_MIMETYPE_TERMINATOR
 
+/*
+ * VMware USB Arbitration Service version definitions
+ */
+
+#define PRODUCT_USB_ARBITRATOR_SHORT_NAME "vmware-usbarbitrator"
+#define PRODUCT_USB_ARBITRATOR_NAME MAKE_NAME("USB Arbitration Service")
+#ifdef _WIN32
+#define PRODUCT_USB_ARBITRATOR_EXECUTABLE PRODUCT_GENERIC_NAME_LOWER "-usbarbitrator.exe"
+#else
+#define PRODUCT_USB_ARBITRATOR_EXECUTABLE PRODUCT_GENERIC_NAME_LOWER "-usbarbitrator"
+#endif
+
 /*
  * TODO: This properly lives in productState, but we need it here to
  * define DEFAULT_LIBDIRECTORY.  If that can be moved to productState,
