@@ -983,9 +983,23 @@ typedef uint32 ToolsVersion;
 #define   TOOLS_VERSION_HOSTED10_BETA1_V_MNR  4
 #define   TOOLS_VERSION_HOSTED10_BETA1_V_BASE 0
 
-#define   TOOLS_VERSION_CURRENT        TOOLS_VERSION_HOSTED10_BETA1
-#define   TOOLS_VERSION_CURRENT_STR    TOOLS_VERSION_TO_STR(TOOLS_VERSION_HOSTED10_BETA1)
-#define   TOOLS_VERSION_CURRENT_CSV    TOOLS_VERSION_TO_CSV(TOOLS_VERSION_HOSTED10_BETA1)
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_HOSTED10_RC TOOLS_VERSION_TO_UINT(TOOLS_VERSION_HOSTED10_RC_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_HOSTED10_RC_V_MJR  8
+#define   TOOLS_VERSION_HOSTED10_RC_V_MNR  4
+#define   TOOLS_VERSION_HOSTED10_RC_V_BASE 1
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_HOSTED10_RELEASE TOOLS_VERSION_TO_UINT(TOOLS_VERSION_HOSTED10_RELEASE_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_HOSTED10_RELEASE_V_MJR  8
+#define   TOOLS_VERSION_HOSTED10_RELEASE_V_MNR  4
+#define   TOOLS_VERSION_HOSTED10_RELEASE_V_BASE 2
+
+#define   TOOLS_VERSION_CURRENT        TOOLS_VERSION_HOSTED10_RELEASE
+#define   TOOLS_VERSION_CURRENT_STR    TOOLS_VERSION_TO_STR(TOOLS_VERSION_HOSTED10_RELEASE)
+#define   TOOLS_VERSION_CURRENT_CSV    TOOLS_VERSION_TO_CSV(TOOLS_VERSION_HOSTED10_RELEASE)
 
 /*
  * The extended Tools version is the current Tools version with the
