@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2005-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2005-2010 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -30,7 +30,6 @@
 #include "includeCheck.h"
 
 #include "vm_basic_types.h"
-#include "vm_basic_defs.h"
 #include "vm_atomic.h"
 #include "vm_assert.h"
 
@@ -89,24 +88,6 @@ typedef enum VMCIIntrType {
 #define VMCI_INTR_DATAGRAM     0
 #define VMCI_INTR_NOTIFICATION 1
 
-
-/*
- * A single VMCI device has an upper limit of 128 MiB on the amount of
- * memory that can be used for queue pairs.
- */
-#define VMCI_MAX_GUEST_QP_MEMORY (128 * 1024 * 1024)
-
-/*
- * Queues with pre-mapped data pages must be small, so that we don't pin
- * too much kernel memory (especially on vmkernel).  We limit a queuepair to
- * 32 KB, or 16 KB per queue for symmetrical pairs.
- *
- * XXX, we are raising this limit to 4MB to support high-throughput workloads
- * with vioi-filter.  Once we switch to rings instead of queuepairs for the
- * page channel, we will drop this limit again.  See PR 852983.
- */
-#define VMCI_MAX_PINNED_QP_MEMORY (4 * 1024 * 1024)
-
 /*
  * We have a fixed set of resource IDs available in the VMX.
  * This allows us to have a very simple implementation since we statically
@@ -118,63 +99,36 @@ typedef enum VMCIIntrType {
 typedef uint32 VMCI_Resource;
 
 /* VMCI reserved hypervisor datagram resource IDs. */
-#define VMCI_RESOURCES_QUERY        0
-#define VMCI_GET_CONTEXT_ID         1
-#define VMCI_SET_NOTIFY_BITMAP      2
-#define VMCI_DOORBELL_LINK          3
-#define VMCI_DOORBELL_UNLINK        4
-#define VMCI_DOORBELL_NOTIFY        5
-/*
- * VMCI_DATAGRAM_REQUEST_MAP and VMCI_DATAGRAM_REMOVE_MAP are
- * obsoleted by the removal of VM to VM communication.
- */
-#define VMCI_DATAGRAM_REQUEST_MAP   6
-#define VMCI_DATAGRAM_REMOVE_MAP    7
-#define VMCI_EVENT_SUBSCRIBE        8
-#define VMCI_EVENT_UNSUBSCRIBE      9
-#define VMCI_QUEUEPAIR_ALLOC        10
-#define VMCI_QUEUEPAIR_DETACH       11
+#define VMCI_RESOURCES_QUERY      0
+#define VMCI_GET_CONTEXT_ID       1
+#define VMCI_SET_NOTIFY_BITMAP    2
+#define VMCI_DOORBELL_LINK        3
+#define VMCI_DOORBELL_UNLINK      4
+#define VMCI_DOORBELL_NOTIFY      5
+#define VMCI_DATAGRAM_REQUEST_MAP 6
+#define VMCI_DATAGRAM_REMOVE_MAP  7
+#define VMCI_EVENT_SUBSCRIBE      8
+#define VMCI_EVENT_UNSUBSCRIBE    9
+#define VMCI_QUEUEPAIR_ALLOC      10
+#define VMCI_QUEUEPAIR_DETACH     11
 /*
  * VMCI_VSOCK_VMX_LOOKUP was assigned to 12 for Fusion 3.0/3.1,
  * WS 7.0/7.1 and ESX 4.1
  */
-#define VMCI_HGFS_TRANSPORT         13
-#define VMCI_UNITY_PBRPC_REGISTER   14
-/*
- * This resource is used for VMCI socket control packets sent to the
- * hypervisor (CID 0) because RID 1 is already reserved.
- */
-#define VSOCK_PACKET_HYPERVISOR_RID 15
-#define VMCI_RESOURCE_MAX           16
-/*
- * The core VMCI device functionality only requires the resource IDs of
- * VMCI_QUEUEPAIR_DETACH and below.
- */
-#define VMCI_CORE_DEVICE_RESOURCE_MAX  VMCI_QUEUEPAIR_DETACH
-
-/*
- * VMCI reserved host datagram resource IDs.
- * vsock control channel has resource id 1.
- */
-#define VMCI_DVFILTER_DATA_PATH_DATAGRAM 2
+#define VMCI_HGFS_TRANSPORT       13
+#define VMCI_RESOURCE_MAX         14
 
 /* VMCI Ids. */
 typedef uint32 VMCIId;
 
-typedef struct VMCIIdRange {
-   int8 action;   // VMCI_FA_X, for use in filters.
-   VMCIId begin;  // Beginning of range
-   VMCIId end;    // End of range
-} VMCIIdRange;
-
 typedef struct VMCIHandle {
    VMCIId context;
    VMCIId resource;
 } VMCIHandle;
 
-static INLINE VMCIHandle
-VMCI_MAKE_HANDLE(VMCIId cid,  // IN:
-                 VMCIId rid)  // IN:
+static INLINE
+VMCIHandle VMCI_MAKE_HANDLE(VMCIId cid,
+			    VMCIId rid)
 {
    VMCIHandle h;
    h.context = cid;
@@ -266,19 +220,14 @@ static const VMCIHandle VMCI_INVALID_HANDLE = {VMCI_INVALID_ID,
 #define VMCI_HYPERVISOR_CONTEXT_ID 0
 
 /*
- * Well-known context id, a logical context that contains a set of
- * well-known services. This context ID is now obsolete.
+ * Well-known context id, a logical context that contains
+ * a set of well-known services.
  */
 #define VMCI_WELL_KNOWN_CONTEXT_ID 1
 
-/*
- * Context ID used by host endpoints.
- */
+/* Todo: Change host context id to dynamic/random id. */
 #define VMCI_HOST_CONTEXT_ID  2
 
-#define VMCI_CONTEXT_IS_VM(_cid) (VMCI_INVALID_ID != _cid && \
-                                  _cid > VMCI_HOST_CONTEXT_ID)
-
 /*
  * The VMCI_CONTEXT_RESOURCE_ID is used together with VMCI_MAKE_HANDLE to make
  * handles that refer to a specific context.
@@ -299,7 +248,7 @@ static const VMCIHandle VMCI_INVALID_HANDLE = {VMCI_INVALID_ID,
 #define VMCI_SUCCESS_LAST_DETACH          3
 #define VMCI_SUCCESS_ACCESS_GRANTED       2
 #define VMCI_SUCCESS_ENTRY_DEAD           1
-#define VMCI_SUCCESS                      0LL
+#define VMCI_SUCCESS                      0
 #define VMCI_ERROR_INVALID_RESOURCE      (-1)
 #define VMCI_ERROR_INVALID_ARGS          (-2)
 #define VMCI_ERROR_NO_MEM                (-3)
@@ -339,8 +288,6 @@ static const VMCIHandle VMCI_INVALID_HANDLE = {VMCI_INVALID_ID,
 #define VMCI_ERROR_BUSMEM_INVALIDATION   (-37)
 #define VMCI_ERROR_MODULE_NOT_LOADED     (-38)
 #define VMCI_ERROR_DEVICE_NOT_FOUND      (-39)
-#define VMCI_ERROR_QUEUEPAIR_NOT_READY   (-40)
-#define VMCI_ERROR_WOULD_BLOCK           (-41)
 
 /* VMCI clients should return error code withing this range */
 #define VMCI_ERROR_CLIENT_MIN     (-500)
@@ -354,43 +301,12 @@ static const VMCIHandle VMCI_INVALID_HANDLE = {VMCI_INVALID_ID,
 /* VMCI reserved events. */
 typedef uint32 VMCI_Event;
 
-#define VMCI_EVENT_CTX_ID_UPDATE  0  // Only applicable to guest endpoints
-#define VMCI_EVENT_CTX_REMOVED    1  // Applicable to guest and host
-#define VMCI_EVENT_QP_RESUMED     2  // Only applicable to guest endpoints
-#define VMCI_EVENT_QP_PEER_ATTACH 3  // Applicable to guest, host and VMX
-#define VMCI_EVENT_QP_PEER_DETACH 4  // Applicable to guest, host and VMX
-#define VMCI_EVENT_MEM_ACCESS_ON  5  // Applicable to VMX and vmk.  On vmk,
-                                     // this event has the Context payload type.
-#define VMCI_EVENT_MEM_ACCESS_OFF 6  // Applicable to VMX and vmk.  Same as
-                                     // above for the payload type.
-#define VMCI_EVENT_GUEST_PAUSED   7  // Applicable to vmk. This event has the
-                                     // Context payload type.
-#define VMCI_EVENT_GUEST_UNPAUSED 8  // Applicable to vmk. Same as above for
-                                     // the payload type.
-#define VMCI_EVENT_MAX            9
-
-/*
- * Of the above events, a few are reserved for use in the VMX, and
- * other endpoints (guest and host kernel) should not use them. For
- * the rest of the events, we allow both host and guest endpoints to
- * subscribe to them, to maintain the same API for host and guest
- * endpoints.
- */
-
-#define VMCI_EVENT_VALID_VMX(_event) (_event == VMCI_EVENT_QP_PEER_ATTACH || \
-                                      _event == VMCI_EVENT_QP_PEER_DETACH || \
-                                      _event == VMCI_EVENT_MEM_ACCESS_ON || \
-                                      _event == VMCI_EVENT_MEM_ACCESS_OFF)
-
-#if defined(VMX86_SERVER)
-#define VMCI_EVENT_VALID(_event) (_event < VMCI_EVENT_MAX)
-#else // VMX86_SERVER
-#define VMCI_EVENT_VALID(_event) (_event < VMCI_EVENT_MAX && \
-                                  _event != VMCI_EVENT_MEM_ACCESS_ON && \
-                                  _event != VMCI_EVENT_MEM_ACCESS_OFF && \
-                                  _event != VMCI_EVENT_GUEST_PAUSED && \
-                                  _event != VMCI_EVENT_GUEST_UNPAUSED)
-#endif // VMX86_SERVER
+#define VMCI_EVENT_CTX_ID_UPDATE  0
+#define VMCI_EVENT_CTX_REMOVED    1
+#define VMCI_EVENT_QP_RESUMED     2
+#define VMCI_EVENT_QP_PEER_ATTACH 3
+#define VMCI_EVENT_QP_PEER_DETACH 4
+#define VMCI_EVENT_MAX            5
 
 /* Reserved guest datagram resource ids. */
 #define VMCI_EVENT_HANDLER 0
@@ -416,12 +332,48 @@ typedef uint32 VMCIPrivilegeFlags;
 #define VMCI_PRIVILEGE_FLAG_RESTRICTED     0x01
 #define VMCI_PRIVILEGE_FLAG_TRUSTED        0x02
 #define VMCI_PRIVILEGE_ALL_FLAGS           (VMCI_PRIVILEGE_FLAG_RESTRICTED | \
-                                            VMCI_PRIVILEGE_FLAG_TRUSTED)
+				            VMCI_PRIVILEGE_FLAG_TRUSTED)
 #define VMCI_NO_PRIVILEGE_FLAGS            0x00
 #define VMCI_DEFAULT_PROC_PRIVILEGE_FLAGS  VMCI_NO_PRIVILEGE_FLAGS
 #define VMCI_LEAST_PRIVILEGE_FLAGS         VMCI_PRIVILEGE_FLAG_RESTRICTED
 #define VMCI_MAX_PRIVILEGE_FLAGS           VMCI_PRIVILEGE_FLAG_TRUSTED
 
+/* VMCI Discovery Service. */
+
+/* Well-known handle to the discovery service. */
+#define VMCI_DS_RESOURCE_ID 1 /* Reserved resource ID for discovery service. */
+#define VMCI_DS_HANDLE VMCI_MAKE_HANDLE(VMCI_WELL_KNOWN_CONTEXT_ID, \
+					VMCI_DS_RESOURCE_ID)
+#define VMCI_DS_CONTEXT VMCI_MAKE_HANDLE(VMCI_WELL_KNOWN_CONTEXT_ID, \
+					 VMCI_CONTEXT_RESOURCE_ID)
+
+/* Maximum length of a DS message. */
+#define VMCI_DS_MAX_MSG_SIZE        300
+
+/* Command actions. */
+#define VMCI_DS_ACTION_LOOKUP         0
+#define VMCI_DS_ACTION_REGISTER       1
+#define VMCI_DS_ACTION_UNREGISTER     2
+
+/* Defines wire-protocol format for a request send to the DS from a context. */
+typedef struct VMCIDsRequestHeader {
+   int32       action;
+   int32       msgid;
+   VMCIHandle  handle;
+   int32       nameLen;
+   char        name[1];
+} VMCIDsRequestHeader;
+
+
+/* Defines the wire-protocol format for a request send from the DS to a context. */
+typedef struct VMCIDsReplyHeader {
+   int32       msgid;
+   int32       code;
+   VMCIHandle  handle;
+   int32       msgLen;
+   int8        msg[1];
+} VMCIDsReplyHeader;
+
 #define VMCI_PUBLIC_GROUP_NAME "vmci public group"
 /* 0 through VMCI_RESERVED_RESOURCE_ID_MAX are reserved. */
 #define VMCI_RESERVED_RESOURCE_ID_MAX 1023
@@ -856,63 +808,5 @@ VMCIQueueHeader_BufReady(const VMCIQueueHeader *consumeQHeader, // IN:
 }
 
 
-/*
- * Defines for the VMCI traffic filter:
- * - VMCI_FA_<name> defines the filter action values
- * - VMCI_FP_<name> defines the filter protocol values
- * - VMCI_FD_<name> defines the direction values (guest or host)
- * - VMCI_FT_<name> are the type values (allow or deny)
- */
+#endif
 
-#define VMCI_FA_INVALID -1
-#define VMCI_FA_ALLOW    0
-#define VMCI_FA_DENY     (VMCI_FA_ALLOW + 1)
-#define VMCI_FA_MAX      (VMCI_FA_DENY + 1)
-
-#define VMCI_FP_INVALID     -1
-#define VMCI_FP_HYPERVISOR   0
-#define VMCI_FP_QUEUEPAIR    (VMCI_FP_HYPERVISOR + 1)
-#define VMCI_FP_DOORBELL     (VMCI_FP_QUEUEPAIR + 1)
-#define VMCI_FP_DATAGRAM     (VMCI_FP_DOORBELL + 1)
-#define VMCI_FP_STREAMSOCK   (VMCI_FP_DATAGRAM + 1)
-#define VMCI_FP_ANY          (VMCI_FP_STREAMSOCK + 1)
-#define VMCI_FP_MAX          (VMCI_FP_ANY + 1)
-
-#define VMCI_FD_INVALID  -1
-#define VMCI_FD_GUEST     0
-#define VMCI_FD_HOST      (VMCI_FD_GUEST + 1)
-#define VMCI_FD_ANY       (VMCI_FD_HOST + 1)
-#define VMCI_FD_MAX       (VMCI_FD_ANY + 1)
-
-/*
- * The filter list tracks VMCI Id ranges for a given filter.
- */
-
-typedef struct {
-   uint32 len;
-   VMCIIdRange *list;
-} VMCIFilterList;
-
-
-/*
- * The filter info is used to communicate the filter configuration
- * from the VMX to the host kernel.
- */
-
-typedef struct {
-   VA64   list;   // List of VMCIIdRange
-   uint32 len;    // Length of list
-   uint8  dir;    // VMCI_FD_X
-   uint8  proto;  // VMCI_FP_X
-} VMCIFilterInfo;
-
-/*
- * In the host kernel, the ingoing and outgoing filters are
- * separated. The VMCIProtoFilters type captures all filters in one
- * direction. The VMCIFilters type captures all filters.
- */
-
-typedef VMCIFilterList VMCIProtoFilters[VMCI_FP_MAX];
-typedef VMCIProtoFilters VMCIFilters[VMCI_FD_MAX];
-
-#endif // _VMCI_DEF_H_
