@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2008,2014-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2008 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -24,7 +24,7 @@
  *
  *    Defines the interface between applications and the underlying GuestRPC
  *    channel. The goal is to have an abstraction so applications can run over
- *    the backdoor, vSockets or TCP/IP sockets by just picking up the
+ *    the backdoor, VMCI sockets or TCP/IP sockets by just picking up the
  *    desired channel at runtime, without the need to modify the code.
  *
  *    For this reason, the behavior of all channels is modeled after the RpcIn
@@ -41,11 +41,10 @@
 
 G_BEGIN_DECLS
 
-/** Aliases. */
-#define RPCIN_SETRETVALS  RpcChannel_SetRetVals
-#define RPCIN_SETRETVALSF RpcChannel_SetRetValsF
+/** Alias for RpcChannel_SetRetVals. */
+#define RPCIN_SETRETVALS RpcChannel_SetRetVals
 
-typedef struct _RpcChannel RpcChannel;
+struct RpcChannel;
 
 /** Data structure passed to RPC callbacks. */
 typedef struct RpcInData {
@@ -76,12 +75,6 @@ typedef struct RpcInData {
    void *clientData;
 } RpcInData;
 
-typedef enum RpcChannelType {
-   RPCCHANNEL_TYPE_INACTIVE,
-   RPCCHANNEL_TYPE_BKDOOR,
-   RPCCHANNEL_TYPE_PRIV_VSOCK,
-   RPCCHANNEL_TYPE_UNPRIV_VSOCK
-} RpcChannelType;
 
 /**
  * Type for RpcIn callbacks. The callback function is responsible for
@@ -114,6 +107,21 @@ typedef struct RpcChannelCallback {
    size_t            xdrInSize;
 } RpcChannelCallback;
 
+
+typedef gboolean (*RpcChannelStartFn)(struct RpcChannel *);
+typedef void (*RpcChannelStopFn)(struct RpcChannel *);
+typedef void (*RpcChannelShutdownFn)(struct RpcChannel *);
+typedef gboolean (*RpcChannelSendFn)(struct RpcChannel *,
+                                     char *data,
+                                     size_t dataLen,
+                                     char **result,
+                                     size_t *resultLen);
+typedef void (*RpcChannelSetupFn)(struct RpcChannel *chan,
+                                  GMainContext *mainCtx,
+                                  const char *appName,
+                                  gpointer appCtx);
+
+
 /**
  * Signature for the callback function called after a channel reset.
  *
@@ -121,28 +129,80 @@ typedef struct RpcChannelCallback {
  * @param[in]  success  Whether reset was successful.
  * @param[in]  data     Client data.
  */
-typedef void (*RpcChannelResetCb)(RpcChannel *chan,
+typedef void (*RpcChannelResetCb)(struct RpcChannel *chan,
                                   gboolean success,
                                   gpointer data);
 
-gboolean
-RpcChannel_Start(RpcChannel *chan);
 
-void
-RpcChannel_Stop(RpcChannel *chan);
+/** Defines the interface between the application and the RPC channel. */
+typedef struct RpcChannel {
+   RpcChannelStartFn       start;
+   RpcChannelStopFn        stop;
+   RpcChannelSendFn        send;
+   RpcChannelSetupFn       setup;
+   RpcChannelShutdownFn    shutdown;
+   gpointer                _private;
+} RpcChannel;
 
-RpcChannelType
-RpcChannel_GetType(RpcChannel *chan);
 
-gboolean
+/**
+ * Wrapper for the start function of an RPC channel struct.
+ *
+ * @param[in]  chan        The RPC channel instance.
+ *
+ * @return TRUE on success.
+ */
+
+G_INLINE_FUNC gboolean
+RpcChannel_Start(RpcChannel *chan)
+{
+   g_return_val_if_fail(chan != NULL, FALSE);
+   g_return_val_if_fail(chan->start != NULL, FALSE);
+
+   return chan->start(chan);
+}
+
+
+/**
+ * Wrapper for the stop function of an RPC channel struct.
+ *
+ * @param[in]  chan        The RPC channel instance.
+ */
+
+G_INLINE_FUNC void
+RpcChannel_Stop(RpcChannel *chan)
+{
+   g_return_if_fail(chan != NULL);
+   g_return_if_fail(chan->stop != NULL);
+
+   chan->stop(chan);
+}
+
+
+/**
+ * Wrapper for the send function of an RPC channel struct.
+ *
+ * @param[in]  chan        The RPC channel instance.
+ * @param[in]  data        Data to send.
+ * @param[in]  dataLen     Number of bytes to send.
+ * @param[out] result      Response from other side (should be freed by caller).
+ * @param[out] resultLen   Number of bytes in response.
+ *
+ * @return The status from the remote end (TRUE if call was successful).
+ */
+
+G_INLINE_FUNC gboolean
 RpcChannel_Send(RpcChannel *chan,
-                char const *data,
+                char *data,
                 size_t dataLen,
                 char **result,
-                size_t *resultLen);
+                size_t *resultLen)
+{
+   g_return_val_if_fail(chan != NULL, FALSE);
+   g_return_val_if_fail(chan->send != NULL, FALSE);
 
-void
-RpcChannel_Free(void *ptr);
+   return chan->send(chan, data, dataLen, result, resultLen);
+}
 
 gboolean
 RpcChannel_BuildXdrCommand(const char *cmd,
@@ -154,9 +214,6 @@ RpcChannel_BuildXdrCommand(const char *cmd,
 RpcChannel *
 RpcChannel_Create(void);
 
-void
-RpcChannel_Shutdown(RpcChannel *chan);
-
 gboolean
 RpcChannel_Destroy(RpcChannel *chan);
 
@@ -177,35 +234,16 @@ RpcChannel_RegisterCallback(RpcChannel *chan,
 
 gboolean
 RpcChannel_SetRetVals(RpcInData *data,
-                      char const *result,
+                      char *result,
                       gboolean retVal);
 
-gboolean
-RpcChannel_SetRetValsF(RpcInData *data,
-                       char *result,
-                       gboolean retVal);
-
 void
 RpcChannel_UnregisterCallback(RpcChannel *chan,
                               RpcChannelCallback *rpc);
 
-gboolean
-RpcChannel_SendOneRaw(const char *data,
-                      size_t dataLen,
-                      char **result,
-                      size_t *resultLen);
-
-gboolean
-RpcChannel_SendOne(char **reply,
-                   size_t *repLen,
-                   const char *reqFmt,
-                   ...);
 
 RpcChannel *
-RpcChannel_New(void);
-
-void
-RpcChannel_SetBackdoorOnly(void);
+BackdoorChannel_New(void);
 
 G_END_DECLS
 
