@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2010-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2010 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -23,7 +23,6 @@
 #endif
 
 #include "vmware.h"
-#include "vm_basic_asm.h"
 #include "str.h"
 #include "err.h"
 #include "util.h"
@@ -55,6 +54,8 @@ struct MXUserCondVar {
 #endif
 };
 
+#define MXUSER_CONDVAR_SIGNATURE 0x444E4F43 // 'COND' in memory
+
 #if defined(_WIN32)
 typedef VOID (WINAPI *InitializeConditionVariableFn)(PCONDITION_VARIABLE cv);
 typedef BOOL (WINAPI *SleepConditionVariableCSFn)(PCONDITION_VARIABLE cv,
@@ -113,8 +114,6 @@ MXUserNativeCVSupported(void)
                                    GetProcAddress(kernel32,
                                                    "WakeConditionVariable");
 
-         COMPILER_MEM_BARRIER();
-
          result = ((pInitializeConditionVariable != NULL) &&
                    (pSleepConditionVariableCS != NULL) &&
                    (pWakeAllConditionVariable != NULL) &&
@@ -160,7 +159,7 @@ MXUserCreateInternal(MXUserCondVar *condVar)  // IN/OUT:
       success = TRUE;
    } else {
       if (InitializeCriticalSectionAndSpinCount(&condVar->x.compat.condVarLock,
-                                                0x80000400) == 0) {
+                                             0x80000400) == 0) {
          success = FALSE;
       } else {
          condVar->x.compat.numWaiters = 0;
@@ -218,24 +217,29 @@ MXUserDestroyInternal(MXUserCondVar *condVar)  // IN/OUT:
  *      MXUserCondVar.
  *
  * Results:
- *      As above
+ *      TRUE   condvar was signalled
+ *      FALSE  timed out waiting for signal
  *
  * Side effects:
- *      It is possible to return from this routine without the condition
- *      variable having been signalled (spurious wake up); code accordingly!
+ *      None
  *
  *-----------------------------------------------------------------------------
  */
 
-static INLINE void
-MXUserWaitInternal(MXRecLock *lock,         // IN/OUT:
-                   MXUserCondVar *condVar,  // IN/OUT:
+static INLINE Bool
+MXUserWaitInternal(MXRecLock *lock,         // IN:
+                   MXUserCondVar *condVar,  // IN:
                    uint32 msecWait)         // IN:
 {
-   int lockCount = MXRecLockCount(lock);
+   DWORD err;
+   Bool signalled;
+
+   uint32 lockCount = MXRecLockCount(lock);
    DWORD waitTime = (msecWait == MXUSER_WAIT_INFINITE) ? INFINITE : msecWait;
 
    if (pSleepConditionVariableCS) {
+      Bool success;
+
       /*
        * When using the native lock found within the MXUser lock, be sure to
        * decrement the count before the wait/sleep and increment it after the
@@ -245,11 +249,25 @@ MXUserWaitInternal(MXRecLock *lock,         // IN/OUT:
        */
 
       MXRecLockDecCount(lock, lockCount);
-      (*pSleepConditionVariableCS)(&condVar->x.condObject, &lock->nativeLock,
-                                   waitTime);
-      MXRecLockIncCount(lock, lockCount);
+      success = (*pSleepConditionVariableCS)(&condVar->x.condObject,
+                                             &lock->nativeLock, waitTime);
+      MXRecLockIncCount(lock, GetReturnAddress(), lockCount);
+
+      if (success) {
+         signalled = TRUE;
+         err = ERROR_SUCCESS;
+      } else {
+         signalled = FALSE;
+         err = GetLastError();
+         if (WAIT_TIMEOUT == err) {
+            if (msecWait == MXUSER_WAIT_INFINITE) {
+               err = ERROR_CALL_NOT_IMPLEMENTED;  // ACK! "IMPOSSIBLE"
+            } else {
+               err = ERROR_SUCCESS;
+            }
+         }
+      }
    } else {
-      DWORD err;
       Bool done = FALSE;
 
       EnterCriticalSection(&condVar->x.compat.condVarLock);
@@ -276,6 +294,7 @@ MXUserWaitInternal(MXRecLock *lock,         // IN/OUT:
                }
 
                err = ERROR_SUCCESS;
+               signalled = TRUE;
                done = TRUE;
             }
          } else {
@@ -294,22 +313,23 @@ MXUserWaitInternal(MXRecLock *lock,         // IN/OUT:
                err = GetLastError();
             }
 
+            signalled = FALSE;
             done = TRUE;
          }
 
          LeaveCriticalSection(&condVar->x.compat.condVarLock);
       } while (!done);
 
-      MXRecLockAcquire(lock,
-                       NULL);  // non-stats
-
-      MXRecLockIncCount(lock, lockCount - 1);
-
-      if (err != ERROR_SUCCESS) {
-         Panic("%s: failure %d on condVar (0x%p; %s)\n", __FUNCTION__, err,
-               condVar, condVar->header->name);
-      }
+      MXRecLockAcquire(lock, GetReturnAddress());
+      MXRecLockIncCount(lock, GetReturnAddress(), lockCount - 1);
    }
+
+   if (err != ERROR_SUCCESS) {
+      Panic("%s: failure %d on condVar (%p; %s)\n", __FUNCTION__, err,
+            condVar, condVar->header->name);
+   }
+
+   return signalled;
 }
 
 
@@ -356,7 +376,7 @@ MXUserSignalInternal(MXUserCondVar *condVar)  // IN/OUT:
  *
  * MXUserBroadcastInternal --
  *
- *      Perform the environmentally specific portion of broadcasting on an
+ *      Perform the environmentally specific portion of broadasting on an
  *      MXUserCondVar.
  *
  * Results:
@@ -448,22 +468,23 @@ MXUserDestroyInternal(MXUserCondVar *condVar)  // IN/OUT:
  *      MXUserCondVar.
  *
  * Results:
- *      As above
+ *      TRUE   condvar was signalled
+ *      FALSE  timed out waiting for signal
  *
  * Side effects:
- *      It is possible to return from this routine without the condition
- *      variable having been signalled (spurious wake up); code accordingly!
+ *      None
  *
  *-----------------------------------------------------------------------------
  */
 
-static INLINE void
-MXUserWaitInternal(MXRecLock *lock,         // IN/OUT:
-                   MXUserCondVar *condVar,  // IN/OUT:
+static INLINE Bool
+MXUserWaitInternal(MXRecLock *lock,         // IN:
+                   MXUserCondVar *condVar,  // IN:
                    uint32 msecWait)         // IN:
 {
    int err;
-   int lockCount = MXRecLockCount(lock);
+   Bool signalled;
+   uint32 lockCount = MXRecLockCount(lock);
 
    /*
     * When using the native lock found within the MXUser lock, be sure to
@@ -477,6 +498,8 @@ MXUserWaitInternal(MXRecLock *lock,         // IN/OUT:
 
    if (msecWait == MXUSER_WAIT_INFINITE) {
       err = pthread_cond_wait(&condVar->condObject, &lock->nativeLock);
+
+      signalled = (err == 0) ? TRUE : FALSE;
    } else {
       struct timeval curTime;
       struct timespec endTime;
@@ -497,18 +520,25 @@ MXUserWaitInternal(MXRecLock *lock,         // IN/OUT:
       endTime.tv_nsec = (long int) (endNS % A_BILLION);
 #undef A_BILLION
 
-      err = pthread_cond_timedwait(&condVar->condObject, &lock->nativeLock,
+      err = pthread_cond_timedwait(&condVar->condObject,
+                                   &lock->nativeLock,
                                    &endTime);
-   }
 
-   MXRecLockIncCount(lock, lockCount);
+      signalled = (err == 0) ? TRUE : FALSE;
 
-   if (err != 0) {
-      if (err != ETIMEDOUT) {
-         Panic("%s: failure %d on condVar (0x%p; %s)\n", __FUNCTION__, err,
-               condVar, condVar->header->name);
+      if (err == ETIMEDOUT) {
+         err = 0;
       }
    }
+
+   MXRecLockIncCount(lock, GetReturnAddress(), lockCount);
+
+   if (err != 0) {
+      Panic("%s: failure %d on condVar (%p; %s)\n", __FUNCTION__, err,
+            condVar, condVar->header->name);
+   }
+
+   return signalled;
 }
 
 
@@ -542,7 +572,7 @@ MXUserSignalInternal(MXUserCondVar *condVar)  // IN/OUT:
  *
  * MXUserBroadcaseInternal --
  *
- *      Perform the environmentally specific portion of broadcasting on an
+ *      Perform the environmentally specific portion of broadasting on an
  *      MXUserCondVar.
  *
  * Results:
@@ -586,10 +616,10 @@ MXUserCondVar *
 MXUserCreateCondVar(MXUserHeader *header,  // IN:
                     MXRecLock *lock)       // IN:
 {
-   MXUserCondVar *condVar = Util_SafeCalloc(1, sizeof *condVar);
+   MXUserCondVar *condVar = Util_SafeCalloc(1, sizeof(*condVar));
 
    if (MXUserCreateInternal(condVar)) {
-      condVar->signature = MXUserGetSignature(MXUSER_TYPE_CONDVAR);
+      condVar->signature = MXUSER_CONDVAR_SIGNATURE;
       condVar->header = header;
       condVar->ownerLock = lock;
    } else {
@@ -609,7 +639,8 @@ MXUserCreateCondVar(MXUserHeader *header,  // IN:
  *      The internal wait on a condition variable routine.
  *
  * Results:
- *      As above
+ *      TRUE   condvar was signalled
+ *      FALSE  timed out waiting for signal
  *
  * Side effects:
  *      An attempt to use a lock other than the one the specified condition
@@ -618,30 +649,33 @@ MXUserCreateCondVar(MXUserHeader *header,  // IN:
  *-----------------------------------------------------------------------------
  */
 
-void
+Bool
 MXUserWaitCondVar(MXUserHeader *header,    // IN:
-                  MXRecLock *lock,         // IN/OUT:
-                  MXUserCondVar *condVar,  // IN/OUT:
+                  MXRecLock *lock,         // IN:
+                  MXUserCondVar *condVar,  // IN:
                   uint32 msecWait)         // IN:
 {
+   Bool signalled;
+
    ASSERT(header);
    ASSERT(lock);
-   ASSERT(condVar);
-   ASSERT(condVar->signature == MXUserGetSignature(MXUSER_TYPE_CONDVAR));
+   ASSERT(condVar && (condVar->signature == MXUSER_CONDVAR_SIGNATURE));
 
    if (condVar->ownerLock != lock) {
-      Panic("%s: invalid use of lock %s with condVar (0x%p; %s)\n",
+      Panic("%s: invalid use of lock %s with condVar (%p; %s)\n",
              __FUNCTION__, header->name, condVar, condVar->header->name);
    }
 
-   if (vmx86_debug && !MXRecLockIsOwner(lock)) {
-      Panic("%s: lock %s for condVar (0x%p) not owned\n",
+   if (!MXRecLockIsOwner(lock)) {
+      Panic("%s: lock %s for condVar (%p) not owned\n",
             __FUNCTION__, condVar->header->name, condVar);
    }
 
    Atomic_Inc(&condVar->referenceCount);
-   MXUserWaitInternal(lock, condVar, msecWait);
+   signalled = MXUserWaitInternal(lock, condVar, msecWait);
    Atomic_Dec(&condVar->referenceCount);
+
+   return signalled;
 }
 
 
@@ -663,17 +697,16 @@ MXUserWaitCondVar(MXUserHeader *header,    // IN:
  */
 
 void
-MXUser_SignalCondVar(MXUserCondVar *condVar)  // IN/OUT:
+MXUser_SignalCondVar(MXUserCondVar *condVar)  // IN:
 {
    int err;
 
-   ASSERT(condVar);
-   ASSERT(condVar->signature == MXUserGetSignature(MXUSER_TYPE_CONDVAR));
+   ASSERT(condVar && (condVar->signature == MXUSER_CONDVAR_SIGNATURE));
 
    err = MXUserSignalInternal(condVar);
 
    if (err != 0) {
-      Panic("%s: failure %d on condVar (0x%p; %s) \n", __FUNCTION__, err,
+      Panic("%s: failure %d on condVar (%p; %s) \n", __FUNCTION__, err,
             condVar, condVar->header->name);
    }
 }
@@ -697,17 +730,16 @@ MXUser_SignalCondVar(MXUserCondVar *condVar)  // IN/OUT:
  */
 
 void
-MXUser_BroadcastCondVar(MXUserCondVar *condVar)  // IN/OUT:
+MXUser_BroadcastCondVar(MXUserCondVar *condVar)  // IN:
 {
    int err;
 
-   ASSERT(condVar);
-   ASSERT(condVar->signature == MXUserGetSignature(MXUSER_TYPE_CONDVAR));
+   ASSERT(condVar && (condVar->signature == MXUSER_CONDVAR_SIGNATURE));
 
    err = MXUserBroadcastInternal(condVar);
 
    if (err != 0) {
-      Panic("%s: failure %d on condVar (0x%p; %s) \n", __FUNCTION__, err,
+      Panic("%s: failure %d on condVar (%p; %s) \n", __FUNCTION__, err,
             condVar, condVar->header->name);
    }
 }
@@ -732,20 +764,19 @@ MXUser_BroadcastCondVar(MXUserCondVar *condVar)  // IN/OUT:
  */
 
 void
-MXUser_DestroyCondVar(MXUserCondVar *condVar)  // IN/OUT:
+MXUser_DestroyCondVar(MXUserCondVar *condVar)  // IN:
 {
    if (condVar != NULL) {
-      ASSERT(condVar->signature == MXUserGetSignature(MXUSER_TYPE_CONDVAR));
+      ASSERT(condVar && (condVar->signature == MXUSER_CONDVAR_SIGNATURE));
 
       if (Atomic_Read(&condVar->referenceCount) != 0) {
-         Panic("%s: Attempted destroy on active condVar (0x%p; %s)\n",
+         Panic("%s: Attempted destroy on active condVar (%p; %s)\n",
                __FUNCTION__, condVar, condVar->header->name);
       }
 
-      condVar->signature = 0;  // just in case...
-
       MXUserDestroyInternal(condVar);
 
+      condVar->signature = 0;  // just in case...
       condVar->header = NULL;
       condVar->ownerLock = NULL;
 
