@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2009-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2009 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -23,15 +23,26 @@
 #include "hostinfo.h"
 #include "ulInt.h"
 
+#define MXUSER_EXCL_SIGNATURE 0x58454B4C // 'LKEX' in memory
+
 struct MXUserExclLock
 {
-   MXUserHeader  header;
-   MXRecLock     recursiveLock;
-   Atomic_Ptr    heldStatsMem;
-   Atomic_Ptr    acquireStatsMem;
+   MXUserHeader            header;
+   MXRecLock               recursiveLock;
+
+#if defined(MXUSER_STATS)
+   VmTimeType              holdStart;
+
+   MXUserAcquisitionStats  acquisitionStats;
+   Atomic_Ptr              acquisitionHisto;
+
+   MXUserBasicStats        heldStats;
+   Atomic_Ptr              heldHisto;
+#endif
 };
 
 
+#if defined(MXUSER_STATS)
 /*
  *-----------------------------------------------------------------------------
  *
@@ -51,231 +62,51 @@ struct MXUserExclLock
 static void
 MXUserStatsActionExcl(MXUserHeader *header)  // IN:
 {
+   Bool isHot;
+   Bool doLog;
+   double contentionRatio;
+
    MXUserExclLock *lock = (MXUserExclLock *) header;
-   MXUserHeldStats *heldStats = Atomic_ReadPtr(&lock->heldStatsMem);
-   MXUserAcquireStats *acquireStats = Atomic_ReadPtr(&lock->acquireStatsMem);
 
-   if (UNLIKELY(heldStats != NULL)) {
-      MXUserDumpBasicStats(&heldStats->data, header);
+   /*
+    * Dump the statistics for the specified lock.
+    */
 
-      if (Atomic_ReadPtr(&heldStats->histo) != NULL) {
-         MXUserHistoDump(Atomic_ReadPtr(&heldStats->histo), header);
-      }
+   MXUserDumpAcquisitionStats(&lock->acquisitionStats, header);
+
+   if (Atomic_ReadPtr(&lock->acquisitionHisto) != NULL) {
+      MXUserHistoDump(Atomic_ReadPtr(&lock->acquisitionHisto), header);
    }
 
-   if (LIKELY(acquireStats != NULL)) {
-      Bool isHot;
-      Bool doLog;
-      double contentionRatio;
+   MXUserDumpBasicStats(&lock->heldStats, header);
 
-      /*
-       * Dump the statistics for the specified lock.
-       */
+   if (Atomic_ReadPtr(&lock->heldHisto) != NULL) {
+      MXUserHistoDump(Atomic_ReadPtr(&lock->heldHisto), header);
+   }
 
-      MXUserDumpAcquisitionStats(&acquireStats->data, header);
+   /*
+    * Has the lock gone "hot"? If so, implement the hot actions.
+    */
 
-      if (Atomic_ReadPtr(&acquireStats->histo) != NULL) {
-         MXUserHistoDump(Atomic_ReadPtr(&acquireStats->histo), header);
-      }
+   MXUserKitchen(&lock->acquisitionStats, &contentionRatio, &isHot, &doLog);
 
-      /*
-       * Has the lock gone "hot"? If so, implement the hot actions.
-       */
+   if (isHot) {
+      MXUserForceHisto(&lock->acquisitionHisto,
+                       MXUSER_STAT_CLASS_ACQUISITION,
+                       MXUSER_DEFAULT_HISTO_MIN_VALUE_NS,
+                       MXUSER_DEFAULT_HISTO_DECADES);
+      MXUserForceHisto(&lock->heldHisto,
+                       MXUSER_STAT_CLASS_HELD,
+                       MXUSER_DEFAULT_HISTO_MIN_VALUE_NS,
+                       MXUSER_DEFAULT_HISTO_DECADES);
 
-      MXUserKitchen(&acquireStats->data, &contentionRatio, &isHot, &doLog);
-
-      if (UNLIKELY(isHot)) {
-         MXUserForceAcquisitionHisto(&lock->acquireStatsMem,
-                                     MXUSER_DEFAULT_HISTO_MIN_VALUE_NS,
-                                     MXUSER_DEFAULT_HISTO_DECADES);
-
-         if (UNLIKELY(heldStats != NULL)) {
-            MXUserForceHeldHisto(&lock->heldStatsMem,
-                                 MXUSER_DEFAULT_HISTO_MIN_VALUE_NS,
-                                 MXUSER_DEFAULT_HISTO_DECADES);
-         }
-
-         if (doLog) {
-            Log("HOT LOCK (%s); contention ratio %f\n",
-                lock->header.name, contentionRatio);
-         }
+      if (doLog) {
+         Log("HOT LOCK (%s); contention ratio %f\n",
+             lock->header.name, contentionRatio);
       }
    }
 }
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * MXUser_EnableStatsExclLock
- *
- *      Enable basic stats on the specified lock.
- *
- * Results:
- *      TRUE    succeeded
- *      FALSE   failed
- *
- * Side effects:
- *      None
- *
- *-----------------------------------------------------------------------------
- */
-
-Bool
-MXUser_EnableStatsExclLock(MXUserExclLock *lock,       // IN/OUT:
-                           Bool trackAcquisitionTime,  // IN:
-                           Bool trackHeldTime)         // IN:
-{
-   ASSERT(lock);
-   MXUserValidateHeader(&lock->header, MXUSER_TYPE_EXCL);
-
-   if (vmx86_stats) {
-      MXUserEnableStats(trackAcquisitionTime ? &lock->acquireStatsMem : NULL,
-                        trackHeldTime        ? &lock->heldStatsMem    : NULL);
-   }
-
-   return vmx86_stats;
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * MXUser_DisableStatsExclLock --
- *
- *      Disable basic stats on the specified lock.
- *
- * Results:
- *      TRUE    succeeded
- *      FALSE   failed
- *
- * Side effects:
- *      None
- *
- *-----------------------------------------------------------------------------
- */
-
-Bool
-MXUser_DisableStatsExclLock(MXUserExclLock *lock)  // IN/OUT:
-{
-   ASSERT(lock);
-   MXUserValidateHeader(&lock->header, MXUSER_TYPE_EXCL);
-
-   if (vmx86_stats) {
-      MXUserDisableStats(&lock->acquireStatsMem, &lock->heldStatsMem);
-   }
-
-   return vmx86_stats;
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * MXUser_SetContentionRatioFloorExclLock --
- *
- *      Set the contention ratio floor for the specified lock.
- *
- * Results:
- *      TRUE    succeeded
- *      FALSE   failed
- *
- * Side effects:
- *      None
- *
- *-----------------------------------------------------------------------------
- */
-
-Bool
-MXUser_SetContentionRatioFloorExclLock(MXUserExclLock *lock,  // IN/OUT:
-                                       double ratio)          // IN:
-{
-   Bool result;
-
-   ASSERT(lock);
-   MXUserValidateHeader(&lock->header, MXUSER_TYPE_EXCL);
-
-   if (vmx86_stats) {
-      result = MXUserSetContentionRatioFloor(&lock->acquireStatsMem, ratio);
-   } else {
-      result = FALSE;
-   }
-
-   return result;
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * MXUser_SetContentionCountFloorExclLock --
- *
- *      Set the contention count floor for the specified lock.
- *
- * Results:
- *      TRUE    succeeded
- *      FALSE   failed
- *
- * Side effects:
- *      None
- *
- *-----------------------------------------------------------------------------
- */
-
-Bool
-MXUser_SetContentionCountFloorExclLock(MXUserExclLock *lock,  // IN/OUT:
-                                       uint64 count)          // IN:
-{
-   Bool result;
-
-   ASSERT(lock);
-   MXUserValidateHeader(&lock->header, MXUSER_TYPE_EXCL);
-
-   if (vmx86_stats) {
-      result = MXUserSetContentionCountFloor(&lock->acquireStatsMem, count);
-   } else {
-      result = FALSE;
-   }
-
-   return result;
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * MXUser_SetContentionDurationFloorExclLock --
- *
- *      Set the contention count floor for the specified lock.
- *
- * Results:
- *      TRUE    succeeded
- *      FALSE   failed
- *
- * Side effects:
- *      None
- *
- *-----------------------------------------------------------------------------
- */
-
-Bool
-MXUser_SetContentionDurationFloorExclLock(MXUserExclLock *lock,  // IN/OUT:
-                                          uint64 duration)       // IN:
-{
-   Bool result;
-
-   ASSERT(lock);
-   MXUserValidateHeader(&lock->header, MXUSER_TYPE_EXCL);
-
-   if (vmx86_stats) {
-      result = MXUserSetContentionDurationFloor(&lock->acquireStatsMem,
-                                                duration);
-   } else {
-      result = FALSE;
-   }
-
-   return result;
-}
+#endif
 
 
 /*
@@ -299,17 +130,18 @@ MXUserDumpExclLock(MXUserHeader *header)  // IN:
 {
    MXUserExclLock *lock = (MXUserExclLock *) header;
 
-   Warning("%s: Exclusive lock @ %p\n", __FUNCTION__, lock);
+   Warning("%s: Exclusive lock @ 0x%p\n", __FUNCTION__, lock);
 
    Warning("\tsignature 0x%X\n", lock->header.signature);
    Warning("\tname %s\n", lock->header.name);
    Warning("\trank 0x%X\n", lock->header.rank);
-   Warning("\tserial number %u\n", lock->header.bits.serialNumber);
 
-   Warning("\tlock count %d\n", MXRecLockCount(&lock->recursiveLock));
+   Warning("\tcount %u\n", lock->recursiveLock.referenceCount);
 
-   Warning("\taddress of owner data %p\n",
-           &lock->recursiveLock.nativeThreadID);
+#if defined(MXUSER_DEBUG)
+   Warning("\tcaller 0x%p\n", lock->recursiveLock.ownerRetAddr);
+   Warning("\tVThreadID %d\n", (int) lock->recursiveLock.portableThreadID);
+#endif
 }
 
 
@@ -334,9 +166,10 @@ MXUserExclLock *
 MXUser_CreateExclLock(const char *userName,  // IN:
                       MX_Rank rank)          // IN:
 {
-   uint32 statsMode;
    char *properName;
-   MXUserExclLock *lock = Util_SafeCalloc(1, sizeof *lock);
+   MXUserExclLock *lock;
+
+   lock = Util_SafeCalloc(1, sizeof(*lock));
 
    if (userName == NULL) {
       properName = Str_SafeAsprintf(NULL, "X-%p", GetReturnAddress());
@@ -351,35 +184,19 @@ MXUser_CreateExclLock(const char *userName,  // IN:
       return NULL;
    }
 
-   lock->header.signature = MXUserGetSignature(MXUSER_TYPE_EXCL);
    lock->header.name = properName;
+   lock->header.signature = MXUSER_EXCL_SIGNATURE;
    lock->header.rank = rank;
-   lock->header.bits.serialNumber = MXUserAllocSerialNumber();
    lock->header.dumpFunc = MXUserDumpExclLock;
 
-   statsMode = MXUserStatsMode();
-
-   switch (MXUserStatsMode()) {
-   case 0:
-      MXUserDisableStats(&lock->acquireStatsMem, &lock->heldStatsMem);
-      lock->header.statsFunc = NULL;
-      break;
-
-   case 1:
-      MXUserEnableStats(&lock->acquireStatsMem, NULL);
-      lock->header.statsFunc = MXUserStatsActionExcl;
-      break;
-
-   case 2:
-      MXUserEnableStats(&lock->acquireStatsMem, &lock->heldStatsMem);
-      lock->header.statsFunc = MXUserStatsActionExcl;
-      break;
-
-   default:
-      Panic("%s: unknown stats mode: %d!\n", __FUNCTION__, statsMode);
-   }
+#if defined(MXUSER_STATS)
+   lock->header.statsFunc = MXUserStatsActionExcl;
+   lock->header.identifier = MXUserAllocID();
 
    MXUserAddToList(&lock->header);
+   MXUserAcquisitionStatsSetUp(&lock->acquisitionStats);
+   MXUserBasicStatsSetUp(&lock->heldStats, MXUSER_STAT_CLASS_HELD);
+#endif
 
    return lock;
 }
@@ -405,7 +222,7 @@ void
 MXUser_DestroyExclLock(MXUserExclLock *lock)  // IN:
 {
    if (lock != NULL) {
-      MXUserValidateHeader(&lock->header, MXUSER_TYPE_EXCL);
+      ASSERT(lock->header.signature == MXUSER_EXCL_SIGNATURE);
 
       if (MXRecLockCount(&lock->recursiveLock) > 0) {
          MXUserDumpAndPanic(&lock->header,
@@ -415,14 +232,16 @@ MXUser_DestroyExclLock(MXUserExclLock *lock)  // IN:
 
       MXRecLockDestroy(&lock->recursiveLock);
 
+#if defined(MXUSER_STATS)
       MXUserRemoveFromList(&lock->header);
-
-      if (vmx86_stats) {
-         MXUserDisableStats(&lock->acquireStatsMem, &lock->heldStatsMem);
-      }
+      MXUserAcquisitionStatsTearDown(&lock->acquisitionStats);
+      MXUserBasicStatsTearDown(&lock->heldStats);
+      MXUserHistoTearDown(Atomic_ReadPtr(&lock->acquisitionHisto));
+      MXUserHistoTearDown(Atomic_ReadPtr(&lock->heldHisto));
+#endif
 
       lock->header.signature = 0;  // just in case...
-      free(lock->header.name);
+      free((void *) lock->header.name);  // avoid const warnings
       lock->header.name = NULL;
       free(lock);
    }
@@ -448,50 +267,46 @@ MXUser_DestroyExclLock(MXUserExclLock *lock)  // IN:
 void
 MXUser_AcquireExclLock(MXUserExclLock *lock)  // IN/OUT:
 {
-   ASSERT(lock);
-   MXUserValidateHeader(&lock->header, MXUSER_TYPE_EXCL);
+#if defined(MXUSER_STATS)
+   VmTimeType begin;
+   VmTimeType value;
+   Bool contended;
+   MXUserHisto *histo;
+#endif
+
+   ASSERT(lock && (lock->header.signature == MXUSER_EXCL_SIGNATURE));
 
    MXUserAcquisitionTracking(&lock->header, TRUE);
 
-   if (vmx86_stats) {
-      VmTimeType value = 0;
-      MXUserHeldStats *heldStats;
-      MXUserAcquireStats *acquireStats;
+#if defined(MXUSER_STATS)
+   begin = Hostinfo_SystemTimerNS();
 
-      acquireStats = Atomic_ReadPtr(&lock->acquireStatsMem);
+   contended =
+#endif
 
-      MXRecLockAcquire(&lock->recursiveLock,
-                       (acquireStats == NULL) ? NULL : &value);
+   MXRecLockAcquire(&lock->recursiveLock, GetReturnAddress());
 
-      if (LIKELY(acquireStats != NULL)) {
-         MXUserHisto *histo;
+#if defined(MXUSER_STATS)
+   value = Hostinfo_SystemTimerNS() - begin;
 
-         MXUserAcquisitionSample(&acquireStats->data, TRUE,
-                            value > acquireStats->data.contentionDurationFloor,
-                                 value);
+   MXUserAcquisitionSample(&lock->acquisitionStats, contended, value);
 
-         histo = Atomic_ReadPtr(&acquireStats->histo);
+   histo = Atomic_ReadPtr(&lock->acquisitionHisto);
 
-         if (UNLIKELY(histo != NULL)) {
-            MXUserHistoSample(histo, value, GetReturnAddress());
-         }
-
-         heldStats = Atomic_ReadPtr(&lock->heldStatsMem);
-
-         if (UNLIKELY(heldStats != NULL)) {
-            heldStats->holdStart = Hostinfo_SystemTimerNS();
-         }
-      }
-   } else {
-      MXRecLockAcquire(&lock->recursiveLock,
-                       NULL);  // non-stats
+   if (UNLIKELY(histo != NULL)) {
+      MXUserHistoSample(histo, value);
    }
+#endif
 
-   if (vmx86_debug && (MXRecLockCount(&lock->recursiveLock) > 1)) {
+   if (MXRecLockCount(&lock->recursiveLock) > 1) {
       MXUserDumpAndPanic(&lock->header,
                          "%s: Acquire on an acquired exclusive lock\n",
                          __FUNCTION__);
    }
+
+#if defined(MXUSER_STATS)
+   lock->holdStart = Hostinfo_SystemTimerNS();
+#endif
 }
 
 
@@ -514,36 +329,32 @@ MXUser_AcquireExclLock(MXUserExclLock *lock)  // IN/OUT:
 void
 MXUser_ReleaseExclLock(MXUserExclLock *lock)  // IN/OUT:
 {
-   ASSERT(lock);
-   MXUserValidateHeader(&lock->header, MXUSER_TYPE_EXCL);
+#if defined(MXUSER_STATS)
+   VmTimeType value;
+   MXUserHisto *histo;
+#endif
 
-   if (vmx86_stats) {
-      MXUserHeldStats *heldStats = Atomic_ReadPtr(&lock->heldStatsMem);
+   ASSERT(lock && (lock->header.signature == MXUSER_EXCL_SIGNATURE));
 
-      if (UNLIKELY(heldStats != NULL)) {
-         VmTimeType value = Hostinfo_SystemTimerNS() - heldStats->holdStart;
-         MXUserHisto *histo = Atomic_ReadPtr(&heldStats->histo);
+#if defined(MXUSER_STATS)
+   value = Hostinfo_SystemTimerNS() - lock->holdStart;
 
-         MXUserBasicStatsSample(&heldStats->data, value);
+   MXUserBasicStatsSample(&lock->heldStats, value);
 
-         if (UNLIKELY(histo != NULL)) {
-            MXUserHistoSample(histo, value, GetReturnAddress());
-         }
-      }
+   histo = Atomic_ReadPtr(&lock->heldHisto);
+
+   if (UNLIKELY(histo != NULL)) {
+      MXUserHistoSample(histo, value);
    }
+#endif
 
-   if (vmx86_debug) {
-      if (MXRecLockCount(&lock->recursiveLock) == 0) {
-         MXUserDumpAndPanic(&lock->header,
-                            "%s: Release of an unacquired exclusive lock\n",
-                             __FUNCTION__);
-      }
+   if (!MXRecLockIsOwner(&lock->recursiveLock)) {
+      uint32 lockCount = MXRecLockCount(&lock->recursiveLock);
 
-      if (!MXRecLockIsOwner(&lock->recursiveLock)) {
-         MXUserDumpAndPanic(&lock->header,
-                            "%s: Non-owner release of a exclusive lock\n",
-                            __FUNCTION__);
-      }
+      MXUserDumpAndPanic(&lock->header,
+                         "%s: Non-owner release of an %s exclusive lock\n",
+                         __FUNCTION__,
+                         lockCount == 0 ? "unacquired" : "acquired");
    }
 
    MXUserReleaseTracking(&lock->header);
@@ -578,34 +389,38 @@ MXUser_TryAcquireExclLock(MXUserExclLock *lock)  // IN/OUT:
 {
    Bool success;
 
-   ASSERT(lock);
-   MXUserValidateHeader(&lock->header, MXUSER_TYPE_EXCL);
+#if defined(MXUSER_STATS)
+   VmTimeType begin;
+#endif
+
+   ASSERT(lock && (lock->header.signature == MXUSER_EXCL_SIGNATURE));
 
    if (MXUserTryAcquireFail(lock->header.name)) {
       return FALSE;
    }
 
-   success = MXRecLockTryAcquire(&lock->recursiveLock);
+#if defined(MXUSER_STATS)
+   begin = Hostinfo_SystemTimerNS();
+#endif
+
+   success = MXRecLockTryAcquire(&lock->recursiveLock, GetReturnAddress());
 
    if (success) {
+#if defined(MXUSER_STATS)
+      VmTimeType end = Hostinfo_SystemTimerNS();
+#endif
+
       MXUserAcquisitionTracking(&lock->header, FALSE);
 
-      if (vmx86_debug && (MXRecLockCount(&lock->recursiveLock) > 1)) {
+      if (MXRecLockCount(&lock->recursiveLock) > 1) {
          MXUserDumpAndPanic(&lock->header,
                             "%s: Acquire on an acquired exclusive lock\n",
                             __FUNCTION__);
       }
-   }
 
-   if (vmx86_stats) {
-      MXUserAcquireStats *acquireStats;
-
-      acquireStats = Atomic_ReadPtr(&lock->acquireStatsMem);
-
-      if (LIKELY(acquireStats != NULL)) {
-         MXUserAcquisitionSample(&acquireStats->data, success, !success,
-                                 0ULL);
-      }
+#if defined(MXUSER_STATS)
+      MXUserAcquisitionSample(&lock->acquisitionStats, FALSE, end - begin);
+#endif
    }
 
    return success;
@@ -630,15 +445,85 @@ MXUser_TryAcquireExclLock(MXUserExclLock *lock)  // IN/OUT:
  */
 
 Bool
-MXUser_IsCurThreadHoldingExclLock(MXUserExclLock *lock)  // IN:
+MXUser_IsCurThreadHoldingExclLock(const MXUserExclLock *lock)  // IN:
 {
-   ASSERT(lock);
-   MXUserValidateHeader(&lock->header, MXUSER_TYPE_EXCL);
+   ASSERT(lock && (lock->header.signature == MXUSER_EXCL_SIGNATURE));
 
    return MXRecLockIsOwner(&lock->recursiveLock);
 }
 
 
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * MXUser_ControlExclLock --
+ *
+ *      Perform the specified command on the specified lock.
+ *
+ * Results:
+ *      TRUE    succeeded
+ *      FALSE   failed
+ *
+ * Side effects:
+ *      Depends on the command, no?
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+Bool
+MXUser_ControlExclLock(MXUserExclLock *lock,  // IN/OUT:
+                       uint32 command,        // IN:
+                       ...)                   // IN:
+{
+   Bool result;
+
+   ASSERT(lock && (lock->header.signature == MXUSER_EXCL_SIGNATURE));
+
+   switch (command) {
+#if defined(MXUSER_STATS)
+   case MXUSER_CONTROL_ACQUISITION_HISTO: {
+      va_list a;
+      uint64 minValue;
+      uint32 decades;
+
+      va_start(a, command);
+      minValue = va_arg(a, uint64);
+      decades = va_arg(a, uint32);
+      va_end(a);
+
+      MXUserForceHisto(&lock->acquisitionHisto, MXUSER_STAT_CLASS_ACQUISITION,
+                       minValue, decades);
+
+      result = TRUE;
+      break;
+   }
+
+   case MXUSER_CONTROL_HELD_HISTO: {
+      va_list a;
+      uint32 minValue;
+      uint32 decades;
+
+      va_start(a, command);
+      minValue = va_arg(a, uint64);
+      decades = va_arg(a, uint32);
+      va_end(a);
+
+      MXUserForceHisto(&lock->heldHisto, MXUSER_STAT_CLASS_HELD,
+                       minValue, decades);
+
+      result = TRUE;
+      break;
+   }
+#endif
+
+   default:
+      result = FALSE;
+   }
+
+   return result;
+}
+
+
 /*
  *-----------------------------------------------------------------------------
  *
@@ -667,17 +552,20 @@ MXUser_CreateSingletonExclLock(Atomic_Ptr *lockStorage,  // IN/OUT:
 
    ASSERT(lockStorage);
 
-   lock = Atomic_ReadPtr(lockStorage);
+   lock = (MXUserExclLock *) Atomic_ReadPtr(lockStorage);
 
    if (UNLIKELY(lock == NULL)) {
-      MXUserExclLock *newLock = MXUser_CreateExclLock(name, rank);
+      MXUserExclLock *before;
 
-      lock = Atomic_ReadIfEqualWritePtr(lockStorage, NULL, (void *) newLock);
+      lock = MXUser_CreateExclLock(name, rank);
 
-      if (lock) {
-         MXUser_DestroyExclLock(newLock);
-      } else {
-         lock = Atomic_ReadPtr(lockStorage);
+      before = (MXUserExclLock *) Atomic_ReadIfEqualWritePtr(lockStorage, NULL,
+                                                           (void *) lock);
+
+      if (before) {
+         MXUser_DestroyExclLock(lock);
+
+         lock = before;
       }
    }
 
@@ -705,8 +593,7 @@ MXUser_CreateSingletonExclLock(Atomic_Ptr *lockStorage,  // IN/OUT:
 MXUserCondVar *
 MXUser_CreateCondVarExclLock(MXUserExclLock *lock)
 {
-   ASSERT(lock);
-   MXUserValidateHeader(&lock->header, MXUSER_TYPE_EXCL);
+   ASSERT(lock && (lock->header.signature == MXUSER_EXCL_SIGNATURE));
 
    return MXUserCreateCondVar(&lock->header, &lock->recursiveLock);
 }
@@ -725,8 +612,7 @@ MXUser_CreateCondVarExclLock(MXUserExclLock *lock)
  *      As above.
  *
  * Side effects:
- *      It is possible to return from this routine without the condtion
- *      variable having been signalled (spurious wake up); code accordingly!
+ *      None.
  *
  *-----------------------------------------------------------------------------
  */
@@ -735,8 +621,7 @@ void
 MXUser_WaitCondVarExclLock(MXUserExclLock *lock,    // IN:
                            MXUserCondVar *condVar)  // IN:
 {
-   ASSERT(lock);
-   MXUserValidateHeader(&lock->header, MXUSER_TYPE_EXCL);
+   ASSERT(lock && (lock->header.signature == MXUSER_EXCL_SIGNATURE));
 
    MXUserWaitCondVar(&lock->header, &lock->recursiveLock, condVar,
                      MXUSER_WAIT_INFINITE);
@@ -753,22 +638,22 @@ MXUser_WaitCondVarExclLock(MXUserExclLock *lock,    // IN:
  *      blocking and is reacquired before returning from this function.
  *
  * Results:
- *      As above
+ *      TRUE   condVar was signalled
+ *      FALSE  timed out waiting for signal
  *
  * Side effects:
- *      It is possible to return from this routine without the condtion
- *      variable having been signalled (spurious wake up); code accordingly!
+ *      None.
  *
  *-----------------------------------------------------------------------------
  */
 
-void
+Bool
 MXUser_TimedWaitCondVarExclLock(MXUserExclLock *lock,    // IN:
                                 MXUserCondVar *condVar,  // IN:
                                 uint32 msecWait)         // IN:
 {
-   ASSERT(lock);
-   MXUserValidateHeader(&lock->header, MXUSER_TYPE_EXCL);
+   ASSERT(lock && (lock->header.signature == MXUSER_EXCL_SIGNATURE));
 
-   MXUserWaitCondVar(&lock->header, &lock->recursiveLock, condVar, msecWait);
+   return MXUserWaitCondVar(&lock->header, &lock->recursiveLock, condVar,
+                            msecWait);
 }
