@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2009-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2009 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -27,28 +27,24 @@
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
-#if defined(__linux__)
-#include <features.h>
-#endif
 
 #if defined(_WIN32)
-typedef DWORD MXUserThreadID;
+typedef DWORD MXThreadID;
 #define MXUSER_INVALID_OWNER 0xFFFFFFFF
 #else
-#include "safetime.h"
 #include <pthread.h>
-typedef pthread_t MXUserThreadID;
+typedef pthread_t MXThreadID;
 #endif
 
 #include "vm_basic_types.h"
-#include "vm_atomic.h"
 #include "vthreadBase.h"
-#include "hostinfo.h"
 
+#if defined(MXUSER_STATS)
 #include "circList.h"
 
 #define MXUSER_STAT_CLASS_ACQUISITION "a"
 #define MXUSER_STAT_CLASS_HELD        "h"
+#endif
 
 /*
  * A portable recursive lock.
@@ -64,7 +60,12 @@ typedef struct {
 #endif
 
    int              referenceCount;   // Acquisition count
-   MXUserThreadID   nativeThreadID;   // Native thread ID
+   MXThreadID       nativeThreadID;   // Native thread ID
+
+#if defined(MXUSER_DEBUG)
+   VThreadID        portableThreadID;  // VThreadID, when available
+   const void      *ownerRetAddr;      // return address of acquisition routine
+#endif
 } MXRecLock;
 
 
@@ -76,16 +77,15 @@ typedef struct {
  *     - exclusive (non-recursive) locks catch the recursion and panic
  *       rather than deadlock.
  *
- * There are 9 environment specific primitives:
+ * There are 8 environment specific primitives:
  *
- * MXUserNativeThreadID         Return native thread ID
  * MXRecLockCreateInternal      Create lock before use
  * MXRecLockDestroyInternal     Destroy lock after use
  * MXRecLockIsOwner             Is lock owned by the caller?
  * MXRecLockSetNoOwner          Set lock as owner by "nobody"
  * MXRecLockSetOwner            Set lock owner
  * MXRecLockAcquireInternal     Lock the lock
- * MXRecLockTryAcquireInternal  Conditionally acquire the lock
+ * MXRecLockTryAcquireInternal  conditionally acquire the lock
  * MXRecLockReleaseInternal     Unlock the lock
  *
  * Windows has a native recursive lock, the CRITICAL_SECTION. POSIXen,
@@ -119,17 +119,10 @@ MXRecLockDestroyInternal(MXRecLock *lock)  // IN/OUT:
 }
 
 
-static INLINE MXUserThreadID
-MXUserNativeThreadID(void)
-{
-   return GetCurrentThreadId();
-}
-
-
 static INLINE Bool
 MXRecLockIsOwner(const MXRecLock *lock)  // IN:
 {
-   return lock->nativeThreadID == MXUserNativeThreadID();
+   return lock->nativeThreadID == GetCurrentThreadId();
 }
 
 
@@ -143,7 +136,7 @@ MXRecLockSetNoOwner(MXRecLock *lock)  // IN:
 static INLINE void
 MXRecLockSetOwner(MXRecLock *lock)  // IN/OUT:
 {
-   lock->nativeThreadID = MXUserNativeThreadID();
+   lock->nativeThreadID = GetCurrentThreadId();
 }
 
 
@@ -181,46 +174,14 @@ MXRecLockCreateInternal(MXRecLock *lock)  // IN/OUT:
 static INLINE int
 MXRecLockDestroyInternal(MXRecLock *lock)  // IN:
 {
-   int err = pthread_mutex_destroy(&lock->nativeLock);
-
-#if defined(__GLIBC__)
-   extern const char *gnu_get_libc_version(void) __attribute__ ((weak));
-
-   if (gnu_get_libc_version != NULL) {
-      /*
-       * Elision locking may be used but only on Haswell and newer (compatible)
-       * processors and with glibc 2.18 or later. Unfortunately there is a
-       * problem with the implemenation that requires ignoring any error here.
-       *
-       * TODO: Change the version checking to use a range once the issue is
-       *       fixed.
-       *
-       * See PR 1200129 for more information.
-       */
-
-      const char *version = gnu_get_libc_version();
-
-      if (strcmp(version, "2.18") >= 0) {
-         err = 0;
-      }
-   }
-#endif
-
-   return err;
-}
-
-
-static INLINE MXUserThreadID
-MXUserNativeThreadID(void)
-{
-   return pthread_self();
+   return pthread_mutex_destroy(&lock->nativeLock);
 }
 
 
 static INLINE Bool
 MXRecLockIsOwner(const MXRecLock *lock)  // IN:
 {
-   return pthread_equal(lock->nativeThreadID, MXUserNativeThreadID());
+   return pthread_equal(lock->nativeThreadID, pthread_self());
 }
 
 
@@ -228,14 +189,14 @@ static INLINE void
 MXRecLockSetNoOwner(MXRecLock *lock)  // IN/OUT:
 {
    /* a hack but it works portably */
-   memset(&lock->nativeThreadID, 0xFF, sizeof lock->nativeThreadID);
+   memset((void *) &lock->nativeThreadID, 0xFF, sizeof(lock->nativeThreadID));
 }
 
 
 static INLINE void
 MXRecLockSetOwner(MXRecLock *lock)  // IN:
 {
-   lock->nativeThreadID = MXUserNativeThreadID();
+   lock->nativeThreadID = pthread_self();
 }
 
 
@@ -270,6 +231,11 @@ MXRecLockInit(MXRecLock *lock)  // IN/OUT:
       MXRecLockSetNoOwner(lock);
 
       lock->referenceCount = 0;
+
+#if defined(MXUSER_DEBUG)
+      lock->portableThreadID = VTHREAD_INVALID_ID;
+      lock->ownerRetAddr = NULL;
+#endif
    }
 
    return success;
@@ -284,118 +250,131 @@ MXRecLockDestroy(MXRecLock *lock)  // IN/OUT:
    if (vmx86_debug && (err != 0)) {
       Panic("%s: MXRecLockDestroyInternal returned %d\n", __FUNCTION__, err);
    }
-}
+} 
 
 
-static INLINE int
+static INLINE uint32
 MXRecLockCount(const MXRecLock *lock)  // IN:
 {
-   ASSERT(lock->referenceCount >= 0);
-   ASSERT(lock->referenceCount < MXUSER_MAX_REC_DEPTH);
-
    return lock->referenceCount;
 }
 
 
 static INLINE void
 MXRecLockIncCount(MXRecLock *lock,  // IN/OUT:
-                  int count)        // IN:
+                  void *location,   // IN:
+                  uint32 count)     // IN:
 {
-   int newCount = (lock->referenceCount += count);
+   if (MXRecLockCount(lock) == 0) {
+#if defined(MXUSER_DEBUG)
+      ASSERT(lock->portableThreadID == VTHREAD_INVALID_ID);
 
-   ASSERT(count >= 0);
+      lock->ownerRetAddr = location;
+      lock->portableThreadID = VThread_CurID();
+#endif
 
-   if (LIKELY(newCount == count)) {
       MXRecLockSetOwner(lock);
    }
-}
 
-
-static INLINE void
-MXRecLockDecCount(MXRecLock *lock,  // IN/OUT:
-                  int count)        // IN:
-{
-   int newCount = (lock->referenceCount -= count);
-
-   ASSERT(count >= 0);
-
-   if (LIKELY(newCount == 0)) {
-      MXRecLockSetNoOwner(lock);
-   }
-}
-
-
-static INLINE void
-MXRecLockAcquire(MXRecLock *lock,       // IN/OUT:
-                 VmTimeType *duration)  // OUT/OPT:
-{
-   int err;
-
-   if (UNLIKELY(MXRecLockCount(lock) > 0) && MXRecLockIsOwner(lock)) {
-      MXRecLockIncCount(lock, 1);
-
-      if (duration != NULL) {
-         *duration = 0ULL;
-      }
-
-      return;  // Uncontended
-   }
-
-   if (LIKELY(duration == NULL)) {
-      err = MXRecLockAcquireInternal(lock);
-   } else {
-      VmTimeType start = Hostinfo_SystemTimerNS();
-
-      err = MXRecLockAcquireInternal(lock);
-
-      *duration = Hostinfo_SystemTimerNS() - start;
-   }
-
-   if (vmx86_debug && (err != 0)) {
-      Panic("%s: MXRecLockAcquireInternal error %d\n", __FUNCTION__, err);
-   }
-
-   ASSERT(MXRecLockCount(lock) == 0);
-
-   MXRecLockIncCount(lock, 1);
-
-   return;
+   lock->referenceCount += count;
 }
 
 
 static INLINE Bool
-MXRecLockTryAcquire(MXRecLock *lock)  // IN/OUT:
+MXRecLockAcquire(MXRecLock *lock,  // IN/OUT:
+                 void *location)   // IN:
+{
+   Bool contended;
+
+   if ((MXRecLockCount(lock) != 0) && MXRecLockIsOwner(lock)) {
+      ASSERT((MXRecLockCount(lock) > 0) &&
+             (MXRecLockCount(lock) < MXUSER_MAX_REC_DEPTH));
+
+      contended = FALSE;
+   } else {
+      int err = MXRecLockTryAcquireInternal(lock);
+
+      if (err == 0) {
+         contended = FALSE;
+      } else {
+         if (vmx86_debug && (err != EBUSY)) {
+            Panic("%s: MXRecLockTryAcquireInternal returned %d\n",
+                  __FUNCTION__, err);
+         }
+
+         err = MXRecLockAcquireInternal(lock);
+         contended = TRUE;
+      }
+
+      if (vmx86_debug && (err != 0)) {
+         Panic("%s: MXRecLockAcquireInternal returned %d\n", __FUNCTION__,
+               err);
+      }
+
+      ASSERT(lock->referenceCount == 0);
+   }
+
+   MXRecLockIncCount(lock, location, 1);
+
+   return contended;
+}
+
+
+static INLINE Bool
+MXRecLockTryAcquire(MXRecLock *lock,  // IN/OUT:
+                    void *location)   // IN:
 {
    int err;
-
-   if (UNLIKELY(MXRecLockCount(lock) > 0) && MXRecLockIsOwner(lock)) {
-      MXRecLockIncCount(lock, 1);
-
-      return TRUE;  // Was acquired
-   }
+   Bool acquired;
 
    err = MXRecLockTryAcquireInternal(lock);
 
-   if (LIKELY(err == 0)) {
-      MXRecLockIncCount(lock, 1);
+   if (err == 0) {
+      MXRecLockIncCount(lock, location, 1);
 
-      return TRUE;  // Was acquired
+      ASSERT((MXRecLockCount(lock) > 0) &&
+             (MXRecLockCount(lock) < MXUSER_MAX_REC_DEPTH));
+
+      acquired = TRUE;
+   } else {
+      if (vmx86_debug && (err != EBUSY)) {
+         Panic("%s: MXRecLockTryAcquireInternal returned %d\n", __FUNCTION__,
+               err);
+      }
+
+      acquired = FALSE;
    }
 
-   if (vmx86_debug && (err != EBUSY)) {
-      Panic("%s: MXRecLockTryAcquireInternal error %d\n", __FUNCTION__, err);
-   }
+   return acquired;
+}
 
-   return FALSE;  // Was not acquired
+static INLINE void
+MXRecLockDecCount(MXRecLock *lock,  // IN/OUT:
+                  uint32 count)     // IN:
+{
+   ASSERT(count <= lock->referenceCount);
+   lock->referenceCount -= count;
+
+   if (MXRecLockCount(lock) == 0) {
+      MXRecLockSetNoOwner(lock);
+
+#if defined(MXUSER_DEBUG)
+      lock->ownerRetAddr = NULL;
+      lock->portableThreadID = VTHREAD_INVALID_ID;
+#endif
+   }
 }
 
 
 static INLINE void
 MXRecLockRelease(MXRecLock *lock)  // IN/OUT:
 {
+   ASSERT((MXRecLockCount(lock) > 0) &&
+          (MXRecLockCount(lock) < MXUSER_MAX_REC_DEPTH));
+
    MXRecLockDecCount(lock, 1);
 
-   if (LIKELY(MXRecLockCount(lock) == 0)) {
+   if (MXRecLockCount(lock) == 0) {
       int err = MXRecLockReleaseInternal(lock);
 
       if (vmx86_debug && (err != 0)) {
@@ -406,70 +385,47 @@ MXRecLockRelease(MXRecLock *lock)  // IN/OUT:
 }
 
 
-/*
- *-----------------------------------------------------------------------------
- *
- * MXUserCastedThreadID --
- *
- *      Obtains a unique thread identifier (ID) which can be stored in a
- *      pointer; typically these thread ID values are used for tracking
- *      purposes.
- *
- *      These values are not typically used with the MXUser MXRecLock
- *      implementation. Native constructs that are very low overhead are
- *      used.
- *
- * Results:
- *      As above
- *
- * Side effects:
- *      None.
- *
- *-----------------------------------------------------------------------------
- */
-
-static INLINE void *
-MXUserCastedThreadID(void)
-{
-   return (void *) (uintptr_t) VThread_CurID();  // unsigned
-}
-
-/*
- * MXUser object type ID values.
- */
-
-typedef enum {
-   MXUSER_TYPE_NEVER_USE  = 0,
-   MXUSER_TYPE_RW         = 1,
-   MXUSER_TYPE_REC        = 2,
-   MXUSER_TYPE_RANK       = 3,
-   MXUSER_TYPE_EXCL       = 4,
-   MXUSER_TYPE_SEMA       = 5,
-   MXUSER_TYPE_CONDVAR    = 6,
-   MXUSER_TYPE_BARRIER    = 7,
-   MXUSER_TYPE_EVENT      = 8
-} MXUserObjectType;
-
 /*
  * MXUser header - all MXUser objects start with this
  */
 
 typedef struct MXUserHeader {
    uint32       signature;
-   char        *name;
    MX_Rank      rank;
-
-   struct {
-      unsigned int serialNumber : 24;
-      unsigned int badHeader: 1;
-   } bits;
-
+   const char  *name;
    void       (*dumpFunc)(struct MXUserHeader *);
+
+#if defined(MXUSER_STATS)
    void       (*statsFunc)(struct MXUserHeader *);
    ListItem     item;
+   uint32       identifier;
+#endif
 } MXUserHeader;
 
 
+/*
+ * The per thread information.
+ */
+
+#if defined(MXUSER_DEBUG) || defined(MXUSER_STATS)
+#define MXUSER_MAX_LOCKS_PER_THREAD (2 * MXUSER_MAX_REC_DEPTH)
+
+typedef struct {
+#if defined(MXUSER_DEBUG)
+   uint32         locksHeld;
+   MXUserHeader  *lockArray[MXUSER_MAX_LOCKS_PER_THREAD];
+#endif
+
+#if defined(MXUSER_STATS)
+   uint64         totalAcquisitions;      // total thread lock acquisitions
+   uint64         contendedAcquisitions;  // contended subset of above
+#endif
+} MXUserPerThread;
+
+MXUserPerThread *MXUserGetPerThread(VThreadID tid,
+                                    Bool mayAlloc);
+#endif
+
 /*
  * Internal functions
  */
@@ -480,16 +436,10 @@ void MXUserDumpAndPanic(MXUserHeader *header,
 
 MXRecLock *MXUserInternalSingleton(Atomic_Ptr *storage);
 
-uint32 MXUserGetSignature(MXUserObjectType objectType);
-
 #if defined(MXUSER_DEBUG)
 void MXUserAcquisitionTracking(MXUserHeader *header,
                            Bool checkRank);
-
 void MXUserReleaseTracking(MXUserHeader *header);
-
-void MXUserValidateHeader(MXUserHeader *header,
-                          MXUserObjectType objectType);
 #else
 static INLINE void
 MXUserAcquisitionTracking(MXUserHeader *header,  // IN:
@@ -503,13 +453,6 @@ MXUserReleaseTracking(MXUserHeader *header)  // IN:
 {
    return;
 }
-
-static INLINE void
-MXUserValidateHeader(MXUserHeader *header,         // IN:
-                     MXUserObjectType objectType)  // IN:
-{
-   return;
-}
 #endif
 
 static INLINE Bool
@@ -524,12 +467,13 @@ MXUserTryAcquireFail(const char *name)  // IN:
 MXUserCondVar *MXUserCreateCondVar(MXUserHeader *header,
                                    MXRecLock *lock);
 
-void MXUserWaitCondVar(MXUserHeader *header,
+Bool MXUserWaitCondVar(MXUserHeader *header,
                        MXRecLock *lock,
                        MXUserCondVar *condVar,
                        uint32 msecWait);
 
 
+#if defined(MXUSER_STATS)
 typedef struct {
    char    *typeName;        // Name
    uint64   numSamples;      // Population sample size
@@ -540,14 +484,10 @@ typedef struct {
 } MXUserBasicStats;
 
 typedef struct {
-   double            contentionRatioFloor;
-   uint64            contentionCountFloor;
-   uint64            contentionDurationFloor;
-   uint64            numAttempts;
-   uint64            numSuccesses;
-   uint64            numSuccessesContended;
-   uint64            successContentionTime;
-   uint64            totalContentionTime;
+   uint64            numContended;     // Number of contended acquires
+   uint64            timeContended;    // Time spent contended on acquires
+   uint64            numUncontended;   // Number of uncontended acquires
+   uint64            timeUncontended;  // Time spent uncontended on acquires
 
    MXUserBasicStats  basicStats;
 } MXUserAcquisitionStats;
@@ -556,26 +496,10 @@ typedef struct {
    MXUserBasicStats  basicStats;       // total held statistics
 } MXUserReleaseStats;
 
-typedef struct
-{
-   MXUserAcquisitionStats  data;
-   Atomic_Ptr              histo;
-} MXUserAcquireStats;
-
-typedef struct
-{
-   VmTimeType        holdStart;
-   MXUserBasicStats  data;
-   Atomic_Ptr        histo;
-} MXUserHeldStats;
-
-uint32 MXUserAllocSerialNumber(void);
-
+uint32 MXUserAllocID(void);
 void MXUserAddToList(MXUserHeader *header);
 void MXUserRemoveFromList(MXUserHeader *header);
 
-uint32 MXUserStatsMode(void);
-
 typedef struct MXUserHisto MXUserHisto;
 
 MXUserHisto *MXUserHistoSetUp(char *typeName,
@@ -585,8 +509,7 @@ MXUserHisto *MXUserHistoSetUp(char *typeName,
 void MXUserHistoTearDown(MXUserHisto *histo);
 
 void MXUserHistoSample(MXUserHisto *histo,
-                       uint64 value,
-                       void *caller);
+                       uint64 value);
 
 void MXUserHistoDump(MXUserHisto *histo,
                      MXUserHeader *header);
@@ -594,9 +517,8 @@ void MXUserHistoDump(MXUserHisto *histo,
 void MXUserAcquisitionStatsSetUp(MXUserAcquisitionStats *stats);
 
 void MXUserAcquisitionSample(MXUserAcquisitionStats *stats,
-                             Bool wasAcquired,
                              Bool wasContended,
-                             uint64 elapsedTime);
+                             uint64 timeToAcquire);
 
 void MXUserDumpAcquisitionStats(MXUserAcquisitionStats *stats,
                                 MXUserHeader *header);
@@ -610,7 +532,7 @@ MXUserBasicStatsSetUp(MXUserBasicStats *stats,
 void
 MXUserBasicStatsSample(MXUserBasicStats *stats,
                        uint64 value);
-
+ 
 void MXUserDumpBasicStats(MXUserBasicStats *stats,
                           MXUserHeader *header);
 
@@ -621,36 +543,15 @@ void MXUserKitchen(MXUserAcquisitionStats *stats,
                    Bool *isHot,
                    Bool *doLog);
 
-#define MXUSER_DEFAULT_HISTO_MIN_VALUE_NS  1000  // 1 usec
-#define MXUSER_DEFAULT_HISTO_DECADES       7     // 1 usec to 10 seconds
+void MXUserForceHisto(Atomic_Ptr *histoPtr,
+                      char *typeName,
+                      uint64 minValue,
+                      uint32 decades);
+#endif
 
-Bool MXUserForceAcquisitionHisto(Atomic_Ptr *mem,
-                                 uint64 minValue,
-                                 uint32 decades);
-
-Bool MXUserForceHeldHisto(Atomic_Ptr *mem,
-                          uint64 minValue,
-                          uint32 decades);
-
-Bool MXUserSetContentionRatioFloor(Atomic_Ptr *mem,
-                                   double ratio);
-
-Bool MXUserSetContentionCountFloor(Atomic_Ptr *mem,
-                                   uint64 count);
-
-Bool MXUserSetContentionDurationFloor(Atomic_Ptr *mem,
-                                      uint64 count);
-
-void MXUserEnableStats(Atomic_Ptr *acquisitionMem,
-                       Atomic_Ptr *heldMem);
-
-void MXUserDisableStats(Atomic_Ptr *acquisitionMem,
-                        Atomic_Ptr *heldMem);
-
-extern void  (*MXUserMX_LockRec)(struct MX_MutexRec *lock);
-extern void  (*MXUserMX_UnlockRec)(struct MX_MutexRec *lock);
-extern Bool  (*MXUserMX_TryLockRec)(struct MX_MutexRec *lock);
-extern Bool  (*MXUserMX_IsLockedByCurThreadRec)(const struct MX_MutexRec *lock);
-extern char *(*MXUserMX_NameRec)(const struct MX_MutexRec *lock);
+extern void (*MXUserMX_LockRec)(struct MX_MutexRec *lock);
+extern void (*MXUserMX_UnlockRec)(struct MX_MutexRec *lock);
+extern Bool (*MXUserMX_TryLockRec)(struct MX_MutexRec *lock);
+extern Bool (*MXUserMX_IsLockedByCurThreadRec)(const struct MX_MutexRec *lock);
 
 #endif
