@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2010-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2010 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -19,37 +19,13 @@
 #ifndef _ULINTSHARED_H_
 #define _ULINTSHARED_H_
 
-
-#if defined(MXUSER_DEBUG)
 extern void MXUserListLocks(void);
-#else
-static INLINE void
-MXUserListLocks(void)
-{
-   return;
-}
-#endif
-
-MX_Rank MXUserCurrentRank(void);
 
 extern void MXUserInstallMxHooks(void (*theLockListFunc)(void),
                                  MX_Rank (*theRankFunc)(void),
                                  void (*theLockFunc)(struct MX_MutexRec *lock),
                                  void (*theUnlockFunc)(struct MX_MutexRec *lock),
                                  Bool (*theTryLockFunc)(struct MX_MutexRec *lock),
-                                 Bool (*theIsLockedFunc)(const struct MX_MutexRec *lock),
-                                 char *(*theNameFunc)(const struct MX_MutexRec *lock),
-                                 void (*theSetInPanicFunc)(void),
-                                 Bool (*theInPanicFunc)(void));
-
-typedef struct MXUserLockTreeNode MXUserLockTreeNode;
-
-void MXUserLockTreeAcquire(void);
-void MXUserLockTreeRelease(void);
-
-MXUserLockTreeNode *MXUserLockTreeAdd(MXUserLockTreeNode *searchNode,
-                                      const char *name,
-                                      uint32 serialNumber,
-                                      MX_Rank rank);
+                                 Bool (*theIsLockedFunc)(const struct MX_MutexRec *lock));
 
 #endif
