@@ -21,7 +21,6 @@
 #endif
 
 #include "vmware.h"
-#include "vm_basic_asm.h"
 #include "str.h"
 #include "util.h"
 #include "hashTable.h"
@@ -29,11 +28,7 @@
 #include "hostinfo.h"
 #include "ulInt.h"
 
-static void
-MXUserFreeHashEntry(void *data)  // IN:
-{
-   free(data);
-}
+#define MXUSER_RW_SIGNATURE 0x57524B4C // 'LKRW' in memory
 
 
 /*
@@ -93,15 +88,11 @@ MXUserNativeRWSupported(void)
                                     GetProcAddress(kernel32,
                                                    "ReleaseSRWLockExclusive");
 
-         COMPILER_MEM_BARRIER();
-
          result = ((pInitializeSRWLock != NULL) &&
                    (pAcquireSRWLockShared != NULL) &&
                    (pReleaseSRWLockShared != NULL) &&
                    (pAcquireSRWLockExclusive != NULL) &&
                    (pReleaseSRWLockExclusive != NULL));
-
-         COMPILER_MEM_BARRIER();
       } else {
          result = FALSE;
       }
@@ -241,22 +232,13 @@ typedef enum {
 } HolderState;
 
 typedef struct {
-   HolderState   state;
-   VmTimeType    holdStart;
+#if defined(MXUSER_STATS)
+   VmTimeType   holdStart;
+#endif
+
+   HolderState  state;
 } HolderContext;
 
-typedef struct
-{
-   MXUserAcquisitionStats  data;
-   Atomic_Ptr              histo;
-} MXUserAcquireStats;
-
-typedef struct
-{
-   MXUserBasicStats  data;
-   Atomic_Ptr        histo;
-} MXUserHeldStats;
-
 struct MXUserRWLock
 {
    MXUserHeader    header;
@@ -268,11 +250,17 @@ struct MXUserRWLock
    Atomic_uint32   holderCount;
    HashTable      *holderTable;
 
-   Atomic_Ptr      heldStatsMem;
-   Atomic_Ptr      acquireStatsMem;
+#if defined(MXUSER_STATS)
+   MXUserAcquisitionStats  acquisitionStats;
+   Atomic_Ptr              acquisitionHisto;
+
+   MXUserBasicStats        heldStats;
+   Atomic_Ptr              heldHisto;
+#endif
 };
 
 
+#if defined(MXUSER_STATS)
 /*
  *-----------------------------------------------------------------------------
  *
@@ -292,206 +280,52 @@ struct MXUserRWLock
 static void
 MXUserStatsActionRW(MXUserHeader *header)  // IN:
 {
+   Bool isHot;
+   Bool doLog;
+   double contentionRatio;
+
    MXUserRWLock *lock = (MXUserRWLock *) header;
-   MXUserHeldStats *heldStats = Atomic_ReadPtr(&lock->heldStatsMem);
-   MXUserAcquireStats *acquireStats = Atomic_ReadPtr(&lock->acquireStatsMem);
 
-   if (heldStats) {
-      MXUserDumpBasicStats(&heldStats->data, header);
+   /*
+    * Dump the statistics for the specified lock.
+    */
 
-      if (Atomic_ReadPtr(&heldStats->histo) != NULL) {
-         MXUserHistoDump(Atomic_ReadPtr(&heldStats->histo), header);
-      }
+   MXUserDumpAcquisitionStats(&lock->acquisitionStats, header);
+
+   if (Atomic_ReadPtr(&lock->acquisitionHisto) != NULL) {
+      MXUserHistoDump(Atomic_ReadPtr(&lock->acquisitionHisto), header);
    }
 
-   if (acquireStats) {
-      Bool isHot;
-      Bool doLog;
-      double contentionRatio;
+   MXUserDumpBasicStats(&lock->heldStats, header);
 
-      /*
-       * Dump the statistics for the specified lock.
-       */
+   if (Atomic_ReadPtr(&lock->heldHisto) != NULL) {
+      MXUserHistoDump(Atomic_ReadPtr(&lock->heldHisto), header);
+   }
 
-      MXUserDumpAcquisitionStats(&acquireStats->data, header);
+   /*
+    * Has the lock gone "hot"? If so, implement the hot actions.
+    * Allow the read and write statistics to go independently hot.
+    */
 
-      if (Atomic_ReadPtr(&acquireStats->histo) != NULL) {
-         MXUserHistoDump(Atomic_ReadPtr(&acquireStats->histo), header);
-      }
+   MXUserKitchen(&lock->acquisitionStats, &contentionRatio, &isHot, &doLog);
 
-      /*
-       * Has the lock gone "hot"? If so, implement the hot actions.
-       */
+   if (isHot) {
+      MXUserForceHisto(&lock->acquisitionHisto,
+                       MXUSER_STAT_CLASS_ACQUISITION,
+                       MXUSER_DEFAULT_HISTO_MIN_VALUE_NS,
+                       MXUSER_DEFAULT_HISTO_DECADES);
+      MXUserForceHisto(&lock->heldHisto,
+                       MXUSER_STAT_CLASS_HELD,
+                       MXUSER_DEFAULT_HISTO_MIN_VALUE_NS,
+                       MXUSER_DEFAULT_HISTO_DECADES);
 
-      MXUserKitchen(&acquireStats->data, &contentionRatio, &isHot, &doLog);
-
-      if (isHot) {
-         MXUserForceHisto(&acquireStats->histo,
-                          MXUSER_STAT_CLASS_ACQUISITION,
-                          MXUSER_DEFAULT_HISTO_MIN_VALUE_NS,
-                          MXUSER_DEFAULT_HISTO_DECADES);
-
-         if (heldStats) {
-            MXUserForceHisto(&heldStats->histo,
-                             MXUSER_STAT_CLASS_HELD,
-                             MXUSER_DEFAULT_HISTO_MIN_VALUE_NS,
-                             MXUSER_DEFAULT_HISTO_DECADES);
-         }
-
-         if (doLog) {
-            Log("HOT LOCK (%s); contention ratio %f\n",
-                lock->header.name, contentionRatio);
-         }
+      if (doLog) {
+         Log("HOT LOCK (%s); contention ratio %f\n",
+             lock->header.name, contentionRatio);
       }
    }
 }
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * MXUser_ControlRWLock --
- *
- *      Perform the specified command on the specified lock.
- *
- * Results:
- *      TRUE    succeeded
- *      FALSE   failed
- *
- * Side effects:
- *      Depends on the command, no?
- *
- *-----------------------------------------------------------------------------
- */
-
-Bool
-MXUser_ControlRWLock(MXUserRWLock *lock,  // IN/OUT:
-                     uint32 command,      // IN:
-                     ...)                 // IN:
-{
-   Bool result;
-
-   ASSERT(lock);
-   MXUserValidateHeader(&lock->header, MXUSER_TYPE_RW);
-
-   switch (command) {
-   case MXUSER_CONTROL_ACQUISITION_HISTO: {
-      if (vmx86_stats) {
-         MXUserAcquireStats *acquireStats;
-
-         acquireStats = Atomic_ReadPtr(&lock->acquireStatsMem);
-
-         if (acquireStats == NULL) {
-            result = FALSE;
-         } else {
-            va_list a;
-            uint32 decades;
-            uint64 minValue;
-
-            va_start(a, command);
-            minValue = va_arg(a, uint64);
-            decades = va_arg(a, uint32);
-            va_end(a);
-
-            MXUserForceHisto(&acquireStats->histo,
-                             MXUSER_STAT_CLASS_ACQUISITION, minValue, decades);
-
-            result = TRUE;
-         }
-      } else {
-         result = FALSE;
-      }
-
-      break;
-   }
-
-   case MXUSER_CONTROL_HELD_HISTO: {
-      if (vmx86_stats) {
-         MXUserHeldStats *heldStats = Atomic_ReadPtr(&lock->heldStatsMem);
-
-         if (heldStats == NULL) {
-            result = FALSE;
-         } else {
-            va_list a;
-            uint32 decades;
-            uint32 minValue;
-
-            va_start(a, command);
-            minValue = va_arg(a, uint64);
-            decades = va_arg(a, uint32);
-            va_end(a);
-
-            MXUserForceHisto(&heldStats->histo, MXUSER_STAT_CLASS_HELD,
-                             minValue, decades);
-
-            result = TRUE;
-         }
-      } else {
-         result = FALSE;
-      }
-
-      break;
-   }
-
-   case MXUSER_CONTROL_ENABLE_STATS: {
-      if (vmx86_stats) {
-         va_list a;
-         Bool trackHeldTimes;
-         MXUserHeldStats *heldStats;
-         MXUserAcquireStats *acquireStats;
-
-         acquireStats = Atomic_ReadPtr(&lock->acquireStatsMem);
-
-         if (LIKELY(acquireStats == NULL)) {
-            MXUserAcquireStats *before;
-
-            acquireStats = Util_SafeCalloc(1, sizeof(*acquireStats));
-            MXUserAcquisitionStatsSetUp(&acquireStats->data);
-
-            before = Atomic_ReadIfEqualWritePtr(&lock->acquireStatsMem, NULL,
-                                                (void *) acquireStats);
-
-            if (before) {
-               free(acquireStats);
-            }
-         }
-
-         va_start(a, command);
-         trackHeldTimes = va_arg(a, int);
-         va_end(a);
-
-         heldStats = Atomic_ReadPtr(&lock->heldStatsMem);
-
-         if ((heldStats == NULL) && trackHeldTimes) {
-            MXUserHeldStats *before;
-
-            heldStats = Util_SafeCalloc(1, sizeof(*heldStats));
-            MXUserBasicStatsSetUp(&heldStats->data, MXUSER_STAT_CLASS_HELD);
-
-            before = Atomic_ReadIfEqualWritePtr(&lock->heldStatsMem, NULL,
-                                                (void *) heldStats);
-
-            if (before) {
-               free(heldStats);
-            }
-         }
-
-         lock->header.statsFunc = MXUserStatsActionRW;
-
-         result = TRUE;
-      } else {
-         result = FALSE;
-      }
-
-      break;
-   }
-
-   default:
-      result = FALSE;
-   }
-
-   return result;
-}
+#endif
 
 
 /*
@@ -520,12 +354,16 @@ MXUserDumpRWLock(MXUserHeader *header)  // IN:
    Warning("\tsignature 0x%X\n", lock->header.signature);
    Warning("\tname %s\n", lock->header.name);
    Warning("\trank 0x%X\n", lock->header.rank);
-   Warning("\tserial number %u\n", lock->header.serialNumber);
 
    if (LIKELY(lock->useNative)) {
-      Warning("\taddress of native lock 0x%p\n", &lock->nativeLock);
+      Warning("\tnativeLock 0x%p\n", &lock->nativeLock);
    } else {
-      Warning("\tcount %d\n", MXRecLockCount(&lock->recursiveLock));
+      Warning("\tcount %u\n", lock->recursiveLock.referenceCount);
+
+#if defined(MXUSER_DEBUG)
+      Warning("\tcaller 0x%p\n", lock->recursiveLock.ownerRetAddr);
+      Warning("\tVThreadID %d\n", (int) lock->recursiveLock.portableThreadID);
+#endif
    }
 
    Warning("\tholderCount %d\n", Atomic_Read(&lock->holderCount));
@@ -552,6 +390,12 @@ MXUserDumpRWLock(MXUserHeader *header)  // IN:
  *-----------------------------------------------------------------------------
  */
 
+static void
+MXUserFreeHashEntry(void *data)  // IN:
+{
+   free(data);
+}
+
 MXUserRWLock *
 MXUser_CreateRWLock(const char *userName,  // IN:
                     MX_Rank rank)          // IN:
@@ -574,12 +418,6 @@ MXUser_CreateRWLock(const char *userName,  // IN:
       properName = Util_SafeStrdup(userName);
    }
 
-   lock->header.signature = MXUserGetSignature(MXUSER_TYPE_RW);
-   lock->header.name = properName;
-   lock->header.rank = rank;
-   lock->header.serialNumber = MXUserAllocSerialNumber();
-   lock->header.dumpFunc = MXUserDumpRWLock;
-
    /*
     * Always attempt to use native locks when they are available. If, for some
     * reason, a native lock should be available but isn't, fall back to using
@@ -588,42 +426,41 @@ MXUser_CreateRWLock(const char *userName,  // IN:
 
    lock->useNative = useNative && MXUserNativeRWInit(&lock->nativeLock);
 
-   lockInited = MXRecLockInit(&lock->recursiveLock);
+   if (lock->useNative) {
+#if defined(MXUSER_STATS)
+      /* stats builds need an internal recursive lock for data integrity */
+
+      lockInited = MXRecLockInit(&lock->recursiveLock);
+
+      if (!lockInited) {
+         MXUserNativeRWDestroy(&lock->nativeLock);
+      }
+#else
+      lockInited = TRUE;
+#endif
+   } else {
+      lockInited = MXRecLockInit(&lock->recursiveLock);
+   }
 
    if (LIKELY(lockInited)) {
-      uint32 statsMode;
-
       lock->holderTable = HashTable_Alloc(256,
                                           HASH_INT_KEY | HASH_FLAG_ATOMIC,
                                           MXUserFreeHashEntry);
 
-      statsMode = MXUserStatsMode();
+      lock->header.name = properName;
+      lock->header.signature = MXUSER_RW_SIGNATURE;
+      lock->header.rank = rank;
+      lock->header.dumpFunc = MXUserDumpRWLock;
 
-      switch (statsMode) {
-      case 0:
-         lock->header.statsFunc = NULL;
-         Atomic_WritePtr(&lock->acquireStatsMem, NULL);
-         Atomic_WritePtr(&lock->heldStatsMem, NULL);
-         break;
-
-      case 1:
-         MXUser_ControlRWLock(lock, MXUSER_CONTROL_ENABLE_STATS, FALSE);
-         break;
-
-      case 2:
-         MXUser_ControlRWLock(lock, MXUSER_CONTROL_ENABLE_STATS, TRUE);
-         break;
-
-      default:
-         Panic("%s: unknown stats mode: %d!\n", __FUNCTION__, statsMode);
-      }
+#if defined(MXUSER_STATS)
+      lock->header.statsFunc = MXUserStatsActionRW;
+      lock->header.identifier = MXUserAllocID();
 
       MXUserAddToList(&lock->header);
+      MXUserAcquisitionStatsSetUp(&lock->acquisitionStats);
+      MXUserBasicStatsSetUp(&lock->heldStats, MXUSER_STAT_CLASS_HELD);
+#endif
    } else {
-      if (lock->useNative) {
-         MXUserNativeRWDestroy(&lock->nativeLock);
-      }
-
       free(properName);
       free(lock);
       lock = NULL;
@@ -653,7 +490,7 @@ void
 MXUser_DestroyRWLock(MXUserRWLock *lock)  // IN:
 {
    if (LIKELY(lock != NULL)) {
-      MXUserValidateHeader(&lock->header, MXUSER_TYPE_RW);
+      ASSERT(lock->header.signature == MXUSER_RW_SIGNATURE);
 
       if (Atomic_Read(&lock->holderCount) != 0) {
          MXUserDumpAndPanic(&lock->header,
@@ -668,39 +505,25 @@ MXUser_DestroyRWLock(MXUserRWLock *lock)  // IN:
             MXUserDumpAndPanic(&lock->header, "%s: Internal error (%d)\n",
                                __FUNCTION__, err);
          }
+
+#if defined(MXUSER_STATS)
+         MXRecLockDestroy(&lock->recursiveLock);  
+#endif
+      } else {
+         MXRecLockDestroy(&lock->recursiveLock);  
       }
 
-      lock->header.signature = 0;  // just in case...
-
-      MXRecLockDestroy(&lock->recursiveLock);
-
+#if defined(MXUSER_STATS)
       MXUserRemoveFromList(&lock->header);
+      MXUserAcquisitionStatsTearDown(&lock->acquisitionStats);
+      MXUserBasicStatsTearDown(&lock->heldStats);
+      MXUserHistoTearDown(Atomic_ReadPtr(&lock->acquisitionHisto));
+      MXUserHistoTearDown(Atomic_ReadPtr(&lock->heldHisto));
+#endif
 
-      if (vmx86_stats) {
-         MXUserHeldStats *heldStats;
-         MXUserAcquireStats *acquireStats;
-
-         acquireStats = Atomic_ReadPtr(&lock->acquireStatsMem);
-
-         if (LIKELY(acquireStats != NULL)) {
-            MXUserAcquisitionStatsTearDown(&acquireStats->data);
-            MXUserHistoTearDown(Atomic_ReadPtr(&acquireStats->histo));
-
-            free(acquireStats);
-         }
-
-         heldStats = Atomic_ReadPtr(&lock->heldStatsMem);
-
-         if (UNLIKELY(heldStats != NULL)) {
-            MXUserBasicStatsTearDown(&heldStats->data);
-            MXUserHistoTearDown(Atomic_ReadPtr(&heldStats->histo));
-
-            free(heldStats);
-         }
-      }
-
-      HashTable_FreeUnsafe(lock->holderTable);
-      free(lock->header.name);
+      HashTable_Free(lock->holderTable);
+      lock->header.signature = 0;  // just in case...
+      free((void *) lock->header.name);  // avoid const warnings
       lock->header.name = NULL;
       free(lock);
    }
@@ -728,17 +551,22 @@ static HolderContext *
 MXUserGetHolderContext(MXUserRWLock *lock)  // IN:
 {
    HolderContext *result;
-   void *threadID = MXUserCastedThreadID();
+   VThreadID tid = VThread_CurID();
 
    ASSERT(lock->holderTable);
 
-   if (!HashTable_Lookup(lock->holderTable, threadID, (void **) &result)) {
+   if (!HashTable_Lookup(lock->holderTable, (void *) (uintptr_t) tid,
+                         (void **) &result)) {
       HolderContext *newContext = Util_SafeMalloc(sizeof(HolderContext));
 
+#if defined(MXUSER_STATS)
       newContext->holdStart = 0;
+#endif
+
       newContext->state = RW_UNLOCKED;
 
-      result = HashTable_LookupOrInsert(lock->holderTable, threadID,
+      result = HashTable_LookupOrInsert(lock->holderTable,
+                                        (void *) (uintptr_t) tid,
                                         (void *) newContext);
 
       if (result != newContext) {
@@ -770,10 +598,17 @@ static INLINE void
 MXUserAcquisition(MXUserRWLock *lock,  // IN/OUT:
                   Bool forRead)        // IN:
 {
+   int err = 0;
+   Bool contended;
    HolderContext *myContext;
 
-   ASSERT(lock);
-   MXUserValidateHeader(&lock->header, MXUSER_TYPE_RW);
+#if defined(MXUSER_STATS)
+   VmTimeType begin;
+   VmTimeType value;
+   MXUserHisto *histo;
+#endif
+
+   ASSERT(lock && (lock->header.signature == MXUSER_RW_SIGNATURE));
 
    MXUserAcquisitionTracking(&lock->header, TRUE);
 
@@ -788,86 +623,50 @@ MXUserAcquisition(MXUserRWLock *lock,  // IN/OUT:
                                                                    "Write");
    }
 
-   if (vmx86_stats) {
-      VmTimeType value;
-      MXUserAcquireStats *acquireStats;
+#if defined(MXUSER_STATS)
+   begin = Hostinfo_SystemTimerNS();
+#endif
 
-      acquireStats = Atomic_ReadPtr(&lock->acquireStatsMem);
-
-      if (lock->useNative) {
-         int err = 0;
-         Bool contended;
-         VmTimeType begin = Hostinfo_SystemTimerNS();
-
-         contended = MXUserNativeRWAcquire(&lock->nativeLock, forRead, &err);
-
-         value = contended ? Hostinfo_SystemTimerNS() - begin : 0;
-
-         if (UNLIKELY(err != 0)) {
-            MXUserDumpAndPanic(&lock->header, "%s: Error %d: contended %d\n",
-                               __FUNCTION__, err, contended);
-         }
-      } else {
-         value = 0;
-
-         MXRecLockAcquire(&lock->recursiveLock,
-                          (acquireStats == NULL) ? NULL : &value);
-      }
-
-      if (LIKELY(acquireStats != NULL)) {
-         MXUserHisto *histo;
-         MXUserHeldStats *heldStats;
-
-         /*
-          * The statistics are not atomically safe so protect them when
-          * necessary.
-          */
-
-         if (forRead && lock->useNative) {
-            MXRecLockAcquire(&lock->recursiveLock,
-                             NULL);  // non-stats
-         }
-
-         MXUserAcquisitionSample(&acquireStats->data, TRUE, value != 0, value);
-
-         histo = Atomic_ReadPtr(&acquireStats->histo);
-
-         if (UNLIKELY(histo != NULL)) {
-            MXUserHistoSample(histo, value, GetReturnAddress());
-         }
-
-         if (forRead && lock->useNative) {
-            MXRecLockRelease(&lock->recursiveLock);
-         }
-
-         heldStats = Atomic_ReadPtr(&lock->heldStatsMem);
-
-         if (UNLIKELY(heldStats != NULL)) {
-            myContext->holdStart = Hostinfo_SystemTimerNS();
-         }
-      }
-   } else {
-      if (LIKELY(lock->useNative)) {
-         int err = 0;
-
-         MXUserNativeRWAcquire(&lock->nativeLock, forRead, &err);
-
-         if (UNLIKELY(err != 0)) {
-            MXUserDumpAndPanic(&lock->header, "%s: Error %d\n",
-                               __FUNCTION__, err);
-         }
-      } else {
-         MXRecLockAcquire(&lock->recursiveLock,
-                          NULL);  // non-stats
-      }
+   contended = lock->useNative ? MXUserNativeRWAcquire(&lock->nativeLock,
+                                                       forRead, &err) :
+                                 MXRecLockAcquire(&lock->recursiveLock,
+                                                  GetReturnAddress());
+   if (UNLIKELY(err != 0)) {
+      MXUserDumpAndPanic(&lock->header, "%s: Error %d: contended %d\n",
+                         __FUNCTION__, err, contended);
    }
 
+#if defined(MXUSER_STATS)
+   value = Hostinfo_SystemTimerNS() - begin;
+
+   /* The statistics are not atomically safe so protect them when necessary */
+   if (forRead && lock->useNative) {
+      MXRecLockAcquire(&lock->recursiveLock, GetReturnAddress());
+   }
+
+   MXUserAcquisitionSample(&lock->acquisitionStats, contended, value);
+
+   histo = Atomic_ReadPtr(&lock->acquisitionHisto);
+
+   if (UNLIKELY(histo != NULL)) {
+      MXUserHistoSample(histo, value);
+   }
+
+   if (forRead && lock->useNative) {
+      MXRecLockRelease(&lock->recursiveLock);
+   }
+#endif
+
    if (!forRead || !lock->useNative) {
       ASSERT(Atomic_Read(&lock->holderCount) == 0);
    }
 
    Atomic_Inc(&lock->holderCount);
    myContext->state = forRead ? RW_LOCKED_FOR_READ : RW_LOCKED_FOR_WRITE;
+
+#if defined(MXUSER_STATS)
+   myContext->holdStart = Hostinfo_SystemTimerNS();
+#endif
 }
 
 
@@ -876,9 +675,7 @@ MXUserAcquisition(MXUserRWLock *lock,  // IN/OUT:
  *
  * MXUser_AcquireForRead --
  *
- *      Acquire a read-write lock for read-shared access. A thread may have
- *      only one read lock outstanding on a read-write lock - no recursive
- *      access.
+ *      Acquire a read-write lock for read-shared access.
  *
  * Results:
  *      The lock is acquired.
@@ -901,8 +698,7 @@ MXUser_AcquireForRead(MXUserRWLock *lock)  // IN/OUT:
  *
  * MXUser_AcquireForWrite --
  *
- *      Acquire a read-write lock for write-exclusive access. A thread may
- *      have only a single write lock outstanding on a read-write lock.
+ *      Acquire a read-write lock for write-exclusive access.
  *
  * Results:
  *      The lock is acquired.
@@ -920,47 +716,50 @@ MXUser_AcquireForWrite(MXUserRWLock *lock)  // IN/OUT:
 }
 
 
-/*
+/*      
  *-----------------------------------------------------------------------------
- *
+ *      
  * MXUser_IsCurThreadHolding --
  *
  *      Is the read-write lock held in the mode queried?
- *
+ *      
  * Results:
  *      TRUE   Yes
  *      FALSE  No
  *
  * Side effects:
  *      None
- *
+ *      
  *-----------------------------------------------------------------------------
- */
-
-Bool
+ */     
+        
+Bool    
 MXUser_IsCurThreadHoldingRWLock(MXUserRWLock *lock,  // IN:
                                 uint32 queryType)    // IN:
 {
    HolderContext *myContext;
 
-   ASSERT(lock);
-   MXUserValidateHeader(&lock->header, MXUSER_TYPE_RW);
+   ASSERT(lock && (lock->header.signature == MXUSER_RW_SIGNATURE));
 
    myContext = MXUserGetHolderContext(lock);
 
    switch (queryType) {
    case MXUSER_RW_FOR_READ:
       return myContext->state == RW_LOCKED_FOR_READ;
-
+        
    case MXUSER_RW_FOR_WRITE:
       return myContext->state == RW_LOCKED_FOR_WRITE;
-
+        
    case MXUSER_RW_LOCKED:
       return myContext->state != RW_UNLOCKED;
 
    default:
+#if defined(MXUSER_DEBUG)
       Panic("%s: unknown query type %d\n", __FUNCTION__, queryType);
-   }
+#else   
+      return FALSE;
+#endif  
+   }    
 }
 
 
@@ -985,55 +784,52 @@ MXUser_ReleaseRWLock(MXUserRWLock *lock)  // IN/OUT:
 {
    HolderContext *myContext;
 
-   ASSERT(lock);
-   MXUserValidateHeader(&lock->header, MXUSER_TYPE_RW);
+#if defined(MXUSER_STATS)
+   VmTimeType holdEnd = Hostinfo_SystemTimerNS();
+   VmTimeType duration;
+   MXUserHisto *histo;
+#endif
+
+   ASSERT(lock && (lock->header.signature == MXUSER_RW_SIGNATURE));
 
    myContext = MXUserGetHolderContext(lock);
 
-   if (vmx86_stats) {
-      MXUserHeldStats *heldStats = Atomic_ReadPtr(&lock->heldStatsMem);
-
-      if (UNLIKELY(heldStats != NULL)) {
-         MXUserHisto *histo;
-         VmTimeType duration = Hostinfo_SystemTimerNS() - myContext->holdStart;
-
-         /*
-          * The statistics are not always atomically safe so protect them
-          * when necessary
-          */
-
-         if ((myContext->state == RW_LOCKED_FOR_READ) && lock->useNative) {
-            MXRecLockAcquire(&lock->recursiveLock,
-                             NULL);  // non-stats
-         }
-
-         MXUserBasicStatsSample(&heldStats->data, duration);
-
-         histo = Atomic_ReadPtr(&heldStats->histo);
-
-         if (UNLIKELY(histo != NULL)) {
-            MXUserHistoSample(histo, duration, GetReturnAddress());
-         }
-
-         if ((myContext->state == RW_LOCKED_FOR_READ) && lock->useNative) {
-            MXRecLockRelease(&lock->recursiveLock);
-         }
-      }
-   }
-
    if (UNLIKELY(myContext->state == RW_UNLOCKED)) {
+      uint32 lockCount = Atomic_Read(&lock->holderCount);
+
       MXUserDumpAndPanic(&lock->header,
-                         "%s: Release of an unacquired read-write lock\n",
-                         __FUNCTION__);
+                         "%s: Non-owner release of an %s read-write lock\n",
+                         __FUNCTION__,
+                         lockCount == 0 ? "unacquired" : "acquired");
    }
 
+#if defined(MXUSER_STATS)
+   duration = holdEnd - myContext->holdStart;
+
+   /* The statistics are not atomically safe so protect them when necessary */
+   if ((myContext->state == RW_LOCKED_FOR_READ) && lock->useNative) {
+      MXRecLockAcquire(&lock->recursiveLock, GetReturnAddress());
+   }
+
+   MXUserBasicStatsSample(&lock->heldStats, duration);
+
+   histo = Atomic_ReadPtr(&lock->heldHisto);
+
+   if (UNLIKELY(histo != NULL)) {
+      MXUserHistoSample(histo, duration);
+   }
+
+   if ((myContext->state == RW_LOCKED_FOR_READ) && lock->useNative) {
+      MXRecLockRelease(&lock->recursiveLock);
+   }
+#endif
+
    MXUserReleaseTracking(&lock->header);
 
    Atomic_Dec(&lock->holderCount);
 
    if (LIKELY(lock->useNative)) {
-      int err = MXUserNativeRWRelease(&lock->nativeLock,
-                                      myContext->state == RW_LOCKED_FOR_READ);
+      int err = MXUserNativeRWRelease(&lock->nativeLock, myContext->state);
 
       if (UNLIKELY(err != 0)) {
          MXUserDumpAndPanic(&lock->header, "%s: Internal error (%d)\n",
@@ -1076,17 +872,20 @@ MXUser_CreateSingletonRWLock(Atomic_Ptr *lockStorage,  // IN/OUT:
 
    ASSERT(lockStorage);
 
-   lock = Atomic_ReadPtr(lockStorage);
+   lock = (MXUserRWLock *) Atomic_ReadPtr(lockStorage);
 
    if (UNLIKELY(lock == NULL)) {
-      MXUserRWLock *newLock = MXUser_CreateRWLock(name, rank);
+      MXUserRWLock *before;
 
-      lock = Atomic_ReadIfEqualWritePtr(lockStorage, NULL, (void *) newLock);
+      lock = MXUser_CreateRWLock(name, rank);
 
-      if (lock) {
-         MXUser_DestroyRWLock(newLock);
-      } else {
-         lock = Atomic_ReadPtr(lockStorage);
+      before = (MXUserRWLock *) Atomic_ReadIfEqualWritePtr(lockStorage, NULL,
+                                                           (void *) lock);
+
+      if (before) {
+         MXUser_DestroyRWLock(lock);
+
+         lock = before;
       }
    }
 
