@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2009-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2009 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -23,13 +23,22 @@
 #include "hostinfo.h"
 #include "ulInt.h"
 
+#define MXUSER_REC_SIGNATURE 0x43524B4C // 'LKRC' in memory
+
 struct MXUserRecLock
 {
-   MXUserHeader         header;
-   MXRecLock            recursiveLock;
-   Atomic_Ptr           heldStatsMem;
-   Atomic_Ptr           acquireStatsMem;
-   Atomic_uint32        refCount;
+   MXUserHeader            header;
+   MXRecLock               recursiveLock;
+
+   uint64                  holdStart;
+
+#if defined(MXUSER_STATS)
+   MXUserAcquisitionStats  acquisitionStats;
+   Atomic_Ptr              acquisitionHisto;
+
+   MXUserBasicStats        heldStats;
+   Atomic_Ptr              heldHisto;
+#endif
 
    /*
     * This is the MX recursive lock override pointer. It is used within the
@@ -42,10 +51,10 @@ struct MXUserRecLock
     *         MXUser_BindMXMutexRec was used to create the lock
     */
 
-   struct MX_MutexRec  *vmmLock;
+   struct MX_MutexRec     *vmmLock;
 };
 
-
+#if defined(MXUSER_STATS)
 /*
  *-----------------------------------------------------------------------------
  *
@@ -65,231 +74,51 @@ struct MXUserRecLock
 static void
 MXUserStatsActionRec(MXUserHeader *header)  // IN:
 {
+   Bool isHot;
+   Bool doLog;
+   double contentionRatio;
+
    MXUserRecLock *lock = (MXUserRecLock *) header;
-   MXUserHeldStats *heldStats = Atomic_ReadPtr(&lock->heldStatsMem);
-   MXUserAcquireStats *acquireStats = Atomic_ReadPtr(&lock->acquireStatsMem);
 
-   if (UNLIKELY(heldStats != NULL)) {
-      MXUserDumpBasicStats(&heldStats->data, header);
+   /*
+    * Dump the statistics for the specified lock.
+    */
 
-      if (Atomic_ReadPtr(&heldStats->histo) != NULL) {
-         MXUserHistoDump(Atomic_ReadPtr(&heldStats->histo), header);
-      }
+   MXUserDumpAcquisitionStats(&lock->acquisitionStats, header);
+
+   if (Atomic_ReadPtr(&lock->acquisitionHisto) != NULL) {
+      MXUserHistoDump(Atomic_ReadPtr(&lock->acquisitionHisto), header);
    }
 
-   if (LIKELY(acquireStats != NULL)) {
-      Bool isHot;
-      Bool doLog;
-      double contentionRatio;
+   MXUserDumpBasicStats(&lock->heldStats, header);
 
-      /*
-       * Dump the statistics for the specified lock.
-       */
+   if (Atomic_ReadPtr(&lock->heldHisto) != NULL) {
+      MXUserHistoDump(Atomic_ReadPtr(&lock->heldHisto), header);
+   }
 
-      MXUserDumpAcquisitionStats(&acquireStats->data, header);
+   /*
+    * Has the lock gone "hot"? If so, implement the hot actions.
+    */
 
-      if (Atomic_ReadPtr(&acquireStats->histo) != NULL) {
-         MXUserHistoDump(Atomic_ReadPtr(&acquireStats->histo), header);
-      }
+   MXUserKitchen(&lock->acquisitionStats, &contentionRatio, &isHot, &doLog);
 
-      /*
-       * Has the lock gone "hot"? If so, implement the hot actions.
-       */
+   if (isHot) {
+      MXUserForceHisto(&lock->acquisitionHisto,
+                       MXUSER_STAT_CLASS_ACQUISITION,
+                       MXUSER_DEFAULT_HISTO_MIN_VALUE_NS,
+                       MXUSER_DEFAULT_HISTO_DECADES);
+      MXUserForceHisto(&lock->heldHisto,
+                       MXUSER_STAT_CLASS_HELD,
+                       MXUSER_DEFAULT_HISTO_MIN_VALUE_NS,
+                       MXUSER_DEFAULT_HISTO_DECADES);
 
-      MXUserKitchen(&acquireStats->data, &contentionRatio, &isHot, &doLog);
-
-      if (UNLIKELY(isHot)) {
-         MXUserForceAcquisitionHisto(&lock->acquireStatsMem,
-                                     MXUSER_DEFAULT_HISTO_MIN_VALUE_NS,
-                                     MXUSER_DEFAULT_HISTO_DECADES);
-
-         if (UNLIKELY(heldStats != NULL)) {
-            MXUserForceHeldHisto(&lock->heldStatsMem,
-                                 MXUSER_DEFAULT_HISTO_MIN_VALUE_NS,
-                                 MXUSER_DEFAULT_HISTO_DECADES);
-         }
-
-         if (doLog) {
-            Log("HOT LOCK (%s); contention ratio %f\n",
-                lock->header.name, contentionRatio);
-         }
+      if (doLog) {
+         Log("HOT LOCK (%s); contention ratio %f\n",
+             lock->header.name, contentionRatio);
       }
    }
 }
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * MXUser_EnableStatsRecLock
- *
- *      Enable basic stats on the specified lock.
- *
- * Results:
- *      TRUE    succeeded
- *      FALSE   failed
- *
- * Side effects:
- *      None
- *
- *-----------------------------------------------------------------------------
- */
-
-Bool
-MXUser_EnableStatsRecLock(MXUserRecLock *lock,        // IN/OUT:
-                          Bool trackAcquisitionTime,  // IN:
-                          Bool trackHeldTime)         // IN:
-{
-   ASSERT(lock);
-   MXUserValidateHeader(&lock->header, MXUSER_TYPE_REC);
-
-   if (vmx86_stats) {
-      MXUserEnableStats(trackAcquisitionTime ? &lock->acquireStatsMem : NULL,
-                        trackHeldTime        ? &lock->heldStatsMem    : NULL);
-   }
-
-   return vmx86_stats;
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * MXUser_DisableStatsRecLock --
- *
- *      Disable basic stats on the specified lock.
- *
- * Results:
- *      TRUE    succeeded
- *      FALSE   failed
- *
- * Side effects:
- *      None
- *
- *-----------------------------------------------------------------------------
- */
-
-Bool
-MXUser_DisableStatsRecLock(MXUserRecLock *lock)  // IN/OUT:
-{
-   ASSERT(lock);
-   MXUserValidateHeader(&lock->header, MXUSER_TYPE_REC);
-
-   if (vmx86_stats) {
-      MXUserDisableStats(&lock->acquireStatsMem, &lock->heldStatsMem);
-   }
-
-   return vmx86_stats;
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * MXUser_SetContentionRatioFloorRecLock --
- *
- *      Set the contention ratio floor for the specified lock.
- *
- * Results:
- *      TRUE    succeeded
- *      FALSE   failed
- *
- * Side effects:
- *      None
- *
- *-----------------------------------------------------------------------------
- */
-
-Bool
-MXUser_SetContentionRatioFloorRecLock(MXUserRecLock *lock,  // IN/OUT:
-                                      double ratio)         // IN:
-{
-   Bool result;
-
-   ASSERT(lock);
-   MXUserValidateHeader(&lock->header, MXUSER_TYPE_REC);
-
-   if (vmx86_stats) {
-      result = MXUserSetContentionRatioFloor(&lock->acquireStatsMem, ratio);
-   } else {
-      result = FALSE;
-   }
-
-   return result;
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * MXUser_SetContentionCountFloorRecLock --
- *
- *      Set the contention count floor for the specified lock.
- *
- * Results:
- *      TRUE    succeeded
- *      FALSE   failed
- *
- * Side effects:
- *      None
- *
- *-----------------------------------------------------------------------------
- */
-
-Bool
-MXUser_SetContentionCountFloorRecLock(MXUserRecLock *lock,  // IN/OUT:
-                                      uint64 count)         // IN:
-{
-   Bool result;
-
-   ASSERT(lock);
-   MXUserValidateHeader(&lock->header, MXUSER_TYPE_REC);
-
-   if (vmx86_stats) {
-      result = MXUserSetContentionCountFloor(&lock->acquireStatsMem, count);
-   } else {
-      result = FALSE;
-   }
-
-   return result;
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * MXUser_SetContentionDurationFloorRecLock --
- *
- *      Set the contention count floor for the specified lock.
- *
- * Results:
- *      TRUE    succeeded
- *      FALSE   failed
- *
- * Side effects:
- *      None
- *
- *-----------------------------------------------------------------------------
- */
-
-Bool
-MXUser_SetContentionDurationFloorRecLock(MXUserRecLock *lock,  // IN/OUT:
-                                         uint64 duration)      // IN:
-{
-   Bool result;
-
-   ASSERT(lock);
-   MXUserValidateHeader(&lock->header, MXUSER_TYPE_REC);
-
-   if (vmx86_stats) {
-      result = MXUserSetContentionDurationFloor(&lock->acquireStatsMem,
-                                                duration);
-   } else {
-      result = FALSE;
-   }
-
-   return result;
-}
+#endif
 
 
 /*
@@ -297,7 +126,7 @@ MXUser_SetContentionDurationFloorRecLock(MXUserRecLock *lock,  // IN/OUT:
  *
  * MXUserDumpRecLock --
  *
- *      Dump a recursive lock.
+ *      Dump an recursive lock.
  *
  * Results:
  *      A dump.
@@ -313,21 +142,21 @@ MXUserDumpRecLock(MXUserHeader *header)  // IN:
 {
    MXUserRecLock *lock = (MXUserRecLock *) header;
 
-   Warning("%s: Recursive lock @ %p\n", __FUNCTION__, lock);
+   Warning("%s: Recursive lock @ 0x%p\n", __FUNCTION__, lock);
 
    Warning("\tsignature 0x%X\n", lock->header.signature);
    Warning("\tname %s\n", lock->header.name);
    Warning("\trank 0x%X\n", lock->header.rank);
-   Warning("\tserial number %u\n", lock->header.bits.serialNumber);
-   Warning("\treference count %u\n", Atomic_Read(&lock->refCount));
 
    if (lock->vmmLock == NULL) {
-      Warning("\tlock count %d\n", MXRecLockCount(&lock->recursiveLock));
+      Warning("\tcount %u\n", lock->recursiveLock.referenceCount);
 
-      Warning("\taddress of owner data %p\n",
-              &lock->recursiveLock.nativeThreadID);
+#if defined(MXUSER_DEBUG)
+      Warning("\tcaller 0x%p\n", lock->recursiveLock.ownerRetAddr);
+      Warning("\tVThreadID %d\n", (int) lock->recursiveLock.portableThreadID);
+#endif
    } else {
-      Warning("\tvmmLock %p\n", lock->vmmLock);
+      Warning("\tvmmLock 0x%p\n", lock->vmmLock);
    }
 }
 
@@ -337,8 +166,7 @@ MXUserDumpRecLock(MXUserHeader *header)  // IN:
  *
  * MXUser_CreateRecLock --
  *
- *      Create a recursive lock specifying if the lock must always be
- *      silent.
+ *      Create a recursive lock.
  *
  *      Only the owner (thread) of a recursive lock may recurse on it.
  *
@@ -356,9 +184,10 @@ MXUserRecLock *
 MXUser_CreateRecLock(const char *userName,  // IN:
                      MX_Rank rank)          // IN:
 {
-   uint32 statsMode;
    char *properName;
-   MXUserRecLock *lock = Util_SafeCalloc(1, sizeof *lock);
+   MXUserRecLock *lock;
+
+   lock = Util_SafeCalloc(1, sizeof(*lock));
 
    if (userName == NULL) {
       properName = Str_SafeAsprintf(NULL, "R-%p", GetReturnAddress());
@@ -374,37 +203,20 @@ MXUser_CreateRecLock(const char *userName,  // IN:
    }
 
    lock->vmmLock = NULL;
-   Atomic_Write(&lock->refCount, 1);
 
-   lock->header.signature = MXUserGetSignature(MXUSER_TYPE_REC);
    lock->header.name = properName;
+   lock->header.signature = MXUSER_REC_SIGNATURE;
    lock->header.rank = rank;
-   lock->header.bits.serialNumber = MXUserAllocSerialNumber();
    lock->header.dumpFunc = MXUserDumpRecLock;
 
-   statsMode = MXUserStatsMode();
-
-   switch (statsMode) {
-   case 0:
-      MXUserDisableStats(&lock->acquireStatsMem, &lock->heldStatsMem);
-      lock->header.statsFunc = NULL;
-      break;
-
-   case 1:
-      MXUserEnableStats(&lock->acquireStatsMem, NULL);
-      lock->header.statsFunc = MXUserStatsActionRec;
-      break;
-
-   case 2:
-      MXUserEnableStats(&lock->acquireStatsMem, &lock->heldStatsMem);
-      lock->header.statsFunc = MXUserStatsActionRec;
-      break;
-
-   default:
-      Panic("%s: unknown stats mode: %d!\n", __FUNCTION__, statsMode);
-   }
+#if defined(MXUSER_STATS)
+   lock->header.statsFunc = MXUserStatsActionRec;
+   lock->header.identifier = MXUserAllocID();
 
    MXUserAddToList(&lock->header);
+   MXUserAcquisitionStatsSetUp(&lock->acquisitionStats);
+   MXUserBasicStatsSetUp(&lock->heldStats, MXUSER_STAT_CLASS_HELD);
+#endif
 
    return lock;
 }
@@ -413,17 +225,16 @@ MXUser_CreateRecLock(const char *userName,  // IN:
 /*
  *-----------------------------------------------------------------------------
  *
- * MXUserCondDestroyRecLock --
+ * MXUser_DestroyRecLock --
  *
- *      Destroy a recursive lock -- but only if its reference count is zero.
+ *      Destroy a recursive lock.
  *
  *      When the lock is bound to a MX lock, only the MXUser "wrapper" is
  *      freed. The caller is responsible for calling MX_DestroyLockRec() on
- *      the MX lock before calling this routine.
+ *      the MX lock.
  *
  * Results:
- *      Lock is destroyed upon correct reference count. Don't use the
- *      pointer again.
+ *      Lock is destroyed. Don't use the pointer again.
  *
  * Side effects:
  *      None
@@ -431,14 +242,12 @@ MXUser_CreateRecLock(const char *userName,  // IN:
  *-----------------------------------------------------------------------------
  */
 
-static void
-MXUserCondDestroyRecLock(MXUserRecLock *lock)  // IN:
+void
+MXUser_DestroyRecLock(MXUserRecLock *lock)  // IN:
 {
-   ASSERT(lock);
-   MXUserValidateHeader(&lock->header, MXUSER_TYPE_REC);
-   ASSERT(Atomic_Read(&lock->refCount) > 0);
+   if (lock != NULL) {
+      ASSERT(lock->header.signature == MXUSER_REC_SIGNATURE);
 
-   if (Atomic_ReadDec32(&lock->refCount) == 1) {
       if (lock->vmmLock == NULL) {
          if (MXRecLockCount(&lock->recursiveLock) > 0) {
             MXUserDumpAndPanic(&lock->header,
@@ -448,28 +257,22 @@ MXUserCondDestroyRecLock(MXUserRecLock *lock)  // IN:
 
          MXRecLockDestroy(&lock->recursiveLock);
 
+#if defined(MXUSER_STATS)
          MXUserRemoveFromList(&lock->header);
-
-         if (vmx86_stats) {
-            MXUserDisableStats(&lock->acquireStatsMem, &lock->heldStatsMem);
-         }
+         MXUserAcquisitionStatsTearDown(&lock->acquisitionStats);
+         MXUserBasicStatsTearDown(&lock->heldStats);
+         MXUserHistoTearDown(Atomic_ReadPtr(&lock->acquisitionHisto));
+         MXUserHistoTearDown(Atomic_ReadPtr(&lock->heldHisto));
+#endif
       }
 
       lock->header.signature = 0;  // just in case...
-      free(lock->header.name);
+      free((void *) lock->header.name);  // avoid const warnings
       lock->header.name = NULL;
       free(lock);
    }
 }
 
-void
-MXUser_DestroyRecLock(MXUserRecLock *lock)  // IN:
-{
-   if (lock != NULL) {
-      MXUserCondDestroyRecLock(lock);
-   }
-}
-
 
 /*
  *-----------------------------------------------------------------------------
@@ -492,52 +295,43 @@ MXUser_DestroyRecLock(MXUserRecLock *lock)  // IN:
 void
 MXUser_AcquireRecLock(MXUserRecLock *lock)  // IN/OUT:
 {
-   ASSERT(lock);
-   MXUserValidateHeader(&lock->header, MXUSER_TYPE_REC);
-   ASSERT(Atomic_Read(&lock->refCount) > 0);
+   ASSERT(lock && (lock->header.signature == MXUSER_REC_SIGNATURE));
 
-   if (UNLIKELY(lock->vmmLock != NULL)) {
+   if (lock->vmmLock) {
       ASSERT(MXUserMX_LockRec);
       (*MXUserMX_LockRec)(lock->vmmLock);
    } else {
+#if defined(MXUSER_STATS)
+      Bool contended;
+      VmTimeType begin;
+#endif
       /* Rank checking is only done on the first acquisition */
       MXUserAcquisitionTracking(&lock->header, TRUE);
 
-      if (vmx86_stats) {
-         VmTimeType value = 0;
-         MXUserAcquireStats *acquireStats;
+#if defined(MXUSER_STATS)
+      begin = Hostinfo_SystemTimerNS();
 
-         acquireStats = Atomic_ReadPtr(&lock->acquireStatsMem);
+      contended =
+#endif
 
-         MXRecLockAcquire(&lock->recursiveLock,
-                          (acquireStats == NULL) ? NULL : &value);
+      MXRecLockAcquire(&lock->recursiveLock, GetReturnAddress());
 
-         if (LIKELY(acquireStats != NULL)) {
-            if (MXRecLockCount(&lock->recursiveLock) == 1) {
-               MXUserHeldStats *heldStats;
-               MXUserHisto *histo;
+#if defined(MXUSER_STATS)
+      if (MXRecLockCount(&lock->recursiveLock) == 1) {
+         MXUserHisto *histo;
+         VmTimeType value = Hostinfo_SystemTimerNS() - begin;
 
-               MXUserAcquisitionSample(&acquireStats->data, TRUE,
-                            value > acquireStats->data.contentionDurationFloor,
-                                       value);
+         MXUserAcquisitionSample(&lock->acquisitionStats, contended, value);
 
-               histo = Atomic_ReadPtr(&acquireStats->histo);
+         histo = Atomic_ReadPtr(&lock->acquisitionHisto);
 
-               if (UNLIKELY(histo != NULL)) {
-                  MXUserHistoSample(histo, value, GetReturnAddress());
-               }
-
-               heldStats = Atomic_ReadPtr(&lock->heldStatsMem);
-
-               if (UNLIKELY(heldStats != NULL)) {
-                  heldStats->holdStart = Hostinfo_SystemTimerNS();
-               }
-            }
+         if (UNLIKELY(histo != NULL)) {
+            MXUserHistoSample(histo, value);
          }
-      } else {
-         MXRecLockAcquire(&lock->recursiveLock,
-                          NULL);  // non-stats
+
+         lock->holdStart = Hostinfo_SystemTimerNS();
       }
+#endif
    }
 }
 
@@ -561,51 +355,32 @@ MXUser_AcquireRecLock(MXUserRecLock *lock)  // IN/OUT:
 void
 MXUser_ReleaseRecLock(MXUserRecLock *lock)  // IN/OUT:
 {
-   ASSERT(lock);
-   MXUserValidateHeader(&lock->header, MXUSER_TYPE_REC);
-   ASSERT(Atomic_Read(&lock->refCount) > 0);
+   ASSERT(lock && (lock->header.signature == MXUSER_REC_SIGNATURE));
 
-   if (UNLIKELY(lock->vmmLock != NULL)) {
+   if (lock->vmmLock) {
       ASSERT(MXUserMX_UnlockRec);
       (*MXUserMX_UnlockRec)(lock->vmmLock);
    } else {
-      if (vmx86_stats) {
-         MXUserHeldStats *heldStats = Atomic_ReadPtr(&lock->heldStatsMem);
+#if defined(MXUSER_STATS)
+      if (MXRecLockCount(&lock->recursiveLock) == 1) {
+         VmTimeType value = Hostinfo_SystemTimerNS() - lock->holdStart;
+         MXUserHisto *histo = Atomic_ReadPtr(&lock->heldHisto);
 
-         if (LIKELY(heldStats != NULL)) {
-            if (MXRecLockCount(&lock->recursiveLock) == 1) {
-               MXUserHeldStats *heldStats;
+         MXUserBasicStatsSample(&lock->heldStats, value);
 
-               heldStats = Atomic_ReadPtr(&lock->heldStatsMem);
-
-               if (UNLIKELY(heldStats != NULL)) {
-                  VmTimeType value;
-                  MXUserHisto *histo = Atomic_ReadPtr(&heldStats->histo);
-
-                  value = Hostinfo_SystemTimerNS() - heldStats->holdStart;
-
-                  MXUserBasicStatsSample(&heldStats->data, value);
-
-                  if (UNLIKELY(histo != NULL)) {
-                     MXUserHistoSample(histo, value, GetReturnAddress());
-                  }
-               }
-            }
+         if (UNLIKELY(histo != NULL)) {
+            MXUserHistoSample(histo, value);
          }
       }
+#endif
 
-      if (vmx86_debug) {
-         if (MXRecLockCount(&lock->recursiveLock) == 0) {
-            MXUserDumpAndPanic(&lock->header,
-                               "%s: Release of an unacquired recursive lock\n",
-                                __FUNCTION__);
-         }
+      if (!MXRecLockIsOwner(&lock->recursiveLock)) {
+         uint32 lockCount = MXRecLockCount(&lock->recursiveLock);
 
-         if (!MXRecLockIsOwner(&lock->recursiveLock)) {
-            MXUserDumpAndPanic(&lock->header,
-                               "%s: Non-owner release of an recursive lock\n",
-                               __FUNCTION__);
-         }
+         MXUserDumpAndPanic(&lock->header,
+                            "%s: Non-owner release of an %s recursive lock\n",
+                            __FUNCTION__,
+                            lockCount == 0 ? "unacquired" : "acquired");
       }
 
       MXUserReleaseTracking(&lock->header);
@@ -643,38 +418,38 @@ MXUser_TryAcquireRecLock(MXUserRecLock *lock)  // IN/OUT:
 {
    Bool success;
 
-   ASSERT(lock);
-   MXUserValidateHeader(&lock->header, MXUSER_TYPE_REC);
-   ASSERT(Atomic_Read(&lock->refCount) > 0);
+   ASSERT(lock && (lock->header.signature == MXUSER_REC_SIGNATURE));
 
-   if (UNLIKELY(lock->vmmLock != NULL)) {
+   if (lock->vmmLock) {
       ASSERT(MXUserMX_TryLockRec);
       success = (*MXUserMX_TryLockRec)(lock->vmmLock);
    } else {
+#if defined(MXUSER_STATS)
+      VmTimeType begin;
+#endif
+
       if (MXUserTryAcquireFail(lock->header.name)) {
-         success = FALSE;
-         goto bail;
+         return FALSE;
       }
 
-      success = MXRecLockTryAcquire(&lock->recursiveLock);
+#if defined(MXUSER_STATS)
+      begin = Hostinfo_SystemTimerNS();
+#endif
+
+      success = MXRecLockTryAcquire(&lock->recursiveLock, GetReturnAddress());
 
       if (success) {
-         MXUserAcquisitionTracking(&lock->header, FALSE);
-      }
-
-      if (vmx86_stats) {
-         MXUserAcquireStats *acquireStats;
-
-         acquireStats = Atomic_ReadPtr(&lock->acquireStatsMem);
-
-         if (LIKELY(acquireStats != NULL)) {
-            MXUserAcquisitionSample(&acquireStats->data, success, !success,
-                                    0ULL);
+#if defined(MXUSER_STATS)
+         if (MXRecLockCount(&lock->recursiveLock) == 1) {
+            MXUserAcquisitionSample(&lock->acquisitionStats, FALSE,
+                                  Hostinfo_SystemTimerNS() - begin);
          }
+#endif
+
+         MXUserAcquisitionTracking(&lock->header, FALSE);
       }
    }
 
-bail:
    return success;
 }
 
@@ -697,15 +472,12 @@ bail:
  */
 
 Bool
-MXUser_IsCurThreadHoldingRecLock(MXUserRecLock *lock)  // IN:
+MXUser_IsCurThreadHoldingRecLock(const MXUserRecLock *lock)  // IN:
 {
    Bool result;
+   ASSERT(lock && (lock->header.signature == MXUSER_REC_SIGNATURE));
 
-   ASSERT(lock);
-   MXUserValidateHeader(&lock->header, MXUSER_TYPE_REC);
-   ASSERT(Atomic_Read(&lock->refCount) > 0);
-
-   if (UNLIKELY(lock->vmmLock != NULL)) {
+   if (lock->vmmLock) {
       ASSERT(MXUserMX_IsLockedByCurThreadRec);
       result = (*MXUserMX_IsLockedByCurThreadRec)(lock->vmmLock);
    } else {
@@ -716,6 +488,78 @@ MXUser_IsCurThreadHoldingRecLock(MXUserRecLock *lock)  // IN:
 }
 
 
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * MXUser_ControlRecLock --
+ *
+ *      Perform the specified command on the specified lock.
+ *
+ * Results:
+ *      TRUE    succeeded
+ *      FALSE   failed
+ *
+ * Side effects:
+ *      Depends on the command, no?
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+Bool
+MXUser_ControlRecLock(MXUserRecLock *lock,  // IN/OUT:
+                      uint32 command,       // IN:
+                      ...)                  // IN:
+{
+   Bool result;
+
+   ASSERT(lock && (lock->header.signature == MXUSER_REC_SIGNATURE));
+   ASSERT(lock->vmmLock == NULL);  // only unbound locks
+
+   switch (command) {
+#if defined(MXUSER_STATS)
+   case MXUSER_CONTROL_ACQUISITION_HISTO: {
+      va_list a;
+      uint64 minValue;
+      uint32 decades;
+
+      va_start(a, command);
+      minValue = va_arg(a, uint64);
+      decades = va_arg(a, uint32);
+      va_end(a);
+
+      MXUserForceHisto(&lock->acquisitionHisto, MXUSER_STAT_CLASS_ACQUISITION,
+                       minValue, decades);
+
+      result = TRUE;
+      break;
+   }
+
+   case MXUSER_CONTROL_HELD_HISTO: {
+      va_list a;
+      uint64 minValue;
+      uint32 decades;
+
+      va_start(a, command);
+      minValue = va_arg(a, uint64);
+      decades = va_arg(a, uint32);
+      va_end(a);
+
+      MXUserForceHisto(&lock->heldHisto, MXUSER_STAT_CLASS_HELD,
+                       minValue, decades);
+      
+      result = TRUE;
+      break; 
+   }
+#endif
+
+   default:
+      result = FALSE;
+   }
+
+   return result;
+}
+
+
 /*
  *-----------------------------------------------------------------------------
  *
@@ -744,17 +588,20 @@ MXUser_CreateSingletonRecLock(Atomic_Ptr *lockStorage,  // IN/OUT:
 
    ASSERT(lockStorage);
 
-   lock = Atomic_ReadPtr(lockStorage);
+   lock = (MXUserRecLock *) Atomic_ReadPtr(lockStorage);
 
    if (UNLIKELY(lock == NULL)) {
-      MXUserRecLock *newLock = MXUser_CreateRecLock(name, rank);
+      MXUserRecLock *before;
 
-      lock = Atomic_ReadIfEqualWritePtr(lockStorage, NULL, (void *) newLock);
+      lock = MXUser_CreateRecLock(name, rank);
 
-      if (lock) {
-         MXUser_DestroyRecLock(newLock);
-      } else {
-         lock = Atomic_ReadPtr(lockStorage);
+      before = (MXUserRecLock *) Atomic_ReadIfEqualWritePtr(lockStorage, NULL,
+                                                            (void *) lock);
+
+      if (before) {
+         MXUser_DestroyRecLock(lock);
+
+         lock = before;
       }
    }
 
@@ -782,15 +629,10 @@ MXUser_CreateSingletonRecLock(Atomic_Ptr *lockStorage,  // IN/OUT:
 MXUserCondVar *
 MXUser_CreateCondVarRecLock(MXUserRecLock *lock)
 {
-   MXUserCondVar *condVar;
-
-   ASSERT(lock);
-   MXUserValidateHeader(&lock->header, MXUSER_TYPE_REC);
+   ASSERT(lock && (lock->header.signature == MXUSER_REC_SIGNATURE));
    ASSERT(lock->vmmLock == NULL);  // only unbound locks
 
-   condVar =  MXUserCreateCondVar(&lock->header, &lock->recursiveLock);
-
-   return condVar;
+   return MXUserCreateCondVar(&lock->header, &lock->recursiveLock);
 }
 
 
@@ -807,8 +649,7 @@ MXUser_CreateCondVarRecLock(MXUserRecLock *lock)
  *      As above.
  *
  * Side effects:
- *      It is possible to return from this routine without the condtion
- *      variable having been signalled (spurious wake up); code accordingly!
+ *      None.
  *
  *-----------------------------------------------------------------------------
  */
@@ -817,8 +658,7 @@ void
 MXUser_WaitCondVarRecLock(MXUserRecLock *lock,     // IN:
                           MXUserCondVar *condVar)  // IN:
 {
-   ASSERT(lock);
-   MXUserValidateHeader(&lock->header, MXUSER_TYPE_REC);
+   ASSERT(lock && (lock->header.signature == MXUSER_REC_SIGNATURE));
    ASSERT(lock->vmmLock == NULL);  // only unbound locks
 
    MXUserWaitCondVar(&lock->header, &lock->recursiveLock, condVar,
@@ -836,51 +676,25 @@ MXUser_WaitCondVarRecLock(MXUserRecLock *lock,     // IN:
  *      blocking and is reacquired before returning from this function.
  *
  * Results:
- *      As above
+ *      TRUE   condVar was signalled
+ *      FALSE  timed out waiting for signal
  *
  * Side effects:
- *      It is possible to return from this routine without the condtion
- *      variable having been signalled (spurious wake up); code accordingly!
+ *      None.
  *
  *-----------------------------------------------------------------------------
  */
 
-void
+Bool
 MXUser_TimedWaitCondVarRecLock(MXUserRecLock *lock,     // IN:
                                MXUserCondVar *condVar,  // IN:
                                uint32 msecWait)         // IN:
 {
-   ASSERT(lock);
-   MXUserValidateHeader(&lock->header, MXUSER_TYPE_REC);
+   ASSERT(lock && (lock->header.signature == MXUSER_REC_SIGNATURE));
    ASSERT(lock->vmmLock == NULL);  // only unbound locks
 
-   MXUserWaitCondVar(&lock->header, &lock->recursiveLock, condVar, msecWait);
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * MXUser_DumpRecLock --
- *
- *      Dump a recursive lock.
- *
- * Results:
- *      None.
- *
- * Side effects:
- *      None.
- *
- *-----------------------------------------------------------------------------
- */
-
-void
-MXUser_DumpRecLock(MXUserRecLock *lock)  // IN:
-{
-   ASSERT(lock);
-   MXUserValidateHeader(&lock->header, MXUSER_TYPE_REC);
-
-   MXUserDumpRecLock(&lock->header);
+   return MXUserWaitCondVar(&lock->header, &lock->recursiveLock, condVar,
+                            msecWait);
 }
 
 
@@ -901,10 +715,9 @@ MXUser_DumpRecLock(MXUserRecLock *lock)  // IN:
  */
 
 struct MX_MutexRec *
-MXUser_GetRecLockVmm(MXUserRecLock *lock)  // IN:
+MXUser_GetRecLockVmm(const MXUserRecLock *lock)  // IN:
 {
-   ASSERT(lock);
-   MXUserValidateHeader(&lock->header, MXUSER_TYPE_REC);
+   ASSERT(lock && (lock->header.signature == MXUSER_REC_SIGNATURE));
 
    return lock->vmmLock;
 }
@@ -927,10 +740,9 @@ MXUser_GetRecLockVmm(MXUserRecLock *lock)  // IN:
  */
 
 MX_Rank
-MXUser_GetRecLockRank(MXUserRecLock *lock)  // IN:
+MXUser_GetRecLockRank(const MXUserRecLock *lock)  // IN:
 {
-   ASSERT(lock);
-   MXUserValidateHeader(&lock->header, MXUSER_TYPE_REC);
+   ASSERT(lock && (lock->header.signature == MXUSER_REC_SIGNATURE));
 
    return lock->header.rank;
 }
@@ -958,7 +770,6 @@ MXUserRecLock *
 MXUser_BindMXMutexRec(struct MX_MutexRec *mutex,  // IN:
                       MX_Rank rank)               // IN:
 {
-   char *name;
    MXUserRecLock *lock;
 
    ASSERT(mutex);
@@ -971,8 +782,7 @@ MXUser_BindMXMutexRec(struct MX_MutexRec *mutex,  // IN:
    if ((MXUserMX_LockRec == NULL) ||
        (MXUserMX_UnlockRec == NULL) ||
        (MXUserMX_TryLockRec == NULL) ||
-       (MXUserMX_IsLockedByCurThreadRec == NULL) ||
-       (MXUserMX_NameRec == NULL)) {
+       (MXUserMX_IsLockedByCurThreadRec == NULL)) {
        return NULL;
     }
 
@@ -982,26 +792,18 @@ MXUser_BindMXMutexRec(struct MX_MutexRec *mutex,  // IN:
     * system will take care of this.
     */
 
-   lock = Util_SafeCalloc(1, sizeof *lock);
+   lock = Util_SafeCalloc(1, sizeof(*lock));
 
-   lock->header.signature = MXUserGetSignature(MXUSER_TYPE_REC);
-
-   name = (*MXUserMX_NameRec)(mutex);
-
-   if (name == NULL) {
-      lock->header.name = Str_SafeAsprintf(NULL, "MX_%p", mutex);
-   } else {
-      lock->header.name = Str_SafeAsprintf(NULL, "%s *", name);
-   }
+   lock->header.name = Str_SafeAsprintf(NULL, "MX_%p", mutex);
 
+   lock->header.signature = MXUSER_REC_SIGNATURE;
    lock->header.rank = rank;
-   lock->header.bits.serialNumber = MXUserAllocSerialNumber();
    lock->header.dumpFunc = NULL;
-   lock->header.statsFunc = NULL;
 
-   Atomic_WritePtr(&lock->acquireStatsMem, NULL);
-   Atomic_WritePtr(&lock->heldStatsMem, NULL);
-   Atomic_Write(&lock->refCount, 1);
+#if defined(MXUSER_STATS)
+   lock->header.statsFunc = NULL;
+   lock->header.identifier = MXUserAllocID();
+#endif
 
    lock->vmmLock = mutex;
 
@@ -1009,56 +811,41 @@ MXUser_BindMXMutexRec(struct MX_MutexRec *mutex,  // IN:
 }
 
 
-/*
- *-----------------------------------------------------------------------------
- *
- * MXUser_IncRefRecLock --
- *
- *      Add a reference to the lock to prevent an immediate destory from
- *      succeeding.
- *
- * Results:
- *      As above
- *
- * Side effects:
- *      None
- *
- *-----------------------------------------------------------------------------
- */
-
-void
-MXUser_IncRefRecLock(MXUserRecLock *lock)  // IN:
-{
-   ASSERT(lock);
-   MXUserValidateHeader(&lock->header, MXUSER_TYPE_REC);
-   ASSERT(Atomic_Read(&lock->refCount) > 0);
-
-   Atomic_Inc(&lock->refCount);
-}
-
+#if defined(VMX86_VMX)
+#include "mutex.h"
+#include "mutexRank.h"
 
 /*
- *-----------------------------------------------------------------------------
+ *----------------------------------------------------------------------------
  *
- * MXUser_DecRefRecLock --
+ * MXUser_InitFromMXRec --
  *
- *      Remove a reference to the lock. If the reference count is zero,
- *      the lock is destroyed.
+ *      Initialize a MX_MutexRec lock and create a MXUserRecLock that binds
+ *      to it.
  *
  * Results:
- *      As above
+ *      Pointer to the MXUserRecLock.
  *
  * Side effects:
- *      None
+ *      None.
  *
- *-----------------------------------------------------------------------------
+ *----------------------------------------------------------------------------
  */
 
-void
-MXUser_DecRefRecLock(MXUserRecLock *lock)  // IN:
+MXUserRecLock *
+MXUser_InitFromMXRec(const char *name,    // IN:
+                     MX_MutexRec *mutex,  // IN:
+                     MX_Rank rank,        // IN:
+                     Bool isBelowBull)    // IN:
 {
-   ASSERT(lock);
-   MXUserValidateHeader(&lock->header, MXUSER_TYPE_REC);
+   MXUserRecLock *userLock;
 
-   MXUserCondDestroyRecLock(lock);
+   ASSERT(isBelowBull == (rank < RANK_userlevelLock));
+
+   MX_InitLockRec(name, rank, mutex);
+   userLock = MXUser_BindMXMutexRec(mutex, rank);
+   ASSERT(userLock);
+
+   return userLock;
 }
+#endif
