@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2010-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2010 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -20,13 +20,16 @@
 #include <windows.h>
 #else
 #if defined(__APPLE__)
-#include <dispatch/dispatch.h>
+#include <mach/mach_init.h>
+#include <mach/task.h>
+#include <mach/semaphore.h>
 #else
 #if (_XOPEN_SOURCE < 600) && !defined(__FreeBSD__) && !defined(sun)
 #undef _XOPEN_SOURCE
 #define _XOPEN_SOURCE 600
 #endif
 #include <semaphore.h>
+#include <time.h>
 #include <sys/time.h>
 #endif
 #endif
@@ -36,24 +39,21 @@
 #include "util.h"
 #include "userlock.h"
 #include "ulInt.h"
-#include "hostinfo.h"
 #if defined(_WIN32)
 #include "win32u.h"
 #endif
+#if defined(__APPLE__)
+#include "hostinfo.h"
+#endif
 
+#define MXUSER_SEMA_SIGNATURE 0x414D4553 // 'SEMA' in memory
 #define MXUSER_A_BILLION (1000 * 1000 * 1000)
 
 #if defined(_WIN32)
 typedef HANDLE NativeSemaphore;
 #else
 #if defined(__APPLE__)
-/*
- * The Mac OS implementation uses dispatch_semaphore_t instead of
- * semaphore_t due to better performance in the uncontended case, by
- * avoiding a system call.  It, also, avoids weird error cases that
- * resulted in bug 916600.
- */
-typedef dispatch_semaphore_t NativeSemaphore;
+typedef semaphore_t NativeSemaphore;
 #else
 typedef sem_t NativeSemaphore;
 #endif
@@ -63,8 +63,8 @@ struct MXUserSemaphore
 {
    MXUserHeader     header;
    Atomic_uint32    activeUserCount;
+
    NativeSemaphore  nativeSemaphore;
-   Atomic_Ptr       acquireStatsMem;
 };
 
 
@@ -99,7 +99,7 @@ struct MXUserSemaphore
 
 #if defined(_WIN32)
 static int
-MXUserInit(NativeSemaphore *sema)  // OUT:
+MXUserInit(NativeSemaphore *sema)  // IN:
 {
    *sema = Win32U_CreateSemaphore(NULL, 0, INT_MAX, NULL);
 
@@ -122,7 +122,7 @@ MXUserTimedDown(NativeSemaphore *sema,  // IN:
 
     status = WaitForSingleObject(*sema, msecWait);
 
-    switch (status) {
+    switch (status) { 
        case WAIT_OBJECT_0:  // The down (decrement) occurred
           *downOccurred = TRUE;
           err = 0;
@@ -180,20 +180,17 @@ MXUserUp(NativeSemaphore *sema)  // IN:
 {
    return ReleaseSemaphore(*sema, 1, NULL) ? 0 : GetLastError();
 }
-
 #elif defined(__APPLE__)
 static int
-MXUserInit(NativeSemaphore *sema)  // OUT:
+MXUserInit(NativeSemaphore *sema)  // IN:
 {
-   *sema = dispatch_semaphore_create(0);
-   return *sema == NULL;
+   return semaphore_create(mach_task_self(), sema, SYNC_POLICY_FIFO, 0);
 }
 
 static int
 MXUserDestroy(NativeSemaphore *sema)  // IN:
 {
-   dispatch_release(*sema);
-   return 0;
+   return semaphore_destroy(mach_task_self(), *sema);
 }
 
 static int
@@ -201,17 +198,63 @@ MXUserTimedDown(NativeSemaphore *sema,  // IN:
                 uint32 msecWait,        // IN:
                 Bool *downOccurred)     // OUT:
 {
-   int64 nsecWait = 1000000LL * (int64)msecWait;
-   dispatch_time_t deadline = dispatch_time(DISPATCH_TIME_NOW, nsecWait);
-   *downOccurred = dispatch_semaphore_wait(*sema, deadline) == 0;
-   return 0;
+   uint64 nsecWait;
+   VmTimeType before;
+   kern_return_t err;
+
+   ASSERT_ON_COMPILE(KERN_SUCCESS == 0);
+
+   /*
+    * Work in nanoseconds. Time the semaphore_timedwait operation in case
+    * it is interrupted (KERN_ABORT). If it is, determine how much time is
+    * necessary to fulfill the specified wait time and retry with a new
+    * and appropriate timeout.
+    */
+
+   nsecWait = 1000000ULL * (uint64) msecWait;
+   before = Hostinfo_SystemTimerNS();
+
+   do {
+      VmTimeType after;
+      mach_timespec_t ts;
+
+      ts.tv_sec = nsecWait / MXUSER_A_BILLION;
+      ts.tv_nsec = nsecWait % MXUSER_A_BILLION;
+
+      err = semaphore_timedwait(*sema, ts);
+      after = Hostinfo_SystemTimerNS();
+
+      if (err == KERN_SUCCESS) {
+         *downOccurred = TRUE;
+      } else {
+         *downOccurred = FALSE;
+
+         if (err == KERN_OPERATION_TIMED_OUT) {
+            /* Really timed out; no down occurred, no error */
+            err = KERN_SUCCESS;
+         } else {
+            if (err == KERN_ABORTED) {
+               VmTimeType duration = after - before;
+
+               if (duration < nsecWait) {
+                  nsecWait -= duration;
+
+                  before = after;
+               } else {
+                  err = KERN_SUCCESS;  // "timed out" anyway... no error
+               }
+            }
+         }
+      }
+   } while (nsecWait && (err == KERN_ABORTED));
+
+   return err;
 }
 
 static int
 MXUserDown(NativeSemaphore *sema)  // IN:
 {
-   dispatch_semaphore_wait(*sema, DISPATCH_TIME_FOREVER);
-   return 0;
+   return semaphore_wait(*sema);
 }
 
 static int
@@ -219,22 +262,24 @@ MXUserTryDown(NativeSemaphore *sema,  // IN:
               Bool *downOccurred)     // OUT:
 {
    /*
-    * Provide 'try' semantics by requesting an immediate timeout.
+    * Use a wait for zero time to implement the try operation. This timed
+    * down will either succeed immediately (down occurred), fail (something
+    * terrible happened) or time out immediately (the down could not be
+    * performed and that is OK).
     */
-   *downOccurred = dispatch_semaphore_wait(*sema, DISPATCH_TIME_NOW) == 0;
-   return 0;
+
+   return MXUserTimedDown(sema, 0, downOccurred);
 }
 
 static int
 MXUserUp(NativeSemaphore *sema)  // IN:
 {
-    dispatch_semaphore_signal(*sema);
-    return 0;
+    return semaphore_signal(*sema);
 }
-
 #else
+
 static int
-MXUserInit(NativeSemaphore *sema)  // OUT:
+MXUserInit(NativeSemaphore *sema)  // IN:
 {
    return (sem_init(sema, 0, 0) == -1) ? errno : 0;
 }
@@ -312,7 +357,7 @@ MXUserTryDown(NativeSemaphore *sema,  // IN:
       *downOccurred = FALSE;
 
       /*
-       * If the error that occurred indicates that the try operation cannot
+       * If the error that occured indicates that the try operation cannot
        * succeed (EAGAIN) or was interrupted (EINTR) suppress the error
        * indicator as these are considered "normal", non-error cases.
        *
@@ -335,63 +380,6 @@ MXUserUp(NativeSemaphore *sema)  // IN:
 #endif  // _WIN32
 
 
-/*
- *-----------------------------------------------------------------------------
- *
- * MXUserStatsActionSema --
- *
- *      Perform the statistics action for the specified semaphore.
- *
- * Results:
- *      As above.
- *
- * Side effects:
- *      None
- *
- *-----------------------------------------------------------------------------
- */
-
-static void
-MXUserStatsActionSema(MXUserHeader *header)  // IN:
-{
-   MXUserSemaphore *sema = (MXUserSemaphore *) header;
-   MXUserAcquireStats *acquireStats = Atomic_ReadPtr(&sema->acquireStatsMem);
-
-   if (LIKELY(acquireStats != NULL)) {
-      Bool isHot;
-      Bool doLog;
-      double contentionRatio;
-
-      /*
-       * Dump the statistics for the specified semaphore.
-       */
-
-      MXUserDumpAcquisitionStats(&acquireStats->data, header);
-
-      if (Atomic_ReadPtr(&acquireStats->histo) != NULL) {
-         MXUserHistoDump(Atomic_ReadPtr(&acquireStats->histo), header);
-      }
-
-      /*
-       * Has the semaphore gone "hot"? If so, implement the hot actions.
-       */
-
-      MXUserKitchen(&acquireStats->data, &contentionRatio, &isHot, &doLog);
-
-      if (isHot) {
-         MXUserForceAcquisitionHisto(&sema->acquireStatsMem,
-                                     MXUSER_DEFAULT_HISTO_MIN_VALUE_NS,
-                                     MXUSER_DEFAULT_HISTO_DECADES);
-
-         if (doLog) {
-            Log("HOT SEMAPHORE (%s); contention ratio %f\n",
-                sema->header.name, contentionRatio);
-         }
-      }
-   }
-}
-
-
 /*
  *-----------------------------------------------------------------------------
  *
@@ -413,15 +401,13 @@ MXUserDumpSemaphore(MXUserHeader *header)  // IN:
 {
    MXUserSemaphore *sema = (MXUserSemaphore *) header;
 
-   Warning("%s: semaphore @ %p\n", __FUNCTION__, sema);
+   Warning("%s: semaphore @ 0x%p\n", __FUNCTION__, sema);
 
    Warning("\tsignature 0x%X\n", sema->header.signature);
    Warning("\tname %s\n", sema->header.name);
    Warning("\trank 0x%X\n", sema->header.rank);
-   Warning("\tserial number %u\n", sema->header.bits.serialNumber);
-
    Warning("\treference count %u\n", Atomic_Read(&sema->activeUserCount));
-   Warning("\taddress of native semaphore %p\n", &sema->nativeSemaphore);
+   Warning("\tnative semaphore 0x%p\n", &sema->nativeSemaphore);
 }
 
 
@@ -450,7 +436,9 @@ MXUser_CreateSemaphore(const char *userName,  // IN:
                        MX_Rank rank)          // IN:
 {
    char *properName;
-   MXUserSemaphore *sema = Util_SafeCalloc(1, sizeof *sema);
+   MXUserSemaphore *sema;
+
+   sema = Util_SafeCalloc(1, sizeof(*sema));
 
    if (userName == NULL) {
       properName = Str_SafeAsprintf(NULL, "Sema-%p", GetReturnAddress());
@@ -459,33 +447,18 @@ MXUser_CreateSemaphore(const char *userName,  // IN:
    }
 
    if (LIKELY(MXUserInit(&sema->nativeSemaphore) == 0)) {
-      uint32 statsMode;
-
-      sema->header.signature = MXUserGetSignature(MXUSER_TYPE_SEMA);
       sema->header.name = properName;
+      sema->header.signature = MXUSER_SEMA_SIGNATURE;
       sema->header.rank = rank;
-      sema->header.bits.serialNumber = MXUserAllocSerialNumber();
       sema->header.dumpFunc = MXUserDumpSemaphore;
 
-      statsMode = MXUserStatsMode();
-
-      switch (MXUserStatsMode()) {
-      case 0:
-         MXUserDisableStats(&sema->acquireStatsMem, NULL);
-         sema->header.statsFunc = NULL;
-         break;
-
-      case 1:
-      case 2:
-         MXUserEnableStats(&sema->acquireStatsMem, NULL);
-         sema->header.statsFunc = MXUserStatsActionSema;
-         break;
-
-      default:
-         Panic("%s: unknown stats mode: %d!\n", __FUNCTION__, statsMode);
-      }
+#if defined(MXUSER_STATS)
+      sema->header.statsFunc = NULL;
+      sema->header.identifier = MXUserAllocID();
 
       MXUserAddToList(&sema->header);
+#endif
+
    } else {
       free(properName);
       free(sema);
@@ -513,12 +486,12 @@ MXUser_CreateSemaphore(const char *userName,  // IN:
  */
 
 void
-MXUser_DestroySemaphore(MXUserSemaphore *sema)  // IN/OUT:
+MXUser_DestroySemaphore(MXUserSemaphore *sema)  // IN:
 {
    if (LIKELY(sema != NULL)) {
       int err;
 
-      MXUserValidateHeader(&sema->header, MXUSER_TYPE_SEMA);
+      ASSERT(sema->header.signature == MXUSER_SEMA_SIGNATURE);
 
       if (Atomic_Read(&sema->activeUserCount) != 0) {
          MXUserDumpAndPanic(&sema->header,
@@ -526,8 +499,6 @@ MXUser_DestroySemaphore(MXUserSemaphore *sema)  // IN/OUT:
                             __FUNCTION__);
       }
 
-      sema->header.signature = 0;  // just in case...
-
       err = MXUserDestroy(&sema->nativeSemaphore);
 
       if (UNLIKELY(err != 0)) {
@@ -535,22 +506,12 @@ MXUser_DestroySemaphore(MXUserSemaphore *sema)  // IN/OUT:
                             __FUNCTION__, err);
       }
 
+#if defined(MXUSER_STATS)
       MXUserRemoveFromList(&sema->header);
+#endif
 
-      if (vmx86_stats) {
-         MXUserAcquireStats *acquireStats;
-
-         acquireStats = Atomic_ReadPtr(&sema->acquireStatsMem);
-
-         if (LIKELY(acquireStats != NULL)) {
-            MXUserAcquisitionStatsTearDown(&acquireStats->data);
-            MXUserHistoTearDown(Atomic_ReadPtr(&acquireStats->histo));
-
-            free(acquireStats);
-         }
-      }
-
-      free(sema->header.name);
+      sema->header.signature = 0;  // just in case...
+      free((void *) sema->header.name);  // avoid const warnings
       sema->header.name = NULL;
       free(sema);
    }
@@ -562,11 +523,11 @@ MXUser_DestroySemaphore(MXUserSemaphore *sema)  // IN/OUT:
  *
  * MXUser_DownSemaphore --
  *
- *      Perform a down (P; probeer te verlagen; "try to reduce") operation
+ *      Perform an down (P; probeer te verlagen; "try to reduce") operation
  *      on a semaphore.
  *
  * Results:
- *      The count will be decremented; a sleep may occur until the decrement
+ *      The count will be decremented; a sleep may occur until the decement
  *      is possible.
  *
  * Side effects:
@@ -580,48 +541,13 @@ MXUser_DownSemaphore(MXUserSemaphore *sema)  // IN/OUT:
 {
    int err;
 
-   ASSERT(sema);
-   MXUserValidateHeader(&sema->header, MXUSER_TYPE_SEMA);
+   ASSERT(sema && (sema->header.signature == MXUSER_SEMA_SIGNATURE));
 
    Atomic_Inc(&sema->activeUserCount);
 
    MXUserAcquisitionTracking(&sema->header, TRUE);  // rank checking
 
-   if (vmx86_stats) {
-      VmTimeType start = 0;
-      Bool tryDownSuccess = FALSE;
-      MXUserAcquireStats *acquireStats;
-
-      acquireStats = Atomic_ReadPtr(&sema->acquireStatsMem);
-
-      if (LIKELY(acquireStats != NULL)) {
-         start = Hostinfo_SystemTimerNS();
-      }
-
-      err = MXUserTryDown(&sema->nativeSemaphore, &tryDownSuccess);
-
-      if (LIKELY(err == 0)) {
-         if (!tryDownSuccess) {
-            err = MXUserDown(&sema->nativeSemaphore);
-         }
-      }
-
-      if (LIKELY((err == 0) && (acquireStats != NULL))) {
-         MXUserHisto *histo;
-         VmTimeType value = Hostinfo_SystemTimerNS() - start;
-
-         MXUserAcquisitionSample(&acquireStats->data, TRUE,
-                                 !tryDownSuccess, value);
-
-         histo = Atomic_ReadPtr(&acquireStats->histo);
-
-         if (UNLIKELY(histo != NULL)) {
-            MXUserHistoSample(histo, value, GetReturnAddress());
-         }
-      }
-   } else {
-      err = MXUserDown(&sema->nativeSemaphore);
-   }
+   err = MXUserDown(&sema->nativeSemaphore);
 
    if (UNLIKELY(err != 0)) {
       MXUserDumpAndPanic(&sema->header, "%s: Internal error (%d)\n",
@@ -639,7 +565,7 @@ MXUser_DownSemaphore(MXUserSemaphore *sema)  // IN/OUT:
  *
  * MXUser_TimedDownSemaphore --
  *
- *      Perform a down (P; probeer te verlagen; "try to reduce") operation
+ *      Perform an down (P; probeer te verlagen; "try to reduce") operation
  *      on a semaphore with a timeout. The wait time will always have elapsed
  *      before the routine returns.
  *
@@ -660,52 +586,13 @@ MXUser_TimedDownSemaphore(MXUserSemaphore *sema,  // IN/OUT:
    int err;
    Bool downOccurred = FALSE;
 
-   ASSERT(sema);
-   MXUserValidateHeader(&sema->header, MXUSER_TYPE_SEMA);
+   ASSERT(sema && (sema->header.signature == MXUSER_SEMA_SIGNATURE));
 
    Atomic_Inc(&sema->activeUserCount);
 
    MXUserAcquisitionTracking(&sema->header, TRUE);  // rank checking
 
-   if (vmx86_stats) {
-      VmTimeType start = 0;
-      Bool tryDownSuccess = FALSE;
-      MXUserAcquireStats *acquireStats;
-
-      acquireStats = Atomic_ReadPtr(&sema->acquireStatsMem);
-
-      if (LIKELY(acquireStats != NULL)) {
-         start = Hostinfo_SystemTimerNS();
-      }
-
-      err = MXUserTryDown(&sema->nativeSemaphore, &tryDownSuccess);
-
-      if (LIKELY(err == 0)) {
-         if (tryDownSuccess) {
-            downOccurred = TRUE;
-         } else {
-            err = MXUserTimedDown(&sema->nativeSemaphore, msecWait,
-                                  &downOccurred);
-         }
-      }
-
-      if (LIKELY((err == 0) && (acquireStats != NULL))) {
-         VmTimeType value = Hostinfo_SystemTimerNS() - start;
-
-         MXUserAcquisitionSample(&acquireStats->data, downOccurred,
-                                 !tryDownSuccess, value);
-
-         if (downOccurred) {
-            MXUserHisto *histo = Atomic_ReadPtr(&acquireStats->histo);
-
-            if (UNLIKELY(histo != NULL)) {
-               MXUserHistoSample(histo, value, GetReturnAddress());
-            }
-         }
-      }
-   } else {
-      err = MXUserTimedDown(&sema->nativeSemaphore, msecWait, &downOccurred);
-   }
+   err = MXUserTimedDown(&sema->nativeSemaphore, msecWait, &downOccurred);
 
    if (UNLIKELY(err != 0)) {
       MXUserDumpAndPanic(&sema->header, "%s: Internal error (%d)\n",
@@ -725,7 +612,7 @@ MXUser_TimedDownSemaphore(MXUserSemaphore *sema,  // IN/OUT:
  *
  * MXUser_TryDownSemaphore --
  *
- *      Perform a try down (P; probeer te verlagen; "try to reduce") operation
+ *      Perform an try down (P; probeer te verlagen; "try to reduce") operation
  *      on a semaphore.
  *
  * Results:
@@ -737,7 +624,7 @@ MXUser_TimedDownSemaphore(MXUserSemaphore *sema,  // IN/OUT:
  *
  * NOTE:
  *      A "TryAcquire" does not rank check should the down operation succeed.
- *      This duplicates the behavior of MX semaphores.
+ *      This duplicates the behavor of MX semaphores.
  *
  *-----------------------------------------------------------------------------
  */
@@ -748,8 +635,7 @@ MXUser_TryDownSemaphore(MXUserSemaphore *sema)  // IN/OUT:
    int err;
    Bool downOccurred = FALSE;
 
-   ASSERT(sema);
-   MXUserValidateHeader(&sema->header, MXUSER_TYPE_SEMA);
+   ASSERT(sema && (sema->header.signature == MXUSER_SEMA_SIGNATURE));
 
    Atomic_Inc(&sema->activeUserCount);
 
@@ -760,17 +646,6 @@ MXUser_TryDownSemaphore(MXUserSemaphore *sema)  // IN/OUT:
                          __FUNCTION__, err);
    }
 
-   if (vmx86_stats) {
-      MXUserAcquireStats *acquireStats;
-
-      acquireStats = Atomic_ReadPtr(&sema->acquireStatsMem);
-
-      if (LIKELY(acquireStats != NULL)) {
-         MXUserAcquisitionSample(&acquireStats->data, downOccurred,
-                                 !downOccurred, 0ULL);
-      }
-   }
-
    Atomic_Dec(&sema->activeUserCount);
 
    return downOccurred;
@@ -799,17 +674,9 @@ MXUser_UpSemaphore(MXUserSemaphore *sema)  // IN/OUT:
 {
    int err;
 
-   ASSERT(sema);
-   MXUserValidateHeader(&sema->header, MXUSER_TYPE_SEMA);
+   ASSERT(sema && (sema->header.signature == MXUSER_SEMA_SIGNATURE));
 
-   /*
-    * The purpose of the activeUserCount tracking is to help catch potentially
-    * fatal cases of destroying an active semaphore; it is not expected to be
-    * (nor it can be) perfect (with low overhead). In this case the time spent
-    * in up is tiny and a decrement at the bottom might not be reached before
-    * another thread comes out of down and does a destroy - so no
-    * activeUserCount tracking here.
-    */
+   Atomic_Inc(&sema->activeUserCount);
 
    err = MXUserUp(&sema->nativeSemaphore);
 
@@ -817,6 +684,8 @@ MXUser_UpSemaphore(MXUserSemaphore *sema)  // IN/OUT:
       MXUserDumpAndPanic(&sema->header, "%s: Internal error (%d)\n",
                          __FUNCTION__, err);
    }
+
+   Atomic_Dec(&sema->activeUserCount);
 }
 
 
@@ -849,17 +718,21 @@ MXUser_CreateSingletonSemaphore(Atomic_Ptr *semaStorage,  // IN/OUT:
 
    ASSERT(semaStorage);
 
-   sema = Atomic_ReadPtr(semaStorage);
+   sema = (MXUserSemaphore *) Atomic_ReadPtr(semaStorage);
 
    if (UNLIKELY(sema == NULL)) {
-      MXUserSemaphore *newSema = MXUser_CreateSemaphore(name, rank);
+      MXUserSemaphore *before;
 
-      sema = Atomic_ReadIfEqualWritePtr(semaStorage, NULL, (void *) newSema);
+      sema = MXUser_CreateSemaphore(name, rank);
 
-      if (sema) {
-         MXUser_DestroySemaphore(newSema);
-      } else {
-         sema = Atomic_ReadPtr(semaStorage);
+      before = (MXUserSemaphore *) Atomic_ReadIfEqualWritePtr(semaStorage,
+                                                              NULL,
+                                                              (void *) sema);
+
+      if (before) {
+         MXUser_DestroySemaphore(sema);
+
+         sema = before;
       }
    }
 
