@@ -182,25 +182,6 @@ strcasecmp(const char *s1,  // IN:
    
    return tolower(*s1) - tolower(*s2);
 }
-
-static int
-ffs(uint32 bits)
-{
-   uint32 i;
-
-   if (bits == 0) {
-      i = 0;
-   } else {
-      i = 1;
-
-      while ((bits & 0x1) == 0) {
-         i++;
-         bits >>= 1;
-      }
-   }
-
-   return i;
-}
 #endif
 
 
@@ -275,7 +256,7 @@ HashTable_Alloc(uint32 numEntries,        // IN: must be a power of 2
    ht = Util_SafeMalloc(sizeof *ht);
    ASSERT_MEM_ALLOC(ht);
 
-   ht->numBits = ffs(numEntries) - 1;
+   ht->numBits = lssb32_0(numEntries);
    ht->numEntries = numEntries;
    ht->keyType = keyType & HASH_TYPE_MASK;
    ht->atomic = (keyType & HASH_FLAG_ATOMIC) != 0;
@@ -829,6 +810,62 @@ HashTable_GetNumElements(const HashTable *ht)  // IN:
 }
 
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * HashTable_KeyArray --
+ *
+ *      Returns an array of pointers to each key in the hash table.
+ *
+ * Results:
+ *      The key array plus its size. If the hash table is empty
+ *      'size' is 0 and 'keys' is NULL. Free 'keys' array with free(). Do *not*
+ *      free the individual elements of the array.
+ *
+ * Side effects:
+ *      None.
+ *
+ *----------------------------------------------------------------------
+ */
+
+void
+HashTable_KeyArray(const HashTable *ht,  // IN:
+                   const void ***keys,   // OUT:
+                   size_t *size)         // OUT:
+{
+   uint32 i;
+   size_t j;
+
+   ASSERT(ht);
+   ASSERT(keys);
+   ASSERT(size);
+
+   ASSERT(!ht->atomic);
+
+   *keys = NULL;
+   *size = HashTable_GetNumElements(ht);
+
+   if (0 == *size) {
+      return;
+   }
+
+   /* alloc array */
+   *keys = Util_SafeMalloc(*size * sizeof **keys);
+
+   /* fill array */
+   for (i = 0, j = 0; i < ht->numEntries; i++) {
+      HashTableEntry *entry;
+
+      for (entry = ENTRY(ht->buckets[i]);
+           entry != NULL;
+           entry = ENTRY(entry->next)) {
+         (*keys)[j++] = entry->keyStr;
+      }
+   }
+}
+
+
+
 /*
  *----------------------------------------------------------------------
  *
