@@ -171,7 +171,7 @@ System_Uptime(void)
  *      Returns the current time as a formatted string, useful for prepending
  *      to debugging output.
  *
- *      For example: "Oct 05 18:03:24.948: "
+ *      For example: "Oct 05 18:03:24.948"
  *
  * Results:
  *      On success, allocates and returns a string containing the formatted
@@ -230,7 +230,7 @@ System_GetTimeAsString(void)
    if (dateTime == NULL) {
       goto out;
    }
-   output = Unicode_Format("%s.%03d: ", dateTime, msec);
+   output = Unicode_Format("%s.%03d", dateTime, msec);
 
   out:
    free(buf);
@@ -239,32 +239,6 @@ System_GetTimeAsString(void)
 }
 
 
-/*
- *----------------------------------------------------------------------
- *
- * System_IsACPI --
- *
- *    Is this an ACPI system?
- *
- * Results:
- *    TRUE if this is an ACPI system.
- *    FALSE if this is not an ACPI system.
- *
- * Side effects:
- *	None.
- *
- *----------------------------------------------------------------------
- */
-
-Bool
-System_IsACPI(void)
-{
-   ASSERT(FALSE);
-
-   return FALSE;
-}
-
-
 /*
  *-----------------------------------------------------------------------------
  *
@@ -401,113 +375,6 @@ System_SetEnv(Bool global,      // IN
 } // System_SetEnv
 
 
-/*
- *----------------------------------------------------------------------
- *
- * System_UnsetEnv --
- *
- *    Unset environment variable.
- *
- * Results:
- *    0 if success, -1 otherwise.
- *
- * Side effects:
- *    Unsets the environment variable.
- *
- *----------------------------------------------------------------------
- */
-
-int
-System_UnsetEnv(const char *valueName) // IN: UTF-8
-{
-#if defined(sun)
-   return(-1);
-#else
-   Posix_Unsetenv(valueName);
-   return 0;
-#endif
-} // System_UnsetEnv
-
-
-/*
- *----------------------------------------------------------------------
- *
- * System_SetLDPath --
- *
- *    Set LD_LIBRARY_PATH. If native is TRUE, use VMWARE_LD_LIBRARY_PATH
- *    as the value (and ignore the path argument, which should be set to
- *    NULL in this case). If native is FALSE, use the passed in path (and
- *    if that path is NULL, unsetenv the value).
- *
- * Results:
- *    The previous value of the environment variable. The caller is
- *    responsible for calling free() on this pointer.
- *
- * Side effects:
- *    Manipulates the value of LD_LIBRARY_PATH variable.
- *
- *----------------------------------------------------------------------
- */
-
-char *
-System_SetLDPath(const char *path,      // IN: UTF-8
-                 const Bool native)     // IN: If TRUE, ignore path and use VMware value
-{
-   char *vmldpath = NULL;
-   char *tmppath = NULL;
-   char *oldpath;
-
-   ASSERT(!native || path == NULL);
-
-   /*
-    * Get the original LD_LIBRARY_PATH, so the installed applications
-    * don't try to use our versions of the libraries.
-    *
-    * From bora/apps/lib/lui/browser.cc::ResetEnvVars():
-    *
-    * For each variable we care about, there are three possible states:
-    * 1) The variable was not considered for saving.
-    *    This is indicated by VMWARE_<foo>'s first character not being
-    *    set to either "1" or "0".
-    *    We should not manipulate <foo>.
-    * 2) The variable was considered but was not set.
-    *    VMWARE_<foo> is set to "0".
-    *    We should unset <foo>.
-    * 3) The variable was considered and was set.
-    *    VMWARE_<foo> is set to "1" + "<value of foo>".
-    *    We should set <foo> back to the saved value.
-    *
-    * XXX: There are other variables that may need resetting (for a list, check
-    * bora/apps/lib/lui/browser.cc or the wrapper scripts), but that would
-    * require some more refactoring of this code (not using system(3), for
-    * example).
-    */
-   oldpath = System_GetEnv(TRUE, "LD_LIBRARY_PATH");
-   if (native == TRUE) {
-      char *p = NULL;
-      vmldpath = tmppath = System_GetEnv(TRUE, "VMWARE_LD_LIBRARY_PATH");
-      if (vmldpath && strlen(vmldpath) && vmldpath[0] == '1') {
-         vmldpath++;
-      } else {
-         vmldpath = p = Util_SafeStrdup("");
-      }
-      if (System_SetEnv(TRUE, "LD_LIBRARY_PATH", vmldpath) == -1) {
-         Debug("%s: failed to set LD_LIBRARY_PATH\n", __FUNCTION__);
-      }
-      free(tmppath);
-      free(p);
-   } else if (path) {
-      /*
-       * Set LD_LIBRARY_PATH to the specified value.
-       */
-      System_SetEnv(TRUE, "LD_LIBRARY_PATH", (char *) path);
-   } else {
-      System_UnsetEnv("LD_LIBRARY_PATH");
-   }
-   return oldpath;
-} // System_SetLDPath
-
-
 /*
  *-----------------------------------------------------------------------------
  *
