@@ -78,7 +78,8 @@ VMToolsLogOutputDebugString(const gchar *domain,
                             gpointer _data);
 #endif
 
-typedef LogHandlerData * (*LogHandlerConfigFn)(const gchar *domain,
+typedef LogHandlerData * (*LogHandlerConfigFn)(const gchar *defaultDomain,
+                                               const gchar *domain,
                                                const gchar *name,
                                                GKeyFile *cfg);
 
@@ -99,10 +100,12 @@ static LogHandler gHandlers[] = {
    { 0,  "std",               VMStdLoggerConfig },
    { 1,  "file",              VMFileLoggerConfig },
    { 2,  "file+",             VMFileLoggerConfig },
+   { 3,  "vmx",               VMXLoggerConfig },
 #if defined(_WIN32)
-   { 3,  "outputdebugstring", VMDebugOutputConfig },
+   { 4,  "outputdebugstring", VMDebugOutputConfig },
    { -1, NULL,                VMDebugOutputConfig },
 #else
+   { 4,  "syslog",            VMSysLoggerConfig },
    { -1, NULL,                VMStdLoggerConfig },
 #endif
 };
@@ -129,7 +132,7 @@ static GPtrArray *gDomains = NULL;
  * @return Number of bytes printed.
  */
 
-static INLINE gint
+gint
 VMToolsAsprintf(gchar **string,
                 gchar const *format,
                 ...)
@@ -232,23 +235,21 @@ VMToolsLogFormat(const gchar *message,
    }
 
    if (msg != NULL && data->convertToLocal) {
-      size_t msgCurrLen;
-      gchar *msgCurr = g_locale_from_utf8(msg, strlen(msg), NULL, &msgCurrLen, NULL);
-
-      /*
-       * The log messages from glib itself (and probably other libraries based
-       * on glib) do not include a trailing new line. Most of our code does. So
-       * we detect whether the original message already had a new line, and
-       * remove it, to avoid having two newlines when printing our log messages.
-       */
-      if (msgCurr != NULL && msgCurr[msgCurrLen - 2] == '\n') {
-         msgCurr[msgCurrLen - 1] = '\0';
-      }
-
+      gchar *msgCurr = g_locale_from_utf8(msg, len, NULL, &len, NULL);
       g_free(msg);
       msg = msgCurr;
    }
 
+   /*
+    * The log messages from glib itself (and probably other libraries based
+    * on glib) do not include a trailing new line. Most of our code does. So
+    * we detect whether the original message already had a new line, and
+    * remove it, to avoid having two newlines when printing our log messages.
+    */
+   if (msg != NULL && msg[len - 2] == '\n') {
+      msg[len - 1] = '\0';
+   }
+
    return msg;
 }
 
@@ -425,7 +426,7 @@ VMToolsConfigLogDomain(const gchar *domain,
          goto exit;
       }
 
-      data = configfn(domain, handler, cfg);
+      data = configfn(gLogDomain, domain, handler, cfg);
    } else if (strcmp(domain, gLogDomain) == 0) {
       /*
        * If no handler defined and we're configuring the default domain,
@@ -433,7 +434,7 @@ VMToolsConfigLogDomain(const gchar *domain,
        */
       hid = DEFAULT_HANDLER->id;
       configfn = DEFAULT_HANDLER->configfn;
-      data = configfn(domain, NULL, cfg);
+      data = configfn(gLogDomain, domain, NULL, cfg);
       ASSERT(data != NULL);
    } else {
       /* An inherited handler. Just create a dummy instance. */
@@ -739,7 +740,7 @@ VMTools_ConfigLogging(const gchar *defaultDomain,
    }
 
    gLogDomain = g_strdup(defaultDomain);
-   gErrorData = DEFAULT_HANDLER->configfn(gLogDomain, NULL, NULL);
+   gErrorData = DEFAULT_HANDLER->configfn(gLogDomain, gLogDomain, NULL, NULL);
 
    /*
     * Configure the default domain first. See function documentation for
@@ -776,6 +777,14 @@ VMTools_ConfigLogging(const gchar *defaultDomain,
                                                "enableCoreDump", NULL);
    }
 
+   /* If needed, restore the old configuration. */
+   if (!reset) {
+      VMToolsRestoreLogging(oldDefault, oldDomains);
+      if (oldDomains != NULL) {
+         g_ptr_array_free(oldDomains, TRUE);
+      }
+   }
+
    /*
     * If core dumps are enabled (default: TRUE), then set up the exception
     * filter on Win32. On POSIX systems, try to modify the resource limit
@@ -825,14 +834,6 @@ VMTools_ConfigLogging(const gchar *defaultDomain,
 #endif
    }
 
-   /* If needed, restore the old configuration. */
-   if (!reset) {
-      VMToolsRestoreLogging(oldDefault, oldDomains);
-      if (oldDomains != NULL) {
-         g_ptr_array_free(oldDomains, TRUE);
-      }
-   }
-
    gLogEnabled |= force;
 
    if (allocDict) {
