@@ -23,16 +23,19 @@
  *      and thread synchronization routines.
  */
 
-#if !defined(_KERNEL)
+#if !defined _KERNEL
 #  error "This os.c file can only be compiled for the FreeBSD kernel."
 #endif
 
+#include <sys/types.h>
 #include <sys/param.h>
 #include <sys/malloc.h>
 #include <sys/lock.h>         // for struct mtx
 #include <sys/kernel.h>
 #include <vm/uma.h>           // for uma_zone_t
 #include <sys/kthread.h>      // for kthread_create()
+#include <vm/vm.h>
+#include <vm/vm_extern.h>     // for vnode_pager_setsize
 
 #include "vm_basic_types.h"
 #include "os.h"
@@ -226,7 +229,7 @@ os_zone_free(OS_ZONE_T *zone, // IN
  *
  * os_malloc --
  *
- *      Malloc some memory in a FreeBSD / OS X kernel independent manner.
+ *      Malloc some memory in a FreeBSD / Mac OS kernel independent manner.
  *      This just calls the internal kernel malloc function. According to the
  *      FreeBSD commments, if M_WAITOK is passed to the flags, malloc will never
  *      return NULL.
@@ -253,7 +256,7 @@ os_malloc(size_t size, // IN
  *
  * os_free --
  *
- *      Free some memory in a FreeBSD / OS X kernel independent manner.
+ *      Free some memory in a FreeBSD / Mac OS kernel independent manner.
  *      This just calls the internal kernel free function.
  *
  * Results:
@@ -279,7 +282,7 @@ os_free(void *mem,   // IN
  * os_mutex_alloc_init --
  *
  *      Allocate and initialize a FreeBSD mutex in an OS independent way.
- *      Mtx_name is not used on OS X.
+ *      Mtx_name is not used on Mac OS.
  *
  * Results:
  *      A new mtx which has been allocated and is ready for use.
@@ -521,7 +524,7 @@ os_rw_lock_unlock_exclusive(OS_RWLOCK_T *lck) // IN
  *
  * os_cv_init --
  *
- *      Initialize a cv under FreeBSD. Under OS X, we are actually passed an
+ *      Initialize a cv under FreeBSD. Under Mac OS, we are actually passed an
  *      object address we will use in place of a cv in later functions. Here
  *      we simply do nothing.
  *
@@ -547,7 +550,7 @@ os_cv_init(OS_CV_T *cv,      // IN
  *
  * os_cv_destroy --
  *
- *      Destroy a cv under FreeBSD. Under OS X, we are actually passed an
+ *      Destroy a cv under FreeBSD. Under Mac OS, we are actually passed an
  *      object address we will use in place of a cv in later functions. Here
  *      we simply do nothing.
  *
@@ -571,7 +574,7 @@ os_cv_destroy(OS_CV_T *cv) // IN
  *
  * os_cv_signal --
  *
- *      Signal a thread to wakeup in a FreeBSD/OS X independent way.
+ *      Signal a thread to wakeup in a FreeBSD/Mac OS independent way.
  *
  * Results:
  *      None.
@@ -626,7 +629,7 @@ os_cv_wait(OS_CV_T *cv,      // IN
  *
  * os_thread_create --
  *
- *      Create an OS X or FreeBSD kernel thread in an OS independent way.
+ *      Create an Mac OS or FreeBSD kernel thread in an OS independent way.
  *
  * Results:
  *      None.
@@ -707,7 +710,7 @@ os_thread_release(OS_THREAD_T thread) // IN
  * Hgfsthreadexit --
  *
  *      Called when a thread is exiting. ErrorCode is returned as the thread exit code
- *      under FreeBSD and ignored under OS X.
+ *      under FreeBSD and ignored under Mac OS.
  *
  * Results:
  *      None.
@@ -861,3 +864,56 @@ os_path_to_utf8_precomposed(char const *bufIn,  // IN
    NOT_IMPLEMENTED();
    return OS_ERR;
 }
+
+
+/*
+ *----------------------------------------------------------------------------
+ *
+ * os_SetSize  --
+ *
+ *      Notifies memory management system that file size has been changed.
+ *
+ * Results:
+ *      None.
+ *
+ * Side effects:
+ *      None.
+ *----------------------------------------------------------------------------
+ */
+
+void
+os_SetSize(struct vnode* vp,          // IN: vnode which size has changed
+           off_t newSize)             // IN: new file size
+{
+   vnode_pager_setsize(vp, newSize);
+}
+
+
+/*
+ *----------------------------------------------------------------------------
+ *
+ * os_FlushRange  --
+ *
+ *      Flushes dirty pages associated with the file.
+ *
+ * Results:
+ *      Always retun 0 (success) for now since it is NOOP.
+ *
+ * Side effects:
+ *      None.
+ *----------------------------------------------------------------------------
+ */
+
+int
+os_FlushRange(struct vnode *vp,    // IN: vnode which data needs flushing
+              off_t start,         // IN: starting offset in the file to flush
+              uint32_t length)     // IN: length of data to flush
+{
+   /*
+    * XXX: NOOP for now. This routine is needed to maintain coherence
+    *      between memory mapped data and data for read/write operations.
+    *      Will need to implement when adding support for memory mapped files to HGFS
+    *      for FreeBsd.
+    */
+   return 0;
+}
