@@ -20,7 +20,7 @@
  * transport.c --
  *
  * Functions that prepare HGFS packages and send them to the host.
- * Implementations are shared between both OS X and FreeBSD.
+ * Implementations are shared between both Mac OS and FreeBSD.
  */
 
 #include <sys/param.h>          // for everything
@@ -90,7 +90,7 @@ HgfsSendOpenDirRequest(HgfsSuperInfo *sip,   // IN: Superinfo pointer
    request->reserved = 0;
 
    reqSize = HGFS_REQ_PAYLOAD_SIZE_V3(request);
-   reqBufferSize = HGFS_NAME_BUFFER_SIZET(reqSize);
+   reqBufferSize = HGFS_NAME_BUFFER_SIZET(HGFS_PACKET_MAX, reqSize);
 
    /*
     * Convert an input string to utf8 precomposed form, convert it to
@@ -190,7 +190,7 @@ HgfsSendOpenRequest(HgfsSuperInfo *sip,   // IN: Superinfo pointer
    request->reserved2 = 0;
 
    reqSize = HGFS_REQ_PAYLOAD_SIZE_V3(request);
-   reqBufferSize = HGFS_NAME_BUFFER_SIZET(reqSize);
+   reqBufferSize = HGFS_NAME_BUFFER_SIZET(HGFS_PACKET_MAX, reqSize);
    request->mode = openMode;
    request->flags = openFlags;
    DEBUG(VM_DEBUG_COMM, "open flags are %x\n", request->flags);
