@@ -229,7 +229,10 @@ int VMCIHost_ContextToHostVmID(VMCIHost *hostContext, VMCIHostVmID *hostVmID);
 void VMCIHost_SetActiveHnd(VMCIHost *hostContext, uintptr_t eventHnd);
 Bool VMCIHost_RemoveHnd(VMCIHost *hostContext, uintptr_t eventHnd);
 Bool VMCIHost_IsActiveHnd(VMCIHost *hostContext, uintptr_t eventHnd);
+void VMCIHost_SetInactiveHnd(VMCIHost *hostContext, uintptr_t eventHnd);
 uint32 VMCIHost_NumHnds(VMCIHost *hostContext);
+uintptr_t VMCIHost_GetActiveHnd(VMCIHost *hostContext);
+void VMCIHost_SignalBitmap(VMCIHost *hostContext);
 #endif
 
 void *VMCI_AllocKernelMem(size_t size, int flags);
@@ -303,18 +306,28 @@ void VMCI_FreePPNSet(PPNSet *ppnSet);
 int VMCI_PopulatePPNList(uint8 *callBuf, const PPNSet *ppnSet);
 #endif
 
-#if !defined(VMX86_TOOLS) && !defined(VMKERNEL)
-struct PageStoreAttachInfo;
+#if !defined(VMX86_TOOLS)
 struct VMCIQueue;
 
+#if  !defined(VMKERNEL)
+struct PageStoreAttachInfo;
+struct VMCIQueue *VMCIHost_AllocQueue(uint64 queueSize);
+void VMCIHost_FreeQueue(struct VMCIQueue *queue, uint64 queueSize);
+
 int VMCIHost_GetUserMemory(struct PageStoreAttachInfo *attach,
                            struct VMCIQueue *produceQ,
                            struct VMCIQueue *detachQ);
 void VMCIHost_ReleaseUserMemory(struct PageStoreAttachInfo *attach,
                                 struct VMCIQueue *produceQ,
                                 struct VMCIQueue *detachQ);
-
+#endif // VMKERNEL
 #ifdef _WIN32
+void VMCIHost_InitQueueMutex(struct VMCIQueue *produceQ,
+                             struct VMCIQueue *consumeQ);
+void VMCIHost_AcquireQueueMutex(struct VMCIQueue *queue);
+void VMCIHost_ReleaseQueueMutex(struct VMCIQueue *queue);
+Bool VMCIHost_EnqueueToDevNull(struct VMCIQueue *queue);
+
 /*
  * Special routine used on the Windows platform to save a queue when
  * its backing memory goes away.
@@ -324,8 +337,13 @@ void VMCIHost_SaveProduceQ(struct PageStoreAttachInfo *attach,
                            struct VMCIQueue *produceQ,
                            struct VMCIQueue *detachQ,
                            const uint64 produceQSize);
+#else // _WIN32
+#  define VMCIHost_InitQueueMutex(_pq, _cq)
+#  define VMCIHost_AcquireQueueMutex(_q)
+#  define VMCIHost_ReleaseQueueMutex(_q)
+#  define VMCIHost_EnqueueToDevNull(_q) FALSE
 #endif // _WIN32
-#endif // !VMX86_TOOLS && !VMKERNEL
+#endif // !VMX86_TOOLS
 
 
 #endif // _VMCI_KERNEL_IF_H_
