@@ -24,7 +24,6 @@
 
 #ifdef __linux__
 #  include "driver-config.h"
-#  define EXPORT_SYMTAB
 #  include <asm/page.h>
 #  include <linux/module.h>
 #elif defined(_WIN32)
@@ -35,7 +34,6 @@
 
 #include "vm_assert.h"
 #include "vmci_kernel_if.h"
-#include "vmci_queue_pair.h"
 #include "vmciQueuePairInt.h"
 #include "vmciUtil.h"
 #include "vmciInt.h"
@@ -235,7 +233,7 @@ VMCIQueuePair_Exit(void)
          detachMsg.hdr.src = VMCI_ANON_SRC_HANDLE;
          detachMsg.hdr.payloadSize = sizeof entry->handle;
          detachMsg.handle = entry->handle;
-         
+
          (void)VMCI_SendDatagram((VMCIDatagram *)&detachMsg);
       }
       /*
@@ -389,10 +387,6 @@ QueuePairList_GetHead(void)
  *-----------------------------------------------------------------------------
  */
 
-#ifdef __linux__
-EXPORT_SYMBOL(VMCIQueuePair_Alloc);
-#endif
-
 int
 VMCIQueuePair_Alloc(VMCIHandle *handle,     // IN/OUT:
                     VMCIQueue  **produceQ,  // OUT:
@@ -403,21 +397,8 @@ VMCIQueuePair_Alloc(VMCIHandle *handle,     // IN/OUT:
                     uint32     flags)       // IN:
 {
    ASSERT_ON_COMPILE(sizeof(VMCIQueueHeader) <= PAGE_SIZE);
-#  define VMCIQP_OFFSET_OF(Struct, field) ((uintptr_t)&(((Struct *)0)->field))
-#ifdef __linux__
-   ASSERT_ON_COMPILE(VMCIQP_OFFSET_OF(VMCIQueue, page) == PAGE_SIZE);
-#elif !defined(SOLARIS)
-   ASSERT_ON_COMPILE(VMCIQP_OFFSET_OF(VMCIQueue, buffer) == PAGE_SIZE);
-#endif
-#  undef VMCIQP_OFFSET_OF
 
-   if (!handle || !produceQ || !consumeQ || (!produceSize && !consumeSize) ||
-       (flags & ~VMCI_QP_ALL_FLAGS)) {
-      return VMCI_ERROR_INVALID_ARGS;
-   }
-
-   return VMCIQueuePairAllocHelper(handle, produceQ, produceSize, consumeQ,
-                                   consumeSize, peer, flags);
+   return VMCIQueuePair_AllocPriv(handle, produceQ, produceSize, consumeQ, consumeSize, peer, flags, VMCI_NO_PRIVILEGE_FLAGS);
 }
 
 
@@ -439,10 +420,6 @@ VMCIQueuePair_Alloc(VMCIHandle *handle,     // IN/OUT:
  *-----------------------------------------------------------------------------
  */
 
-#ifdef __linux__
-EXPORT_SYMBOL(VMCIQueuePair_AllocPriv);
-#endif
-
 int
 VMCIQueuePair_AllocPriv(VMCIHandle *handle,           // IN/OUT:
                         VMCIQueue  **produceQ,        // OUT:
@@ -453,7 +430,17 @@ VMCIQueuePair_AllocPriv(VMCIHandle *handle,           // IN/OUT:
                         uint32     flags,             // IN:
                         VMCIPrivilegeFlags privFlags) // IN:
 {
-   return VMCI_ERROR_NO_ACCESS;
+   if (privFlags != VMCI_NO_PRIVILEGE_FLAGS) {
+      return VMCI_ERROR_NO_ACCESS;
+   }
+
+   if (!handle || !produceQ || !consumeQ || (!produceSize && !consumeSize) ||
+       (flags & ~VMCI_QP_ALL_FLAGS)) {
+      return VMCI_ERROR_INVALID_ARGS;
+   }
+
+   return VMCIQueuePairAllocHelper(handle, produceQ, produceSize, consumeQ,
+                                   consumeSize, peer, flags);
 }
 
 
@@ -474,10 +461,6 @@ VMCIQueuePair_AllocPriv(VMCIHandle *handle,           // IN/OUT:
  *-----------------------------------------------------------------------------
  */
 
-#ifdef __linux__
-EXPORT_SYMBOL(VMCIQueuePair_Detach);
-#endif
-
 int
 VMCIQueuePair_Detach(VMCIHandle handle) // IN:
 {
@@ -523,7 +506,7 @@ QueuePairEntryCreate(VMCIHandle handle,  // IN:
                           2; /* One page each for the queue headers. */
 
    ASSERT((produceSize || consumeSize) && produceQ && consumeQ);
-   
+
    if (VMCI_HANDLE_INVALID(handle)) {
       VMCIId contextID = VMCI_GetContextID();
       VMCIId oldRID = queuePairRID;
@@ -813,8 +796,8 @@ out:
     */
    if ((queuePairEntry->flags & VMCI_QPFLAG_LOCAL) &&
        queuePairEntry->refCount == 1) {
-      VMCIQueue_Init(*handle, *produceQ);
-      VMCIQueue_Init(*handle, *consumeQ);
+      VMCIQueueHeader_Init((*produceQ)->qHeader, *handle);
+      VMCIQueueHeader_Init((*consumeQ)->qHeader, *handle);
    }
 
    QueuePairList_Unlock();
@@ -912,11 +895,10 @@ out:
 
    /* If we didn't remove the entry, this could change once we unlock. */
    refCount = entry ? entry->refCount :
-                      0xffffffff; /* 
+                      0xffffffff; /*
                                    * Value does not matter, silence the
                                    * compiler.
                                    */
-                                       
 
    QueuePairList_Unlock();
 
