@@ -25,10 +25,20 @@
 #ifndef _VMCI_QUEUE_PAIR_INT_H_
 #define _VMCI_QUEUE_PAIR_INT_H_
 
-#include "vmci_queue_pair.h"
-#include "vmciGuestKernelAPI.h"
+#include "vmci_defs.h"
+#include "vmciQueue.h"
 
 void VMCIQueuePair_Init(void);
 void VMCIQueuePair_Exit(void);
+int VMCIQueuePair_Alloc(VMCIHandle *handle, VMCIQueue **produceQ,
+                        uint64 produceSize, VMCIQueue **consumeQ,
+                        uint64 consumeSize, VMCIId peer, uint32 flags);
+int VMCIQueuePair_AllocPriv(VMCIHandle *handle, VMCIQueue **produceQ,
+                            uint64 produceSize, VMCIQueue **consumeQ,
+                            uint64 consumeSize, VMCIId peer, uint32 flags,
+                            VMCIPrivilegeFlags privFlags);
+int VMCIQueuePair_Detach(VMCIHandle handle);
+
 
 #endif /* !_VMCI_QUEUE_PAIR_INT_H_ */
+
