@@ -46,6 +46,7 @@
 #include "vmciProcess.h"
 #include "vmciUtil.h"
 #include "vmciEvent.h"
+#include "vmciNotifications.h"
 #include "vmciQueuePairInt.h"
 #include "vmci_version.h"
 #include "vmciCommonInt.h"
@@ -54,14 +55,17 @@
 #define VMCI_DEVICE_MINOR_NUM 0
 
 typedef struct vmci_device {
-   compat_mutex_t lock;
+   compat_mutex_t    lock;
 
-   unsigned int ioaddr;
-   unsigned int ioaddr_size;
-   unsigned int irq;
+   unsigned int      ioaddr;
+   unsigned int      ioaddr_size;
+   unsigned int      irq;
+   unsigned int      intr_type;
+   Bool              exclusive_vectors;
+   struct msix_entry msix_entries[VMCI_MAX_INTRS];
 
-   Bool         enabled;
-   spinlock_t   dev_spinlock;
+   Bool              enabled;
+   spinlock_t        dev_spinlock;
 } vmci_device;
 
 static int vmci_probe_device(struct pci_dev *pdev,
@@ -74,11 +78,15 @@ static int vmci_ioctl(struct inode *inode, struct file *file,
 static unsigned int vmci_poll(struct file *file, poll_table *wait);
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 19)
 static compat_irqreturn_t vmci_interrupt(int irq, void *dev_id,
-                                         struct pt_regs * regs);
+                                           struct pt_regs * regs);
+static compat_irqreturn_t vmci_interrupt_bm(int irq, void *dev_id,
+                                            struct pt_regs * regs);
 #else
 static compat_irqreturn_t vmci_interrupt(int irq, void *dev_id);
+static compat_irqreturn_t vmci_interrupt_bm(int irq, void *dev_id);
 #endif
 static void dispatch_datagrams(unsigned long data);
+static void process_bitmap(unsigned long data);
 
 static const struct pci_device_id vmci_ids[] = {
    { PCI_DEVICE(PCI_VENDOR_ID_VMWARE, PCI_DEVICE_ID_VMWARE_VMCI), },
@@ -105,7 +113,10 @@ static vmci_device vmci_dev;
 /* We dynamically request the device major number at init time. */
 static int device_major_nr = 0;
 
-DECLARE_TASKLET(vmci_tasklet, dispatch_datagrams,
+DECLARE_TASKLET(vmci_dg_tasklet, dispatch_datagrams,
+                (unsigned long)&vmci_dev);
+
+DECLARE_TASKLET(vmci_bm_tasklet, process_bitmap,
                 (unsigned long)&vmci_dev);
 
 /*
@@ -116,6 +127,13 @@ DECLARE_TASKLET(vmci_tasklet, dispatch_datagrams,
 static uint8 *data_buffer = NULL;
 static uint32 data_buffer_size = VMCI_MAX_DG_SIZE;
 
+/*
+ * If the VMCI hardware supports the notification bitmap, we allocate
+ * and register a page with the device.
+ */
+
+static uint8 *notification_bitmap = NULL;
+
 
 /*
  *-----------------------------------------------------------------------------
@@ -150,6 +168,8 @@ vmci_init(void)
 
    /* Initialize device data. */
    compat_mutex_init(&vmci_dev.lock);
+   vmci_dev.intr_type = VMCI_INTR_TYPE_INTX;
+   vmci_dev.exclusive_vectors = FALSE;
    spin_lock_init(&vmci_dev.dev_spinlock);
    vmci_dev.enabled = FALSE;
 
@@ -200,6 +220,43 @@ vmci_exit(void)
 }
 
 
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * vmci_enable_msix --
+ *
+ *      Enable MSI-X.  Try exclusive vectors first, then shared vectors.
+ *
+ * Results:
+ *      0 on success, other error codes on failure.
+ *
+ * Side effects:
+ *      None.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+static int
+vmci_enable_msix(struct pci_dev *pdev) // IN
+{
+   int i;
+   int result;
+
+   for (i = 0; i < VMCI_MAX_INTRS; ++i) {
+      vmci_dev.msix_entries[i].entry = i;
+      vmci_dev.msix_entries[i].vector = i;
+   }
+
+   result = pci_enable_msix(pdev, vmci_dev.msix_entries, VMCI_MAX_INTRS);
+   if (!result) {
+      vmci_dev.exclusive_vectors = TRUE;
+   } else if (result > 0) {
+      result = pci_enable_msix(pdev, vmci_dev.msix_entries, 1);
+   }
+   return result;
+}
+
+
 /*
  *-----------------------------------------------------------------------------
  *
@@ -266,8 +323,26 @@ vmci_probe_device(struct pci_dev *pdev,           // IN: vmci PCI device
       goto release;
    }
 
+   /*
+    * If the hardware supports notifications, we will use that as
+    * well.
+    */
+   if (capabilities & VMCI_CAPS_NOTIFICATIONS) {
+      capabilities = VMCI_CAPS_DATAGRAM;
+      notification_bitmap = vmalloc(PAGE_SIZE);
+      if (notification_bitmap == NULL) {
+         printk(KERN_ERR "VMCI device unable to allocate notification bitmap.\n");
+      } else {
+         memset(notification_bitmap, 0, PAGE_SIZE);
+         capabilities |= VMCI_CAPS_NOTIFICATIONS;
+      }
+   } else {
+      capabilities = VMCI_CAPS_DATAGRAM;
+   }
+   printk(KERN_INFO "VMCI: using capabilities 0x%x.\n", capabilities);
+
    /* Let the host know which capabilities we intend to use. */
-   outl(VMCI_CAPS_DATAGRAM, ioaddr + VMCI_CAPS_ADDR);
+   outl(capabilities, ioaddr + VMCI_CAPS_ADDR);
 
    /* Device struct initialization. */
    compat_mutex_lock(&vmci_dev.lock);
@@ -278,11 +353,24 @@ vmci_probe_device(struct pci_dev *pdev,           // IN: vmci PCI device
 
    vmci_dev.ioaddr = ioaddr;
    vmci_dev.ioaddr_size = ioaddr_size;
-   vmci_dev.irq = pdev->irq;
+
+   /*
+    * Register notification bitmap with device if that capability is
+    * used
+    */
+   if (capabilities & VMCI_CAPS_NOTIFICATIONS) {
+      unsigned long bitmapPPN;
+      bitmapPPN = page_to_pfn(vmalloc_to_page(notification_bitmap));
+      if (!VMCI_RegisterNotificationBitmap(bitmapPPN)) {
+         printk(KERN_ERR "VMCI device unable to register notification bitmap "
+                "with PPN 0x%x.\n", (uint32)bitmapPPN);
+         goto unlock;
+      }
+   }
 
    /* Check host capabilities. */
    if (!VMCI_CheckHostCapabilities()) {
-      goto unlock;
+      goto remove_bitmap;
    }
 
    /* Enable device. */
@@ -303,20 +391,61 @@ vmci_probe_device(struct pci_dev *pdev,           // IN: vmci PCI device
    VMCIDatagram_Init();
    VMCIEvent_Init();
    VMCIUtil_Init();
+   VMCINotifications_Init();
    VMCIQueuePair_Init();
 
-   if (request_irq(vmci_dev.irq, vmci_interrupt, COMPAT_IRQF_SHARED,
-                   "vmci", &vmci_dev)) {
-      printk(KERN_ERR "vmci: irq %u in use\n", vmci_dev.irq);
+   /*
+    * Enable interrupts.  Try MSI-X first, then MSI, and then fallback on
+    * legacy interrupts.
+    */
+   if (!vmci_enable_msix(pdev)) {
+      vmci_dev.intr_type = VMCI_INTR_TYPE_MSIX;
+      vmci_dev.irq = vmci_dev.msix_entries[0].vector;
+   } else if (!pci_enable_msi(pdev)) {
+      vmci_dev.intr_type = VMCI_INTR_TYPE_MSI;
+      vmci_dev.irq = pdev->irq;
+   } else {
+      vmci_dev.intr_type = VMCI_INTR_TYPE_INTX;
+      vmci_dev.irq = pdev->irq;
+   }
+
+   /* Request IRQ for legacy or MSI interrupts, or for first MSI-X vector. */
+   result = request_irq(vmci_dev.irq, vmci_interrupt, COMPAT_IRQF_SHARED,
+                        "vmci", &vmci_dev);
+   if (result) {
+      printk(KERN_ERR "vmci: irq %u in use: %d\n", vmci_dev.irq, result);
       goto components_exit;
    }
 
+   /*
+    * For MSI-X with exclusive vectors we need to request an interrupt for each
+    * vector so that we get a separate interrupt handler routine.  This allows
+    * us to distinguish between the vectors.
+    */
+
+   if (vmci_dev.exclusive_vectors) {
+      ASSERT(vmci_dev.intr_type == VMCI_INTR_TYPE_MSIX);
+      result = request_irq(vmci_dev.msix_entries[1].vector, vmci_interrupt_bm,
+                           0, "vmci", &vmci_dev);
+      if (result) {
+         printk(KERN_ERR "vmci: irq %u in use: %d\n",
+                vmci_dev.msix_entries[1].vector, result);
+         free_irq(vmci_dev.irq, &vmci_dev);
+         goto components_exit;
+      }
+   }
+
    printk(KERN_INFO "Registered vmci device.\n");
 
    compat_mutex_unlock(&vmci_dev.lock);
 
    /* Enable specific interrupt bits. */
-   outl(VMCI_IMR_DATAGRAM, vmci_dev.ioaddr + VMCI_IMR_ADDR);
+   if (capabilities & VMCI_CAPS_NOTIFICATIONS) {
+      outl(VMCI_IMR_DATAGRAM | VMCI_IMR_NOTIFICATION,
+           vmci_dev.ioaddr + VMCI_IMR_ADDR);
+   } else {
+      outl(VMCI_IMR_DATAGRAM, vmci_dev.ioaddr + VMCI_IMR_ADDR);
+   }
 
    /* Enable interrupts. */
    outl(VMCI_CONTROL_INT_ENABLE, vmci_dev.ioaddr + VMCI_CONTROL_ADDR);
@@ -325,12 +454,25 @@ vmci_probe_device(struct pci_dev *pdev,           // IN: vmci PCI device
 
  components_exit:
    VMCIQueuePair_Exit();
+   VMCINotifications_Exit();
    VMCIUtil_Exit();
    VMCIEvent_Exit();
    VMCIProcess_Exit();
+   if (vmci_dev.intr_type == VMCI_INTR_TYPE_MSIX) {
+      pci_disable_msix(pdev);
+   } else if (vmci_dev.intr_type == VMCI_INTR_TYPE_MSI) {
+      pci_disable_msi(pdev);
+   }
+ remove_bitmap:
+   if (notification_bitmap) {
+      outl(VMCI_CONTROL_RESET, vmci_dev.ioaddr + VMCI_CONTROL_ADDR);
+   }
  unlock:
    compat_mutex_unlock(&vmci_dev.lock);
  release:
+   if (notification_bitmap) {
+      vfree(notification_bitmap);
+   }
    release_region(ioaddr, ioaddr_size);
  pci_disable:
    compat_pci_disable_device(pdev);
@@ -372,9 +514,30 @@ vmci_remove_device(struct pci_dev* pdev)
    compat_mutex_lock(&dev->lock);
    printk(KERN_INFO "Resetting vmci device\n");
    outl(VMCI_CONTROL_RESET, vmci_dev.ioaddr + VMCI_CONTROL_ADDR);
+
+   /*
+    * Free IRQ and then disable MSI/MSI-X as appropriate.  For MSI-X, we might
+    * have multiple vectors, each with their own IRQ, which we must free too.
+    */
+
    free_irq(dev->irq, dev);
+   if (dev->intr_type == VMCI_INTR_TYPE_MSIX) {
+      if (dev->exclusive_vectors) {
+         free_irq(dev->msix_entries[1].vector, dev);
+      }
+      pci_disable_msix(pdev);
+   } else if (dev->intr_type == VMCI_INTR_TYPE_MSI) {
+      pci_disable_msi(pdev);
+   }
+   dev->exclusive_vectors = FALSE;
+   dev->intr_type = VMCI_INTR_TYPE_INTX;
+
    release_region(dev->ioaddr, dev->ioaddr_size);
    dev->enabled = FALSE;
+   VMCINotifications_Exit();
+   if (notification_bitmap) {
+      vfree(notification_bitmap);
+   }
 
    printk(KERN_INFO "Unregistered vmci device.\n");
    compat_mutex_unlock(&dev->lock);
@@ -658,7 +821,7 @@ vmci_ioctl(struct inode *inode,  // IN
       retval = copy_to_user((void *)arg, &cid, sizeof cid);
       break;
    }
-
+ 
    default:
       printk(KERN_DEBUG "vmci_ioctl(): unknown ioctl 0x%x.\n", cmd);
       retval = -EINVAL;
@@ -726,10 +889,12 @@ vmci_poll(struct file *file, // IN
  *
  * vmci_interrupt --
  *
- *      Interrupt handler.
+ *      Interrupt handler for legacy or MSI interrupt, or for first MSI-X
+ *      interrupt (vector VMCI_INTR_DATAGRAM).
  *
  * Results:
- *      None.
+ *      COMPAT_IRQ_HANDLED if the interrupt is handled, COMPAT_IRQ_NONE if
+ *      not an interrupt.
  *
  * Side effects:
  *      None.
@@ -749,27 +914,89 @@ vmci_interrupt(int irq,               // IN
 #endif
 {
    vmci_device *dev = clientdata;
-   unsigned int icr = 0;
 
    if (dev == NULL) {
-      printk (KERN_DEBUG "vmci_interrupt(): irq %d for unknown device.\n",
-              irq);
+      printk(KERN_DEBUG "vmci_interrupt(): irq %d for unknown device.\n", irq);
       return COMPAT_IRQ_NONE;
    }
 
-   /* Acknowledge interrupt and determine what needs doing. */
-   icr = inl(dev->ioaddr + VMCI_ICR_ADDR);
-   if (icr == 0) {
+   /*
+    * If we are using MSI-X with exclusive vectors then we simply schedule
+    * the datagram tasklet, since we know the interrupt was meant for us.
+    * Otherwise we must read the ICR to determine what to do.
+    */
+
+   if (dev->intr_type == VMCI_INTR_TYPE_MSIX && dev->exclusive_vectors) {
+      tasklet_schedule(&vmci_dg_tasklet);
+   } else {
+      unsigned int icr;
+
+      ASSERT(dev->intr_type == VMCI_INTR_TYPE_INTX ||
+             dev->intr_type == VMCI_INTR_TYPE_MSI);
+
+      /* Acknowledge interrupt and determine what needs doing. */
+      icr = inl(dev->ioaddr + VMCI_ICR_ADDR);
+      if (icr == 0 || icr == 0xffffffff) {
+         return COMPAT_IRQ_NONE;
+      }
+
+      if (icr & VMCI_ICR_DATAGRAM) {
+         tasklet_schedule(&vmci_dg_tasklet);
+         icr &= ~VMCI_ICR_DATAGRAM;
+      }
+      if (icr & VMCI_ICR_NOTIFICATION) {
+         tasklet_schedule(&vmci_bm_tasklet);
+         icr &= ~VMCI_ICR_NOTIFICATION;
+      }
+      if (icr != 0) {
+         printk(KERN_INFO LGPFX"Ignoring unknown interrupt cause (%d).\n", icr);
+      }
+   }
+
+   return COMPAT_IRQ_HANDLED;
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * vmci_interrupt_bm --
+ *
+ *      Interrupt handler for MSI-X interrupt vector VMCI_INTR_NOTIFICATION,
+ *      which is for the notification bitmap.  Will only get called if we are
+ *      using MSI-X with exclusive vectors.
+ *
+ * Results:
+ *      COMPAT_IRQ_HANDLED if the interrupt is handled, COMPAT_IRQ_NONE if
+ *      not an interrupt.
+ *
+ * Side effects:
+ *      None.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 19)
+static compat_irqreturn_t
+vmci_interrupt_bm(int irq,               // IN
+                  void *clientdata,      // IN
+                  struct pt_regs *regs)  // IN
+#else
+static compat_irqreturn_t
+vmci_interrupt_bm(int irq,               // IN
+                  void *clientdata)      // IN
+#endif
+{
+   vmci_device *dev = clientdata;
+
+   if (dev == NULL) {
+      printk(KERN_DEBUG "vmci_interrupt_bm(): irq %d for unknown device.\n", irq);
       return COMPAT_IRQ_NONE;
    }
 
-   if (icr & VMCI_ICR_DATAGRAM) {
-      tasklet_schedule(&vmci_tasklet);
-      icr &= ~VMCI_ICR_DATAGRAM;
-   }
-   if (icr != 0) {
-      printk(KERN_INFO LGPFX"Ignoring unknown interrupt cause (%d).\n", icr);
-   }
+   /* For MSI-X we can just assume it was meant for us. */
+   ASSERT(dev->intr_type == VMCI_INTR_TYPE_MSIX && dev->exclusive_vectors);
+   tasklet_schedule(&vmci_bm_tasklet);
 
    return COMPAT_IRQ_HANDLED;
 }
@@ -850,7 +1077,7 @@ VMCI_SendDatagram(VMCIDatagram *dg)
       "rep outsb\n\t"
       : /* No output. */
       : "d"(vmci_dev.ioaddr + VMCI_DATA_OUT_ADDR),
-	"c"(VMCI_DG_SIZE(dg)), "S"(dg)
+        "c"(VMCI_DG_SIZE(dg)), "S"(dg)
       );
 
    /*
@@ -902,6 +1129,44 @@ dispatch_datagrams(unsigned long data)
 }
 
 
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * process_bitmap --
+ *
+ *      Scans the notification bitmap for raised flags, clears them
+ *      and handles the notifications.
+ *
+ * Results:
+ *      None.
+ *
+ * Side effects:
+ *      None.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+void
+process_bitmap(unsigned long data)
+{
+   vmci_device *dev = (vmci_device *)data;
+
+   if (dev == NULL) {
+      printk(KERN_DEBUG "vmci: process_bitmaps(): no vmci device"
+	     "present.\n");
+      return;
+   }
+
+   if (notification_bitmap == NULL) {
+      printk(KERN_DEBUG "vmci: process_bitmaps(): no bitmap present.\n");
+      return;
+   }
+
+
+   VMCI_ScanNotificationBitmap(notification_bitmap);
+}
+
+
 module_init(vmci_init);
 module_exit(vmci_exit);
 MODULE_DEVICE_TABLE(pci, vmci_ids);
