@@ -238,7 +238,7 @@ HgfsDelete(struct inode *dir,      // IN: Parent dir of file/dir to delete
    }
 
    /* Build full name to send to server. */
-   if (HgfsBuildPath(fileName, HGFS_NAME_BUFFER_SIZET(reqSize),
+   if (HgfsBuildPath(fileName, HGFS_NAME_BUFFER_SIZET(req->bufferSize, reqSize),
                      dentry) < 0) {
       LOG(4, (KERN_DEBUG "VMware hgfs: HgfsDelete: build path failed\n"));
       result = -EINVAL;
@@ -249,7 +249,7 @@ HgfsDelete(struct inode *dir,      // IN: Parent dir of file/dir to delete
 
    /* Convert to CP name. */
    result = CPName_ConvertTo(fileName,
-                             HGFS_NAME_BUFFER_SIZET(reqSize),
+                             HGFS_NAME_BUFFER_SIZET(req->bufferSize, reqSize),
                              fileName);
    if (result < 0) {
       LOG(4, (KERN_DEBUG "VMware hgfs: HgfsDelete: CP conversion failed\n"));
@@ -449,7 +449,7 @@ HgfsPackSetattrRequest(struct iattr *iattr,   // IN: Inode attrs to update from
       }
       requestV3->reserved = 0;
       reqSize = HGFS_REQ_PAYLOAD_SIZE_V3(requestV3);
-      reqBufferSize = HGFS_NAME_BUFFER_SIZET(reqSize);
+      reqBufferSize = HGFS_NAME_BUFFER_SIZET(req->bufferSize, reqSize);
 
       /*
        * We only support changing these attributes:
@@ -552,7 +552,7 @@ HgfsPackSetattrRequest(struct iattr *iattr,   // IN: Inode attrs to update from
 	 fileNameLength = &requestV2->fileName.length;
       }
       reqSize = sizeof *requestV2;
-      reqBufferSize = HGFS_NAME_BUFFER_SIZE(requestV2);
+      reqBufferSize = HGFS_NAME_BUFFER_SIZE(req->bufferSize, requestV2);
 
       /*
        * We only support changing these attributes:
@@ -626,7 +626,7 @@ HgfsPackSetattrRequest(struct iattr *iattr,   // IN: Inode attrs to update from
       fileName = request->fileName.name;
       fileNameLength = &request->fileName.length;
       reqSize = sizeof *request;
-      reqBufferSize = HGFS_NAME_BUFFER_SIZE(request);
+      reqBufferSize = HGFS_NAME_BUFFER_SIZE(req->bufferSize, request);
 
 
       /*
@@ -811,7 +811,7 @@ HgfsPackCreateDirRequest(struct dentry *dentry, // IN: Directory to create
 
    /* Build full name to send to server. */
    if (HgfsBuildPath(fileName,
-                     HGFS_PACKET_MAX - (requestSize - 1),
+                     req->bufferSize - (requestSize - 1),
                      dentry) < 0) {
       LOG(4, (KERN_DEBUG "VMware hgfs: HgfsPackCreateDirRequest: build path "
               "failed\n"));
@@ -822,7 +822,7 @@ HgfsPackCreateDirRequest(struct dentry *dentry, // IN: Directory to create
 
    /* Convert to CP name. */
    result = CPName_ConvertTo(fileName,
-                             HGFS_PACKET_MAX - (requestSize - 1),
+                             req->bufferSize - (requestSize - 1),
                              fileName);
    if (result < 0) {
       LOG(4, (KERN_DEBUG "VMware hgfs: HgfsPackCreateDirRequest: CP "
@@ -1364,7 +1364,7 @@ retry:
    }
 
    /* Build full old name to send to server. */
-   if (HgfsBuildPath(oldName, HGFS_NAME_BUFFER_SIZET(reqSize),
+   if (HgfsBuildPath(oldName, HGFS_NAME_BUFFER_SIZET(req->bufferSize, reqSize),
                      oldDentry) < 0) {
       LOG(4, (KERN_DEBUG "VMware hgfs: HgfsRename: build old path failed\n"));
       result = -EINVAL;
@@ -1375,7 +1375,7 @@ retry:
 
    /* Convert old name to CP format. */
    result = CPName_ConvertTo(oldName,
-                             HGFS_NAME_BUFFER_SIZET(reqSize),
+                             HGFS_NAME_BUFFER_SIZET(req->bufferSize, reqSize),
                              oldName);
    if (result < 0) {
       LOG(4, (KERN_DEBUG "VMware hgfs: HgfsRename: oldName CP "
@@ -1413,7 +1413,7 @@ retry:
       newNameLength = &newNameP->length;
    }
 
-   if (HgfsBuildPath(newName, HGFS_NAME_BUFFER_SIZET(reqSize) - result,
+   if (HgfsBuildPath(newName, HGFS_NAME_BUFFER_SIZET(req->bufferSize, reqSize) - result,
                      newDentry) < 0) {
       LOG(4, (KERN_DEBUG "VMware hgfs: HgfsRename: build new path failed\n"));
       result = -EINVAL;
@@ -1424,7 +1424,7 @@ retry:
 
    /* Convert new name to CP format. */
    result = CPName_ConvertTo(newName,
-                             HGFS_NAME_BUFFER_SIZET(reqSize) - result,
+                             HGFS_NAME_BUFFER_SIZET(req->bufferSize, reqSize) - result,
                              newName);
    if (result < 0) {
       LOG(4, (KERN_DEBUG "VMware hgfs: HgfsRename: newName CP "
@@ -1545,7 +1545,7 @@ HgfsPackSymlinkCreateRequest(struct dentry *dentry,   // IN: File pointer for th
       return -EPROTO;
    }
 
-   if (HgfsBuildPath(symlinkName, HGFS_PACKET_MAX - (requestSize - 1),
+   if (HgfsBuildPath(symlinkName, req->bufferSize - (requestSize - 1),
                      dentry) < 0) {
       LOG(4, (KERN_DEBUG "VMware hgfs: HgfsPackSymlinkCreateRequest: build symlink path "
               "failed\n"));
@@ -1557,7 +1557,7 @@ HgfsPackSymlinkCreateRequest(struct dentry *dentry,   // IN: File pointer for th
 
    /* Convert symlink name to CP format. */
    result = CPName_ConvertTo(symlinkName,
-                             HGFS_PACKET_MAX - (requestSize - 1),
+                             req->bufferSize - (requestSize - 1),
                              symlinkName);
    if (result < 0) {
       LOG(4, (KERN_DEBUG "VMware hgfs: HgfsPackSymlinkCreateRequest: symlinkName CP "
@@ -1597,7 +1597,7 @@ HgfsPackSymlinkCreateRequest(struct dentry *dentry,   // IN: File pointer for th
    targetNameBytes = strlen(symname) + 1;
 
    /* Copy target name into request packet. */
-   if (targetNameBytes > HGFS_PACKET_MAX - (requestSize - 1)) {
+   if (targetNameBytes > req->bufferSize - (requestSize - 1)) {
       LOG(4, (KERN_DEBUG "VMware hgfs: HgfsPackSymlinkCreateRequest: target name is too "
               "big\n"));
       return -EINVAL;
