@@ -43,15 +43,17 @@
 #include "inode.h"
 #include "vm_assert.h"
 #include "vm_basic_types.h"
+#include "hgfsTransport.h"
+
 
 /* Private functions. */
 static int HgfsDoWrite(HgfsHandle handle,
-                       const char *buf,
-                       size_t count,
+                       HgfsDataPacket dataPacket[],
+                       uint32 numEntries,
                        loff_t offset);
 static int HgfsDoRead(HgfsHandle handle,
-                      char *buf,
-                      size_t count,
+                      HgfsDataPacket dataPacket[],
+                      uint32 numEntries,
                       loff_t offset);
 static int HgfsDoReadpage(HgfsHandle handle,
                           struct page *page,
@@ -143,6 +145,8 @@ struct address_space_operations HgfsAddressSpaceOperations = {
  *    It is assumed that this function is never called with a larger read than
  *    what can be sent in one request.
  *
+ *    HgfsDataPacket is an array of pages into which data will be read.
+ *
  * Results:
  *    Returns the number of bytes read on success, or an error on failure.
  *
@@ -153,10 +157,10 @@ struct address_space_operations HgfsAddressSpaceOperations = {
  */
 
 static int
-HgfsDoRead(HgfsHandle handle,  // IN:  Handle for this file
-           char *buf,          // OUT: Buffer to copy data into
-           size_t count,       // IN:  Number of bytes to read
-           loff_t offset)      // IN:  Offset at which to read
+HgfsDoRead(HgfsHandle handle,             // IN:  Handle for this file
+           HgfsDataPacket dataPacket[],   // IN/OUT: Data description
+           uint32 numEntries,             // IN: Number of entries in dataPacket
+           loff_t offset)                 // IN:  Offset at which to read
 {
    HgfsReq *req;
    HgfsOp opUsed;
@@ -164,8 +168,11 @@ HgfsDoRead(HgfsHandle handle,  // IN:  Handle for this file
    uint32 actualSize = 0;
    char *payload = NULL;
    HgfsStatus replyStatus;
+   char *buf;
+   uint32 count;
+   ASSERT(numEntries == 1);
 
-   ASSERT(buf);
+   count = dataPacket[0].len;
 
    req = HgfsGetNewRequest();
    if (!req) {
@@ -177,7 +184,7 @@ HgfsDoRead(HgfsHandle handle,  // IN:  Handle for this file
 
  retry:
    opUsed = hgfsVersionRead;
-   if (opUsed == HGFS_OP_READ_V3) {
+   if (opUsed == HGFS_OP_READ_FAST_V4) {
       HgfsRequest *header;
       HgfsRequestReadV3 *request;
 
@@ -190,6 +197,31 @@ HgfsDoRead(HgfsHandle handle,  // IN:  Handle for this file
       request->offset = offset;
       request->requiredSize = count;
       request->reserved = 0;
+      req->dataPacket = kmalloc(numEntries * sizeof req->dataPacket[0],
+                                GFP_KERNEL);
+      if (!req->dataPacket) {
+         LOG(4, (KERN_DEBUG "%s: Failed to allocate mem\n", __func__));
+         result = -ENOMEM;
+         goto out;
+      }
+      memcpy(req->dataPacket, dataPacket, numEntries * sizeof req->dataPacket[0]);
+      req->numEntries = numEntries;
+
+      LOG(4, (KERN_DEBUG "VMware hgfs: Fast Read V4\n"));
+   } else if (opUsed == HGFS_OP_READ_V3) {
+      HgfsRequest *header;
+      HgfsRequestReadV3 *request;
+
+      header = (HgfsRequest *)(HGFS_REQ_PAYLOAD(req));
+      header->id = req->id;
+      header->op = opUsed;
+
+      request = (HgfsRequestReadV3 *)(HGFS_REQ_PAYLOAD_V3(req));
+      request->file = handle;
+      request->offset = offset;
+      request->requiredSize = count;
+      request->reserved = 0;
+      req->dataPacket = NULL;
       req->payloadSize = HGFS_REQ_PAYLOAD_SIZE_V3(request);
    } else {
       HgfsRequestRead *request;
@@ -200,10 +232,10 @@ HgfsDoRead(HgfsHandle handle,  // IN:  Handle for this file
       request->file = handle;
       request->offset = offset;
       request->requiredSize = count;
+      req->dataPacket = NULL;
       req->payloadSize = sizeof *request;
    }
 
-
    /* Send the request and process the reply. */
    result = HgfsSendRequest(req);
    if (result == 0) {
@@ -213,7 +245,9 @@ HgfsDoRead(HgfsHandle handle,  // IN:  Handle for this file
 
       switch (result) {
       case 0:
-         if (opUsed == HGFS_OP_READ_V3) {
+         if (opUsed == HGFS_OP_READ_FAST_V4) {
+            actualSize = ((HgfsReplyReadV3 *)HGFS_REP_PAYLOAD_V3(req))->actualSize;
+         } else if (opUsed == HGFS_OP_READ_V3) {
             actualSize = ((HgfsReplyReadV3 *)HGFS_REP_PAYLOAD_V3(req))->actualSize;
             payload = ((HgfsReplyReadV3 *)HGFS_REP_PAYLOAD_V3(req))->payload;
          } else {
@@ -229,7 +263,7 @@ HgfsDoRead(HgfsHandle handle,  // IN:  Handle for this file
          }
 
          if (!actualSize) {
-            /* We got no bytes, so don't need to copy to user. */
+            /* We got no bytes. */
             LOG(6, (KERN_DEBUG "VMware hgfs: HgfsDoRead: server returned "
                    "zero\n"));
             result = actualSize;
@@ -237,19 +271,34 @@ HgfsDoRead(HgfsHandle handle,  // IN:  Handle for this file
          }
 
          /* Return result. */
-         memcpy(buf, payload, actualSize);
-         LOG(6, (KERN_DEBUG "VMware hgfs: HgfsDoRead: copied %u\n",
-                 actualSize));
+         if (opUsed == HGFS_OP_READ_V3 || opUsed == HGFS_OP_READ) {
+            buf = kmap(dataPacket[0].page) + dataPacket[0].offset;
+            ASSERT(buf);
+            memcpy(buf, payload, actualSize);
+            LOG(6, (KERN_DEBUG "VMware hgfs: HgfsDoRead: copied %u\n",
+                    actualSize));
+            kunmap(dataPacket[0].page);
+         }
          result = actualSize;
 	 break;
 
       case -EPROTO:
          /* Retry with older version(s). Set globally. */
-         if (opUsed == HGFS_OP_READ_V3) {
+         switch (opUsed) {
+         case HGFS_OP_READ_FAST_V4:
+            LOG(4, (KERN_DEBUG "VMware hgfs: HgfsDoRead: Fast Read V4 not "
+                    "supported. Falling back to V3 Read.\n"));
+            hgfsVersionRead = HGFS_OP_READ_V3;
+            goto retry;
+
+         case HGFS_OP_READ_V3:
             LOG(4, (KERN_DEBUG "VMware hgfs: HgfsDoRead: Version 3 not "
                     "supported. Falling back to version 1.\n"));
             hgfsVersionRead = HGFS_OP_READ;
             goto retry;
+
+         default:
+            break;
          }
 	 break;
 
@@ -267,6 +316,9 @@ HgfsDoRead(HgfsHandle handle,  // IN:  Handle for this file
    }
 
 out:
+   if (req->dataPacket) {
+      kfree(req->dataPacket);
+   }
    HgfsFreeRequest(req);
    return result;
 }
@@ -286,6 +338,9 @@ out:
  *    It is assumed that this function is never called with a larger write
  *    than what can be sent in one request.
  *
+ *    HgfsDataPacket is an array of pages from which data will be written
+ *    to file.
+ *
  * Results:
  *    Returns the number of bytes written on success, or an error on failure.
  *
@@ -296,10 +351,10 @@ out:
  */
 
 static int
-HgfsDoWrite(HgfsHandle handle,       // IN: Handle for this file
-            const char *buf,         // IN: Buffer containing data
-            size_t count,            // IN: Number of bytes to write
-            loff_t offset)           // IN: Offset to begin writing at
+HgfsDoWrite(HgfsHandle handle,             // IN: Handle for this file
+            HgfsDataPacket dataPacket[],   // IN: Data description
+            uint32 numEntries,             // IN: Number of entries in dataPacket
+            loff_t offset)                 // IN: Offset to begin writing at
 {
    HgfsReq *req;
    int result = 0;
@@ -309,8 +364,11 @@ HgfsDoWrite(HgfsHandle handle,       // IN: Handle for this file
    char *payload = NULL;
    uint32 reqSize;
    HgfsStatus replyStatus;
+   char *buf;
+   uint32 count;
+   ASSERT(numEntries == 1);
 
-   ASSERT(buf);
+   count = dataPacket[0].len;
 
    req = HgfsGetNewRequest();
    if (!req) {
@@ -322,7 +380,36 @@ HgfsDoWrite(HgfsHandle handle,       // IN: Handle for this file
 
  retry:
    opUsed = hgfsVersionWrite;
-   if (opUsed == HGFS_OP_WRITE_V3) {
+   if (opUsed == HGFS_OP_WRITE_FAST_V4) {
+      HgfsRequest *header;
+      HgfsRequestWriteV3 *request;
+
+      header = (HgfsRequest *)(HGFS_REQ_PAYLOAD(req));
+      header->id = req->id;
+      header->op = opUsed;
+
+      request = (HgfsRequestWriteV3 *)(HGFS_REQ_PAYLOAD_V3(req));
+      request->file = handle;
+      request->flags = 0;
+      request->offset = offset;
+      request->requiredSize = count;
+      request->reserved = 0;
+      payload = request->payload;
+      requiredSize = request->requiredSize;
+
+      req->dataPacket = kmalloc(numEntries * sizeof req->dataPacket[0],
+                                GFP_KERNEL);
+      if (!req->dataPacket) {
+         LOG(4, (KERN_DEBUG "%s: Failed to allocate mem\n", __func__));
+         result = -ENOMEM;
+         goto out;
+      }
+      memcpy(req->dataPacket, dataPacket, numEntries * sizeof req->dataPacket[0]);
+      req->numEntries = numEntries;
+      reqSize = HGFS_REQ_PAYLOAD_SIZE_V3(request);
+      req->payloadSize = reqSize;
+      LOG(4, (KERN_DEBUG "VMware hgfs: Fast Write V4\n"));
+   } else if (opUsed == HGFS_OP_WRITE_V3) {
       HgfsRequest *header;
       HgfsRequestWriteV3 *request;
 
@@ -339,6 +426,12 @@ HgfsDoWrite(HgfsHandle handle,       // IN: Handle for this file
       payload = request->payload;
       requiredSize = request->requiredSize;
       reqSize = HGFS_REQ_PAYLOAD_SIZE_V3(request);
+      req->dataPacket = NULL;
+      buf = kmap(dataPacket[0].page) + dataPacket[0].offset;
+      memcpy(payload, buf, requiredSize);
+      kunmap(dataPacket[0].page);
+
+      req->payloadSize = reqSize + requiredSize - 1;
    } else {
       HgfsRequestWrite *request;
 
@@ -352,10 +445,13 @@ HgfsDoWrite(HgfsHandle handle,       // IN: Handle for this file
       payload = request->payload;
       requiredSize = request->requiredSize;
       reqSize = sizeof *request;
-   }
+      req->dataPacket = NULL;
+      buf = kmap(dataPacket[0].page) + dataPacket[0].offset;
+      memcpy(payload, buf, requiredSize);
+      kunmap(dataPacket[0].page);
 
-   memcpy(payload, buf, requiredSize);
-   req->payloadSize = reqSize + requiredSize - 1;
+      req->payloadSize = reqSize + requiredSize - 1;
+   }
 
    /* Send the request and process the reply. */
    result = HgfsSendRequest(req);
@@ -366,7 +462,7 @@ HgfsDoWrite(HgfsHandle handle,       // IN: Handle for this file
 
       switch (result) {
       case 0:
-         if (opUsed == HGFS_OP_WRITE_V3) {
+         if (opUsed == HGFS_OP_WRITE_V3 || opUsed == HGFS_OP_WRITE_FAST_V4) {
             actualSize = ((HgfsReplyWriteV3 *)HGFS_REP_PAYLOAD_V3(req))->actualSize;
          } else {
             actualSize = ((HgfsReplyWrite *)HGFS_REQ_PAYLOAD(req))->actualSize;
@@ -380,11 +476,21 @@ HgfsDoWrite(HgfsHandle handle,       // IN: Handle for this file
 
       case -EPROTO:
          /* Retry with older version(s). Set globally. */
-         if (opUsed == HGFS_OP_WRITE_V3) {
+         switch (opUsed) {
+         case HGFS_OP_WRITE_FAST_V4:
+            LOG(4, (KERN_DEBUG "VMware hgfs: HgfsDoWrite: Fast Write V4 not "
+                    "supported. Falling back to V3 write.\n"));
+            hgfsVersionRead = HGFS_OP_WRITE_V3;
+            goto retry;
+
+         case HGFS_OP_WRITE_V3:
             LOG(4, (KERN_DEBUG "VMware hgfs: HgfsDoWrite: Version 3 not "
                     "supported. Falling back to version 1.\n"));
             hgfsVersionWrite = HGFS_OP_WRITE;
             goto retry;
+
+         default:
+            break;
          }
          break;
 
@@ -404,6 +510,9 @@ HgfsDoWrite(HgfsHandle handle,       // IN: Handle for this file
    }
 
 out:
+   if (req->dataPacket) {
+      kfree(req->dataPacket);
+   }
    HgfsFreeRequest(req);
    return result;
 }
@@ -436,9 +545,9 @@ HgfsDoReadpage(HgfsHandle handle,  // IN:     Handle to use for reading
                unsigned pageTo)    // IN:     Where to stop reading
 {
    int result = 0;
-   char *buffer = kmap(page) + pageFrom;
    loff_t curOffset = ((loff_t)page->index << PAGE_CACHE_SHIFT) + pageFrom;
    size_t nextCount, remainingCount = pageTo - pageFrom;
+   HgfsDataPacket dataPacket[1];
 
    LOG(6, (KERN_DEBUG "VMware hgfs: HgfsDoReadpage: read %Zu bytes from fh %u "
            "at offset %Lu\n", remainingCount, handle, curOffset));
@@ -452,7 +561,10 @@ HgfsDoReadpage(HgfsHandle handle,  // IN:     Handle to use for reading
    do {
       nextCount = (remainingCount > HGFS_IO_MAX) ?
          HGFS_IO_MAX : remainingCount;
-      result = HgfsDoRead(handle, buffer, nextCount, curOffset);
+      dataPacket[0].page = page;
+      dataPacket[0].offset = pageFrom;
+      dataPacket[0].len = nextCount;
+      result = HgfsDoRead(handle, dataPacket, 1, curOffset);
       if (result < 0) {
          LOG(4, (KERN_DEBUG "VMware hgfs: HgfsDoReadpage: read error %d\n",
                  result));
@@ -460,7 +572,7 @@ HgfsDoReadpage(HgfsHandle handle,  // IN:     Handle to use for reading
       }
       remainingCount -= result;
       curOffset += result;
-      buffer += result;
+      pageFrom += result;
    } while ((result > 0) && (remainingCount > 0));
 
    /*
@@ -468,7 +580,11 @@ HgfsDoReadpage(HgfsHandle handle,  // IN:     Handle to use for reading
     * than a page in the file from this offset, so we should zero the rest of
     * the page's memory.
     */
-   memset(buffer, 0, remainingCount);
+   if (remainingCount) {
+      char *buffer = kmap(page) + pageTo;
+      memset(buffer - remainingCount, 0, remainingCount);
+      kunmap(page);
+   }
 
    /*
     * We read a full page (or all of the page that actually belongs to the
@@ -480,7 +596,6 @@ HgfsDoReadpage(HgfsHandle handle,  // IN:     Handle to use for reading
    result = 0;
 
   out:
-   kunmap(page);
    return result;
 }
 
@@ -526,11 +641,11 @@ HgfsDoWritepage(HgfsHandle handle,  // IN: Handle to use for writing
                 unsigned pageTo)    // IN: Ending page offset
 {
    int result = 0;
-   char *buffer = kmap(page) + pageFrom;
    loff_t curOffset = ((loff_t)page->index << PAGE_CACHE_SHIFT) + pageFrom;
    size_t nextCount;
    size_t remainingCount = pageTo - pageFrom;
    struct inode *inode;
+   HgfsDataPacket dataPacket[1];
 
    ASSERT(page->mapping);
    ASSERT(page->mapping->host);
@@ -545,7 +660,10 @@ HgfsDoWritepage(HgfsHandle handle,  // IN: Handle to use for writing
    do {
       nextCount = (remainingCount > HGFS_IO_MAX) ?
          HGFS_IO_MAX : remainingCount;
-      result = HgfsDoWrite(handle, buffer, nextCount, curOffset);
+      dataPacket[0].page = page;
+      dataPacket[0].offset = pageFrom;
+      dataPacket[0].len = nextCount;
+      result = HgfsDoWrite(handle, dataPacket, 1, curOffset);
       if (result < 0) {
          LOG(4, (KERN_DEBUG "VMware hgfs: HgfsDoWritepage: write error %d\n",
                  result));
@@ -553,7 +671,7 @@ HgfsDoWritepage(HgfsHandle handle,  // IN: Handle to use for writing
       }
       remainingCount -= result;
       curOffset += result;
-      buffer += result;
+      pageFrom += result;
 
       /* Update the inode's size now rather than waiting for a revalidate. */
       if (curOffset > compat_i_size_read(inode)) {
@@ -564,7 +682,6 @@ HgfsDoWritepage(HgfsHandle handle,  // IN: Handle to use for writing
    result = 0;
 
   out:
-   kunmap(page);
    return result;
 }
 
