@@ -44,16 +44,18 @@
 #include <linux/timer.h>
 /* Must be included after sched.h. */
 #include <linux/smp_lock.h>
+#include <linux/interrupt.h> /* for spin_lock_bh */
+
 
-#include "bdhandler.h"
 #include "hgfsDevLinux.h"
 #include "hgfsProto.h"
 #include "module.h"
 #include "request.h"
-#include "tcp.h"
 #include "transport.h"
 #include "vm_assert.h"
 
+extern int USE_VMCI;
+
 COMPAT_KTHREAD_DECLARE_STOP_INFO();
 static HgfsTransportChannel *hgfsChannel;     /* Current active channel. */
 static compat_mutex_t hgfsChannelLock;        /* Lock to protect hgfsChannel. */
@@ -157,10 +159,22 @@ HgfsTransportSetupNewChannel(void)
 {
    HgfsTransportChannel *newChannel;
 
+   newChannel = HgfsGetVmciChannel();
+   if (newChannel != NULL) {
+      if (HgfsTransportOpenChannel(newChannel)) {
+         hgfsChannel = newChannel;
+         LOG(10, ("CHANNEL: Vmci channel\n"));
+         return TRUE;
+      }
+   }
+
+   USE_VMCI = 0;
+
    newChannel = HgfsGetVSocketChannel();
    if (newChannel != NULL) {
       if (HgfsTransportOpenChannel(newChannel)) {
          hgfsChannel = newChannel;
+         LOG(10, ("CHANNEL: Vsocket channel\n"));
          return TRUE;
       }
    }
@@ -169,11 +183,13 @@ HgfsTransportSetupNewChannel(void)
    if (newChannel != NULL) {
       if (HgfsTransportOpenChannel(newChannel)) {
          hgfsChannel = newChannel;
+         LOG(10, ("CHANNEL: Tcp channel\n"));
          return TRUE;
       }
    }
 
    newChannel = HgfsGetBdChannel();
+   LOG(10, ("CHANNEL: Bd channel\n"));
    ASSERT(newChannel);
    hgfsChannel = newChannel;
    return HgfsTransportOpenChannel(newChannel);
@@ -201,9 +217,9 @@ HgfsTransportAddPendingRequest(HgfsReq *req)   // IN: Request to add
 {
    ASSERT(req);
 
-   spin_lock(&hgfsRepQueueLock);
+   spin_lock_bh(&hgfsRepQueueLock);
    list_add_tail(&req->list, &hgfsRepPending);
-   spin_unlock(&hgfsRepQueueLock);
+   spin_unlock_bh(&hgfsRepQueueLock);
 }
 
 
@@ -223,14 +239,14 @@ HgfsTransportAddPendingRequest(HgfsReq *req)   // IN: Request to add
  *----------------------------------------------------------------------
  */
 
-static void
+void
 HgfsTransportRemovePendingRequest(HgfsReq *req)   // IN: Request to dequeue
 {
    ASSERT(req);
 
-   spin_lock(&hgfsRepQueueLock);
+   spin_lock_bh(&hgfsRepQueueLock);
    list_del_init(&req->list);
-   spin_unlock(&hgfsRepQueueLock);
+   spin_unlock_bh(&hgfsRepQueueLock);
 }
 
 
@@ -256,7 +272,7 @@ HgfsTransportFlushPendingRequests(void)
 {
    struct HgfsReq *req;
 
-   spin_lock(&hgfsRepQueueLock);
+   spin_lock_bh(&hgfsRepQueueLock);
 
    list_for_each_entry(req, &hgfsRepPending, list) {
       if (req->state == HGFS_REQ_STATE_SUBMITTED) {
@@ -266,7 +282,7 @@ HgfsTransportFlushPendingRequests(void)
       }
    }
 
-   spin_unlock(&hgfsRepQueueLock);
+   spin_unlock_bh(&hgfsRepQueueLock);
 }
 
 /*
@@ -292,7 +308,7 @@ HgfsTransportGetPendingRequest(HgfsHandle id)   // IN: id of the request
 {
    HgfsReq *cur, *req = NULL;
 
-   spin_lock(&hgfsRepQueueLock);
+   spin_lock_bh(&hgfsRepQueueLock);
 
    list_for_each_entry(cur, &hgfsRepPending, list) {
       if (cur->id == id) {
@@ -302,7 +318,7 @@ HgfsTransportGetPendingRequest(HgfsHandle id)   // IN: id of the request
       }
    }
 
-   spin_unlock(&hgfsRepQueueLock);
+   spin_unlock_bh(&hgfsRepQueueLock);
 
    return req;
 }
@@ -345,6 +361,36 @@ HgfsTransportAllocateRequest(size_t bufferSize)   // IN: size of the buffer
    return req;
 }
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * HgfsTransportFreeRequest --
+ *
+ *     Free HGFS request structre using channel-specific free function.
+ *
+ * Results:
+ *     None.
+ *
+ * Side effects:
+ *     None.
+ *
+ *----------------------------------------------------------------------
+ */
+
+void
+HgfsTransportFreeRequest(HgfsReq *req)   // IN: size of the buffer
+{
+   /*
+    * We cannot use hgfsChannel structre because global channel could
+    * changes in the meantime. We remember the channel when we do
+    * allocation and call the same channel for de-allocation. Smart.
+    */
+
+   HgfsTransportChannel *channel = (HgfsTransportChannel *)req->transportId;
+   channel->ops.free(req);
+   return;
+}
+
 
 /*
  *----------------------------------------------------------------------
@@ -370,7 +416,7 @@ HgfsTransportSendRequest(HgfsReq *req)   // IN: Request to send
 
    ASSERT(req);
    ASSERT(req->state == HGFS_REQ_STATE_UNSENT);
-   ASSERT(req->payloadSize <= HGFS_PACKET_MAX);
+   ASSERT(req->payloadSize <= req->bufferSize);
 
    compat_mutex_lock(&hgfsChannelLock);
 
@@ -393,7 +439,7 @@ HgfsTransportSendRequest(HgfsReq *req)   // IN: Request to send
       ASSERT(hgfsChannel->ops.send);
 
       /* If channel changed since we created request we need to adjust */
-      if (req->transportId != hgfsChannel) {
+     if (req->transportId != hgfsChannel) {
 
          HgfsTransportRemovePendingRequest(req);
 
@@ -434,11 +480,13 @@ out:
    compat_mutex_unlock(&hgfsChannelLock);
 
    if (likely(ret == 0)) {
-      /* Send succeeded, wait for the reply */
-      if (wait_event_interruptible(req->queue,
-                                   req->state == HGFS_REQ_STATE_COMPLETED)) {
-         ret = -EINTR; /* Interrupted by some signal. */
-      }
+      /*
+       * Send succeeded, wait for the reply.
+       * Right now, we cannot cancel request once they
+       * are dispatched to the host.
+       */
+      wait_event(req->queue,
+                 req->state == HGFS_REQ_STATE_COMPLETED);
    }
 
    HgfsTransportRemovePendingRequest(req);
