@@ -39,7 +39,7 @@
 #include "vm_assert.h"
 #include "vmci_call_defs.h"
 #include "vmci_defs.h"
-#include "vmciKernelAPI1.h"
+#include "vmciKernelAPI.h"
 
 static Bool HgfsVmciChannelOpen(HgfsTransportChannel *channel);
 static void HgfsVmciChannelClose(HgfsTransportChannel *channel);
@@ -223,7 +223,7 @@ HgfsRequestAsyncShmemDispatch(HgfsAsyncIov *iov, // IN: request vectors
  *      Passes down free pages to the hgfs Server. HgfsServer will use this pages
  *      for sending change notification, oplock breaks etc.
  *
- *      XXX It seems safe to call vmci_datagram_send in atomic context.
+ *      XXX It seems safe to call VMCIDatagram_Send in atomic context.
  *
  * Results:
  *      None
@@ -281,7 +281,7 @@ HgfsVmciChannelPassGuestPages(HgfsTransportChannel *channel) // IN:
    transportHeader->pktType = HGFS_TH_REP_GET_PAGES;
 
    LOG(10, (KERN_WARNING "Sending %d Guest pages \n", i));
-   if ((ret = vmci_datagram_send(dg)) < VMCI_SUCCESS) {
+   if ((ret = VMCIDatagram_Send(dg)) < VMCI_SUCCESS) {
       if (ret == HGFS_VMCI_TRANSPORT_ERROR) {
          LOG(0, (KERN_WARNING "HGFS Transport error occured. Don't blame VMCI\n"));
       }
@@ -328,13 +328,11 @@ HgfsVmciChannelCompleteRequest(uint64 id) // IN: Request ID
    /* Reference is taken here */
    req = HgfsTransportGetPendingRequest(id);
    if (!req) {
-      LOG(0, (KERN_WARNING "No request with id %"FMT64"u \n", id));
       goto exit;
    }
 
    transportStatus = (HgfsVmciTransportStatus *)req->buffer;
    if (transportStatus->status != HGFS_TS_IO_COMPLETE) {
-      LOG(0, (KERN_WARNING "Request not completed with id %"FMT64"u \n", id));
       goto exit;
    }
 
@@ -434,7 +432,6 @@ HgfsVmciChannelOpen(HgfsTransportChannel *channel) // IN: Channel
 
    ASSERT(channel->status == HGFS_CHANNEL_NOTCONNECTED);
    ASSERT(channel->priv == NULL);
-   memset(&gHgfsShmemPages, 0, sizeof gHgfsShmemPages);
 
    if (USE_VMCI == 0) {
       goto error;
@@ -447,24 +444,23 @@ HgfsVmciChannelOpen(HgfsTransportChannel *channel) // IN: Channel
       goto error;
    }
 
-   ret = vmci_datagram_create_handle(
-                          VMCI_INVALID_ID,        /* Resource ID */
-                          VMCI_FLAG_DG_NONE,      /* Flags */
-                          HgfsVmciChannelCallback,/* Datagram Recv Callback */
-                          NULL,                   /* Callback data */
-                          channel->priv);         /* VMCI outhandle */
+   ret = VMCIDatagram_CreateHnd(VMCI_INVALID_ID,        /* Resource ID */
+                                VMCI_FLAG_DG_NONE,      /* Flags */
+                                HgfsVmciChannelCallback,/* Datagram Recv Callback */
+                                NULL,                   /* Callback data */
+                                channel->priv);         /* VMCI outhandle */
    if (ret != VMCI_SUCCESS) {
       LOG(1, (KERN_WARNING "Failed to create VMCI handle %d\n", ret));
       goto error;
    }
 
-   gHgfsShmemPages.list = kmalloc(sizeof *gHgfsShmemPages.list * HGFS_VMCI_SHMEM_PAGES,
+   gHgfsShmemPages.list = kmalloc(sizeof gHgfsShmemPages.list * HGFS_VMCI_SHMEM_PAGES,
                                   GFP_KERNEL);
    if (!gHgfsShmemPages.list) {
       goto error;
    }
 
-   memset(gHgfsShmemPages.list, 0, sizeof *gHgfsShmemPages.list * HGFS_VMCI_SHMEM_PAGES);
+   memset(gHgfsShmemPages.list, 0, sizeof gHgfsShmemPages.list * HGFS_VMCI_SHMEM_PAGES);
 
    for (i = 0; i < HGFS_VMCI_SHMEM_PAGES; i++) {
       gHgfsShmemPages.list[i].va = __get_free_page(GFP_KERNEL);
@@ -486,10 +482,6 @@ HgfsVmciChannelOpen(HgfsTransportChannel *channel) // IN: Channel
 
    ret = HgfsVmciChannelPassGuestPages(channel);
    if (!ret) {
-      for (i = 0; i < gHgfsShmemPages.totalPageCount; i++) {
-         LOG(1, (KERN_WARNING "Freeing pages\n"));
-         free_page(gHgfsShmemPages.list[i].va);
-      }
       LOG(1, (KERN_WARNING "Failed to pass pages to the guest %d\n", ret));
       goto error;
    }
@@ -543,16 +535,10 @@ HgfsVmciChannelTerminateSession(HgfsTransportChannel *channel) {
    transportHeader->pktType = HGFS_TH_TERMINATE_SESSION;
 
    LOG(1, (KERN_WARNING "Terminating session with host \n"));
-   if ((ret = vmci_datagram_send(dg)) < VMCI_SUCCESS) {
+   if ((ret = VMCIDatagram_Send(dg)) < VMCI_SUCCESS) {
       if (ret == HGFS_VMCI_TRANSPORT_ERROR) {
          LOG(0, (KERN_WARNING "HGFS Transport error occured. Don't blame VMCI\n"));
       }
-      LOG(0, (KERN_WARNING "Cannot communicate with Server.\n"));
-   } else {
-      int i;
-      for (i = 0; i < gHgfsShmemPages.totalPageCount; i++) {
-         free_page(gHgfsShmemPages.list[i].va);
-      }
    }
 
    kfree(dg);
@@ -581,7 +567,7 @@ HgfsVmciChannelClose(HgfsTransportChannel *channel) // IN: Channel
 {
    ASSERT(channel->priv != NULL);
    HgfsVmciChannelTerminateSession(channel);
-   vmci_datagram_destroy_handle(*(VMCIHandle *)channel->priv);
+   VMCIDatagram_DestroyHnd(*(VMCIHandle *)channel->priv);
    kfree(channel->priv);
    kfree(gHgfsShmemPages.list);
    channel->priv = NULL;
@@ -683,7 +669,6 @@ HgfsVmciChannelSend(HgfsTransportChannel *channel, // IN: Channel
    int j;
 
    ASSERT(req);
-   ASSERT(req->buffer);
    ASSERT(req->state == HGFS_REQ_STATE_UNSENT || req->state == HGFS_REQ_STATE_ALLOCATED);
    ASSERT(req->payloadSize <= req->bufferSize);
 
@@ -733,11 +718,10 @@ HgfsVmciChannelSend(HgfsTransportChannel *channel, // IN: Channel
    ASSERT(total == 0);
    ASSERT(bufferSize == req->bufferSize + sizeof (HgfsVmciTransportStatus));
 
-   LOG(0, (KERN_WARNING "Size of request is %Zu\n", req->payloadSize));
+   LOG(8, ("Size of request is %Zu\n", req->payloadSize));
 
    for (j = 0; j < req->numEntries; j++, iovCount++) {
       /* I will have to probably do page table walk here, haven't figured it out yet */
-      ASSERT(req->dataPacket);
       transportHeader->iov[iovCount].pa = page_to_phys(req->dataPacket[j].page);
       transportHeader->iov[iovCount].pa += req->dataPacket[j].offset;
       transportHeader->iov[iovCount].len = req->dataPacket[j].len;
@@ -755,14 +739,14 @@ HgfsVmciChannelSend(HgfsTransportChannel *channel, // IN: Channel
    transportStatus->size = req->bufferSize + sizeof (HgfsVmciTransportStatus);
 
    /*
-    * Don't try to set req->state after vmci_datagram_send().
+    * Don't try to set req->state after VMCIDatagram_Send().
     * It may be too late then. We could have received a datagram by then and
     * datagram handler expects request's state to be submitted.
     */
    req->state = HGFS_REQ_STATE_SUBMITTED;
    id = req->id;
 
-   if ((ret = vmci_datagram_send(dg)) < VMCI_SUCCESS) {
+   if ((ret = VMCIDatagram_Send(dg)) < VMCI_SUCCESS) {
       if (ret == HGFS_VMCI_TRANSPORT_ERROR) {
          LOG(0, (KERN_WARNING "HGFS Transport error occured. Don't blame VMCI\n"));
       } else if (ret == HGFS_VMCI_VERSION_MISMATCH) {
