@@ -78,7 +78,7 @@ static int vmxnet_close(struct net_device *dev);
 static void vmxnet_set_multicast_list(struct net_device *dev);
 static int vmxnet_set_mac_address(struct net_device *dev, void *addr);
 static struct net_device_stats *vmxnet_get_stats(struct net_device *dev);
-#ifdef HAVE_CHANGE_MTU
+#if defined(HAVE_CHANGE_MTU) || defined(HAVE_NET_DEVICE_OPS)
 static int vmxnet_change_mtu(struct net_device *dev, int new_mtu);
 #endif
 
@@ -90,6 +90,9 @@ static void vmxnet_release_private_data(Vmxnet_Private *lp,
 static int vmxnet_probe_device(struct pci_dev *pdev, const struct pci_device_id *id);
 static void vmxnet_remove_device(struct pci_dev *pdev);
 
+static Bool vmxnet_alloc_shared_mem(struct pci_dev *pdev, size_t size,
+				    void **vaOut, dma_addr_t *paOut);
+
 #ifdef CONFIG_PM
 
 /*
@@ -271,7 +274,7 @@ static struct pci_driver vmxnet_driver = {
 #endif
                                          };
 
-#ifdef HAVE_CHANGE_MTU
+#if defined(HAVE_CHANGE_MTU) || defined(HAVE_NET_DEVICE_OPS)
 static int
 vmxnet_change_mtu(struct net_device *dev, int new_mtu)
 {
@@ -1364,7 +1367,7 @@ vmxnet_probe_features(struct net_device *dev, // IN:
    /* check if this is enhanced vmxnet device */
    if ((lp->features & VMXNET_FEATURE_TSO) && 
        (lp->features & VMXNET_FEATURE_JUMBO_FRAME)) {
-	enhanced = TRUE;
+      enhanced = TRUE;
    }
 
    /* determine rx/tx ring sizes */ 
@@ -1391,7 +1394,8 @@ vmxnet_probe_features(struct net_device *dev, // IN:
       numRxBuffers2 = 1;
    }
 
-   printk("numRxBuffers = %d, numRxBuffers2 = %d\n", numRxBuffers, numRxBuffers2);
+   printk(KERN_INFO "numRxBuffers = %d, numRxBuffers2 = %d\n", 
+	  numRxBuffers, numRxBuffers2);
    if (lp->tso || lp->jumboFrame) {
       maxNumTxBuffers = VMXNET2_MAX_NUM_TX_BUFFERS_TSO;
       defNumTxBuffers = VMXNET2_DEFAULT_NUM_TX_BUFFERS_TSO;
@@ -1413,13 +1417,10 @@ vmxnet_probe_features(struct net_device *dev, // IN:
               numRxBuffers, numRxBuffers2, (uint32)sizeof(Vmxnet2_RxRingEntry),
               numTxBuffers, (uint32)sizeof(Vmxnet2_TxRingEntry),
               (int)lp->ddSize);
-   lp->dd = kmalloc(lp->ddSize, GFP_KERNEL | GFP_DMA);
-   if (!lp->dd) {
+   if (!vmxnet_alloc_shared_mem(lp->pdev, lp->ddSize, (void **)&lp->dd, &lp->ddPA)) {
       printk(KERN_ERR "Unable to allocate memory for driver data\n");
       return FALSE;
    }
-   lp->ddPA = compat_pci_map_single(lp->pdev, lp->dd, lp->ddSize,
-                                    PCI_DMA_BIDIRECTIONAL);
    memset(lp->dd, 0, lp->ddSize);
    spin_lock_init(&lp->txLock);
    lp->numRxBuffers = numRxBuffers;
@@ -1460,12 +1461,11 @@ vmxnet_probe_features(struct net_device *dev, // IN:
 
    if (lp->partialHeaderCopyEnabled) {
       lp->txBufferSize = numTxBuffers * TX_PKT_HEADER_SIZE;
-      lp->txBufferStart = kmalloc(lp->txBufferSize, GFP_KERNEL | GFP_DMA);
-      if (lp->txBufferStart) {
-         lp->txBufferPA = compat_pci_map_single(lp->pdev, lp->txBufferStart,
-                                                lp->txBufferSize, PCI_DMA_TODEVICE);
-         lp->dd->txBufferPhysStart = lp->txBufferPA;
-         lp->dd->txBufferPhysLength = lp->txBufferSize;
+
+      if (vmxnet_alloc_shared_mem(lp->pdev, lp->txBufferSize,
+				  (void **)&lp->txBufferStart, &lp->txBufferPA)) {
+	 lp->dd->txBufferPhysStart = (uint32)lp->txBufferPA;
+         lp->dd->txBufferPhysLength = (uint32)lp->txBufferSize;
          lp->dd->txPktMaxSize = TX_PKT_HEADER_SIZE;
       } else {
          lp->partialHeaderCopyEnabled = FALSE;
@@ -1477,6 +1477,49 @@ vmxnet_probe_features(struct net_device *dev, // IN:
    return TRUE;
 }
 
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * vmxnet_alloc_shared_mem --
+ *
+ *      Attempts to allocate dma-able memory that uses a 32-bit PA.
+ *
+ * Results:
+ *      TRUE on success, otherwise FALSE.
+ *
+ * Side effects:
+ *      None.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+#define FITS_IN_32_BITS(_x) ((_x) == ((_x) & 0xFFFFFFFF))
+
+static Bool 
+vmxnet_alloc_shared_mem(struct pci_dev *pdev, // IN:
+			size_t size,          // IN:
+			void **vaOut,         // OUT:
+			dma_addr_t *paOut)    // OUT:
+{
+   void *va = NULL;
+   dma_addr_t pa = 0;
+
+   /* DMA-mapping.txt says 32-bit DMA by default */
+   va = compat_pci_alloc_consistent(pdev, size, &pa);
+   if (!va) {
+      *vaOut = NULL;
+      *paOut = 0;
+      return FALSE;
+   }
+
+   VMXNET_ASSERT(FITS_IN_32_BITS(pa) &&
+		 FITS_IN_32_BITS((uint64)pa + (size - 1)));
+   *vaOut = va;
+   *paOut = pa;
+
+   return TRUE;
+}
+
 
 /*
  *-----------------------------------------------------------------------------
@@ -1547,17 +1590,14 @@ vmxnet_release_private_data(Vmxnet_Private *lp,   // IN:
 {
 #ifdef VMXNET_DO_ZERO_COPY
    if (lp->partialHeaderCopyEnabled && lp->txBufferStart) {
-      compat_pci_unmap_single(pdev, lp->txBufferPA, lp->txBufferSize,
-                              PCI_DMA_TODEVICE);
-      kfree(lp->txBufferStart);
+      compat_pci_free_consistent(pdev, lp->txBufferSize, 
+				 lp->txBufferStart, lp->txBufferPA);
       lp->txBufferStart = NULL;
    }
 #endif
 
    if (lp->dd) {
-      compat_pci_unmap_single(lp->pdev, lp->ddPA, lp->ddSize,
-                              PCI_DMA_BIDIRECTIONAL);
-      kfree(lp->dd);
+      compat_pci_free_consistent(pdev, lp->ddSize, lp->dd, lp->ddPA);
       lp->dd = NULL;
    }
 }
@@ -1886,7 +1926,7 @@ vmxnet_open(struct net_device *dev)
       return -ENOMEM;
    }
 
-   ddPA = VMXNET_GET_LO_ADDR(lp->ddPA); // lp->dd was allocated out of ZONE_DMA32
+   ddPA = VMXNET_GET_LO_ADDR(lp->ddPA);
    outl(ddPA, ioaddr + VMXNET_INIT_ADDR);
    outl(lp->dd->length, ioaddr + VMXNET_INIT_LENGTH);
 
@@ -2003,7 +2043,7 @@ vmxnet_map_pkt(struct sk_buff *skb,
 
    VMXNET_ASSERT(startSgIdx < VMXNET2_SG_DEFAULT_LENGTH);
 
-   lp->numTxPending ++;
+   lp->numTxPending++;
    tb = &lp->txBufInfo[dd->txDriverNext];
    xre = &lp->txRing[dd->txDriverNext];
 
@@ -2846,9 +2886,10 @@ vmxnet_close(struct net_device *dev)
       //Will go ahead and free these skb's anyways (possibly dangerous,
       //but seems to work in practice)
       if (lp->numTxPending > 0) {
-         printk(KERN_EMERG "vmxnet_close: Failed to finish all pending tx.\n"
+         printk(KERN_EMERG "vmxnet_close: %s failed to finish all pending tx (%d).\n"
 	        "Is the related vmxnet device disabled?\n"
-                "This virtual machine may be in an inconsistent state.\n");
+                "This virtual machine may be in an inconsistent state.\n", 
+		dev->name, lp->numTxPending);
          lp->numTxPending = 0;
       }
    }
