@@ -105,8 +105,7 @@ VSockVmciNotifyWaitingWrite(VSockVmciSock *vsk)    // IN
     *   if writeNotifyWindow > bufferReady then notify
     * as freeSpace == ConsumeSize - bufferReady.
     */
-   retval = VMCIQueue_FreeSpace(vsk->consumeQ, vsk->produceQ, vsk->consumeSize) >
-            notifyLimit;
+   retval = VMCIQPair_ConsumeFreeSpace(vsk->qpair) > notifyLimit;
 #ifdef VSOCK_OPTIMIZATION_FLOW_CONTROL
    if (retval) {
       /*
@@ -154,8 +153,7 @@ VSockVmciNotifyWaitingRead(VSockVmciSock *vsk)  // IN
     * not require a protocol change and will retain compatibility between
     * endpoints with mixed versions of this function.
     */
-   return VMCIQueue_BufReady(vsk->produceQ,
-			     vsk->consumeQ, vsk->produceSize) > 0;
+   return VMCIQPair_ProduceBufReady(vsk->qpair) > 0;
 #else
    return TRUE;
 #endif
@@ -339,7 +337,7 @@ VSockVmciSendWaitingRead(struct sock *sk,    // IN
              vsk->consumeSize);
    }
 
-   VMCIQueue_GetPointers(vsk->consumeQ, vsk->produceQ, &tail, &head);
+   VMCIQPair_GetConsumeIndexes(vsk->qpair, &tail, &head);
    roomLeft = vsk->consumeSize - head;
    if (roomNeeded >= roomLeft) {
       waitingInfo.offset = roomNeeded - roomLeft;
@@ -398,7 +396,7 @@ VSockVmciSendWaitingWrite(struct sock *sk,   // IN
       return TRUE;
    }
 
-   VMCIQueue_GetPointers(vsk->produceQ, vsk->consumeQ, &tail, &head);
+   VMCIQPair_GetProduceIndexes(vsk->qpair, &tail, &head);
    roomLeft = vsk->produceSize - tail;
    if (roomNeeded + 1 >= roomLeft) {
       /* Wraps around to current generation. */
@@ -831,8 +829,9 @@ VSockVmciNotifyPktRecvPreDequeue(struct sock *sk,               // IN
     * socket locked we should copy at least ready bytes.
     */
 #if defined(VSOCK_OPTIMIZATION_WAITING_NOTIFY)
-   VMCIQueue_GetPointers(vsk->consumeQ, vsk->produceQ,
-                         &data->produceTail, &data->consumeHead);
+   VMCIQPair_GetConsumeIndexes(vsk->qpair,
+                               &data->produceTail,
+                               &data->consumeHead);
 #endif
 
    return 0;
@@ -989,8 +988,9 @@ VSockVmciNotifyPktSendPreEnqueue(struct sock *sk,               // IN
    vsk = vsock_sk(sk);
 
 #if defined(VSOCK_OPTIMIZATION_WAITING_NOTIFY)
-      VMCIQueue_GetPointers(vsk->produceQ, vsk->consumeQ,
-                            &data->produceTail, &data->consumeHead);
+      VMCIQPair_GetProduceIndexes(vsk->qpair,
+                                  &data->produceTail,
+                                  &data->consumeHead);
 #endif
 
    return 0;;
