@@ -86,7 +86,6 @@ extern "C" {
 #define BALLOON_DEBUG_VERBOSE   0
 
 #define BALLOON_STATS
-#define BALLOON_STATS_PROCFS
 
 /*
  * Includes
@@ -95,6 +94,7 @@ extern "C" {
 #include "os.h"
 #include "backdoor.h"
 #include "backdoor_balloon.h"
+#include "dbllnklst.h"
 #include "vmballoon.h"
 
 /*
@@ -105,9 +105,6 @@ extern "C" {
 #define NULL 0
 #endif
 
-#define BALLOON_NAME                    "vmmemctl"
-#define BALLOON_NAME_VERBOSE            "VMware memory control driver"
-
 #define BALLOON_PROTOCOL_VERSION        2
 
 #define BALLOON_CHUNK_PAGES             1000
@@ -138,7 +135,7 @@ extern "C" {
  */
 #define BALLOON_PAGE_ALLOC_FAILURE      1000
 
-// Maximum number of page allocations without yielding processor
+/* Maximum number of page allocations without yielding processor */
 #define BALLOON_ALLOC_YIELD_THRESHOLD   1024
 
 
@@ -148,24 +145,25 @@ extern "C" {
 
 typedef struct BalloonChunk {
    PageHandle page[BALLOON_CHUNK_PAGES];
-   uint32 nextPage;
-   struct BalloonChunk *prev;
-   struct BalloonChunk *next;
+   uint32 pageCount;
+   DblLnkLst_Links node;
 } BalloonChunk;
 
 typedef struct {
    PageHandle page[BALLOON_ERROR_PAGES];
-   uint32 nextPage;
+   uint32 pageCount;
 } BalloonErrorPages;
 
 typedef struct {
    /* sets of reserved physical pages */
-   BalloonChunk *chunks;
+   DblLnkLst_Links chunks;
    int nChunks;
 
    /* transient list of non-balloonable pages */
    BalloonErrorPages errors;
 
+   BalloonGuest guestType;
+
    /* balloon size */
    int nPages;
    int nPagesTarget;
@@ -189,7 +187,6 @@ typedef struct {
  */
 
 static Balloon globalBalloon;
-static Bool timerStarted;
 
 /*
  * Balloon operations
@@ -198,7 +195,6 @@ static int  BalloonPageAlloc(Balloon *b, BalloonPageAllocType allocType);
 static int  BalloonPageFree(Balloon *b, int monitorUnlock);
 static int  BalloonAdjustSize(Balloon *b, uint32 target);
 static void BalloonReset(Balloon *b);
-static void BalloonTimerHandler(void *clientData);
 
 /*
  * Backdoor Operations
@@ -219,122 +215,6 @@ static int BalloonMonitorUnlockPage(Balloon *b, PageHandle handle);
 #define STATS_INC(stat)
 #endif
 
-/*
- * Macros to generate operations for simple lists of OBJ.
- * OBJ must contain next and prev fields.
- */
-
-#define GENERATE_LIST_INSERT(OBJ)                       \
-static void OBJ ## _Insert(OBJ **head, OBJ *obj)        \
-{                                                       \
-   OBJ *h = *head;                                      \
-                                                        \
-   /* add element to head of list */                    \
-   obj->next = h;                                       \
-   if (h != NULL) {                                     \
-      h->prev = obj;                                    \
-   }                                                    \
-   *head = obj;                                         \
-   obj->prev = NULL;                                    \
-}
-
-#define GENERATE_LIST_REMOVE(OBJ)                       \
-static void OBJ ## _Remove(OBJ **head, OBJ *obj)        \
-{                                                       \
-    /* splice element out of list */                    \
-    if (obj->prev != NULL) {                            \
-      obj->prev->next = obj->next;                      \
-    } else {                                            \
-      *head = obj->next;                                \
-    }                                                   \
-    if (obj->next != NULL) {                            \
-      obj->next->prev = obj->prev;                      \
-    }                                                   \
-}
-
-/*
- * List Operations
- */
-
-GENERATE_LIST_INSERT(BalloonChunk);
-GENERATE_LIST_REMOVE(BalloonChunk);
-
-
-/*
- *----------------------------------------------------------------------
- *
- * BalloonProcRead --
- *
- *      Ballon driver status reporting routine.  Note that this is only
- *      used for Linux.
- *
- * Results:
- *      Writes ASCII status information into "buf".
- *      Returns number of bytes written.
- *
- * Side effects:
- *      None.
- *
- *----------------------------------------------------------------------
- */
-
-static int
-BalloonProcRead(char *buf,   // OUT
-                size_t size) // IN
-{
-   int len = 0;
-   BalloonStats stats;
-
-   Balloon_GetStats(&stats);
-
-   /* format size info */
-   len += OS_Snprintf(buf + len, size - len,
-                     "target:             %8d pages\n"
-                     "current:            %8d pages\n",
-                     stats.nPagesTarget,
-                     stats.nPages);
-
-   /* format rate info */
-   len += OS_Snprintf(buf + len, size - len,
-                     "rateNoSleepAlloc:   %8d pages/sec\n"
-                     "rateSleepAlloc:     %8d pages/sec\n"
-                     "rateFree:           %8d pages/sec\n",
-                     BALLOON_NOSLEEP_ALLOC_MAX,
-                     stats.rateAlloc,
-                     stats.rateFree);
-
-#ifdef BALLOON_STATS_PROCFS
-   len += OS_Snprintf(buf + len, size - len,
-                     "\n"
-                     "timer:              %8u\n"
-                     "start:              %8u (%4u failed)\n"
-                     "guestType:          %8u (%4u failed)\n"
-                     "lock:               %8u (%4u failed)\n"
-                     "unlock:             %8u (%4u failed)\n"
-                     "target:             %8u (%4u failed)\n"
-                     "primNoSleepAlloc:   %8u (%4u failed)\n"
-                     "primCanSleepAlloc:  %8u (%4u failed)\n"
-                     "primFree:           %8u\n"
-                     "errAlloc:           %8u\n"
-                     "errFree:            %8u\n",
-                     stats.timer,
-                     stats.start, stats.startFail,
-                     stats.guestType, stats.guestTypeFail,
-                     stats.lock,  stats.lockFail,
-                     stats.unlock, stats.unlockFail,
-                     stats.target, stats.targetFail,
-                     stats.primAlloc[BALLOON_PAGE_ALLOC_NOSLEEP],
-                     stats.primAllocFail[BALLOON_PAGE_ALLOC_NOSLEEP],
-                     stats.primAlloc[BALLOON_PAGE_ALLOC_CANSLEEP],
-                     stats.primAllocFail[BALLOON_PAGE_ALLOC_CANSLEEP],
-                     stats.primFree,
-                     stats.primErrorPageAlloc,
-                     stats.primErrorPageFree);
-#endif
-
-   return len;
-}
-
 
 /*
  *----------------------------------------------------------------------
@@ -354,15 +234,11 @@ BalloonProcRead(char *buf,   // OUT
  *----------------------------------------------------------------------
  */
 
-void
-Balloon_GetStats(BalloonStats *stats) // OUT
+const BalloonStats *
+Balloon_GetStats(void)
 {
    Balloon *b = &globalBalloon;
-
-   /*
-    * Copy statistics out of global structure.
-    */
-   OS_MemCopy(stats, &b->stats, sizeof *stats);
+   BalloonStats *stats = &b->stats;
 
    /*
     * Fill in additional information about size and rates, which is
@@ -370,8 +246,11 @@ Balloon_GetStats(BalloonStats *stats) // OUT
     */
    stats->nPages = b->nPages;
    stats->nPagesTarget = b->nPagesTarget;
+   stats->rateNoSleepAlloc = BALLOON_NOSLEEP_ALLOC_MAX;
    stats->rateAlloc = b->rateAlloc;
    stats->rateFree = b->rateFree;
+
+   return stats;
 }
 
 
@@ -410,6 +289,7 @@ BalloonChunk_Create(void)
 
    /* initialize */
    OS_MemZero(chunk, sizeof *chunk);
+   DblLnkLst_Init(&chunk->node);
 
    return chunk;
 }
@@ -439,39 +319,6 @@ BalloonChunk_Destroy(BalloonChunk *chunk) // IN
 }
 
 
-/*
- *----------------------------------------------------------------------
- *
- * Balloon_Init --
- *
- *      Initializes device state for balloon "b".
- *
- * Results:
- *      Returns BALLOON_SUCCESS.
- *
- * Side effects:
- *      None.
- *
- *----------------------------------------------------------------------
- */
-
-static int
-Balloon_Init(Balloon *b) // IN
-{
-   /* clear state */
-   OS_MemZero(b, sizeof *b);
-
-   /* initialize rates */
-   b->rateAlloc = BALLOON_RATE_ALLOC_MAX;
-   b->rateFree  = BALLOON_RATE_FREE_MAX;
-
-   /* initialize reset flag */
-   b->resetFlag = 1;
-
-   return BALLOON_SUCCESS;
-}
-
-
 /*
  *----------------------------------------------------------------------
  *
@@ -544,26 +391,25 @@ BalloonReset(Balloon *b) // IN
 /*
  *----------------------------------------------------------------------
  *
- * BalloonTimerHandler --
+ * Balloon_QueryAndExecute --
  *
- *      Balloon bottom half handler.  Contacts monitor via backdoor
- *      to obtain balloon size target, and starts adjusting balloon
- *      size to achieve target by allocating or deallocating pages.
- *      Resets balloon if requested by the monitor.
+ *      Contacts monitor via backdoor to obtain balloon size target,
+ *      and starts adjusting balloon size to achieve target by allocating
+ *      or deallocating pages. Resets balloon if requested by the monitor.
  *
  * Results:
  *      None.
  *
  * Side effects:
- *      Schedules next execution of balloon timer handler.
+ *      None.
  *
  *----------------------------------------------------------------------
  */
 
-static void
-BalloonTimerHandler(void *clientData) // IN
+void
+Balloon_QueryAndExecute(void)
 {
-   Balloon *b = (Balloon *) clientData;
+   Balloon *b = &globalBalloon;
    uint32 target = 0; // Silence compiler warning.
    int status;
 
@@ -614,15 +460,14 @@ BalloonErrorPagesAlloc(Balloon *b,      // IN
                        PageHandle page) // IN
 {
    /* fail if list already full */
-   if (b->errors.nextPage >= BALLOON_ERROR_PAGES) {
-      return(BALLOON_FAILURE);
+   if (b->errors.pageCount >= BALLOON_ERROR_PAGES) {
+      return BALLOON_FAILURE;
    }
 
    /* add page to list */
-   b->errors.page[b->errors.nextPage] = page;
-   b->errors.nextPage++;
+   b->errors.page[b->errors.pageCount++] = page;
    STATS_INC(b->stats.primErrorPageAlloc);
-   return(BALLOON_SUCCESS);
+   return BALLOON_SUCCESS;
 }
 
 /*
@@ -648,12 +493,12 @@ BalloonErrorPagesFree(Balloon *b) // IN
    unsigned int i;
 
    /* free all non-balloonable "error" pages */
-   for (i = 0; i < b->errors.nextPage; i++) {
+   for (i = 0; i < b->errors.pageCount; i++) {
       OS_ReservedPageFree(b->errors.page[i]);
       b->errors.page[i] = PAGE_HANDLE_INVALID;
       STATS_INC(b->stats.primErrorPageFree);
    }
-   b->errors.nextPage = 0;
+   b->errors.pageCount = 0;
 }
 
 
@@ -678,7 +523,7 @@ static int
 BalloonPageAlloc(Balloon *b,                     // IN
                  BalloonPageAllocType allocType) // IN
 {
-   BalloonChunk *chunk;
+   BalloonChunk *chunk = NULL;
    PageHandle page;
    int status;
 
@@ -704,9 +549,17 @@ BalloonPageAlloc(Balloon *b,                     // IN
       return BALLOON_PAGE_ALLOC_FAILURE;
    }
 
-   /* find chunk with space, create if necessary */
-   chunk = b->chunks;
-   if ((chunk == NULL) || (chunk->nextPage >= BALLOON_CHUNK_PAGES)) {
+   /* Find chunk with free space, create it if necessary. */
+   if (DblLnkLst_IsLinked(&b->chunks)) {
+      /* Get first chunk from the list */
+      chunk = DblLnkLst_Container(b->chunks.next, BalloonChunk, node);
+      if (chunk->pageCount >= BALLOON_CHUNK_PAGES) {
+         /* This chunk is full. */
+         chunk = NULL;
+      }
+   }
+
+   if (chunk == NULL) {
       /* create new chunk */
       chunk = BalloonChunk_Create();
       if (chunk == NULL) {
@@ -714,7 +567,8 @@ BalloonPageAlloc(Balloon *b,                     // IN
          OS_ReservedPageFree(page);
          return BALLOON_PAGE_ALLOC_FAILURE;
       }
-      BalloonChunk_Insert(&b->chunks, chunk);
+
+      DblLnkLst_LinkFirst(&b->chunks, &chunk->node);
 
       /* update stats */
       b->nChunks++;
@@ -735,8 +589,7 @@ BalloonPageAlloc(Balloon *b,                     // IN
    }
 
    /* track allocated page */
-   chunk->page[chunk->nextPage] = page;
-   chunk->nextPage++;
+   chunk->page[chunk->pageCount++] = page;
 
    /* update balloon size */
    b->nPages++;
@@ -767,38 +620,41 @@ static int
 BalloonPageFree(Balloon *b,        // IN
                 int monitorUnlock) // IN
 {
-   BalloonChunk *chunk;
+   DblLnkLst_Links *node, *next;
+   BalloonChunk *chunk = NULL;
    PageHandle page;
    int status;
 
-   chunk = b->chunks;
+   DblLnkLst_ForEachSafe(node, next, &b->chunks) {
+      chunk = DblLnkLst_Container(node, BalloonChunk, node);
+      if (chunk->pageCount > 0) {
+         break;
+      }
 
-   while ((chunk != NULL) && (chunk->nextPage == 0)) {
       /* destroy empty chunk */
-      BalloonChunk_Remove(&b->chunks, chunk);
+      DblLnkLst_Unlink1(node);
       BalloonChunk_Destroy(chunk);
+      chunk = NULL;
 
       /* update stats */
       b->nChunks--;
-
-      chunk = b->chunks;
    }
 
-   if (chunk == NULL) {
-      return(BALLOON_FAILURE);
+   if (!chunk) {
+      /* We could not find a single non-empty chunk. */
+      return BALLOON_FAILURE;
    }
 
-   /* select page to deallocate */
-   chunk->nextPage--;
-   page = chunk->page[chunk->nextPage];
+   /* deallocate last page */
+   page = chunk->page[--chunk->pageCount];
 
    /* inform monitor via backdoor */
    if (monitorUnlock) {
       status = BalloonMonitorUnlockPage(b, page);
       if (status != BALLOON_SUCCESS) {
          /* reset next pointer, fail */
-         chunk->nextPage++;
-         return(status);
+         chunk->pageCount++;
+         return status;
       }
    }
 
@@ -810,9 +666,9 @@ BalloonPageFree(Balloon *b,        // IN
    b->nPages--;
 
    /* reclaim chunk, if empty */
-   if (chunk->nextPage == 0) {
+   if (chunk->pageCount == 0) {
       /* destroy empty chunk */
-      BalloonChunk_Remove(&b->chunks, chunk);
+      DblLnkLst_Unlink1(&chunk->node);
       BalloonChunk_Destroy(chunk);
 
       /* update stats */
@@ -947,7 +803,7 @@ BalloonInflate(Balloon *b,    // IN
              * pages, and fail.
              */
             BalloonErrorPagesFree(b);
-            return(status);
+            return status;
          }
          /*
           * NOSLEEP page allocation failed, so the guest is under memory
@@ -972,7 +828,7 @@ BalloonInflate(Balloon *b,    // IN
       BalloonIncreaseRateAlloc(b, nAllocNoSleep);
       /* release non-balloonable pages, succeed */
       BalloonErrorPagesFree(b);
-      return(BALLOON_SUCCESS);
+      return BALLOON_SUCCESS;
    } else {
       /*
        * NOSLEEP allocation failed, so the guest is under memory pressure.
@@ -983,7 +839,7 @@ BalloonInflate(Balloon *b,    // IN
          BalloonIncreaseRateAlloc(b, i);
          /* release non-balloonable pages, succeed */
          BalloonErrorPagesFree(b);
-         return(BALLOON_SUCCESS);
+         return BALLOON_SUCCESS;
       } else {
          /* update successful NOSLEEP allocations, and proceed */
          nAllocNoSleep = i;
@@ -1015,7 +871,7 @@ BalloonInflate(Balloon *b,    // IN
          }
          /* release non-balloonable pages, fail */
          BalloonErrorPagesFree(b);
-         return(status);
+         return status;
       }
 
       if (++allocations > BALLOON_ALLOC_YIELD_THRESHOLD) {
@@ -1032,7 +888,7 @@ BalloonInflate(Balloon *b,    // IN
 
    /* release non-balloonable pages, succeed */
    BalloonErrorPagesFree(b);
-   return(BALLOON_SUCCESS);
+   return BALLOON_SUCCESS;
 }
 
 
@@ -1070,7 +926,7 @@ BalloonDeflate(Balloon *b,    // IN
             /* quickly decrease rate if error */
             b->rateFree = MAX(b->rateFree / 2, BALLOON_RATE_FREE_MIN);
          }
-         return(status);
+         return status;
       }
    }
 
@@ -1105,23 +961,14 @@ static int
 BalloonAdjustSize(Balloon *b,    // IN
                   uint32 target) // IN
 {
-   /* done if already at target */
-   if (b->nPages == target) {
-      return(BALLOON_SUCCESS);
-   }
-
-   /* inflate balloon if below target */
    if (b->nPages < target) {
       return BalloonInflate(b, target);
-   }
-
-   /* deflate balloon if above target */
-   if (b->nPages > target) {
+   } else if (b->nPages > target) {
       return BalloonDeflate(b, target);
+   } else {
+      /* already at target */
+      return BALLOON_SUCCESS;
    }
-
-   /* not reached */
-   return(BALLOON_FAILURE);
 }
 
 
@@ -1188,15 +1035,13 @@ static int
 BalloonMonitorGuestType(Balloon *b) // IN
 {
    uint32 status, target;
-   BalloonGuest identity;
    Backdoor_proto bp;
 
-   identity = OS_Identity();
-   ASSERT(identity == (uint32) identity);
+   ASSERT(b->guestType == (uint32) b->guestType);
 
    /* prepare backdoor args */
    bp.in.cx.halfs.low = BALLOON_BDOOR_CMD_GUEST_ID;
-   bp.in.size = identity;
+   bp.in.size = b->guestType;
 
    /* invoke backdoor */
    Backdoor_Balloon(&bp);
@@ -1414,57 +1259,45 @@ BalloonMonitorUnlockPage(Balloon *b,        // IN
 /*
  *----------------------------------------------------------------------
  *
- * Balloon_ModuleInit --
+ * Balloon_Init --
  *
- *      Startup the balloon module.
+ *      Initializes state oif the balloon.
  *
  * Results:
- *      On success: BALLOON_SUCCESS
- *      On failure: BALLOON_FAILURE
+ *      Returns TRUE on success (always).
  *
  * Side effects:
- *      None
+ *      None.
  *
  *----------------------------------------------------------------------
  */
 
-int
-Balloon_ModuleInit(void)
+Bool
+Balloon_Init(BalloonGuest guestType)
 {
-   static int initialized = 0;
    Balloon *b = &globalBalloon;
 
-   /* initialize only once */
-   if (initialized++) {
-      return BALLOON_FAILURE;
-   }
+   DblLnkLst_Init(&b->chunks);
 
-   /* os-specific initialization */
-   if (!OS_Init(BALLOON_NAME, BALLOON_NAME_VERBOSE, BalloonProcRead)) {
-      return BALLOON_FAILURE;
-   }
+   b->guestType = guestType;
 
-   /* initialize global state */
-   Balloon_Init(b);
+   /* initialize rates */
+   b->rateAlloc = BALLOON_RATE_ALLOC_MAX;
+   b->rateFree  = BALLOON_RATE_FREE_MAX;
 
-   /* start timer */
-   timerStarted = FALSE;
-   if (!OS_TimerStart(BalloonTimerHandler, b)) {
-      Balloon_ModuleCleanup();
-      return BALLOON_FAILURE;
-   }
+   /* initialize reset flag */
+   b->resetFlag = TRUE;
 
-   timerStarted = TRUE;
-   return BALLOON_SUCCESS;
+   return TRUE;
 }
 
 
 /*
  *----------------------------------------------------------------------
  *
- * Balloon_ModuleCleanup --
+ * Balloon_Cleanup --
  *
- *      Terminate and cleanup the balloon module.
+ *      Cleanup after ballooning.
  *
  * Results:
  *      None
@@ -1476,15 +1309,10 @@ Balloon_ModuleInit(void)
  */
 
 void
-Balloon_ModuleCleanup(void)
+Balloon_Cleanup(void)
 {
    Balloon *b = &globalBalloon;
 
-   /* stop timer */
-   if (timerStarted) {
-      OS_TimerStop();
-   }
-
    /*
     * Deallocate all reserved memory, and reset connection with monitor.
     * Reset connection before deallocating memory to avoid potential for
@@ -1492,9 +1320,6 @@ Balloon_ModuleCleanup(void)
     */
    BalloonMonitorStart(b);
    Balloon_Deallocate(b);
-
-   /* os-specific cleanup */
-   OS_Cleanup();
 }
 
 #ifdef __cplusplus
