@@ -39,6 +39,7 @@
 #include "str.h"
 #include "vmtoolsd_version.h"
 #include "vmware/tools/utils.h"
+#include "vmware/tools/i18n.h"
 
 
 /**
@@ -197,38 +198,53 @@ ToolsCore_ParseCommandLine(ToolsServiceState *state,
 #endif
    GOptionEntry clOptions[] = {
       { "name", 'n', 0, G_OPTION_ARG_STRING, &state->name,
-         N_("Name of the service being started."), N_("svcname") },
+         SU_(cmdline.name, "Name of the service being started."),
+         SU_(cmdline.name.argument, "svcname") },
       { "common-path", '\0', 0, G_OPTION_ARG_FILENAME, &state->commonPath,
-         N_("Path to the common plugin directory."), N_("path") },
+         SU_(cmdline.commonpath, "Path to the common plugin directory."),
+         SU_(cmdline.path, "path") },
       { "plugin-path", 'p', 0, G_OPTION_ARG_FILENAME, &state->pluginPath,
-         N_("Path to the plugin directory."), N_("path") },
+         SU_(cmdline.pluginpath, "Path to the plugin directory."),
+         SU_(cmdline.path, "path") },
       { "cmd", '\0', 0, G_OPTION_ARG_CALLBACK, ToolsCoreRunCommand,
-         N_("Sends an RPC command to the host and exits."), N_("command") },
+         SU_(cmdline.rpc, "Sends an RPC command to the host and exits."),
+         SU_(cmdline.rpc.command, "command") },
 #if defined(G_PLATFORM_WIN32)
       { "dump-state", 's', 0, G_OPTION_ARG_NONE, &dumpState,
-         N_("Dumps the internal state of a running service instance to the logs."), 0 },
+         SU_(cmdline.state, "Dumps the internal state of a running service instance to the logs."),
+         NULL },
       { "kill", 'k', 0, G_OPTION_ARG_NONE, &kill,
-         N_("Stops a running instance of a tools service."), 0 },
+         SU_(cmdline.kill, "Stops a running instance of a tools service."),
+         NULL },
       { "install", 'i', G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, ToolsCoreIgnoreArg,
-         N_("Installs the service with the Service Control Manager."), N_("args") },
+         SU_(cmdline.install, "Installs the service with the Service Control Manager."),
+         SU_(cmdline.install.args, "args") },
       { "uninstall", 'u', G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, ToolsCoreIgnoreArg,
-         N_("Uninstalls the service from the Service Control Manager."), NULL },
+         SU_(cmdline.uninstall, "Uninstalls the service from the Service Control Manager."),
+         NULL },
       { "displayname", 'd', 0, G_OPTION_ARG_STRING, &state->displayName,
-         N_("Service display name (only used with -i)."), N_("name") },
+         SU_(cmdline.displayname, "Service display name (only used with -i)."),
+         SU_(cmdline.displayname.argument, "name") },
 #else
       { "background", 'b', 0, G_OPTION_ARG_FILENAME, &state->pidFile,
-         N_("Runs in the background and creates a pid file."), N_("pidfile") },
+         SU_(cmdline.background, "Runs in the background and creates a pid file."),
+         SU_(cmdline.background.pidfile, "pidfile") },
       { "blockFd", '\0', 0, G_OPTION_ARG_INT, &state->ctx.blockFD,
-         N_("File descriptor for the VMware blocking fs."), N_("fd") },
+         SU_(cmdline.blockfd, "File descriptor for the VMware blocking fs."),
+         SU_(cmdline.blockfd.fd, "fd") },
 #endif
       { "config", 'c', 0, G_OPTION_ARG_FILENAME, &state->configFile,
-         N_("Uses the config file at the given path."), N_("path") },
+         SU_(cmdline.config, "Uses the config file at the given path."),
+         SU_(cmdline.path, "path") },
       { "debug", 'g', 0, G_OPTION_ARG_FILENAME, &state->debugPlugin,
-         N_("Runs in debug mode, using the given plugin."), N_("path") },
+         SU_(cmdline.debug, "Runs in debug mode, using the given plugin."),
+         SU_(cmdline.path, "path") },
       { "log", 'l', 0, G_OPTION_ARG_NONE, &state->log,
-         N_("Turns on logging. Overrides the config file."), NULL },
+         SU_(cmdline.log, "Turns on logging. Overrides the config file."),
+         NULL },
       { "version", 'v', 0, G_OPTION_ARG_NONE, &version,
-         N_("Prints the daemon version and exits."), NULL },
+         SU_(cmdline.version, "Prints the daemon version and exits."),
+         NULL },
       { NULL }
    };
    GError *error = NULL;
