@@ -25,7 +25,8 @@
  *    Internal functions for the tools daemon.
  */
 
-#define G_LOG_DOMAIN       "vmtoolsd"
+#define VMW_TEXT_DOMAIN    "vmtoolsd"
+#define G_LOG_DOMAIN       VMW_TEXT_DOMAIN
 #define TOOLSCORE_COMMON   "common"
 
 #include <glib-object.h>
@@ -75,6 +76,7 @@ typedef struct ToolsServiceState {
    gchar         *name;
    gchar         *configFile;
    time_t         configMtime;
+   guint          configCheckTask;
    gboolean       log;
    gboolean       mainService;
    gchar         *commonPath;
