@@ -27,27 +27,25 @@
 #include <string.h>
 #include <stdio.h>
 #include <locale.h>
-
-#ifdef _WIN32
-#include "resource.h"
-#define MAX_STRING_BUFFER   2048
-static char * GetString(unsigned int idString);
-#define GETSTR(str)   GetString(ID##str)
-#else
-#include "messages.h"
-#define GETSTR(str)   str
-#endif
+#include <glib/gstdio.h>
 
 #include "toolboxCmdInt.h"
 #include "toolboxcmd_version.h"
 #include "system.h"
+#include "vmware/tools/guestrpc.h"
+#include "vmware/tools/i18n.h"
+#include "vmware/tools/utils.h"
 
 #include "embed_version.h"
 VM_EMBED_VERSION(TOOLBOXCMD_VERSION_STRING);
 
 
-typedef int (*ToolboxCmdFunc)(char **argv, int argc);
-typedef void (*ToolboxHelpFunc)(const char *progName);
+typedef int (*ToolboxCmdFunc)(char **argv,
+                              int argc,
+                              gboolean quiet);
+
+typedef void (*ToolboxHelpFunc)(const char *progName,
+                                const char *cmd);
 
 
 /*
@@ -62,7 +60,6 @@ const typedef struct CmdTable {
    ToolboxHelpFunc helpFunc;  /* The help function associated with the command. */
 } CmdTable;
 
-static Bool quiet_flag; /* Flag set by `--quiet'. */
 
 /*
  * Sadly, our home-brewed implementation of getopt() doesn't come with an
@@ -79,25 +76,21 @@ static struct option long_options[] = {
    { 0, 0, 0, 0 } };
 #endif
 
+static gboolean gQuiet = FALSE;
 static const char *options = "hqv";
 
 /*
  * Local Functions
  */
 
-static int HelpCommand(char **argv, int argc);
-static int DeviceCommand(char **argv, int argc);
-static int DiskCommand(char **argv, int argc);
-static int StatCommand(char **argv, int argc);
-static int ScriptCommand(char **argv, int argc);
-static int TimeSyncCommand(char **argv, int argc);
-static void DeviceHelp(const char *progName);
-static void DiskHelp(const char *progName);
-static void ScriptHelp(const char *progName);
-static void StatHelp(const char *progName);
-static void TimeSyncHelp(const char *progName);
-static void ToolboxCmdHelp(const char *progName);
-static CmdTable *ParseCommand(char **argv, int argc);
+static int
+HelpCommand(char **argv,
+            int argc,
+            gboolean quiet);
+
+static void
+ToolboxCmdHelp(const char *progName,
+               const char *cmd);
 
 
 /*
@@ -105,51 +98,22 @@ static CmdTable *ParseCommand(char **argv, int argc);
  * Must go after function declarations
  */
 static CmdTable commands[] = {
-   { "timesync", TimeSyncCommand, TRUE, FALSE, TimeSyncHelp},
-   { "script", ScriptCommand, FALSE /* We will handle argument checks ourselves */,
-      TRUE, ScriptHelp},
-   { "disk", DiskCommand, TRUE, TRUE, DiskHelp},
-   { "stat", StatCommand, TRUE, FALSE, StatHelp},
-   { "device", DeviceCommand, TRUE, FALSE, DeviceHelp},
-   { "help", HelpCommand, FALSE, FALSE, ToolboxCmdHelp},
+   { "timesync",  TimeSync_Command, TRUE,    FALSE,   TimeSync_Help},
+   { "script",    Script_Command,   FALSE,   TRUE,    Script_Help},
+   { "disk",      Disk_Command,     TRUE,    TRUE,    Disk_Help},
+   { "stat",      Stat_Command,     TRUE,    FALSE,   Stat_Help},
+   { "device",    Device_Command,   TRUE,    FALSE,   Device_Help},
+#if (defined(_WIN32) || defined(linux)) && !defined(OPEN_VM_TOOLS)
+   { "upgrade",   Upgrade_Command,  TRUE,    FALSE,   Upgrade_Help},
+#endif
+   { "help",      HelpCommand,      FALSE,   FALSE,   ToolboxCmdHelp},
 };
 
-#ifdef _WIN32
-/*
- *-----------------------------------------------------------------------------
- *
- * GetString --
- *
- *      Gets a localized string based on an identifier.
- *
- * Results:
- *      A pointer to the string. I should not be freed, being static allocated.
- *
- * Side effects:
- *      None.
- *
- *-----------------------------------------------------------------------------
- */
-
-static char *
-GetString(unsigned int idString)     // IN: the identifier of the string
-{
-   static char szResourceString[MAX_STRING_BUFFER];
-   const int sizeResourceString = ARRAYSIZE(szResourceString);
-
-   szResourceString[0] = '\0';
-
-   LoadString(GetModuleHandle(NULL), idString,
-              szResourceString, sizeResourceString);
-
-   return szResourceString;
-}
-#endif
 
 /*
  *-----------------------------------------------------------------------------
  *
- * ToolboxMissingEntityError --
+ * ToolsCmd_MissingEntityError --
  *
  *      Print out error message regarding missing argument.
  *
@@ -162,18 +126,125 @@ GetString(unsigned int idString)     // IN: the identifier of the string
  *-----------------------------------------------------------------------------
  */
 
-static void
-ToolboxMissingEntityError(const char *name,     // IN: command name (argv[0])
-                          const char *entity)   // IN: what is missing
+void
+ToolsCmd_MissingEntityError(const char *name,     // IN: command name (argv[0])
+                            const char *entity)   // IN: what is missing
 {
-   fprintf(stderr, "%s: Missing %s\n", name, entity);
+   g_printerr(SU_(error.missing, "%s: Missing %s\n"), name, entity);
 }
 
 
 /*
  *-----------------------------------------------------------------------------
  *
- * ToolboxUnknownEntityError --
+ * ToolsCmd_Print --
+ *
+ *      Prints a message to stdout unless quiet output was requested.
+ *
+ * Results:
+ *      None.
+ *
+ * Side effects:
+ *      None.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+void
+ToolsCmd_Print(const char *fmt,
+               ...)
+{
+   if (!gQuiet) {
+      gchar *str;
+      va_list args;
+
+      va_start(args, fmt);
+      g_vasprintf(&str, fmt, args);
+      va_end(args);
+
+      g_print("%s", str);
+      g_free(str);
+   }
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * ToolsCmd_PrintErr --
+ *
+ *      Prints a message to stderr unless quiet output was requested.
+ *
+ * Results:
+ *      None.
+ *
+ * Side effects:
+ *      None.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+void
+ToolsCmd_PrintErr(const char *fmt,
+                  ...)
+{
+   if (!gQuiet) {
+      gchar *str;
+      va_list args;
+
+      va_start(args, fmt);
+      g_vasprintf(&str, fmt, args);
+      va_end(args);
+
+      g_printerr("%s", str);
+      g_free(str);
+   }
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * ToolsCmd_SendRPC --
+ *
+ *    Sends an RPC message to the host.
+ *
+ * Results:
+ *    The return value from the RPC.
+ *
+ * Side effects:
+ *    None.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+gboolean
+ToolsCmd_SendRPC(const char *rpc,      // IN
+                 size_t rpcLen,        // IN
+                 char **result,        // OUT
+                 size_t *resultLen)    // OUT
+{
+   char *lrpc = (char *) rpc;
+   RpcChannel *chan = BackdoorChannel_New();
+   gboolean ret = RpcChannel_Start(chan);
+
+   if (!ret) {
+      g_warning("Error starting RPC channel.");
+      goto exit;
+   }
+
+   ret = RpcChannel_Send(chan, lrpc, rpcLen, result, resultLen);
+
+exit:
+   RpcChannel_Destroy(chan);
+   return ret;
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * ToolsCmd_UnknownEntityError --
  *
  *      Print out error message regarding unknown argument.
  *
@@ -186,35 +257,12 @@ ToolboxMissingEntityError(const char *name,     // IN: command name (argv[0])
  *-----------------------------------------------------------------------------
  */
 
-static void
-ToolboxUnknownEntityError(const char *name,    // IN: command name (argv[0])
-                          const char *entity,  // IN: what is unknown
-                          const char *str)     // IN: errorneous string
+void
+ToolsCmd_UnknownEntityError(const char *name,    // IN: command name (argv[0])
+                            const char *entity,  // IN: what is unknown
+                            const char *str)     // IN: errorneous string
 {
-   fprintf(stderr, "%s: Unknown %s '%s'\n", name, entity, str);
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * DeviceHelp --
- *
- *      Prints the help for device commands.
- *
- * Results:
- *      None.
- *
- * Side effects:
- *      None.
- *
- *-----------------------------------------------------------------------------
- */
-
-static void
-DeviceHelp(const char *progName) // IN: The name of the program obtained from argv[0]
-{
-   printf(GETSTR(S_HELP_DEVICE), progName);
+   g_printerr(SU_(error.unknown, "%s: Unknown %s '%s'\n"), name, entity, str);
 }
 
 
@@ -235,102 +283,24 @@ DeviceHelp(const char *progName) // IN: The name of the program obtained from ar
  */
 
 static void
-ToolboxCmdHelp(const char *progName)
+ToolboxCmdHelp(const char *progName,   // IN
+               const char *cmd)        // IN
 {
-   printf(GETSTR(S_HELP_TOOLBOXCMD),
-          progName, progName, progName);
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * TimeSyncHelp --
- *
- *      Prints the help for timesync command.
- *
- * Results:
- *      None.
- *
- * Side effects:
- *      None
- *
- *-----------------------------------------------------------------------------
- */
-
-static void
-TimeSyncHelp(const char *progName) // IN: The name of the program obtained from argv[0]
-{
-   printf(GETSTR(S_HELP_TIMESYNC), progName);
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * ScriptHelp --
- *
- *      Prints the help for the script command.
- *
- * Results:
- *      None.
- *
- * Side effects:
- *      None.
- *
- *-----------------------------------------------------------------------------
- */
-
-static void
-ScriptHelp(const char *progName) // IN: The name of the program obtained from argv[0]
-{
-   printf(GETSTR(S_HELP_SCRIPT), progName);
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * DiskHelp --
- *
- *      Prints the help for the disk command.
- *
- * Results:
- *      None.
- *
- * Side effects:
- *      None.
- *
- *-----------------------------------------------------------------------------
- */
-
-static void
-DiskHelp(const char *progName) // IN: The name of the program obtained from argv[0]
-{
-   printf(GETSTR(S_HELP_DISK), progName);
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * StatHelp --
- *
- *      Prints the help for the stat command.
- *
- * Results:
- *      None.
- *
- * Side effects:
- *      None.
- *
- *-----------------------------------------------------------------------------
- */
-
-static void
-StatHelp(const char *progName) // IN: The name of the program obtained from argv[0]
-{
-   printf(GETSTR(S_HELP_STAT), progName);
+   g_print(SU_(help.main, "Usage: %s <command> [options] [subcommand]\n"
+                          "Type '%s %s <command>' for help on a specific command.\n"
+                          "Type '%s -v' to see the VMware Tools version.\n"
+                          "Use '-q' option to suppress stdout output.\n"
+                          "Most commands take a subcommand.\n\n"
+                          "Available commands:\n"
+                          "   device\n"
+                          "   disk\n"
+                          "   script\n"
+                          "   stat\n"
+                          "   timesync\n"
+                          "   upgrade (not available on all operating systems)\n"
+                          "\n"
+                          "For additional information please visit http://www.vmware.com/support/\n\n"),
+           progName, progName, cmd, progName);
 }
 
 
@@ -352,8 +322,9 @@ StatHelp(const char *progName) // IN: The name of the program obtained from argv
  */
 
 static int
-HelpCommand(char **argv, // IN: Command line arguments
-            int argc)    // IN: Length of argv
+HelpCommand(char **argv,      // IN: Command line arguments
+            int argc,         // IN: Length of argv
+            gboolean quiet)   // IN
 {
    int retval = EXIT_SUCCESS;
 
@@ -362,244 +333,21 @@ HelpCommand(char **argv, // IN: Command line arguments
 
       for (i = 0; i < ARRAYSIZE(commands); i++) {
          if (toolbox_strcmp(commands[i].command, argv[optind]) == 0) {
-            commands[i].helpFunc(argv[0]);
+            commands[i].helpFunc(argv[0], commands[i].command);
             return EXIT_SUCCESS;
          }
       }
-      ToolboxUnknownEntityError(argv[0], "subcommand", argv[optind]);
+      ToolsCmd_UnknownEntityError(argv[0],
+                                  SU_(arg.subcommand, "subcommand"),
+                                  argv[optind]);
       retval = EX_USAGE;
    }
 
-   ToolboxCmdHelp(argv[0]);
+   ToolboxCmdHelp(argv[0], argv[optind - 1]);
    return retval;
 }
 
 
-/*
- *-----------------------------------------------------------------------------
- *
- * DeviceCommand --
- *
- *      Handle and parse device commands.
- *
- * Results:
- *      Returns EXIT_SUCCESS on success.
- *      Returns the exit code on errors.
- *
- * Side effects:
- *      Might enable or disable a device.
- *
- *-----------------------------------------------------------------------------
- */
-
-static int
-DeviceCommand(char **argv, // IN: Command line arguments
-              int argc)    // IN: Length of command line arguments
-{
-   char *subcommand = argv[optind];
-   Bool haveDeviceArg = optind + 1 < argc;
-
-   if (toolbox_strcmp(subcommand, "list") == 0) {
-      return Devices_ListDevices();
-   } else if (toolbox_strcmp(subcommand, "status") == 0) {
-      if (haveDeviceArg) {
-         return Devices_DeviceStatus(argv[optind + 1]);
-      }
-   } else if (toolbox_strcmp(subcommand, "enable") == 0) {
-      if (haveDeviceArg) {
-         return Devices_EnableDevice(argv[optind + 1], quiet_flag);
-      }
-   } else if (toolbox_strcmp(subcommand, "disable") == 0) {
-      if (haveDeviceArg) {
-         return Devices_DisableDevice(argv[optind + 1], quiet_flag);
-      }
-   } else {
-      ToolboxUnknownEntityError(argv[0], "subcommand", subcommand);
-      return EX_USAGE;
-   }
-
-   ToolboxMissingEntityError(argv[0], "device name");
-   return EX_USAGE;
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * DiskCommand --
- *
- *      Handle and parse disk commands.
- *
- * Results:
- *      Returns EXIT_SUCCESS on success.
- *      Returns the appropriate exit code on errors.
- *
- * Side effects:
- *      Might shrink disk
- *
- *-----------------------------------------------------------------------------
- */
-
-static int
-DiskCommand(char **argv, // IN: command line arguments
-            int argc)    // IN: The length of the command line arguments
-{
-   if (toolbox_strcmp(argv[optind], "list") == 0) {
-      return Shrink_List();
-   } else if (toolbox_strcmp(argv[optind], "shrink") == 0) {
-      if (++optind >= argc) {
-         ToolboxMissingEntityError(argv[0], "mount point");
-      } else {
-         return Shrink_DoShrink(argv[optind], quiet_flag);
-      }
-   } else {
-      ToolboxUnknownEntityError(argv[0], "subcommand", argv[optind]);
-   }
-   return EX_USAGE;
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * StatCommand --
- *
- *      Handle and parse stat commands.
- *
- * Results:
- *      Returns EXIT_SUCCESS on success.
- *      Returns the appropriate exit codes on errors.
- *
- * Side effects:
- *      None.
- *
- *-----------------------------------------------------------------------------
- */
-
-static int
-StatCommand(char **argv, // IN: Command line arguments
-            int argc)    // IN: Length of command line arguments
-{
-   if (toolbox_strcmp(argv[optind], "hosttime") == 0) {
-      return Stat_HostTime();
-   } else if (toolbox_strcmp(argv[optind], "sessionid") == 0) {
-      return Stat_GetSessionID();
-   } else if (toolbox_strcmp(argv[optind], "balloon") == 0) {
-      return Stat_GetMemoryBallooned();
-   } else if (toolbox_strcmp(argv[optind], "swap") == 0) {
-      return Stat_GetMemorySwapped();
-   } else if (toolbox_strcmp(argv[optind], "memlimit") == 0) {
-      return Stat_GetMemoryLimit();
-   } else if (toolbox_strcmp(argv[optind], "memres") == 0) {
-      return Stat_GetMemoryReservation();
-   } else if (toolbox_strcmp(argv[optind], "cpures") == 0) {
-      return Stat_GetCpuReservation();
-   } else if (toolbox_strcmp(argv[optind], "cpulimit") == 0) {
-      return Stat_GetCpuLimit();
-   } else if (toolbox_strcmp(argv[optind], "speed") == 0) {
-      return Stat_ProcessorSpeed();
-   } else {
-      ToolboxUnknownEntityError(argv[0], "subcommand", argv[optind]);
-      return EX_USAGE;
-   }
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * ScriptCommand --
- *
- *      Handle and parse script commands.
- *
- * Results:
- *      Returns EXIT_SUCCESS on success.
- *      Returns the exit code on errors.
- *
- * Side effects:
- *      Might enables, disables, or change APM scripts.
- *
- *-----------------------------------------------------------------------------
- */
-
-static int
-ScriptCommand(char **argv, // IN: command line arguments.
-              int argc)    // IN: the length of the command line arguments.
-{
-   const char *apm;
-
-   if (++optind >= argc) {
-      ToolboxMissingEntityError(argv[0], "script type");
-      return EX_USAGE;
-   }
-
-   apm = argv[optind++];
-
-   if (!Script_CheckName(apm)) {
-      ToolboxUnknownEntityError(argv[0], "script type", apm);
-      return EX_USAGE;
-   }
-
-   if (optind >= argc) {
-      ToolboxMissingEntityError(argv[0], "subcommand");
-      return EX_USAGE;
-   }
-
-   if (toolbox_strcmp(argv[optind], "default") == 0) {
-      return Script_GetDefault(apm);
-   } else if (toolbox_strcmp(argv[optind], "current") == 0) {
-      return Script_GetCurrent(apm);
-   } else if (toolbox_strcmp(argv[optind], "set") == 0) {
-      if (++optind >= argc) {
-         ToolboxMissingEntityError(argv[0], "script path");
-         return EX_USAGE;
-      }
-      return Script_Set(apm, argv[optind], quiet_flag);
-   } else if (toolbox_strcmp(argv[optind], "enable") == 0) {
-      return Script_Enable(apm, quiet_flag);
-   } else if (toolbox_strcmp(argv[optind], "disable") == 0) {
-      return Script_Disable(apm, quiet_flag);
-   } else {
-      ToolboxUnknownEntityError(argv[0], "subcommand", argv[optind]);
-      return EX_USAGE;
-   }
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * TimeSyncCommand --
- *
- *      Parse and Handle timesync commands.
- *
- * Results:
- *      Returns EXIT_SUCCESS on success.
- *      Returns the appropriate exit code errors.
- *
- * Side effects:
- *      Might enable time sync, which would change the time in the guest os.
- *
- *-----------------------------------------------------------------------------
- */
-
-static int
-TimeSyncCommand(char **argv, // IN: command line arguments
-                int argc)    // IN: The length of the command line arguments
-{
-   if (toolbox_strcmp(argv[optind], "enable") == 0) {
-      return TimeSync_Enable(quiet_flag);
-   } else if (toolbox_strcmp(argv[optind], "disable") == 0) {
-      return TimeSync_Disable(quiet_flag);
-   } else if (toolbox_strcmp(argv[optind], "status") == 0) {
-      return TimeSync_Status();
-   } else {
-      ToolboxUnknownEntityError(argv[0], "subcommand", argv[optind]);
-      return EX_USAGE;
-   }
-}
-
-
 /*
  *-----------------------------------------------------------------------------
  *
@@ -661,12 +409,15 @@ main(int argc,    // IN: length of command line arguments
    int retval;
 
    setlocale(LC_ALL, "");
+   VMTools_ConfigLogging("toolboxcmd", NULL, FALSE, FALSE);
+   VMTools_BindTextDomain(VMW_TEXT_DOMAIN, NULL, NULL);
 
    /*
     * Check if we are in a VM
     */
    if (!VmCheck_IsVirtualWorld()) {
-      fprintf(stderr, GETSTR(S_WARNING_VWORLD), argv[0]);
+      g_printerr(SU_(error.novirtual, "%s must be run inside a virtual machine.\n"),
+                 argv[0]);
       exit(EXIT_FAILURE);
    }
 
@@ -697,12 +448,13 @@ main(int argc,    // IN: length of command line arguments
          break;
 
       case 'q':
-         quiet_flag = TRUE;
+         gQuiet = TRUE;
          break;
 
       case '?':
          /* getopt_long already printed an error message. */
-         fprintf(stderr, GETSTR(S_HELP_MAIN), argv[0], "-h", "", "");
+         g_printerr(SU_(help.hint, "Try '%s %s%s%s' for more information.\n"),
+                    argv[0], "-h", "", "");
          return EXIT_FAILURE;
 
       default:
@@ -713,30 +465,40 @@ main(int argc,    // IN: length of command line arguments
    if (show_version) {
       printf("%s (%s)\n", TOOLBOXCMD_VERSION_STRING, BUILD_NUMBER);
    } else if (show_help) {
-      ToolboxCmdHelp(argv[0]);
+      ToolboxCmdHelp(argv[0], "help");
    } else {
       /* Process any remaining command line arguments (not options), and
        * execute corresponding command
        */
       if (optind >= argc) {
-         ToolboxMissingEntityError(argv[0], "command");
+         ToolsCmd_MissingEntityError(argv[0], SU_(arg.command, "command"));
          retval = EX_USAGE;
       } else if ((cmd = ParseCommand(argv, argc)) == NULL) {
-         ToolboxUnknownEntityError(argv[0], "command", argv[optind]);
+         ToolsCmd_UnknownEntityError(argv[0], SU_(arg.command, "command"), argv[optind]);
          retval = EX_USAGE;
       } else if (cmd->requireRoot && !System_IsUserAdmin()) {
-         fprintf(stderr, GETSTR(S_WARNING_ADMIN), argv[0], cmd->command);
+#if defined(_WIN32)
+         g_printerr(SU_(error.noadmin.win,
+                        "%s: Administrator permissions are needed to perform %s operations.\n"
+                        "Use an administrator command prompt to complete these tasks.\n"),
+                    argv[0], cmd->command);
+
+#else
+         g_printerr(SU_(error.noadmin.posix,
+                        "%s: You must be root to perform %s operations.\n"),
+                    argv[0], cmd->command);
+#endif
          retval = EX_NOPERM;
       } else if (cmd->requireArguments && ++optind >= argc) {
-         ToolboxMissingEntityError(argv[0], "subcommand");
+         ToolsCmd_MissingEntityError(argv[0], SU_(arg.subcommand, "subcommand"));
          retval = EX_USAGE;
       } else {
-         retval = cmd->func(argv, argc);
+         retval = cmd->func(argv, argc, gQuiet);
       }
 
       if (retval == EX_USAGE && (cmd == NULL || strcmp(cmd->command, "help"))) {
-         fprintf(stderr, GETSTR(S_HELP_MAIN),
-                 argv[0], "help", cmd ? " " : "", cmd ? cmd->command : "");
+         g_printerr(SU_(help.hint, "Try '%s %s%s%s' for more information.\n"),
+                    argv[0], "help", cmd ? " " : "", cmd ? cmd->command : "");
       }
 
       return retval;
