@@ -30,6 +30,7 @@
 
 #include "system.h"
 #include "toolboxCmdInt.h"
+#include "vmware/tools/i18n.h"
 #include "vmware/tools/utils.h"
 
 
@@ -43,10 +44,6 @@ typedef enum ScriptType {
    Current
 } ScriptType;
 
-static int ScriptToggle(const char *apm, Bool enable, int quiet_flag);
-static const char* GetConfName(const char *apm);
-static int GetConfEntry(const char *apm, ScriptType type);
-
 
 /*
  *-----------------------------------------------------------------------------
@@ -133,8 +130,9 @@ LoadConfFile(void)
  */
 
 static int
-GetConfEntry(const char *apm,  // IN: apm name
-             ScriptType type)  // IN: Script type (default or current)
+GetConfEntry(const char *progName,  // IN: program name (argv[0])
+             const char *apm,       // IN: apm name
+             ScriptType type)       // IN: Script type (default or current)
 {
    gchar *entry = NULL;
    GKeyFile *confDict = NULL;
@@ -143,7 +141,9 @@ GetConfEntry(const char *apm,  // IN: apm name
 
    confName = GetConfName(apm);
    if (!confName) {
-      fprintf(stderr, "Unknown operation\n");
+      ToolsCmd_UnknownEntityError(progName,
+                                  SU_(script.operation, "operation"),
+                                  apm);
       return EX_USAGE;
    }
 
@@ -162,7 +162,8 @@ GetConfEntry(const char *apm,  // IN: apm name
       printf("%s\n", entry);
       ret = EXIT_SUCCESS;
    } else {
-      fprintf(stderr, "No script for operation %s\n", apm);
+      ToolsCmd_PrintErr(SU_(script.unknownop, "No script for operation %s.\n"),
+                        apm);
       ret = EX_TEMPFAIL;
    }
 
@@ -175,7 +176,7 @@ GetConfEntry(const char *apm,  // IN: apm name
 /*
  *-----------------------------------------------------------------------------
  *
- * Script_GetDefault  --
+ * ScriptGetDefault  --
  *
  *      Gets the path to default script.
  *
@@ -190,17 +191,18 @@ GetConfEntry(const char *apm,  // IN: apm name
  *-----------------------------------------------------------------------------
  */
 
-int
-Script_GetDefault(const char *apm) // IN: APM name
+static int
+ScriptGetDefault(const char *progName, // IN: program name (argv[0])
+                 const char *apm)      // IN: APM name
 {
-   return GetConfEntry(apm, Default);
+   return GetConfEntry(progName, apm, Default);
 }
 
 
 /*
  *-----------------------------------------------------------------------------
  *
- * Script_GetCurrent  --
+ * ScriptGetCurrent  --
  *
  *      Gets the path to Current script.
  *
@@ -215,10 +217,11 @@ Script_GetDefault(const char *apm) // IN: APM name
  *-----------------------------------------------------------------------------
  */
 
-int
-Script_GetCurrent(const char *apm) // IN: apm function name
+static int
+ScriptGetCurrent(const char *progName, // IN: program name (argv[0])
+                 const char *apm)      // IN: apm function name
 {
-   return GetConfEntry(apm, Current);
+   return GetConfEntry(progName, apm, Current);
 }
 
 
@@ -242,9 +245,9 @@ Script_GetCurrent(const char *apm) // IN: apm function name
  */
 
 static int
-ScriptToggle(const char *apm, // IN: APM name
-             Bool enable,     // IN: status
-             int quiet_flag)  // IN: Verbosity flag
+ScriptToggle(const char *progName,  // IN: program name (argv[0])
+             const char *apm,       // IN: APM name
+             Bool enable)           // IN: status
 {
    const char *path;
    const char *confName;
@@ -255,7 +258,9 @@ ScriptToggle(const char *apm, // IN: APM name
    confName = GetConfName(apm);
 
    if (!confName) {
-      fprintf(stderr, "Unknown operation\n");
+      ToolsCmd_UnknownEntityError(progName,
+                                  SU_(script.operation, "operation"),
+                                  apm);
       return EX_USAGE;
    }
 
@@ -269,7 +274,8 @@ ScriptToggle(const char *apm, // IN: APM name
 
    g_key_file_set_string(confDict, "powerops", confName, path);
    if (!VMTools_WriteConfig(NULL, confDict, &err)) {
-      fprintf(stderr, "Error writing config: %s\n", err->message);
+      ToolsCmd_PrintErr(SU_(script.write.error, "Error writing config: %s\n"),
+                        err->message);
       g_clear_error(&err);
       ret = EX_TEMPFAIL;
    }
@@ -282,59 +288,12 @@ ScriptToggle(const char *apm, // IN: APM name
 /*
  *-----------------------------------------------------------------------------
  *
- * Script_Enable  --
+ * ScriptSet  --
  *
- *      enables script.
- *
- * Results:
- *      Same as ScriptToggle.
- *
- * Side effects:
- *      Same as ScriptToggle.
- *
- *-----------------------------------------------------------------------------
- */
-
-int
-Script_Enable(const char *apm,   // IN: APM name
-              int quiet_flag)    // IN: Verbosity flag
-{
-   return ScriptToggle(apm, TRUE, quiet_flag);
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * Script_Disable  --
- *
- *      disable script
- *
- * Results:
- *      Same as ScriptToggle.
- *
- * Side effects:
- *      Same as ScriptToggle.
- *
- *-----------------------------------------------------------------------------
- */
-
-int
-Script_Disable(const char *apm,  // IN: APM name
-               int quiet_flag)   // IN: Verbosity Flag
-{
-   return ScriptToggle(apm, FALSE, quiet_flag);
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * sets a script to the given path  --
- *
- *      disable script.
+ *      Sets a script to the given path.
  *
  * Results:
+ *      EX_OSFILE if path doesn't exist.
  *      EXIT_SUCCESS on success.
  *      EX_USAGE on parse errors.
  *      EX_TEMPFAIL on failure.
@@ -346,10 +305,10 @@ Script_Disable(const char *apm,  // IN: APM name
  *-----------------------------------------------------------------------------
  */
 
-int
-Script_Set(const char *apm,   // IN: APM name
-           const char *path,  // IN: path to script
-           int quiet_flag)    // IN: Verbosity flag
+static int
+ScriptSet(const char *progName,  // IN: program name (argv[0])
+          const char *apm,       // IN: APM name
+          const char *path)      // IN: Verbosity flag
 {
    const char *confName;
    int ret = EXIT_SUCCESS;
@@ -357,13 +316,15 @@ Script_Set(const char *apm,   // IN: APM name
    GError *err = NULL;
 
    if (!File_Exists(path)) {
-      fprintf(stderr, "%s doesn't exist\n", path);
+      ToolsCmd_PrintErr(SU_(script.notfound, "%s doesn't exist.\n"), path);
       return EX_OSFILE;
    }
 
    confName = GetConfName(apm);
    if (!confName) {
-      fprintf(stderr, "Unknown operation\n");
+      ToolsCmd_UnknownEntityError(progName,
+                                  SU_(script.operation, "operation"),
+                                  apm);
       return EX_USAGE;
    }
 
@@ -371,7 +332,8 @@ Script_Set(const char *apm,   // IN: APM name
    g_key_file_set_string(confDict, "powerops", confName, path);
 
    if (!VMTools_WriteConfig(NULL, confDict, &err)) {
-      fprintf(stderr, "Error writing config: %s\n", err->message);
+      ToolsCmd_PrintErr(SU_(script.write.error, "Error writing config: %s\n"),
+                        err->message);
       g_clear_error(&err);
       ret = EX_TEMPFAIL;
    }
@@ -384,7 +346,7 @@ Script_Set(const char *apm,   // IN: APM name
 /*
  *-----------------------------------------------------------------------------
  *
- * Script_CheckName  --
+ * ScriptCheckName  --
  *
  *      Check if it is known script
  *
@@ -397,8 +359,106 @@ Script_Set(const char *apm,   // IN: APM name
  *-----------------------------------------------------------------------------
  */
 
-Bool
-Script_CheckName(const char *apm) // IN: script name
+static Bool
+ScriptCheckName(const char *apm) // IN: script name
 {
    return GetConfName(apm) != NULL;
 }
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * Script_Command --
+ *
+ *      Handle and parse script commands.
+ *
+ * Results:
+ *      Returns EXIT_SUCCESS on success.
+ *      Returns the exit code on errors.
+ *
+ * Side effects:
+ *      Might enables, disables, or change APM scripts.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+int
+Script_Command(char **argv,    // IN: command line arguments.
+               int argc,       // IN: the length of the command line arguments.
+               gboolean quiet) // IN
+{
+   const char *apm;
+
+   if (++optind >= argc) {
+      ToolsCmd_MissingEntityError(argv[0], SU_(arg.scripttype, "script type"));
+      return EX_USAGE;
+   }
+
+   apm = argv[optind++];
+
+   if (!ScriptCheckName(apm)) {
+      ToolsCmd_UnknownEntityError(argv[0], SU_(arg.scripttype, "script type"), apm);
+      return EX_USAGE;
+   }
+
+   if (optind >= argc) {
+      ToolsCmd_MissingEntityError(argv[0], SU_(arg.subcommand, "subcommand"));
+      return EX_USAGE;
+   }
+
+   if (toolbox_strcmp(argv[optind], "default") == 0) {
+      return ScriptGetDefault(argv[0], apm);
+   } else if (toolbox_strcmp(argv[optind], "current") == 0) {
+      return ScriptGetCurrent(argv[0], apm);
+   } else if (toolbox_strcmp(argv[optind], "set") == 0) {
+      if (++optind >= argc) {
+         ToolsCmd_MissingEntityError(argv[0], SU_(arg.scriptpath, "script path"));
+         return EX_USAGE;
+      }
+      return ScriptSet(argv[0], apm, argv[optind]);
+   } else if (toolbox_strcmp(argv[optind], "enable") == 0) {
+      return ScriptToggle(argv[0], apm, TRUE);
+   } else if (toolbox_strcmp(argv[optind], "disable") == 0) {
+      return ScriptToggle(argv[0], apm, FALSE);
+   } else {
+      ToolsCmd_UnknownEntityError(argv[0],
+                                  SU_(arg.subcommand, "subcommand"),
+                                  argv[optind]);
+      return EX_USAGE;
+   }
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * Script_Help --
+ *
+ *      Prints the help for the script command.
+ *
+ * Results:
+ *      None.
+ *
+ * Side effects:
+ *      None.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+void
+Script_Help(const char *progName, // IN: The name of the program obtained from argv[0]
+            const char *cmd)      // IN
+{
+   g_print(SU_(help.script,
+               "%s: control the scripts run in response to power operations\n"
+               "Usage: %s %s <power|resume|suspend|shutdown> <subcommand> [args]\n\n"
+               "Subcommands:\n"
+               "   enable: enable the given script and restore its path to the default\n"
+               "   disable: disable the given script\n"
+               "   set <full_path>: set the given script to the given path\n"
+               "   default: print the default path of the given script\n"
+               "   current: print the current path of the given script\n"),
+           cmd, progName, cmd);
+}
+
