@@ -24,13 +24,7 @@
 
 #include <time.h>
 #include "toolboxCmdInt.h"
-
-
-/*
- * Local Functions
- */
-
-static int OpenHandle(VMGuestLibHandle *glHandle,VMGuestLibError *glError);
+#include "vmware/tools/i18n.h"
 
 
 /*
@@ -57,12 +51,16 @@ OpenHandle(VMGuestLibHandle *glHandle, // OUT: The guestlib handle
 {
    *glError = VMGuestLib_OpenHandle(glHandle);
    if (*glError != VMGUESTLIB_ERROR_SUCCESS) {
-      fprintf(stderr, "OpenHandle failed: %s\n", VMGuestLib_GetErrorText(*glError));
+      ToolsCmd_PrintErr(SU_(stat.openhandle.failed,
+                            "OpenHandle failed: %s\n"),
+                        VMGuestLib_GetErrorText(*glError));
       return EX_UNAVAILABLE;
    }
    *glError = VMGuestLib_UpdateInfo(*glHandle);
    if (*glError != VMGUESTLIB_ERROR_SUCCESS) {
-      fprintf(stderr, "UpdateInfo failed: %s\n", VMGuestLib_GetErrorText(*glError));
+      ToolsCmd_PrintErr(SU_(stat.update.failed,
+                            "UpdateInfo failed: %s\n"),
+                        VMGuestLib_GetErrorText(*glError));
       return EX_TEMPFAIL;
    }
    return 0;  // We don't return EXIT_SUCCESSS to indicate that this is not
@@ -74,7 +72,7 @@ OpenHandle(VMGuestLibHandle *glHandle, // OUT: The guestlib handle
 /*
  *-----------------------------------------------------------------------------
  *
- * Stat_ProcessorSpeed  --
+ * StatProcessorSpeed  --
  *
  *      Gets the Processor Speed.
  *
@@ -88,8 +86,8 @@ OpenHandle(VMGuestLibHandle *glHandle, // OUT: The guestlib handle
  *-----------------------------------------------------------------------------
  */
 
-int
-Stat_ProcessorSpeed(void)
+static int
+StatProcessorSpeed(void)
 {
    uint32 speed;
    Backdoor_proto bp;
@@ -97,7 +95,8 @@ Stat_ProcessorSpeed(void)
    Backdoor(&bp);
    speed = bp.out.ax.word;
    if (speed < 0) {
-      fprintf(stderr, "Unable to get processor speed\n");
+      ToolsCmd_PrintErr("%s",
+                        SU_(stat.getspeed.failed, "Unable to get processor speed.\n"));
       return EX_TEMPFAIL;
    }
    printf("%u MHz\n", speed);
@@ -108,41 +107,7 @@ Stat_ProcessorSpeed(void)
 /*
  *-----------------------------------------------------------------------------
  *
- * Stat_MemorySize  --
- *
- *      Gets the virtual machine's memory in MBs.
- *
- * Results:
- *      EXIT_SUCCESS on success.
- *      EX_TEMPFAIL on failure.
- *
- * Side effects:
- *      None.
- *
- *-----------------------------------------------------------------------------
- */
-
-int
-Stat_MemorySize(void)
-{
-   uint32 memsize;
-   Backdoor_proto bp;
-   bp.in.cx.halfs.low = BDOOR_CMD_GETMEMSIZE;
-   Backdoor(&bp);
-   memsize = bp.out.ax.word;
-   if (memsize < 0) {
-      fprintf(stderr, "Unable to get memory size\n");
-      return EX_TEMPFAIL;
-   }
-   printf("%u MB\n", memsize);
-   return EXIT_SUCCESS;
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * Stat_HostTime  --
+ * StatHostTime  --
  *
  *      Gets the host machine's time.
  *
@@ -156,8 +121,8 @@ Stat_MemorySize(void)
  *-----------------------------------------------------------------------------
  */
 
-int
-Stat_HostTime(void)
+static int
+StatHostTime(void)
 {
    int64 hostSecs;
    int64 hostUsecs;
@@ -178,13 +143,15 @@ Stat_HostTime(void)
    hostUsecs = bp.out.bx.word;
 
    if (hostSecs <= 0) {
-      fprintf(stderr, "Unable to get host time\n");
+      ToolsCmd_PrintErr("%s",
+                        SU_(stat.gettime.failed, "Unable to get host time.\n"));
       return EX_TEMPFAIL;
    }
-   
+
    sec = hostSecs + (hostUsecs / 1000000);
    if (strftime(buf, sizeof buf, "%d %b %Y %H:%M:%S", localtime(&sec)) == 0) {
-      fprintf(stderr, "Unable to format host time\n");
+      ToolsCmd_PrintErr("%s",
+                        SU_(stat.formattime.failed, "Unable to format host time.\n"));
       return EX_TEMPFAIL;
    }
    printf("%s\n", buf);
@@ -195,7 +162,7 @@ Stat_HostTime(void)
 /*
  *-----------------------------------------------------------------------------
  *
- * Stat_GetSessionID --
+ * StatGetSessionID --
  *
  *      Gets the Session ID for the virtual machine
  *      Works only if the host is ESX.
@@ -210,8 +177,8 @@ Stat_HostTime(void)
  *-----------------------------------------------------------------------------
  */
 
-int
-Stat_GetSessionID(void)
+static int
+StatGetSessionID(void)
 {
    int exitStatus = EXIT_SUCCESS;
    uint64 session;
@@ -224,8 +191,9 @@ Stat_GetSessionID(void)
    }
    glError = VMGuestLib_GetSessionId(glHandle, &session);
    if (glError != VMGUESTLIB_ERROR_SUCCESS) {
-      fprintf(stderr, "Failed to get session ID: %s\n",
-              VMGuestLib_GetErrorText(glError));
+      ToolsCmd_PrintErr(SU_(stat.getsession.failed,
+                            "Failed to get session ID: %s\n"),
+                        VMGuestLib_GetErrorText(glError));
       exitStatus = EX_TEMPFAIL;
    } else {
       printf("0x%"FMT64"x\n", session);
@@ -238,7 +206,7 @@ Stat_GetSessionID(void)
 /*
  *-----------------------------------------------------------------------------
  *
- * Stat_GetMemoryBallooned  --
+ * StatGetMemoryBallooned  --
  *
  *      Retrieves memory ballooned.
  *      Works only if the host is ESX.
@@ -253,8 +221,8 @@ Stat_GetSessionID(void)
  *-----------------------------------------------------------------------------
  */
 
-int
-Stat_GetMemoryBallooned(void)
+static int
+StatGetMemoryBallooned(void)
 {
    int exitStatus = EXIT_SUCCESS;
    uint32 memBallooned;
@@ -267,7 +235,9 @@ Stat_GetMemoryBallooned(void)
    }
    glError = VMGuestLib_GetMemBalloonedMB(glHandle, &memBallooned);
    if (glError != VMGUESTLIB_ERROR_SUCCESS) {
-      fprintf(stderr, "Failed to get CPU Limit: %s\n", VMGuestLib_GetErrorText(glError));
+      ToolsCmd_PrintErr(SU_(stat.balloon.failed,
+                            "Failed to get ballooned memory: %s\n"),
+                        VMGuestLib_GetErrorText(glError));
       exitStatus = EX_TEMPFAIL;
    } else {
       printf("%u MHz\n", memBallooned);
@@ -280,7 +250,7 @@ Stat_GetMemoryBallooned(void)
 /*
  *-----------------------------------------------------------------------------
  *
- * Stat_GetMemoryReservation  --
+ * StatGetMemoryReservation  --
  *
  *      Retrieves min memory.
  *      Works only if the host is ESX.
@@ -295,8 +265,8 @@ Stat_GetMemoryBallooned(void)
  *-----------------------------------------------------------------------------
  */
 
-int
-Stat_GetMemoryReservation(void)
+static int
+StatGetMemoryReservation(void)
 {
    int exitStatus = EXIT_SUCCESS;
    uint32  memReservation;
@@ -309,7 +279,9 @@ Stat_GetMemoryReservation(void)
    }
    glError = VMGuestLib_GetMemReservationMB(glHandle, &memReservation);
    if (glError != VMGUESTLIB_ERROR_SUCCESS) {
-      fprintf(stderr, "Failed to get CPU Limit: %s\n", VMGuestLib_GetErrorText(glError));
+      ToolsCmd_PrintErr(SU_(stat.memres.failed,
+                            "Failed to get memory reservation: %s\n"),
+                        VMGuestLib_GetErrorText(glError));
       exitStatus = EX_TEMPFAIL;
    } else {
       printf("%u MB\n", memReservation);
@@ -322,7 +294,7 @@ Stat_GetMemoryReservation(void)
 /*
  *-----------------------------------------------------------------------------
  *
- * Stat_GetMemorySwapped  --
+ * StatGetMemorySwapped  --
  *
  *      Retrieves swapped memory.
  *      Works only if the host is ESX.
@@ -338,8 +310,8 @@ Stat_GetMemoryReservation(void)
  *-----------------------------------------------------------------------------
  */
 
-int
-Stat_GetMemorySwapped(void)
+static int
+StatGetMemorySwapped(void)
 {
    int exitStatus = EXIT_SUCCESS;
    uint32 memSwapped;
@@ -352,7 +324,9 @@ Stat_GetMemorySwapped(void)
    }
    glError = VMGuestLib_GetMemSwappedMB(glHandle, &memSwapped);
    if (glError != VMGUESTLIB_ERROR_SUCCESS) {
-      fprintf(stderr, "Failed to get CPU Limit: %s\n", VMGuestLib_GetErrorText(glError));
+      ToolsCmd_PrintErr(SU_(stat.memswap.failed,
+                            "Failed to get swapped memory: %s\n"),
+                        VMGuestLib_GetErrorText(glError));
       exitStatus = EX_TEMPFAIL;
    } else {
       printf("%u MB\n", memSwapped);
@@ -365,7 +339,7 @@ Stat_GetMemorySwapped(void)
 /*
  *-----------------------------------------------------------------------------
  *
- * Stat_GetMemoryLimit --
+ * StatGetMemoryLimit --
  *
  *      Retrieves max memory.
  *      Works only if the host is ESX.
@@ -380,8 +354,8 @@ Stat_GetMemorySwapped(void)
  *-----------------------------------------------------------------------------
  */
 
-int
-Stat_GetMemoryLimit(void)
+static int
+StatGetMemoryLimit(void)
 {
    int exitStatus = EXIT_SUCCESS;
    uint32 memLimit;
@@ -394,7 +368,9 @@ Stat_GetMemoryLimit(void)
    }
    glError = VMGuestLib_GetMemLimitMB(glHandle, &memLimit);
    if (glError != VMGUESTLIB_ERROR_SUCCESS) {
-      fprintf(stderr, "Failed to get CPU Limit: %s\n", VMGuestLib_GetErrorText(glError));
+      ToolsCmd_PrintErr(SU_(stat.maxmem.failed,
+                            "Failed to get memory limit: %s\n"),
+                        VMGuestLib_GetErrorText(glError));
       exitStatus = EX_TEMPFAIL;
    } else {
       printf("%u MB\n", memLimit);
@@ -407,7 +383,7 @@ Stat_GetMemoryLimit(void)
 /*
  *-----------------------------------------------------------------------------
  *
- * Stat_GetCpuReservation  --
+ * StatGetCpuReservation  --
  *
  *      Retrieves cpu min speed.
  *      Works only if the host is ESX.
@@ -422,8 +398,8 @@ Stat_GetMemoryLimit(void)
  *-----------------------------------------------------------------------------
  */
 
-int
-Stat_GetCpuReservation(void)
+static int
+StatGetCpuReservation(void)
 {
    int exitStatus = EXIT_SUCCESS;
    uint32 cpuReservation;
@@ -436,7 +412,9 @@ Stat_GetCpuReservation(void)
    }
    glError = VMGuestLib_GetCpuReservationMHz(glHandle, &cpuReservation);
    if (glError != VMGUESTLIB_ERROR_SUCCESS) {
-      fprintf(stderr, "Failed to get CPU Limit: %s\n", VMGuestLib_GetErrorText(glError));
+      ToolsCmd_PrintErr(SU_(stat.cpumin.failed,
+                            "Failed to get CPU minimum: %s\n"),
+                        VMGuestLib_GetErrorText(glError));
       exitStatus = EX_TEMPFAIL;
    } else {
       printf("%u MHz\n", cpuReservation);
@@ -449,7 +427,7 @@ Stat_GetCpuReservation(void)
 /*
  *-----------------------------------------------------------------------------
  *
- * Stat_GetCpuLimit  --
+ * StatGetCpuLimit  --
  *
  *      Retrieves cpu max speed .
  *      Works only if the host is ESX.
@@ -464,8 +442,8 @@ Stat_GetCpuReservation(void)
  *-----------------------------------------------------------------------------
  */
 
-int
-Stat_GetCpuLimit(void)
+static int
+StatGetCpuLimit(void)
 {
    int exitStatus = EXIT_SUCCESS;
    uint32 cpuLimit;
@@ -478,7 +456,9 @@ Stat_GetCpuLimit(void)
    }
    glError = VMGuestLib_GetCpuLimitMHz(glHandle, &cpuLimit);
    if (glError != VMGUESTLIB_ERROR_SUCCESS) {
-      fprintf(stderr, "Failed to get CPU Limit: %s\n", VMGuestLib_GetErrorText(glError));
+      ToolsCmd_PrintErr(SU_(stat.cpumax.failed,
+                            "Failed to get CPU limit: %s\n"),
+                        VMGuestLib_GetErrorText(glError));
       exitStatus = EX_TEMPFAIL;
    } else {
       printf("%u MHz\n", cpuLimit);
@@ -486,3 +466,90 @@ Stat_GetCpuLimit(void)
    VMGuestLib_CloseHandle(glHandle);
    return exitStatus;
 }
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * Stat_Command --
+ *
+ *      Handle and parse stat commands.
+ *
+ * Results:
+ *      Returns EXIT_SUCCESS on success.
+ *      Returns the appropriate exit codes on errors.
+ *
+ * Side effects:
+ *      None.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+int
+Stat_Command(char **argv,      // IN: Command line arguments
+             int argc,         // IN: Length of command line arguments
+             gboolean quiet)   // IN
+{
+   if (toolbox_strcmp(argv[optind], "hosttime") == 0) {
+      return StatHostTime();
+   } else if (toolbox_strcmp(argv[optind], "sessionid") == 0) {
+      return StatGetSessionID();
+   } else if (toolbox_strcmp(argv[optind], "balloon") == 0) {
+      return StatGetMemoryBallooned();
+   } else if (toolbox_strcmp(argv[optind], "swap") == 0) {
+      return StatGetMemorySwapped();
+   } else if (toolbox_strcmp(argv[optind], "memlimit") == 0) {
+      return StatGetMemoryLimit();
+   } else if (toolbox_strcmp(argv[optind], "memres") == 0) {
+      return StatGetMemoryReservation();
+   } else if (toolbox_strcmp(argv[optind], "cpures") == 0) {
+      return StatGetCpuReservation();
+   } else if (toolbox_strcmp(argv[optind], "cpulimit") == 0) {
+      return StatGetCpuLimit();
+   } else if (toolbox_strcmp(argv[optind], "speed") == 0) {
+      return StatProcessorSpeed();
+   } else {
+      ToolsCmd_UnknownEntityError(argv[0],
+                                  SU_(arg.subcommand, "subcommand"),
+                                  argv[optind]);
+      return EX_USAGE;
+   }
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * Stat_Help --
+ *
+ *      Prints the help for the stat command.
+ *
+ * Results:
+ *      None.
+ *
+ * Side effects:
+ *      None.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+void
+Stat_Help(const char *progName, // IN: The name of the program obtained from argv[0]
+          const char *cmd)      // IN
+{
+   g_print(SU_(help.stat, "%s: print useful guest and host information\n"
+                          "Usage: %s %s <subcommand>\n\n"
+                          "Subcommands:\n"
+                          "   hosttime: print the host time\n"
+                          "   speed: print the CPU speed in MHz\n"
+                          "ESX guests only subcommands:\n"
+                          "   sessionid: print the current session id\n"
+                          "   balloon: print memory ballooning information\n"
+                          "   swap: print memory swapping information\n"
+                          "   memlimit: print memory limit information\n"
+                          "   memres: print memory reservation information\n"
+                          "   cpures: print CPU reservation information\n"
+                          "   cpulimit: print CPU limit information\n"),
+           cmd, progName, cmd);
+}
+
