@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2008-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2008 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -38,8 +38,8 @@
 
 #include "fsops.h"
 #include "block.h"
+#include "stubs.h"
 #include "vm_basic_types.h"
-#include "vm_assert.h"
 
 /* Regular directories on a linux ext3 partition are 4K. */
 
@@ -724,6 +724,7 @@ VMBlockRelease(const char *path,                   // IN: Must be control file.
 void *
 VMBlockInit(void)
 {
+   Atomic_Init();
    BlockInit();
    return NULL;
 }
