@@ -97,6 +97,49 @@ string::string(ConstUnicode s) // IN
 
 #ifdef _WIN32
 
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * utf::string::init_bstr_t --
+ *
+ *      Utility function to construct from a _bstr_t object.
+ *      Copies the UTF-16 representation of the _bstr_t.
+ *
+ * Results:
+ *      None.
+ *
+ * Side effects:
+ *      Makes a copy of the _bstr_t data and frees that data when
+ *      the utf::string is destroyed.
+ *
+ * Note:
+ *      WIN32 only call
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+void
+string::init_bstr_t(const _bstr_t &s) // IN
+{
+   // If the input is empty, then there's nothing to do.
+   if (s.length() == 0) {
+      return;
+   }
+
+   Unicode utf8 = Unicode_AllocWithUTF16(static_cast<const utf16_t *>(s));
+
+   try {
+      mUstr = utf8;
+      Unicode_Free(utf8);
+   } catch (...) {
+      Unicode_Free(utf8);
+      throw;
+   }
+
+   ASSERT(Validate(mUstr));
+}
+
+
 /*
  *-----------------------------------------------------------------------------
  *
@@ -128,7 +171,7 @@ string::string(const ubstr_t &s) // IN
       return;
    }
 
-   mUstr = Glib::ustring(static_cast<const char *>(s));
+   mUstr = static_cast<const char *>(s);
    ASSERT(Validate(mUstr));
 }
 
@@ -159,22 +202,91 @@ string::string(const _bstr_t &s) // IN
      mUtf16Cache(NULL),
      mUtf16Length(npos)
 {
+   init_bstr_t(s);
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * utf::string::string --
+ *
+ *      Constructor from a uvariant_t object. Copies the UTF-16 representation
+ *      of the ubstr_t interface.
+ *
+ * Results:
+ *      None.
+ *
+ * Side effects:
+ *      Makes a copy of the uvariant_t data and frees that data when the
+ *      utf::string is destroyed.
+ *
+ * Note:
+ *      WIN32 only call
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+string::string(const uvariant_t &v) // IN
+   : mUstr(),
+     mUtf16Cache(NULL),
+     mUtf16Length(npos)
+{
+   ubstr_t s;
+
+   try {
+      s = v;
+   } catch (...) {
+      Warning("Invalid uvariant_t to ubstr_t conversion.\n");
+      throw;
+   }
+
    // If the input is empty, then there's nothing to do.
    if (s.length() == 0) {
       return;
    }
 
-   Unicode utf8 = Unicode_AllocWithUTF16(static_cast<const utf16_t *>(s));
+   mUstr = static_cast<const char *>(s);
+   ASSERT(Validate(mUstr));
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * utf::string::string --
+ *
+ *      Constructor from a _variant_t object. Copies the UTF-16 representation
+ *      of the _variant_t.
+ *
+ * Results:
+ *      None.
+ *
+ * Side effects:
+ *      Makes a copy of the _variant_t data and frees that data when
+ *      the utf::string is destroyed.
+ *
+ * Note:
+ *      WIN32 only call
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+string::string(const _variant_t &v) // IN
+   : mUstr(),
+     mUtf16Cache(NULL),
+     mUtf16Length(npos)
+{
+   _bstr_t s;
 
    try {
-      mUstr = Glib::ustring(utf8);
-      Unicode_Free(utf8);
+      s = v;
    } catch (...) {
-      Unicode_Free(utf8);
+      Warning("Invalid _variant_t to _bstr_t conversion.\n");
       throw;
    }
 
-   ASSERT(Validate(mUstr));
+   init_bstr_t(s);
 }
 
 #endif
@@ -243,7 +355,7 @@ string::string(const utf16_t *s) // IN
    Unicode utf8 = Unicode_AllocWithUTF16(s);
 
    try {
-      mUstr = Glib::ustring(utf8);
+      mUstr = utf8;
       Unicode_Free(utf8);
    } catch (...) {
       Unicode_Free(utf8);
@@ -281,7 +393,7 @@ string::string(const char *s,           // IN
    Unicode utf8 = Unicode_Alloc(s, encoding);
 
    try {
-      mUstr = Glib::ustring(utf8);
+      mUstr = utf8;
       Unicode_Free(utf8);
    } catch (...) {
       Unicode_Free(utf8);
@@ -445,9 +557,10 @@ string::operator const ubstr_t()
  */
 
 string&
-string::operator=(const string &s) // IN
+string::operator=(string copy) // IN
 {
-   return assign(s);
+   swap(copy);
+   return *this;
 }
 
 
@@ -1013,6 +1126,8 @@ string::push_back(value_type uc) // IN
  *
  *      Assigns the passed in string to this string.
  *
+ *      Callers should prefer using operator= instead of assign().
+ *
  * Results:
  *      A reference to this object
  *
@@ -1025,9 +1140,7 @@ string::push_back(value_type uc) // IN
 string&
 string::assign(const string &s) // IN
 {
-   string copy(s);
-   swap(copy);
-   return *this;
+   return operator=(s);
 }
 
 
@@ -1226,10 +1339,11 @@ string::replace(size_type i,     // IN
  *
  * utf::string::replace --
  *
- *      Replace all occurences of the given string with another string.
+ *      Mutates this string by replacing all occurrences of one string with
+ *      another.
  *
  * Results:
- *      A utf::string created with the substring replaced.
+ *      A reference to this object.
  *
  * Side effects:
  *      None
@@ -1262,6 +1376,30 @@ string::replace(const string &from, // IN
 }
 
 
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * utf::string::replace_copy --
+ *
+ * Results:
+ *      Returns a new string with all occurrences of one string replaced by
+ *      another.
+ *
+ * Side effects:
+ *      None
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+string
+string::replace_copy(const string& from, // IN
+                     const string& to)   // IN
+   const
+{
+   return string(*this).replace(from, to);
+}
+
+
 /*
  *-----------------------------------------------------------------------------
  *
@@ -1433,7 +1571,7 @@ string::rfind(value_type uc,     // IN
  *
  * utf::string::find_first_of --
  *
- *      Find the first occurence of 's' in this string.  'i' determines where in
+ *      Find the first occurrence of 's' in this string.  'i' determines where in
  *      the current string we start searching for 's'
  *
  * Results:
@@ -1470,7 +1608,7 @@ string::find_first_of(value_type uc,   // IN
  *
  * utf::string::find_first_not_of --
  *
- *      Find the first occurence of a string NOT in 's' in this string.  'i'
+ *      Find the first occurrence of a string NOT in 's' in this string.  'i'
  *      determines where in this string we start searching to NOT 's'.
  *
  * Results:
